#include "i_ftdi.h"

#undef _WIN32_WINNT
#define _WIN32_WINNT 0x0502
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#ifndef NO_QT
#include <qglobal.h>
#endif
#ifndef Q_CC_CLANG
#include <malloc.h>
#else
#include <cstdlib>
#endif

#include <string.h>

#if !defined(PRINT_PRINTF) && !defined(PRINT_NO)
#include <QDebug>
#endif
#ifdef PRINT_PRINTF
#include <stdlib.h>
#define qWarning printf
#define qCritical printf
#endif
#ifdef PRINT_NO
#define qWarning(...)
#define qCritical(...)
#endif

#include "ftd2xx.h"

#if defined __cplusplus
extern "C" {
#endif

BYTE ftdi_get_number(WORD vid, WORD pid, WORD *size)
{
    return ftdi_list(vid, pid, size, NULL, 0);
}

BYTE ftdi_list(WORD vid, WORD pid, WORD *size, struct s_info *arr_flags, WORD array_size)
{
    FT_STATUS ftStatus;
    DWORD number = 0;
    WORD devices = 0;

    *size = 0;

#ifndef _WIN32
    ftStatus = FT_SetVIDPID(vid, pid);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_list: FT_SetVIDPID error %lu", ftStatus);
        return RET_FALSE;
    }
#endif

    ftStatus = FT_CreateDeviceInfoList(&number);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_list: FT_CreateDeviceInfoList error %lu", ftStatus);
        return RET_FALSE;
    }

    if (number == 0)
        return RET_TRUE;

    FT_DEVICE_LIST_INFO_NODE *pDest = (FT_DEVICE_LIST_INFO_NODE*)malloc(number*sizeof(FT_DEVICE_LIST_INFO_NODE));
    if (pDest == NULL) {
        qCritical("ftdi_list: malloc failed");
        return RET_FALSE;
    }

    ftStatus = FT_GetDeviceInfoList(pDest, &number);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_list: FT_GetDeviceInfoList error %lu", ftStatus);
        free(pDest);
        return RET_FALSE;
    }

    for (unsigned int i = 0; i < number; i++) {
        if (pDest[i].ID != ((vid << 16) + pid) || (pDest[i].Flags & 1))
            continue;

        if (arr_flags != NULL && devices >= array_size)
            break;

        if (arr_flags != NULL) {
            arr_flags[devices].handle = (void *)devices;
            arr_flags[devices].flags = F_NO;
        }

        devices++;
    }

    free(pDest);
    *size = devices;

    return RET_TRUE;
}
BYTE ftdi_open(void *handle, WORD vid, WORD pid, void **ret_handle)
{
    FT_STATUS ftStatus;
    DWORD number = 0;
    DWORD devices = 0;

#ifndef _WIN32
    ftStatus = FT_SetVIDPID(vid, pid);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_open: FT_SetVIDPID error %lu", ftStatus);
        return RET_FALSE;
    }
#endif

    ftStatus = FT_CreateDeviceInfoList(&number);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_open: FT_CreateDeviceInfoList error %lu", ftStatus);
        return RET_FALSE;
    }

    if (number == 0)
        return RET_FALSE;

    FT_DEVICE_LIST_INFO_NODE *pDest = (FT_DEVICE_LIST_INFO_NODE *)malloc(number*sizeof(FT_DEVICE_LIST_INFO_NODE));
    if (pDest == NULL) {
        qCritical("ftdi_open: malloc failed");
        return RET_FALSE;
    }

    ftStatus = FT_GetDeviceInfoList(pDest, &number);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_open: FT_GetDeviceInfoList error %lu", ftStatus);
        free(pDest);
        return RET_FALSE;
    }

    char *serial = NULL;

    for (unsigned int i = 0; i < number; i++) {
        if (pDest[i].ID != ((vid << 16) + pid) || (pDest[i].Flags & 1))
            continue;

        if ((void *)devices == handle) {
            serial = pDest[i].SerialNumber;
            break;
        }

        devices++;
    }

    if (serial == NULL) {
        qWarning("ftdi_open: target device not found");
        free(pDest);
        return RET_FALSE;
    }

    ftStatus = FT_OpenEx(serial, FT_OPEN_BY_SERIAL_NUMBER, ret_handle);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_open: FT_OpenEx error %lu", ftStatus);
        free(pDest);
        return RET_FALSE;
    }

    free(pDest);

    ftStatus = FT_SetTimeouts(*ret_handle, 1, 1000);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_open: FT_SetTimeouts error %lu", ftStatus);
        return RET_FALSE;
    }

    return RET_TRUE;
}

BYTE ftdi_close(void *handle)
{
    FT_STATUS ftStatus;
    ftStatus = FT_Close(handle);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_close: FT_Close error %lu", ftStatus);
        return RET_FALSE;
    }

    return RET_TRUE;
}

BYTE ftdi_clear(void *handle)
{
    FT_STATUS ftStatus;
    ftStatus = FT_Purge(handle, FT_PURGE_RX | FT_PURGE_TX);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_clear: FT_Purge error %lu", ftStatus);
        return RET_FALSE;
    }

    return RET_TRUE;
}

BYTE ftdi_set_timeout(void *handle, WORD timeout)
{
    FT_STATUS ftStatus;
    ftStatus = FT_SetTimeouts(handle, 1, timeout);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_set_timeout: FT_SetTimeouts error %lu", ftStatus);
        return RET_FALSE;
    }
    return RET_TRUE;
}

//write
BYTE ftdi_write(void *handle, const void *buffer, WORD data_size)
{
    FT_STATUS ftStatus;
    DWORD sent = 0;
    DWORD last_sent;

    while (sent < data_size) {
        ftStatus = FT_Write(handle, (BYTE*)buffer + sent, data_size - sent, &last_sent);
        if (ftStatus != FT_OK) {
            qWarning("ftdi_write: FT_Write error %lu, %ld/%d sent", ftStatus, sent, data_size);
            return RET_FALSE;
        }
        if (last_sent == 0) {
            qWarning("ftdi_write: FT_Write sent 0 bytes, returning");
            return RET_FALSE;
        }
        sent += last_sent;
    }

    return RET_TRUE;
}

//read
BYTE ftdi_read(void *handle, void *buffer, WORD buf_size, DWORD *p_bytes_read)
{
    FT_STATUS ftStatus;
    DWORD null;
    DWORD bytes;

    *p_bytes_read = 0;

    ftStatus = FT_GetStatus(handle, &bytes, &null, &null);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_read: FT_GetStatus error %lu", ftStatus);
        return RET_FALSE;
    }

    if (bytes > buf_size)
        bytes = buf_size;

    if (bytes == 0) return RET_TRUE;

    ftStatus = FT_Read(handle, buffer, bytes, p_bytes_read);
    if (ftStatus != FT_OK) {
        qWarning("ftdi_read: FT_Read error %lu", ftStatus);
        return RET_FALSE;
    }

    return RET_TRUE;
}

#if defined __cplusplus
} // extern "C" {
#endif
