#include "plot_2dplot.h"
#include "plot_2dplot_p.h"
#include "math/plot_MathProcessing.h"

#include "../../src/gui/flowlayout.hpp"
#include <qwt_plot_directpainter.h>
#include <qwt_plot_layout.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_canvas.h>
#include <algorithm>
#include <limits>

#include <QApplication>
#include <QDesktopWidget>
#include <QtEvents>
#include <QColorDialog>
#include <QDrag>
#include <QTimer>
#include <QFileDialog>
#include <QMessageBox>
#include <QSettings>

namespace plot {

const char *ManualFitStr = "Manual fit";

void TCalcLimitsByFrameContent::operator ()(TInfoForAllPlots *InfoForAll, TCurves *Src, const int ApplyTo, int N)
{
    PlotInfo = InfoForAll;
    Curves = Src;
    MinPlotsX = PlotInfo->CurrentXLimits[0];
    MaxPlotsX = PlotInfo->CurrentXLimits[1];
    MinPlotsY = PlotInfo->CurrentYLimits[0];
    MaxPlotsY = PlotInfo->CurrentYLimits[1];

    Clear();
    CalcScrLimits(N);

    if (ApplyTo == TPlotEnums::A_Both || ApplyTo == TPlotEnums::A_XAxis) {
        PlotInfo->CurrentXLimits[0] = min(XMins);
        PlotInfo->CurrentXLimits[1] = max(XMaxs);
    }

    if (ApplyTo == TPlotEnums::A_Both || ApplyTo == TPlotEnums::A_YAxis) {
        PlotInfo->CurrentYLimits[0] = min(YMins);
        PlotInfo->CurrentYLimits[1] = max(YMaxs);
    }
}
inline void TCalcLimitsByFrameContent::Clear()
{
    XMins.clear(); XMaxs.clear();
    YMins.clear(); YMaxs.clear();
}
void TCalcLimitsByFrameContent::CalcScrLimits(int N)
{
    if (N < 0)
        for (int i = 0; i < Curves->QwtCurves.size(); i++)
            CalcScrLimits(*Curves->QwtCurves.at(i));
    else
        CalcScrLimits(*Curves->QwtCurves.at(N));
}
inline bool TCalcLimitsByFrameContent::IsIn(const QwtPlotCurve &CurveData, const int Index) const
{
    return CurveData.sample(Index).x() >= MinPlotsX && CurveData.sample(Index).x() <= MaxPlotsX;
}
void TCalcLimitsByFrameContent::CalcScrLimits(const QwtPlotCurve &CurveData)
{
    //TODO in-place
    QVector<double> X, Y;
    for (int i = 0; i < CurveData.dataSize(); i++) {
        if (IsIn(CurveData, i)) {
            //X.push_back(CurveData.sample(i).x());
            Y.push_back(CurveData.sample(i).y());
        }
    }

    XMins.push_back(CurveData.sample(0).x()); // min(X);
    XMaxs.push_back(CurveData.sample(CurveData.dataSize() - 1).x()); // max(X);
    QPair<double, double> _minmax = minmax(Y);
    YMins.push_back(_minmax.first);
    YMaxs.push_back(_minmax.second);
}

//================================================================================================================================
void T2dPlotControl::GUICreate()
{
    TCommonPlotControl::GUICreate();

    BAutoScaleX = new QPushButton;
    BAutoScaleX->setIcon(makeHugeIcon(":/plot/AutoscX_ones.png"));
    BAutoScaleX->setToolTip(QObject::tr("Autoscale in X axis"));

    BAutoScaleY = new QPushButton;
    BAutoScaleY->setIcon(makeHugeIcon(":/plot/AutoscY_ones.png"));
    BAutoScaleY->setToolTip(QObject::tr("Autoscale in Y axis"));

    TBCurveThickness = new QToolButton;
    TBCurveThickness->setIcon(makeHugeIcon(":/plot/line_thickness.png"));
    TBCurveThickness->setToolTip(QObject::tr("Thickness of line"));
    TBCurveThickness->setPopupMode(QToolButton::InstantPopup);
    QMenu *MCurveThickness = new QMenu;
    TBCurveThickness->setMenu(MCurveThickness);

    TBCurveColour = new QToolButton;
    TBCurveColour->setIcon(makeHugeIcon(":/plot/linecolor.png"));
    TBCurveColour->setToolTip(QObject::tr("Colour of line"));
    TBCurveColour->setPopupMode(QToolButton::InstantPopup);
    QMenu *MCurveColour = new QMenu;
    TBCurveColour->setMenu(MCurveColour);

    CreateActionsAndMenus();

    CBChooseActiveCurve = new TComboBox;
    CBChooseActiveCurve->addItem("All");

    BResetMCCurve = new QPushButton;
    BResetMCCurve->setIcon(makeHugeIcon(":/plot/Reset.png"));
    BResetMCCurve->setToolTip(tr("Reset manual curve fit"));
    BResetMCCurve->hide();

    if (!show_additional_gui()) {
        TBCurveColour->hide();
        TBCurveThickness->hide();
        BAutoScaleX->hide();
        BAutoScaleY->hide();
    }
    if (!show_advanced_gui()) {
        CBChooseActiveCurve->hide();
    }

    ControlMainLayout->insertWidget(7, BAutoScaleX);
    ControlMainLayout->insertWidget(8, BAutoScaleY);

    addDelimeterH(ControlMainLayout, BAutoScaleX->sizeHint().width());
    ControlMainLayout->addWidget(CBChooseActiveCurve);
    ControlMainLayout->addWidget(TBCurveColour);
    ControlMainLayout->addWidget(TBCurveThickness);
    ControlMainLayout->addWidget(TBFontSize);
    //addDelimeterH(ControlMainLayout, BAutoScaleX->sizeHint().width());

    ControlMainLayout->addWidget(BCancel);
    ControlMainLayout->addWidget(BApply);
    ControlMainLayout->addWidget(BResetMCCurve);

    setTitle("2D");
}

void T2dPlotControl::CreateActionsAndMenus()
{
    AGFitOrders = new QActionGroup(this);
    const char *FitOrders[] = {"Fit. None", "Fit. 0 order", "Fit. 1 order", "Fit. 2 order", "Fit. 3 order", "Fit. 4 order", "Fit. 5 order", ManualFitStr};
    for (unsigned int i = 0; i < sizeof(FitOrders) / sizeof(FitOrders[0]); i++)
    {
        AGFitOrders->addAction(MMathProcessing->addAction(tr(FitOrders[i])));
        AGFitOrders->actions()[i]->setCheckable(true);
    }
    AGFitOrders->actions()[0]->setChecked(true);

    AGCurveColours = new QActionGroup(this);
    const char *CurveColours[] = {"Black", "Red", "Green", "Blue", "Magenta", "Cyan", "Other..."};
    for (unsigned int i = 0; i < sizeof(CurveColours)/sizeof(CurveColours[0]); i++)
    {
        AGCurveColours->addAction(TBCurveColour->menu()->addAction(tr(CurveColours[i])));
        AGCurveColours->actions()[i]->setCheckable(true);
    }
    AGCurveColours->actions()[0]->setChecked(true);

    AGCurveThickness = new QActionGroup(this);
    const char *CurveThickness[] = {"1", "2", "3", "4", "5", "6", "8", "10", "15", "20"};
    for (unsigned int i = 0; i < sizeof(CurveThickness)/sizeof(CurveThickness[0]); i++)
    {
        AGCurveThickness->addAction(TBCurveThickness->menu()->addAction(tr(CurveThickness[i])));
        AGCurveThickness->actions()[i]->setCheckable(true);
    }
    AGCurveThickness->actions()[0]->setChecked(true);
}

//================================================================================================================================
//================================================================================================================================

TPure2dPlotPrivate::TPure2dPlotPrivate()
{
    AutoScaleEnabled = true;
    ZoomBySelection = true;
    SelectedPointForMC = -1;
    State = TPure2dPlot::Mode_Normal;
}
QPointF TPure2dPlotPrivate::FindAltitude(const QPointF &Click, const QPointF &Closest, const QPointF &p) const
{
    QPointF Clk = Click - Closest;
    QPointF P = p - Closest;
    double Alpha = (Clk.x()*P.x() + Clk.y()*P.y()) / (P.x()*P.x() + P.y()*P.y());
    return P*Alpha + Closest;
}
bool TPure2dPlotPrivate::PointBelongToLine(const QPointF &P, const QLineF &Line) const
{
    if (P.x() > qMax(Line.x1(), Line.x2()) || P.x() < qMin(Line.x1(), Line.x2()) ||
       P.y() > qMax(Line.y1(), Line.y2()) || P.y() < qMin(Line.y1(), Line.y2())) return false;
    else return true;
}
QPointF TPure2dPlotPrivate::GetIntersectPoint(const QwtPlotCurve *Curve, const QPoint &ScrPos, int &Idx) const
{
    if (!Curve) return QPointF();
    if (Curve->dataSize() < 2) return QPointF();

    Q_Q(const TPure2dPlot);
    double minv = std::numeric_limits<double>::max();
    int minidx = -1;
    for (int i = 0; i < Curve->dataSize() - 1; i++)
    {
        QPointF p1 = q->PlotToScreen(Curve->sample(i));
        QPointF p2 = q->PlotToScreen(Curve->sample(i+1));
        const QPointF AltIntersection = FindAltitude(ScrPos, p1, p2);
        if (PointBelongToLine(AltIntersection, QLineF(p1, p2))) {
            double v = GetDist(ScrPos, AltIntersection);
            if (v < minv) {
                minv = v;
                minidx = i;
            }
        }
    }
    if (minidx == -1) return QPointF();

    Idx = minidx;
    QPointF p1 = q->PlotToScreen(Curve->sample(Idx));
    QPointF p2 = q->PlotToScreen(Curve->sample(Idx+1));
    QPointF p3 = FindAltitude(ScrPos, p1, p2);
    return q->ScreenToPlot(p3.toPoint());
}

bool TPure2dPlot::isMathReady()
{
    return !MathProcessing->isRunning();
}
TPure2dPlot::~TPure2dPlot()
{
    delete d_ptr;
    delete MathProcessing;
}
TPure2dPlot::TPure2dPlot(QObject *_Parent, QWidget *parent) : TCommonPlot(parent), TCurves(QPlot), d_ptr(new TPure2dPlotPrivate)
{
    Parent = _Parent;

    Q_D(TPure2dPlot);
    d->q_ptr = this;
    FitOrder = TPlotEnums::FO_None;

    MathProcessing = new math::TMathProcessing(&MathData);
    MathProcessing->MCCurve = MCCurve;
    connect(MathProcessing, SIGNAL(finished()), this, SLOT(s_Refresh()), Qt::QueuedConnection);

    setAcceptDrops(true);
    ((QWidget *)QPlot->titleLabel())->setToolTip(tr("Drag & Drop as data to plots, as image"));
}

void TPure2dPlot::paintEvent(QPaintEvent *e)
{
    TCommonPlot::paintEvent(e);
    incRefreshGuard = true;
}
void TPure2dPlot::RefreshInc()
{
    if (blockRefresh)
        return;

    Q_D(TPure2dPlot);
    QwtPlotDirectPainter p;
    bool full_refresh = d->AutoScaleEnabled ||
            FitOrder != TPlotEnums::FO_None ||
            MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt() >= 0;
    incRefreshGuard = false;
    for (int i = 0; i < QwtCurves.size() && !full_refresh && !incRefreshGuard; i++) {
        if (PaintData.at(i)) {
            int from = LastPaintSize.at(i);
            int size = PaintData.at(i).data()->size();
            LastPaintSize[i] = size;
            if (from > size) {
                full_refresh = true;
                break;
            }
            if (from != size)
                p.drawSeries(QwtCurves.at(i), qMax(0, from-1), size);
        }
    }
    if (full_refresh)
        Refresh();
    else if (incRefreshGuard)
        //QPlot->canvas()->replot(); ///Replot();
        QPlot->canvas()->update(); //update including frame, not only contentrect
}

void TPure2dPlot::Refresh()
{
    if (blockRefresh)
        return;

    if (FitOrder == TPlotEnums::FO_None && MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt() < 0) {
        PaintData = SourceData;
        ReSyncDataAndQwtCurves();

        MathProcessing->wait();
        MathData.clear();

        s_Refresh();
    } else if (!MathProcessing->isRunning()) {
        MathData.clear();
        ResizeOwner(MathData, SourceData.size());
        PaintData = getUnLock(MathData);

        for (int i = 0; i < SourceData.size(); i++)
            if (SourceData[i]) {
                *MathData[i] = *SourceData[i].data();
                MathData[i]->detach();
            }
        ReSyncDataAndQwtCurves();

        if (FitOrder != TPlotEnums::FO_None)
            MathProcessing->DoFit(FitOrder);

        if (MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt() >= 0)
            MathProcessing->DoScale(MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt());

        MathProcessing->start();
    }
}
void TPure2dPlot::s_Refresh()
{
    if (blockRefresh)
        return;

    ReSetAxisScales();
    ReAssignPlotOptions();
    Replot();

    PlotWasChanged();
    emit MathUpdated();
}

void TPure2dPlot::setVertLineVisible(int N, bool v)
{
    VertLines[N]->setVisible(v);
    Replot();
}
void TPure2dPlot::MoveVertLine(double XPos, int N)
{
    VertLines[N]->setXValue(XPos);
    Replot();
}
void TPure2dPlot::SetVertLinePen(QPen &Pen, int N)
{
    VertLines[N]->setLinePen(Pen);
    Replot();
}
void TPure2dPlot::SetNumOfVertLines(int N)
{
    VertLines.resize(N);

    for (int i = 0; i < VertLines.size(); i++) {
        VertLines[i] = new QwtPlotMarker;
        VertLines[i]->setXValue(0);
        VertLines[i]->setLinePen(QPen(QColor(0, 0, 255), 0, Qt::SolidLine));
        VertLines[i]->setLineStyle(QwtPlotMarker::VLine);
        VertLines[i]->attach(QPlot);
        VertLines[i]->hide();
    }
}
void TPure2dPlot::DefineClickInfo(const QPoint &Pos)
{
    Q_D(TPure2dPlot);
    if (d->State == TPure2dPlot::Mode_ManualConstruct) {
        d->SelectedPointForMC = -1;

        if (MCCurve->dataSize() <= 1)
            return;

        if ((d->SelectedPointForMC = MCCurve->closestPoint(Pos, 0)) == -1)
            return;

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)) && !defined(_ADROID)
    double scale = qApp->desktop()->logicalDpiX();
#else
    double scale = qApp->desktop()->physicalDpiX();
#endif

    int Range = 1.5*qMax<int>(20, 0.25 * scale);
        if ((PlotToScreen(MCCurve->sample(d->SelectedPointForMC)) - Pos).manhattanLength() > Range)
        {
            int Idx;
            QPointF p2 = d->GetIntersectPoint(MCCurve, Pos, Idx);
            if ((PlotToScreen(p2) - Pos).manhattanLength() < Range)
                AddPointToMCCurve(Idx, p2);
            else { d->SelectedPointForMC = -1; TCommonPlot::DefineClickInfo(Pos); }
        }
    }
    else TCommonPlot::DefineClickInfo(Pos);
}
void TPure2dPlot::dragEnterEvent(QDragEnterEvent *event)
{
    const T2dMime *data = qobject_cast<const T2dMime *>(event->mimeData());
    if (data && event->source() != this)
        event->acceptProposedAction();
}
void TPure2dPlot::dropEvent(QDropEvent *event)
{
    const T2dMime *data = qobject_cast<const T2dMime *>(event->mimeData());
    if (!data || event->source() == this) return;
    SourceData << data->data;
    Refresh();
}
bool TPure2dPlot::eventFilter(QObject *Object, QEvent *Event)
{
    if (!isEnabled()) {
        Action = ExternalAct;
    } else
    if (Enabled)
    {
        Range PlotXLimits, PlotYLimits;
        PlotXLimits[0] = qMin(DefaultXLimits[0], PlotsXLimits[0]);
        PlotXLimits[1] = qMax(DefaultXLimits[1], PlotsXLimits[1]);
        PlotYLimits[0] = qMin(DefaultYLimits[0], PlotsYLimits[0]);
        PlotYLimits[1] = qMax(DefaultYLimits[1], PlotsYLimits[1]);

        PlotXLimits[0] -= (PlotXLimits[1] - PlotXLimits[0]);
        PlotXLimits[1] += (PlotXLimits[1] - PlotXLimits[0])/2.;
        PlotYLimits[0] -= (PlotYLimits[1] - PlotYLimits[0]);
        PlotYLimits[1] += (PlotYLimits[1] - PlotYLimits[0])/2.;

        if (Event->type() == QEvent::Wheel) {
            QPointF Pos = mapToCanvas(((QWheelEvent *)Event)->pos());
            Pos = ScreenToPlot(Pos.toPoint());
            ZoomPlot(Pos, ((QWheelEvent *)Event)->delta(), &PlotXLimits, &PlotYLimits);
        } else if (Event->type() == QEvent::MouseButtonPress) {
            QPoint Pos = mapToCanvas(((QMouseEvent *)Event)->pos());
            DefineClickInfo(Pos);
            if (Action == Panning && QPlot->plotLayout()->titleRect().contains(((QMouseEvent *)Event)->pos())) {
                Action = ExternalAct;

                QDrag *drag = new QDrag(this);
                T2dMime *mimeData = new T2dMime;
                mimeData->data = ActiveCurve < 0 || ActiveCurve >= SourceData.size() ?
                            SourceData : (TCurvesData() << SourceData.at(ActiveCurve));
                mimeData->setImageData(QPixmap::grabWidget(this).toImage());
                drag->setMimeData(mimeData);
                drag->setPixmap(QPixmap::fromImage(MakePreview()));
                drag->exec(Qt::CopyAction);
            }
        } else if (Event->type() == QEvent::MouseButtonRelease) {
            Action = ExternalAct;
        } else if (Event->type() == QEvent::MouseMove) {
            QPoint Pos = mapToCanvas(((QMouseEvent *)Event)->pos());

            Q_D(TPure2dPlot);
            if (d->State == TPure2dPlot::Mode_ManualConstruct && d->SelectedPointForMC != -1)
                MoveSelectedPointInMCCurve(Pos);
            else if (Action == TCommonPlot::MoveC)
                MoveMarker(Pos.x(), Pos.y(), false);
            else if (Action == MoveA || Action == MoveB)
                TraceCurve(Pos, ActiveCurve);
            else if (Action == Panning) {
                double kx = (ScreenXToPlotX(20) - ScreenXToPlotX(0)) / 20.;
                double ky = (ScreenYToPlotY(20) - ScreenYToPlotY(0)) / 20.;
                QPointF Delta((Pos.x() - ClickPos.x())*kx,
                              (Pos.y() - ClickPos.y())*ky);
                PanPlot(Delta, &PlotXLimits, &PlotYLimits);
            }
        }
    }
    if (Event->type() == QEvent::MouseButtonPress)
        emit ShowPlotControl();

    return QWidget::eventFilter(Object, Event);
}
void TPure2dPlot::MoveSelectedPointInMCCurve(const QPoint &Pos)
{
    Q_D(TPure2dPlot);

    int sz = MCCurve->dataSize();
    if (sz <= 1)
        return;
//((QwtPointSeriesData *)MCCurve->data())->samples()[d->SelectedPointForMC] = Pos;

    double *x = new double [sz];
    double *y = new double [sz];

    for (int i = 0; i < sz; i++) {
        if (i == d->SelectedPointForMC) {
            x[i] = ScreenXToPlotX(Pos.x());
            y[i] = ScreenYToPlotY(Pos.y());
        } else {
            x[i] = MCCurve->sample(i).x();
            y[i] = MCCurve->sample(i).y();
        }
    }

    MCCurve->setSamples(x, y, sz);

    delete [] x;
    delete [] y;

    Replot();
}
void TPure2dPlot::SetActiveCurve(int Active)
{
    TCurves::SetActiveCurve(Active);
    emit sync_active_curve(ActiveCurve);
}
void TPure2dPlot::TraceCurve(const QPoint &Pos, int NOfCurve)
{
    if (NOfCurve < 0)
        MoveMarker(Pos.x(), Pos.y(), false);
    else {
        double x = 0., y = 0.;
        GetClosestPoint(&x, &y, Pos, NOfCurve);
        MoveMarker(x, y, true);
    }
}

void TPure2dPlot::on_BAutoScaleX_clicked()
{
    LimitsStretcher(this, this, TPlotEnums::A_XAxis, ActiveCurve);
    SetCurrentXScales();
    HandleAxisMultiplier();
    ReAssignPlotOptions();
    Replot();
}
void TPure2dPlot::on_BAutoScaleY_clicked()
{
    LimitsStretcher(this, this, TPlotEnums::A_YAxis, ActiveCurve);
    SetCurrentYScales();
    HandleAxisMultiplier();
    ReAssignPlotOptions();
    Replot();
}
void TPure2dPlot::ReSetAxisScales()
{
    Q_D(TPure2dPlot);
    if (d->AutoScaleEnabled) {
        if (ActiveCurve < PaintData.size()) {
            QVector<double> MaxX, MaxY, MinX, MinY;
            DefineLimits(MaxX, MaxY, MinX, MinY, ActiveCurve);
            if (MinX.size() && MaxX.size()) {
                Range TmpXLimits;
                TmpXLimits[0] = *std::min_element(MinX.begin(), MinX.end());
                TmpXLimits[1] = *std::max_element(MaxX.begin(), MaxX.end());

                if (TmpXLimits[0] == std::numeric_limits<double>::max() && TmpXLimits[1] == -std::numeric_limits<double>::max()) {
                    HandleAxisMultiplier();
                    return;
                }

                HandleZeroDiff(TmpXLimits[1], TmpXLimits[0]);
                PlotsXLimits = TmpXLimits;
            }

            if (MinY.size() && MaxY.size()) {
                Range TmpYLimits;
                TmpYLimits[0] = *std::min_element(MinY.begin(), MinY.end());
                TmpYLimits[1] = *std::max_element(MaxY.begin(), MaxY.end());

                if (TmpYLimits[0] == std::numeric_limits<double>::max() && TmpYLimits[1] == -std::numeric_limits<double>::max()) {
                    HandleAxisMultiplier();
                    return;
                }

                HandleZeroDiff(TmpYLimits[1], TmpYLimits[0]);
                PlotsYLimits = TmpYLimits;
            }
        }

        TPlotCore::ReSetAxisScales();
    }
    else HandleAxisMultiplier();
}
void TPure2dPlot::SetFitOrder(int Order)
{
    FitOrder = Order;
}

void TPure2dPlot::SetAutoScaleEnabled(bool Enabled)
{
    Q_D(TPure2dPlot);
    d->AutoScaleEnabled = Enabled;
    //TODO plotslimits = defaultplotslimits
    Refresh();
}
void TPure2dPlot::SelectionApplied()
{
    Q_D(TPure2dPlot);
    if (d->ZoomBySelection)
    {
        QRectF Rect = GetSelectedRect();
        SetCurrentXScales(Rect.x(), Rect.right());
        SetCurrentYScales(Rect.y(), Rect.bottom());
        Replot();
    }
}
void TPure2dPlot::EnableSelectMode(const bool st)
{
    Q_D(TPure2dPlot);
    QPlot->setVisible(!st);
    SetSelectionModeEnabled(st);
    setFocus(Qt::MouseFocusReason);
    d->State = st ? Mode_Select : Mode_Normal;
}

void TPure2dPlot::SelectRectChanged(const QRectF &Rect)
{
    if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(Rect, TSelectEvent::St_SelectChanged, (TSelectEvent::TSelectRectChangedOrigins)SelectRectEventOrigin));
    SelectRectEventOrigin = TSelectEvent::SRO_Unknown;
}


void TPure2dPlot::HandleAxisMultiplier()
{
    XAxisUnit.UpdateLegend(CurrentXLimits[0], CurrentXLimits[1]);
    YAxisUnit.UpdateLegend(CurrentYLimits[0], CurrentYLimits[1]);
    UpdateAxisLegends();
}
void TPure2dPlot::UpdateAxisLegends()
{
    UpdateLabels();
    TInfoFor2dPlot::UpdateAxisLegends();

    QPlot->axisScaleDraw(QwtPlot::xBottom)->Multiplier = AutoLegendEnabled ? XAxisUnit.Multiplier : 1.;
    QPlot->axisScaleDraw(QwtPlot::yLeft)->Multiplier = AutoLegendEnabled ? YAxisUnit.Multiplier : 1.;
}
void TPure2dPlot::UpdateLabels(bool IsDetailedInfoVisible)
{
    Q_D(TPure2dPlot);
    d->AMarkerInfo(AutoLegendEnabled, XAxisUnit, YAxisUnit, MarkerA->xValue(), MarkerA->yValue());
    d->BMarkerInfo(AutoLegendEnabled, XAxisUnit, YAxisUnit, MarkerB->xValue(), MarkerB->yValue());
    TValueInfoCreator difx, dify;

    QString AText, BText, MText;
    if (IsDetailedInfoVisible) { AText = "A"; BText = "B"; }
    else
    {
        AText = d->AMarkerInfo.FormUniteString();
        BText = d->BMarkerInfo.FormUniteString();

        difx(AutoLegendEnabled, XAxisUnit, d->BMarkerInfo.XInfo.SrcValue - d->AMarkerInfo.XInfo.SrcValue);
        dify(AutoLegendEnabled, YAxisUnit, d->BMarkerInfo.YInfo.SrcValue - d->AMarkerInfo.YInfo.SrcValue);
        MText = QString(QChar(0x2206)) + "X = " + difx.Value + difx.Unit + '\n' +
                QString(QChar(0x2206)) + "Y = " + dify.Value + dify.Unit;
    }

    QwtText QwtAText(AText),
            QwtBText(BText),
            QwtMText(MText);

    QFont font = qApp->font(); font.setPointSize(FontSize);
    QwtAText.setFont(font);
    QwtBText.setFont(font);
    QwtMText.setFont(font);

    QwtAText.setColor(QColor(190, 255, 190));
    QwtBText.setColor(QColor(190, 255, 190));
    QwtMText.setColor(QColor(190, 255, 190));

    MarkerA->setLabel(QwtAText);
    MarkerB->setLabel(QwtBText);
    MarkerM->setLabel(QwtMText);
}
void TPure2dPlot::ResetMCCurve()
{
    int idx = qMax(0, ActiveCurve);
    if (!QwtCurves.size() || QwtCurves.at(idx)->dataSize() <= 1)
        return;

    MCCurve->show();
    MCCurve->setSamples(QVector<QPointF>() <<
                        QwtCurves.at(idx)->sample(0) <<
                        QwtCurves.at(idx)->sample(
                            QwtCurves.at(idx)->dataSize() - 1));

    Replot();
}
void TPure2dPlot::StartMCMode()
{
    Q_D(TPure2dPlot);
    d->State = TPure2dPlot::Mode_ManualConstruct;

    if (MCCurve->dataSize()) {
        MCCurve->show();
        Replot();
    }
    else
        ResetMCCurve();
}
void TPure2dPlot::SetZoomBySelection(const bool State)
{
    Q_D(TPure2dPlot);
    d->ZoomBySelection = State;
}
void TPure2dPlot::AddPointToMCCurve(const int After, const QPointF &p2)
{
    Q_D(TPure2dPlot);

    int i = 0;
    double *x = new double [MCCurve->dataSize() + 1], *y = new double [MCCurve->dataSize() + 1];

    //TODO QVector.insert?
    for (; i <= After; i++) {
        x[i] = MCCurve->sample(i).x();
        y[i] = MCCurve->sample(i).y();
    }
    x[i] = p2.x();
    y[i] = p2.y();
    d->SelectedPointForMC = i;

    for (; i < MCCurve->dataSize(); i++) {
        x[i + 1] = MCCurve->sample(i).x();
        y[i + 1] = MCCurve->sample(i).y();
    }

    MCCurve->setSamples(x, y, MCCurve->dataSize() + 1);
    delete [] x; delete [] y;

    Replot();
}
void TPure2dPlot::ExitMCMode()
{
    MCCurve->hide();
    Replot();

    Q_D(TPure2dPlot);
    d->State = TPure2dPlot::Mode_Normal;
}
TCurveData TPure2dPlot::GetMCCurve() const
{
    TCurveData Result;

    for (int i = 0; i < MCCurve->dataSize(); i++)
        Result.AddPoint(MCCurve->sample(i).x(), MCCurve->sample(i).y());

    return Result;
}

//================================================================================================================================
T2dPlot::T2dPlot(QObject *_Parent, bool child, QWidget *wparent) : QWidget(wparent)
{
    Parent = _Parent;
    GUICreate(child);

    connect(Pure2dPlot, SIGNAL(UpdateAllMarkersInfo()), this, SLOT(UpdateAllMarkersInfo()));

    SyncTimer = new QTimer(this); //TODO replace!!!
    SyncTimer->start(850);
    connect(SyncTimer,  SIGNAL(timeout()),              this, SLOT(on_SyncTimer_timeout()));
}
void T2dPlot::ConnectPlotControlSlots()
{
    PlotControl->BAutoScale->setChecked(true);

    connect(PlotControl->BAutoScaleX, SIGNAL(clicked()), Pure2dPlot, SLOT(on_BAutoScaleX_clicked()));
    connect(PlotControl->BAutoScaleY, SIGNAL(clicked()), Pure2dPlot, SLOT(on_BAutoScaleY_clicked()));
    connect(PlotControl->BRestoreScale, SIGNAL(clicked()), Pure2dPlot, SLOT(BRestoreScale_clicked()));
    connect(PlotControl->BExpandSelectRect, SIGNAL(clicked()), Pure2dPlot, SLOT(on_BExpandSelectRect_clicked()));

    connect(PlotControl->BMarkers, SIGNAL(toggled(bool)), this, SLOT(on_BMarkers_clicked(bool)));
    connect(PlotControl->BDetailedMarkersInfo, SIGNAL(toggled(bool)), this, SLOT(on_BDetailedMarkersInfo_clicked()));
    connect(PlotControl->BGrid, SIGNAL(toggled(bool)), Pure2dPlot, SLOT(BGrid_clicked(bool)));
    connect(PlotControl->BFixX, SIGNAL(toggled(bool)), this, SLOT(on_BFixX_toggled()));
    connect(PlotControl->BFixY, SIGNAL(toggled(bool)), this, SLOT(on_BFixY_toggled()));

    connect(PlotControl->BSelectByRect, SIGNAL(clicked()), this, SLOT(on_BSelectByRect_clicked()));
    connect(PlotControl->BCancel, SIGNAL(clicked()), this, SLOT(on_BCancel_clicked()));
    connect(PlotControl->BApply, SIGNAL(clicked()), this, SLOT(on_BApply_clicked()));

    connect(PlotControl->BAutoScale, SIGNAL(toggled(bool)), this, SLOT(on_BAutoScale_toggled()));
    connect(PlotControl->BSavePlotPixmap, SIGNAL(clicked()), this, SLOT(on_BSavePlotPixmap_clicked()));
    connect(PlotControl->BSave,  SIGNAL(clicked()), this, SLOT(on_BSave_clicked()));

    connect(PlotControl->CBChooseActiveCurve, SIGNAL(highlighted(int)), this, SLOT(on_CBChooseActiveCurve_highlighted(int)));
    connect(PlotControl->CBChooseActiveCurve, SIGNAL(activated(int)), this, SLOT(on_CBChooseActiveCurve_activated(int)));

    connect(PlotControl->BResetMCCurve, SIGNAL(clicked()), Pure2dPlot, SLOT(ResetMCCurve()));

    connect(PlotControl->AGCurveColours, SIGNAL(triggered(QAction*)), this, SLOT(on_CurveColour_changed()));
    connect(PlotControl->AGCurveThickness, SIGNAL(triggered(QAction*)), this, SLOT(on_CurveThickness_changed()));
    connect(PlotControl->AGFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitOrder_changed()));
    connect(PlotControl->AGFontSizes, SIGNAL(triggered(QAction*)), this, SLOT(on_FontSize_changed()));
}
void T2dPlot::GUICreate(bool child)
{
    Pure2dPlot = new TPure2dPlot(Parent, this);
    PlotControl = new T2dPlotControl;
    MarkersInfo = new TMarkersInfo;

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->setContentsMargins(0, 3, 3, 0);
    MainLayout->addWidget(MarkersInfo, 0, Qt::AlignTop);
    MainLayout->addWidget(Pure2dPlot);

    MarkersInfo->GUICreate();
    MarkersInfo->LAngleLbl->setText(tr("dY/dX"));
    MarkersInfo->LAngleUnit->setText("     ");

    if (!child) {
        MainLayout->addWidget(PlotControl);
        PlotControl->GUICreate();

        FillCBChooseActiveCurve();
        ConnectPlotControlSlots();
    } else {
        Pure2dPlot->SetActiveCurve(-1);
    }

    connect(Pure2dPlot, SIGNAL(sync_active_curve(int)), this, SLOT(sync_active_curve(int)), Qt::QueuedConnection);
}
void T2dPlot::FillCBChooseActiveCurve()
{
    PlotControl->CBChooseActiveCurve->blockSignals(true);
    PlotControl->CBChooseActiveCurve->clear();

    for (int i = 0; i < Pure2dPlot->QwtCurves.size(); i++)
        PlotControl->CBChooseActiveCurve->addItem(QString::number(i+1), i);

    PlotControl->CBChooseActiveCurve->addItem(tr("All"), QVariant(-1));
    PlotControl->CBChooseActiveCurve->blockSignals(false);

    int idx = (PlotControl->CBChooseActiveCurve->count() <= 2) ? 0 : (PlotControl->CBChooseActiveCurve->count() - 1);
    idx = PlotControl->CBChooseActiveCurve->itemData(idx).toInt();
    Pure2dPlot->SetActiveCurve(idx);

    bool Enabled = (Pure2dPlot->QwtCurves.size() != 0);
    PlotControl->CBChooseActiveCurve->setEnabled(Enabled);
    PlotControl->BAutoScaleX->setEnabled(Enabled);
    PlotControl->BAutoScaleY->setEnabled(Enabled);
    PlotControl->TBCurveColour->setEnabled(Enabled);
    PlotControl->TBCurveThickness->setEnabled(Enabled);
}
void T2dPlot::UpdateAllMarkersInfo()
{
    if (!PlotControl->BMarkers->isChecked())
        return;

    Pure2dPlot->UpdateLabels(PlotControl->BDetailedMarkersInfo->isChecked());
    if (!PlotControl->BDetailedMarkersInfo->isChecked())
        return;

    MarkersInfo->LEdXA->setText  (Pure2dPlot->d_func()->AMarkerInfo.XInfo.Value);
    MarkersInfo->LXAUnit->setText(Pure2dPlot->d_func()->AMarkerInfo.XInfo.Unit);
    MarkersInfo->LEdYA->setText  (Pure2dPlot->d_func()->AMarkerInfo.YInfo.Value);
    MarkersInfo->LYAUnit->setText(Pure2dPlot->d_func()->AMarkerInfo.YInfo.Unit);
    MarkersInfo->LEdXB->setText  (Pure2dPlot->d_func()->BMarkerInfo.XInfo.Value);
    MarkersInfo->LXBUnit->setText(Pure2dPlot->d_func()->BMarkerInfo.XInfo.Unit);
    MarkersInfo->LEdYB->setText  (Pure2dPlot->d_func()->BMarkerInfo.YInfo.Value);
    MarkersInfo->LYBUnit->setText(Pure2dPlot->d_func()->BMarkerInfo.YInfo.Unit);

    MarkersInfo->LEdQ->setText(QString::number((int)fabs(0.5*(Pure2dPlot->d_func()->AMarkerInfo.XInfo.SrcValue+
                                                              Pure2dPlot->d_func()->BMarkerInfo.XInfo.SrcValue)/
                                                         (Pure2dPlot->d_func()->AMarkerInfo.XInfo.SrcValue-
                                                          Pure2dPlot->d_func()->BMarkerInfo.XInfo.SrcValue)) ));

    double xdist = Pure2dPlot->d_func()->BMarkerInfo.XInfo.SrcValue - Pure2dPlot->d_func()->AMarkerInfo.XInfo.SrcValue;
    double ydist = Pure2dPlot->d_func()->BMarkerInfo.YInfo.SrcValue - Pure2dPlot->d_func()->AMarkerInfo.YInfo.SrcValue;

    TValueInfoCreator ValueInfo;
    ValueInfo(Pure2dPlot->AutoLegendEnabled, Pure2dPlot->XAxisUnit, xdist);
    MarkersInfo->LEdDeltaX->setText(ValueInfo.Value); MarkersInfo->LDeltaXUnit->setText(ValueInfo.Unit);

    ValueInfo(Pure2dPlot->AutoLegendEnabled, Pure2dPlot->YAxisUnit, ydist);
    MarkersInfo->LEdDeltaY->setText(ValueInfo.Value); MarkersInfo->LDeltaYUnit->setText(ValueInfo.Unit);

    ValueInfo(Pure2dPlot->AutoLegendEnabled, Pure2dPlot->XAxisUnit, sqrt(xdist * xdist + ydist * ydist));
    MarkersInfo->LEdDistance->setText(ValueInfo.Value);

    TSIUnit Temp;
    if (Pure2dPlot->YAxisUnit.SIUnit != Pure2dPlot->XAxisUnit.SIUnit) { //prevents both null too
        Temp.SetSIUnit(Pure2dPlot->YAxisUnit.SIUnit + " / " + Pure2dPlot->XAxisUnit.SIUnit);
        Temp.UpdateMultiplier(ydist / xdist);
    }
    ValueInfo(Pure2dPlot->AutoLegendEnabled, Temp, ydist / xdist);
    MarkersInfo->LEdAngle->setText(ValueInfo.Value);
    MarkersInfo->LAngleUnit->setText(ValueInfo.Unit);
}

void T2dPlot::on_SyncTimer_timeout()
{
    if (Pure2dPlot->QwtCurves.size() + 1 != PlotControl->CBChooseActiveCurve->count())
        FillCBChooseActiveCurve();
}
void T2dPlot::on_FitOrder_changed()
{
    int i = FindCheckedItemIn(PlotControl->AGFitOrders);
    if (PlotControl->AGFitOrders->actions()[i]->text() == QString(ManualFitStr))
    {
        StartMCCurveConstructing();
        Pure2dPlot->SetFitOrder(TPlotEnums::FO_Manual);
    } else {
        Pure2dPlot->SetFitOrder(i - 1);
        Pure2dPlot->Refresh();
    }
}
void T2dPlot::on_CurveColour_changed()
{
    QColor Colour;
    if (PlotControl->AGCurveColours->checkedAction()->text() == "Black") Colour = Qt::black;
    else if (PlotControl->AGCurveColours->checkedAction()->text() == "Red") Colour = Qt::red;
    else if (PlotControl->AGCurveColours->checkedAction()->text() == "Green") Colour = Qt::green;
    else if (PlotControl->AGCurveColours->checkedAction()->text() == "Blue") Colour = Qt::blue;
    else if (PlotControl->AGCurveColours->checkedAction()->text() == "Magenta") Colour = Qt::magenta;
    else if (PlotControl->AGCurveColours->checkedAction()->text() == "Cyan") Colour = Qt::cyan;
    else if (PlotControl->AGCurveColours->checkedAction()->text() == "Other...") {
        Pure2dPlot->VerifyActiveCurve();
        int act = Pure2dPlot->ActiveCurve;
        Colour = Pure2dPlot->ActiveCurve > 0 ? Pure2dPlot->QwtCurves.at(Pure2dPlot->ActiveCurve)->pen().color() : Qt::black;
        Colour = QColorDialog::getColor(Colour);
        if (!Colour.isValid())
            return;
        if (act != Pure2dPlot->ActiveCurve) {
            Pure2dPlot->ActiveCurve = act;
            Pure2dPlot->VerifyActiveCurve();
        }
        if (act != Pure2dPlot->ActiveCurve)
            return;
    }
    if (Pure2dPlot->ActiveCurve >= 0 && Pure2dPlot->SourceData.size() > Pure2dPlot->ActiveCurve && Pure2dPlot->SourceData.at(Pure2dPlot->ActiveCurve))
        Pure2dPlot->SourceData.at(Pure2dPlot->ActiveCurve).data()->Pen.setColor(Colour);
    emit CurveColourChanged(Colour, Pure2dPlot->ActiveCurve);
    Refresh();
}
void T2dPlot::on_BAutoScale_toggled()
{
    Pure2dPlot->SetAutoScaleEnabled(PlotControl->BAutoScale->isChecked());
}
void T2dPlot::on_BSave_clicked()
{
    TCurvesDataOwner src;
    ResizeOwner(src, Pure2dPlot->ActiveCurve >= 0 ? 1 : Pure2dPlot->SourceData.size());
    if (Pure2dPlot->ActiveCurve >= 0) {
        if (Pure2dPlot->SourceData.at(Pure2dPlot->ActiveCurve))
            *src[0] = *Pure2dPlot->SourceData.at(Pure2dPlot->ActiveCurve).data();
    } else for (int i = 0; i < src.size(); i++)
        if (Pure2dPlot->SourceData.at(i))
            *src[i] = *Pure2dPlot->SourceData.at(i).data();

    TMetaStorage attributes;

    attributes.insert("Preview", QVariant::fromValue(MakePreview()));
    attributes.insert("Fit mode", FindCheckedItemIn(PlotControl->AGFitOrders));

    QStringList clrs;
    for (int i = 0; i < src.size(); i++)
        clrs.append(src.at(i)->Pen.color().name());

    attributes.insert("Curve colour", clrs.join(","));
    attributes.insert("Curve thickness", FindCheckedItemIn(PlotControl->AGCurveThickness));

    attributes.insert("XLabel",
                      (Pure2dPlot->AutoLegendEnabled ? "" : Pure2dPlot->XLegend)+
                      Pure2dPlot->XAxisUnit.Description);

    attributes.insert("XUnit", Pure2dPlot->XAxisUnit.SIUnit);

    attributes.insert("YLabel",
                      (Pure2dPlot->AutoLegendEnabled ? "" : Pure2dPlot->YLegend) +
                      Pure2dPlot->YAxisUnit.Description);

    attributes.insert("YUnit", Pure2dPlot->YAxisUnit.SIUnit);

    emit AddDataToContainer(src, attributes);
}
void T2dPlot::SetSelectionModeEnabled(bool Enabled)
{
    PlotControl->BSelectByRect->setChecked(Enabled);
    on_BSelectByRect_clicked();
}
void T2dPlot::on_BSelectByRect_clicked()
{
    bool State = PlotControl->BSelectByRect->isChecked();

    if (State && Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(Pure2dPlot->GetSelectedRect(), TSelectEvent::St_SelectionStarted));

    PlotControl->SetSelectionModeEnabled(State);
    if (show_additional_gui()) {
        PlotControl->BAutoScaleX->setVisible(!State);
        PlotControl->BAutoScaleY->setVisible(!State);
    }

    Pure2dPlot->EnableSelectMode(State);
}
void T2dPlot::on_BApply_clicked()
{
    if (Pure2dPlot->d_func()->State == TPure2dPlot::Mode_Select)
    {
        if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(Pure2dPlot->GetSelectedRect(), TSelectEvent::St_SelectionApplied));
        PlotControl->BSelectByRect->setVisible(true);
        PlotControl->BSelectByRect->setChecked(false);
        on_BSelectByRect_clicked();
        Pure2dPlot->SelectionApplied();
    }
    else if (Pure2dPlot->d_func()->State == TPure2dPlot::Mode_ManualConstruct)
    {
        PreparePlotControlForMC(false);
        Pure2dPlot->ExitMCMode();
        emit ManualConstructingDone();
        RestoreEnvironmentState();
        Pure2dPlot->Refresh();
    }
}
void T2dPlot::on_BCancel_clicked()
{
    if (Pure2dPlot->d_func()->State == TPure2dPlot::Mode_Select)
    {
        if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(Pure2dPlot->GetSelectedRect(), TSelectEvent::St_SelectionCancled));
        PlotControl->BSelectByRect->setVisible(true);
        PlotControl->BSelectByRect->setChecked(false);
        on_BSelectByRect_clicked();
    }
    else if (Pure2dPlot->d_func()->State == TPure2dPlot::Mode_ManualConstruct)
    {
        PreparePlotControlForMC(false);
        Pure2dPlot->ExitMCMode();
        Pure2dPlot->SetFitOrder(TPlotEnums::FO_None);
        Pure2dPlot->Refresh();
    }
}
void T2dPlot::on_BDetailedMarkersInfo_clicked()
{
    MarkersInfo->setVisible(PlotControl->BDetailedMarkersInfo->isChecked());
    UpdateAllMarkersInfo();
}
void T2dPlot::on_BMarkers_clicked(bool on)
{
    if (!on)
        PlotControl->BDetailedMarkersInfo->setChecked(false);

    Pure2dPlot->BMarkers_clicked(on);

    PlotControl->BDetailedMarkersInfo->setVisible(on);
}
void T2dPlot::on_BSavePlotPixmap_clicked()
{
    QString SelectedFilter, FileName = QFileDialog::getSaveFileName(this, tr("Save plot as picture..."), QString(), "*.png;;*.jpg;;*.tiff;;*.bmp", &SelectedFilter, QFileDialog::DontUseNativeDialog);
    if (FileName.isEmpty()) return;
    if (!FileName.endsWith(SelectedFilter.remove(0, 1))) FileName += SelectedFilter;

    QPixmap::grabWidget(Pure2dPlot).toImage().save(FileName, SelectedFilter.remove(0, 1).toAscii().data());
}
void T2dPlot::RestoreEnvironmentState()
{
    if (Pure2dPlot->QwtCurves.size()) {
        int Thickness = Pure2dPlot->QwtCurves.at(qMax(0, Pure2dPlot->ActiveCurve))->pen().width();
        if (Thickness > 0 && Thickness - 1 < PlotControl->AGCurveThickness->actions().size())
            PlotControl->AGCurveThickness->actions()[Thickness - 1]->setChecked(true);
    }

    int FO = Pure2dPlot->FitOrder;
    QList<QAction *> Acts = PlotControl->AGFitOrders->actions();
    if (FO == TPlotEnums::FO_Manual && Acts.size())    Acts[Acts.size() - 1]->setChecked(true);
    else if (FO == TPlotEnums::FO_None && Acts.size()) Acts[0]->setChecked(true);
    else if (FO >= 0 && FO < Acts.size() - 2 && Acts.size() > 2) Acts[FO + 1]->setChecked(true);
}
void T2dPlot::on_CBChooseActiveCurve_highlighted(int Index)
{
    bool ok = false;
    if (Index == -1 ||
        PlotControl->CBChooseActiveCurve->itemData(Index).toInt(&ok) == -1 || !ok)
        Pure2dPlot->UnselectCurves();
    else
        Pure2dPlot->SelectCurve(PlotControl->CBChooseActiveCurve->itemData(Index).toInt());

    Pure2dPlot->Replot();
}
void T2dPlot::on_CBChooseActiveCurve_activated(int Index)
{
    Pure2dPlot->UnselectCurves();

    Pure2dPlot->ActiveCurve = (Index == -1) ? -1 : PlotControl->CBChooseActiveCurve->itemData(Index).toInt();

    RestoreEnvironmentState();

    Pure2dPlot->Refresh();
}
void T2dPlot::sync_active_curve(int active)
{
    for (int i = 0; i < PlotControl->CBChooseActiveCurve->count(); i++) {
        if (active == PlotControl->CBChooseActiveCurve->itemData(i).toInt()) {
            PlotControl->CBChooseActiveCurve->setCurrentIndex(i);
            on_CBChooseActiveCurve_activated(i);
        }
    }
}


void T2dPlot::SetGridVisible(const bool Visible)
{
    PlotControl->BGrid->setChecked(Visible);
}
void T2dPlot::SetAutoScalingEnabled(const bool AutoScaling)
{
    PlotControl->BAutoScale->setChecked(AutoScaling);
    on_BAutoScale_toggled();
}
void T2dPlot::PreparePlotControlForMC(bool MCStarts)
{
    PlotControl->BApplyAndCancelSetVisible(MCStarts);
    PlotControl->BResetMCCurve->setVisible(MCStarts);
    PlotControl->CBChooseActiveCurve->setEnabled(!MCStarts);
    for (int i = 0; i < PlotControl->AGFitOrders->actions().size() - 1; PlotControl->AGFitOrders->actions()[i++]->setEnabled(!MCStarts));
}
void T2dPlot::StartMCCurveConstructing()
{
    if (!Pure2dPlot->QwtCurves.size())
        return;

    Pure2dPlot->StartMCMode();
    PreparePlotControlForMC(true);
}

}
