#pragma once

#include "core/plot_core.h"

double max(const QVector<double> &data);
double min(const QVector<double> &data);
QPair<double, double> minmax(const QVector<double> &data);
double rms(const QVector<double> &data);
double avg(const QVector<double> &data);

namespace plot {

const QPen DefaultPenForInactiveCurves(QColor(220, 220, 220), 1, Qt::DashDotLine);

class TCurveData
{
public:
    QVector<double> XSeq;
    QVector<double> YSeq;
    QPen Pen;

    TCurveData() { Pen.setWidth(1); }
    void detach() { XSeq.detach(); YSeq.detach(); }
    void clear() { XSeq.clear(); YSeq.clear(); }
    void AddY(double y) { YSeq.append(y); }
    void AddPoint(double x, double y) { XSeq.append(x); YSeq.append(y); }

    int size() const { return (!isCompressedXCurve() && XSeq.size() < YSeq.size()) ? XSeq.size() : YSeq.size(); }
    bool isCompressedXCurve() const { return XSeq.size() == 2; }
    void setXCalib(double x0, double dx) { XSeq.resize(2); XSeq.squeeze(); XSeq[0] = x0; XSeq[1] = dx; }
    double getXat(int i) const { return !isCompressedXCurve() ? XSeq.at(i) : XSeq.at(0) + i * XSeq.at(1); }
};

typedef QVector<QWeakPointer<TCurveData> > TCurvesData;
typedef QVector<QSharedPointer<TCurveData> > TCurvesDataOwner;

class TCurves
{
public:
    void DeleteAllCurves();
    void ReSyncDataAndQwtCurves();
    void DefineLimits(QVector<double> &MaxX, QVector<double> &MaxY, QVector<double> &MinX, QVector<double> &MinY, int N);

    TCurves(QwtPlot *_Plot);
    ~TCurves() { DeleteAllCurves(); }
    virtual void SetActiveCurve(int Active) { ActiveCurve = Active; }
    void VerifyActiveCurve();

    void GetClosestPoint(double *, double *, const QPoint &Pos, int N = 0) const;
    void UnselectCurves();
    void SelectCurve(int N);

    void CleanData();

    QwtPlot *Plot;
    TCurvesData SourceData;
    TCurvesData PaintData;
    TCurvesDataOwner MathData;
    QVector<QwtPlotCurve *> QwtCurves;
    QwtPlotCurve *MCCurve;
    QVector<int> LastPaintSize; //incremental painter
    int ActiveCurve;//-1 - all curves; 0,1,... - n of active curve
};
}
