#pragma once

#include "widgets/plot_widgets.h"
#include "qwt_slider.h"

#include <QDialog>
#include <QLabel>
#include <QActionGroup>
#include <QToolButton>

namespace plot {

class ControlPanelHistory
{
public:
    ControlPanelHistory();
    bool BGridIsChecked;
    bool BAngleIsChecked;
    bool BCrossIsChecked;
    bool BMarkersIsChecked;
    bool BCrossectionIsChecked;
    bool BDetailedMarkersInfoIsChecked;
    bool MakeCorrectLineByMedianaXDirIsChecked;
    bool MakeCorrectLineByMedianaYDirIsChecked;
    bool BFixOXDirIsChecked;
    bool BFixOYDirIsChecked;
    bool BFixOXDirIsEnabled;
    bool BFixOYDirIsEnabled;
    double MinSliderVal;
    double MaxSliderVal;
    int NOfPalette;
    int CurrentXFitOrder;
    int CurrentYFitOrder;
    int CurrentSubtrOrder;

    int MeanValueFilterSize;
    int MedianValueFilterSize;
    int GaussianFilterSize;
    bool HistogramEqualization;

    bool AutoZScaleOn;
};

class TFilterSizeWidget: public QDialog
{
    Q_OBJECT

signals:
    void SliderValueChanged(int);

private slots:
    void onSliderMoved(double val);

private:
    QwtSlider    *SFilterSize;
    QLabel       *LFilterSize;

public:
    static const int DefaultFilterSize = 5;
    TFilterSizeWidget(QWidget *parent = 0);
    int FilterSize() const { return SFilterSize->value; }
    void SetFilterSize(int V);
};

class T3dPlotControl: public TCommonPlotControl
{
    Q_OBJECT

    void CreateActionsAndMenus();
public slots:
    void SetSelectionModeEnabled(const bool Enabled);
    void on_BApply_clicked() { BSelectByRect->setVisible(true); BSelectByRect->setChecked(false); }

public:
    void GUICreate();

    int GetNumOfAction(const QString &ActName) const;

    TwoStateButton *BCross;
    TwoStateButton  *BAreaZzoom;
    QPushButton  *BPalettes;
    QToolButton  *TBPalettes;
    QToolButton  *TBMath2;
    QMenu        *MMath2;
    QActionGroup *AGPalettes;
    QActionGroup *AGXFitOrders;
    QActionGroup *AGYFitOrders;
    QActionGroup *AGSubtr;
    QPushButton  *BCrossection;
    QPushButton  *BFixOXDir;
    QPushButton  *BFixOYDir;

    QString NameOfMeanValueFilter() const { return tr("Mean value"); }
    QString NameOfMedianValueFilter() const { return tr("Median value"); }
    QString NameOfGaussianFilter() const { return tr("Gaussian"); }
    QString NameOfHistogramFilter() const { return tr("Histogram Equalization"); }
};

class T3dMarkersInfo: public TMarkersInfo
{
public:
    void GUICreate();

    QLineEdit *LEdZA;
    QLabel *LZAUnit;

    QLineEdit *LEdZB;
    QLabel *LZBUnit;

    QLineEdit *LEdDeltaZ;
    QLabel *LDeltaZUnit;
};

}
