#include "plot_3dplot.h"
#include "math/plot_MathProcessing.h"

#include <qwt_plot_layout.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_canvas.h>

#include <QtEvents>
#include <QDrag>
#include <QApplication>
#include <QSplitter>
#include <QVBoxLayout>
#include <QFileDialog>
#include <QInputDialog>
#include <QMessageBox>
#include <QDebug>

namespace plot {

void T3dMarkerInfoHandler::operator()(bool AutoLegendEnabled, const TSIUnit &XYUnitInfo, const TSIUnit &ZUnitInfo, double XVal, double YVal, double ZVal)
{ TGeneralMarkerInfoHandler::operator ()(AutoLegendEnabled, XYUnitInfo, XYUnitInfo, XVal, YVal); ZInfo(AutoLegendEnabled, ZUnitInfo, ZVal); }

bool TPure3dPlot::isMathReady()
{
    return !MathProcessing->isRunning();
}
TPure3dPlot::~TPure3dPlot()
{
    delete MathProcessing;
}
TPure3dPlot::TPure3dPlot(int Num, T3dPlot *Chld, QObject *_Parent, QWidget *parent) : TCommonPlot(parent), Child(Chld), ZoomBySelection(true)
{
    Parent = _Parent;

    MathProcessing = new math::TMathProcessing(&MathData);
    N = Num;
    QSpectrogram->attach(QPlot);
    QSpectrogram->setRenderThreadCount(QThread::idealThreadCount());

    Cross = new QwtPlotMarker;
    Cross->setValue(0, 0);
    Cross->setSymbol(new QwtSymbol(QwtSymbol::Cross, QColor(255, 100, 255), QColor(255, 100, 255), QSize(15, 15)));
    Cross->setLineStyle(QwtPlotMarker::NoLine);
    Cross->attach(QPlot);
    Cross->hide();

    areaRB = new QRubberBand(QRubberBand::Rectangle, QPlot);
    areaRB->hide();

    AreaZzoom = false;
    zoom_active = 0;

    wait_for_math = false;
    math_delayed_refresh = false;

    connect(MathProcessing, SIGNAL(finished()), this, SLOT(s_Refresh()), Qt::QueuedConnection);

    setAcceptDrops(true);
    ((QWidget *)QPlot->titleLabel())->setToolTip(tr("Drag & Drop as data to plots, as image"));

    QPlot->canvas()->installEventFilter(this);
}
void TPure3dPlot::SelectDone()
{
    if (ZoomBySelection)
    {
        QRectF Rect = GetSelectedRect();
        if (!Rect.isValid() || Rect.width() < 9e-9 || Rect.height() < 9e-9) {
            qDebug() << Rect;
            return;
        }
        SetPlotsXScales(Rect.left(), Rect.right());
        SetPlotsYScales(Rect.top(),  Rect.bottom());
        ResizeQPlot();
    }
}
inline void TPure3dPlot::MakeCrossection() { if (BCrossectionIsChecked && BMarkersIsChecked) Child->MakeCrossection(); }

void TPure3dPlot::AreaZzoomST(double x, double y, double xo, double yo, int st)
{
    if (st == 1) {
        areazA = areazB = QPointF(x, y);
        areazAo = QPointF(xo, yo);
        areaRB->show();
        areaRB->setGeometry(xo, yo, 1, 1);
    } else if (st == 2) {
        areazB = QPointF(x, y);

        double xA = qMin<qreal>(areazAo.x(), xo);
        double xB = qMax<qreal>(areazAo.x(), xo);
        double yA = qMin<qreal>(areazAo.y(), yo);
        double yB = qMax<qreal>(areazAo.y(), yo);
        areaRB->setGeometry(xA, yA, xB-xA, yB-yA);
    } else if (st == 3) {
        areazB = QPointF(x, y);

        if (fabs(xo - areazAo.x()) <= 4 || fabs(yo - areazAo.y()) <= 4 ) {
            areaRB->hide();
            return;
        }

        double xA = qMin<qreal>(areazA.x(), areazB.x());
        double xB = qMax<qreal>(areazA.x(), areazB.x());
        double yA = qMin<qreal>(areazA.y(), areazB.y());
        double yB = qMax<qreal>(areazA.y(), areazB.y());
        double DeltaX = xB - xA, DeltaY = yB - yA;
        double NumberOfSteps = 1000; //GetMaxNumberOfPointsXY(xA, yA, xB, yB);
        double StepX, StepY;

        double min = 1e100;
        double max = -1e100;

        StepX = DeltaX / NumberOfSteps;
        StepY = DeltaY / NumberOfSteps;

        for (int i = 0; i < NumberOfSteps; i++) {
            for (int j = 0; j < NumberOfSteps; j++) {
                double val = value(xA + i*StepX, yA + i*StepY);
                min = qMin(min, val);
                max = qMax(max, val);
            }
        }

        if (!qFuzzyCompare(1. + min, 1. + max)) {
            Child->Slider->Sliders->SetMinMaxSlidersValues(
                        min / ZAxisUnit.Multiplier,
                        max / ZAxisUnit.Multiplier);
            AutoZScaleOn = false;
        }

        areaRB->hide();
    }
}

void TPure3dPlot::dragEnterEvent(QDragEnterEvent *event)
{
    const T3dMime *data = qobject_cast<const T3dMime *>(event->mimeData());
    if (data && event->source() != this)
        event->acceptProposedAction();
}
void TPure3dPlot::dropEvent(QDropEvent *event)
{
    const T3dMime *data = qobject_cast<const T3dMime *>(event->mimeData());
    if (!data || event->source() == this) return;

    SourceData = data->data + SourceData; //new on top
    //TODO limits bounding rect
    Refresh();
}
bool TPure3dPlot::eventFilter(QObject *Object, QEvent *Event)
{
    if (Object == QPlot->canvas()) {
        if (Event->type() == QEvent::Paint) {
            wait_for_math = MathProcessing->isRunning();
            if (math_delayed_refresh && !wait_for_math) {
                math_delayed_refresh = false;
                QMetaObject::invokeMethod(this, "Refresh", Qt::QueuedConnection);
            }
            return wait_for_math;
        }
        return false;
    }

    if (!isEnabled()) {
        Action = ExternalAct;
    } else
    if (Enabled && !SelectionModeEnabled && AreaZzoom && Child && (int)Child->ActivePlot == N) {
        Action = ExternalAct;

        int AreaZzoom_state = 0;

        if (Event->type() == QEvent::MouseButtonPress && zoom_active == 0) {
            AreaZzoom_state = 1;
            zoom_active = 1;
            Child->SetActivePlot(N); //2d->3d plotcontrol
        }
        if (Event->type() == QEvent::MouseMove && zoom_active == 1)
            AreaZzoom_state = 2;
        if (Event->type() == QEvent::MouseButtonRelease && zoom_active == 1) {
            AreaZzoom_state = 3;
            zoom_active = 0;
        }

        if (AreaZzoom_state != 0) {
            QPoint Pos = mapToCanvas(((QMouseEvent *)Event)->pos());
            double X = ScreenXToPlotX(Pos.x());
            double Y = ScreenYToPlotY(Pos.y() - 5);// y rounding error

            X = qBound(PlotsXLimits[0], X, PlotsXLimits[1]);
            Y = qBound(PlotsYLimits[0], Y, PlotsYLimits[1]);

            AreaZzoomST(X, Y, ((QMouseEvent *)Event)->pos().x(), ((QMouseEvent *)Event)->pos().y(), AreaZzoom_state);
        }
        return false;
    }

    if (!isEnabled()) {
        Action = ExternalAct;
    } else
    if (Enabled)
    {
        if (Event->type() == QEvent::Wheel) {
            if (Child) Child->SetActivePlot(N);
            QPointF Pos = mapToCanvas(((QWheelEvent *)Event)->pos());
            Pos = ScreenToPlot(Pos.toPoint());

            int Increment = ((QWheelEvent *) Event)->delta();
            ZoomPlot(Pos, Increment);
            MakeCrossection();
            if (Child) Child->SyncZoom(N, Pos, Increment, BFixXIsChecked, BFixYIsChecked); //TODO sync only curxylims
        } else if (Event->type() == QEvent::MouseButtonPress) {
            QPoint Pos = mapToCanvas(((QMouseEvent *)Event)->pos());
            DefineClickInfo(Pos);
            if (Child) Child->SyncDefineClickInfo(N, Pos);

            if (Action == Panning && QPlot->plotLayout()->titleRect().contains(((QMouseEvent *)Event)->pos())) {
                Action = ExternalAct;

                QDrag *drag = new QDrag(this);
                T3dMime *mimeData = new T3dMime;
                mimeData->data = SourceData;
                if (Child) {
                    QImage graph = QPixmap::grabWidget(this).toImage();
                    QImage slider = QPixmap::grabWidget(Child->Slider).toImage().scaledToHeight(graph.height(), Qt::SmoothTransformation);

                    QImage composition(graph.width() + slider.width(), graph.height(), graph.format());
                    QPainter painter(&composition);
                    painter.drawImage(0, 0, graph);
                    painter.drawImage(graph.width(), 0, slider);

                    mimeData->setImageData(composition);
                }
                drag->setMimeData(mimeData);
                drag->setPixmap(QPixmap::fromImage(MakePreview()));
                drag->exec(Qt::CopyAction);
            }
            if (Child) {
                if (BCrossIsChecked && ((QMouseEvent *)Event)->buttons() == Qt::LeftButton &&
                        (int)Child->ActivePlot == N) {
                    Action = CrossAct;
                    double X = ScreenXToPlotX(Pos.x());
                    double Y = ScreenYToPlotY(Pos.y());

                    X = qBound(CurrentXLimits[0], X, CurrentXLimits[1]);
                    Y = qBound(CurrentYLimits[0], Y, CurrentYLimits[1]);

                    SetCrossCoords(X, Y);
                    if (Parent) QCoreApplication::postEvent(Parent, new TCrossEvent(QPointF(X, Y)));
                }
                Child->SetActivePlot(N);
            }
        } else if (Event->type() == QEvent::MouseMove) {
            QPoint Pos = mapToCanvas(((QMouseEvent *)Event)->pos());
            if (Action == CrossAct && ((QMouseEvent *)Event)->buttons() == Qt::LeftButton) {
                double X = ScreenXToPlotX(Pos.x());
                double Y = ScreenYToPlotY(Pos.y());

                X = qBound(CurrentXLimits[0], X, CurrentXLimits[1]);
                Y = qBound(CurrentYLimits[0], Y, CurrentYLimits[1]);

                SetCrossCoords(X, Y);
                if (Parent) QCoreApplication::postEvent(Parent, new TCrossEvent(QPointF(X, Y)));
            }
            if (Action == MoveA || Action == MoveB || Action == MoveC) {
                if (!BFixOXDirIsChecked && !BFixOYDirIsChecked)
                    MoveMarker(Pos.x(), Pos.y(), false);
                else {
                    QPointF APos, BPos;
                    if (Action == MoveA) {
                        APos = ScreenToPlot(Pos);
                        if (BFixOXDirIsChecked)      { BPos.setX(APos.x()); BPos.setY(MarkerB->value().y()); }
                        else if (BFixOYDirIsChecked) { BPos.setY(APos.y()); BPos.setX(MarkerB->value().x()); }
                    }
                    else if (Action == MoveB) {
                        BPos = ScreenToPlot(Pos);
                        if (BFixOXDirIsChecked)      { APos.setX(BPos.x()); APos.setY(MarkerA->value().y()); }
                        else if (BFixOYDirIsChecked) { APos.setY(BPos.y()); APos.setX(MarkerA->value().x()); }
                    }
                    HandleMoveMarkers(APos, BPos);
                }
                MakeCrossection();
            }
            if (Action == Panning) {
                double kx = (ScreenXToPlotX(20) - ScreenXToPlotX(0)) / 20.;
                double ky = (ScreenYToPlotY(20) - ScreenYToPlotY(0)) / 20.;
                QPointF Delta((Pos.x() - ClickPos.x())*kx,
                              (Pos.y() - ClickPos.y())*ky);

                PanPlot(Delta);
                MakeCrossection();
                if (Child) Child->SyncPan(N, Delta); //TODO sync only curxylims
            }
        }
        else if (Event->type() == QEvent::MouseButtonRelease) {
            Action = ExternalAct;

        }
    }
    return QWidget::eventFilter(Object, Event);
}

void TPure3dPlot::SelectRectChanged(const QRectF &Rect)
{
    if (Parent)
        QCoreApplication::postEvent(Parent, new TSelectEvent(Rect, TSelectEvent::St_SelectChanged));
}

static bool _is_resize = false;
void TPure3dPlot::ResizeQPlot(bool moved)
{
    if (_is_resize) return;
    _is_resize = true;

    //Init various indents
    QPlot->plotLayout()->activate(QPlot, QPlot->rect());

    int UpSpace = QPlot->plotLayout()->titleRect().height();
    int DownSpace = QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
    int LeftSpace = QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width();
    int RightSpace = 0;

    //fit plot
    double NewDeltaX, NewDeltaY;
    const QSize AvailableSpace = FindAvailableSpaceForQPlot();
    FindSizeWithAspectRatio(AvailableSpace.width() - LeftSpace - RightSpace,
                            AvailableSpace.height() - UpSpace - DownSpace,
                            &NewDeltaX, &NewDeltaY);

    int xsize = NewDeltaX + LeftSpace + RightSpace - 10;
    int ysize = NewDeltaY + UpSpace + DownSpace - 10;
    QPlot->resize(xsize, ysize);

    //set minimum size
    /*
    FindSizeWithAspectRatio(1, 1, &NewDeltaX, &NewDeltaY);
    setMinimumSize(qMin<int>(xsize+10, 100/NewDeltaY + LeftSpace + RightSpace - 10),
                   qMin<int>(ysize+10, 100/NewDeltaX + UpSpace + DownSpace - 10));
                   */

    //scale division update to fit all in size
    UpdateScalesDiv();

    if (Child && Child->ActivePlot == N)
        Child->ResizeSliders();

    //Resize ArthurFrame:
    QSize NewSizeForArtFrame = QPlot->rect().size() + QSize(10, 10);

    if (AvailableSpace.height() < NewSizeForArtFrame.height())
        NewSizeForArtFrame.setHeight(AvailableSpace.height());

    if (AvailableSpace.width() < NewSizeForArtFrame.width())
        NewSizeForArtFrame.setWidth(AvailableSpace.width());

    resize(NewSizeForArtFrame);
    if (moved) last_resize_x = pos().x();
    move(last_resize_x + (AvailableSpace.width() - NewSizeForArtFrame.width()), pos().y());

    _is_resize = false;
}
QSize TPure3dPlot::FindAvailableSpaceForQPlot() const
{
    QSplitter *Spl = NULL;

    for (int YC = 0; YC < Child->Split3d->count(); YC++)
        for (int XC = 0; XC < ((QSplitter *)Child->Split3d->widget(YC))->count(); XC++)
            if ((QWidget *)this == ((QSplitter *)Child->Split3d->widget(YC))->widget(XC) && (Spl = ((QSplitter *)Child->Split3d->widget(YC))))
                return QSize(Spl->sizes()[XC], Spl->contentsRect().height() );

    return QSize(150, 150);
}
void TPure3dPlot::UpdateLabels(bool IsDetailedInfoVisible)
{
    AMarkerInfo(AutoLegendEnabled, XYAxisUnit, ZAxisUnit, MarkerA->xValue(), MarkerA->yValue(), value(MarkerA->xValue(), MarkerA->yValue()));
    BMarkerInfo(AutoLegendEnabled, XYAxisUnit, ZAxisUnit, MarkerB->xValue(), MarkerB->yValue(), value(MarkerB->xValue(), MarkerB->yValue()));
    TValueInfoCreator difx, dify, difz;

    QString AText, BText, MText;
    if (IsDetailedInfoVisible) { AText = "A"; BText = "B"; }
    else
    {
        AText = AMarkerInfo.FormUniteString();
        BText = BMarkerInfo.FormUniteString();

        difx(AutoLegendEnabled, XYAxisUnit, BMarkerInfo.XInfo.SrcValue - AMarkerInfo.XInfo.SrcValue);
        dify(AutoLegendEnabled, XYAxisUnit, BMarkerInfo.YInfo.SrcValue - AMarkerInfo.YInfo.SrcValue);
        difz(AutoLegendEnabled, ZAxisUnit, BMarkerInfo.ZInfo.SrcValue - AMarkerInfo.ZInfo.SrcValue);
        MText = QString(QChar(0x2206)) + "X = " + difx.Value + difx.Unit + '\n' +
                QString(QChar(0x2206)) + "Y = " + dify.Value + dify.Unit + '\n' +
                QString(QChar(0x2206)) + "Z = " + difz.Value + difz.Unit;
    }

    QwtText QwtAText(AText),
            QwtBText(BText),
            QwtMText(MText);

    QFont font = qApp->font(); font.setPointSize(FontSize);
    QwtAText.setFont(font);
    QwtBText.setFont(font);
    QwtMText.setFont(font);

    QwtAText.setColor(QColor(255, 160, 160));
    QwtBText.setColor(QColor(255, 160, 160));
    QwtMText.setColor(QColor(255, 160, 160));

    MarkerA->setLabel(QwtAText);
    MarkerB->setLabel(QwtBText);
    MarkerM->setLabel(QwtMText);
}

void TPure3dPlot::Refresh()
{
    if (blockRefresh)
        return;

    if (!IsMathEnabled()) {
        PaintData = SourceData;

        MathProcessing->wait();
        MathData.clear();

        s_Refresh();
    } else {
        if ((math_delayed_refresh = (MathProcessing->isRunning() || wait_for_math)))
            return;
        qApp->sendPostedEvents(this, QEvent::MetaCall); //flush all previous s_Refresh,Refresh events

        MathData.clear();
        ResizeOwner(MathData, SourceData.size());
        PaintData = getUnLock(MathData);

        for (int i = 0; i < SourceData.size(); i++)
            if (SourceData[i]) {
                *MathData[i] = *SourceData[i].data();
                MathData[i]->detach();
            }

        if (CurrentXFitOrder >= 0)  MathProcessing->DoFitX(CurrentXFitOrder);
        if (CurrentYFitOrder >= 0)  MathProcessing->DoFitY(CurrentYFitOrder);
        if (CurrentSubtrOrder >= 0) MathProcessing->DoSubtr(CurrentSubtrOrder);
        if (MakeCorrectLineByMedianaXDirIsChecked) MathProcessing->DoLineCorrectByMedianaX();
        if (MakeCorrectLineByMedianaYDirIsChecked) MathProcessing->DoLineCorrectByMedianaY();
        if (MeanValueFilterSize > 0)               MathProcessing->DoMeanValue(MeanValueFilterSize);
        if (MedianValueFilterSize > 0)             MathProcessing->DoMedianValue(MedianValueFilterSize);
        if (GaussianFilterSize > 0)                MathProcessing->DoGaussian(GaussianFilterSize);
        MathProcessing->start();
    }
}
template <int N>
void ColorMapHistogramEqualization(const THistogram<N> &zhistogram, QwtLinearColorMap *map, double from, double to);
void TPure3dPlot::s_Refresh()
{
    if (blockRefresh || (IsMathEnabled() && MathProcessing->isRunning()))
        return;

    UpdatePlotsZLimits();
    //if (0/*AutoScale*/) {plotsxylims = topdata.size(); ReSetAxisScales();} else
        HandleAxisMultiplier(); //for labels
    ReAssignPlotOptions();

    UpdateHistogram(zhistogram);
    if (HistogramEqualization) {
        SetPalette(active_palette, MinSliderVal, MaxSliderVal);
        blockRefresh = true;
        ColorMapHistogramEqualization(zhistogram, colormap, MinSliderVal, MaxSliderVal);
        blockRefresh = false;
        if (N == Child->ActivePlot)
            Child->Slider->update();
    }

    Replot();

    if (N == Child->ActivePlot) {
        Child->UpdateMinMaxInSlider();
        Child->UpdateSliderHistogram(zhistogram);
        Child->UpdateAllMarkersInfo();
        MakeCrossection();
    }

    PlotWasChanged();
    emit MathUpdated();
}

void TPure3dPlot::SetCrossCoords(double x, double y)
{
    Cross->setValue(x, y);
    Cross->show();
    Replot();
}
void TPure3dPlot::HideCross()
{
    Cross->hide();
    Replot();
}

bool TPure3dPlot::IsMathEnabled() const
{
    return CurrentXFitOrder >= 0 ||
            CurrentYFitOrder >= 0 ||
            CurrentSubtrOrder >= 0 ||
            MakeCorrectLineByMedianaXDirIsChecked ||
            MakeCorrectLineByMedianaYDirIsChecked ||
            MeanValueFilterSize > 0 ||
            MedianValueFilterSize > 0 ||
            GaussianFilterSize > 0;
}
void TPure3dPlot::MakeXFit(const int Order) { if (CurrentXFitOrder != Order) { CurrentXFitOrder = Order; Refresh(); } }
void TPure3dPlot::MakeYFit(const int Order) { if (CurrentYFitOrder != Order) { CurrentYFitOrder = Order; Refresh(); } }
void TPure3dPlot::MakeSubtr(const int Order) { if (CurrentSubtrOrder != Order) { CurrentSubtrOrder = Order; Refresh(); } }
void TPure3dPlot::MakeCorrectLineByMedianaXDir(const bool Make)
{ if (MakeCorrectLineByMedianaXDirIsChecked != Make) { MakeCorrectLineByMedianaXDirIsChecked = Make; Refresh(); } }
void TPure3dPlot::MakeCorrectLineByMedianaYDir(const bool Make)
{ if (MakeCorrectLineByMedianaYDirIsChecked != Make) { MakeCorrectLineByMedianaYDirIsChecked = Make; Refresh(); } }
void TPure3dPlot::MakeMeanValue(const int Size) { if (MeanValueFilterSize != Size) { MeanValueFilterSize = Size; Refresh(); } }
void TPure3dPlot::MakeMedianValue(const int Size) { if (MedianValueFilterSize != Size) { MedianValueFilterSize = Size; Refresh(); } }
void TPure3dPlot::MakeGaussian(const int Size) { if (GaussianFilterSize != Size) { GaussianFilterSize = Size; Refresh(); } }
void TPure3dPlot::MakeHistogram(const bool Make) {
    if (HistogramEqualization != Make) {
        HistogramEqualization = Make;

        SetPalette(active_palette, MinSliderVal, MaxSliderVal);
        if (HistogramEqualization)
            ColorMapHistogramEqualization(zhistogram, colormap, MinSliderVal, MaxSliderVal);
        if (N == Child->ActivePlot)
            Child->Slider->update();
        ReplotChanged();
    }
}

void TPure3dPlot::HandleAxisMultiplier()
{
    XYAxisUnit.UpdateLegend(CurrentXLimits[1], CurrentYLimits[1]);
    UpdateAxisLegends();
}
void TPure3dPlot::UpdateAxisLegends()
{
    UpdateLabels();
    TInfoFor3dPlot::UpdateAxisLegends();
    if (AutoLegendEnabled)
        QPlot->axisScaleDraw(QwtPlot::yLeft)->Multiplier = QPlot->axisScaleDraw(QwtPlot::xBottom)->Multiplier = XYAxisUnit.Multiplier;
}
void TPure3dPlot::HandleMoveMarkers(const QPointF &APos, const QPointF &BPos)
{
    MoveMarker(MarkerA, APos.x(), APos.y());
    MoveMarker(MarkerB, BPos.x(), BPos.y());

    ReSyncMarkers();

    UpdateLabels();
    Replot();
}
void TPure3dPlot::on_BFixOXDir_clicked()
{
    if (BFixOXDirIsChecked)
    {
        if (MarkerA->value().y() == MarkerB->value().y()) {
            MoveMarker(MarkerA, MarkerA->value().x(), PlotsYLimits[0]);
            MoveMarker(MarkerB, MarkerB->value().x(), PlotsYLimits[1]);
        };
        double C = (MarkerA->value().x() + MarkerB->value().x()) / 2.;
        HandleMoveMarkers(QPointF(C, MarkerA->value().y()), QPointF(C, MarkerB->value().y()));
        MakeCrossection();
    }
}
void TPure3dPlot::on_BFixOYDir_clicked()
{
    if (BFixOYDirIsChecked)
    {
        if (MarkerA->value().x() == MarkerB->value().x()) {
            MoveMarker(MarkerA, PlotsXLimits[0], MarkerA->value().y());
            MoveMarker(MarkerB, PlotsXLimits[1], MarkerB->value().y());
        };
        double C = (MarkerA->value().y() + MarkerB->value().y()) / 2.;
        HandleMoveMarkers(QPointF(MarkerA->value().x(), C), QPointF(MarkerB->value().x(), C));
        MakeCrossection();
    }
}

//================================================================================================================================
T3dPlot::T3dPlot(int N, QObject *_Parent, QWidget *wparent) : QWidget(wparent)
{
    Parent = _Parent;

    GUICreate(N);
    InitPalettes();

    ActivePlot = -1;
    SetActivePlot(0);
    installEventFilter(this);
}
void T3dPlot::ConnectPlotControlSlots()
{
    connect(PlotControl->BMarkers,             SIGNAL(toggled(bool)), this, SLOT(on_BMarkers_clicked(bool)));
    connect(PlotControl->BDetailedMarkersInfo, SIGNAL(toggled(bool)), this, SLOT(on_BDetailedMarkersInfo_clicked()));
    connect(PlotControl->BAngle,               SIGNAL(toggled(bool)), this, SLOT(on_BAngle_clicked()));
    connect(PlotControl->BGrid,                SIGNAL(toggled(bool)), this, SLOT(on_BGrid_clicked()));
    connect(PlotControl->BFixX,                SIGNAL(toggled(bool)), this, SLOT(on_BFixX_toggled()));
    connect(PlotControl->BFixY,                SIGNAL(toggled(bool)), this, SLOT(on_BFixY_toggled()));

    connect(PlotControl->BRestoreScale,        SIGNAL(clicked()), this, SLOT(on_BRestoreScale_clicked()));
    connect(PlotControl->BCross,               SIGNAL(state_changed(bool)), this, SLOT(on_BCross_clicked()));
    connect(PlotControl->BSelectByRect,        SIGNAL(clicked()), this, SLOT(on_BSelectByRect_clicked()));
    connect(PlotControl->BExpandSelectRect,    SIGNAL(clicked()), this, SLOT(on_BExpandSelectRect_clicked()));
    connect(PlotControl->BCancel,              SIGNAL(clicked()), this, SLOT(on_BCancel_clicked()));
    connect(PlotControl->BApply,               SIGNAL(clicked()), this, SLOT(on_BApply_clicked()));

    connect(PlotControl->BAreaZzoom,           SIGNAL(clicked()), this, SLOT(on_BAreaZzoom_clicked()));

    connect(PlotControl->BSavePlotPixmap,      SIGNAL(clicked()), this, SLOT(on_BSavePlotPixmap_clicked()));
    connect(PlotControl->BSave,                SIGNAL(clicked()), this, SLOT(on_BSave_clicked()));
    connect(PlotControl->BCrossection,         SIGNAL(toggled(bool)), this, SLOT(on_BCrossection_clicked()));
    connect(PlotControl->BFixOXDir,            SIGNAL(clicked()), this, SLOT(on_BFixOXDir_clicked()));
    connect(PlotControl->BFixOYDir,            SIGNAL(clicked()), this, SLOT(on_BFixOYDir_clicked()));

    connect(PlotControl->MMathProcessing->actions()[0], SIGNAL(toggled(bool)), this, SLOT(on_GwyXFit_triggered(bool)));
    connect(PlotControl->MMathProcessing->actions()[1], SIGNAL(toggled(bool)), this, SLOT(on_GwyYFit_triggered(bool)));

    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())],   SIGNAL(triggered(bool)), this, SLOT(on_MeanValue_triggered(bool)));
    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())], SIGNAL(triggered(bool)), this, SLOT(on_MedianValue_triggered(bool)));
    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())],    SIGNAL(triggered(bool)), this, SLOT(on_Gaussian_triggered(bool)));
    connect(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())],   SIGNAL(toggled(bool)), this, SLOT(on_Histogram_triggered(bool)));

    connect(PlotControl->AGXFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitXOrder_changed()));
    connect(PlotControl->AGYFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitYOrder_changed()));
    connect(PlotControl->AGSubtr, SIGNAL(triggered(QAction*)), this, SLOT(on_SubtrOrder_changed()));
    connect(PlotControl->AGFontSizes, SIGNAL(triggered(QAction*)), this, SLOT(on_FontSize_changed()));
}
void T3dPlot::ConnectSlots()
{
    ConnectPlotControlSlots();
    connect(TwoDPlot, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), this, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)));
    connect(Slider->Sliders, SIGNAL(ValuesChanged(double, double)), this, SLOT(on_Sliders_ValuesChanged(double, double)));
    connect(Slider->Sliders, SIGNAL(AutoChanged(bool)), this, SLOT(on_AutoZScale_changed(bool)));

    for (int i = 0; i < ThreeDPlots.size(); i++)
        connect(ThreeDPlots.at(i), SIGNAL(UpdateAllMarkersInfo()), this, SLOT(UpdateAllMarkersInfo()));
}
void T3dPlot::GUICreate(int N)
{
    int YNum = qSqrt(N);
    if (N % YNum != 0)
        YNum = 1;
    int XNum = N / YNum;

    Split2d = new QSplitter(Qt::Vertical, this);
    Split2d->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    QVBoxLayout *ControlLayout = new QVBoxLayout;

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->setContentsMargins(0, 0, 6, 0);
    MainLayout->setSpacing(0);
    MainLayout->addWidget(Split2d);
    MainLayout->addLayout(ControlLayout);

    QWidget *down_wt = new QWidget(Split2d);
    QHBoxLayout *MainDownLayout = new QHBoxLayout(down_wt);
    MainDownLayout->setContentsMargins(0, 0, 0, 0);

    Split2d->addWidget(down_wt);
    Split2d->setCollapsible(0, false);


    MarkersInfo = new T3dMarkersInfo;
    Split3d = new QSplitter(Qt::Vertical, down_wt);
    Split3d->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    Slider = new TDoubleSlider;

    MainDownLayout->addWidget(MarkersInfo, 0, Qt::AlignTop);
    MainDownLayout->addWidget(Split3d);
    MainDownLayout->addWidget(Slider, 0, Qt::AlignTop);


    ThreeDPlots.resize(N);
    for (int i = 0; i < YNum; i++) {
        QSplitter *spl = new QSplitter(Split3d);
        spl->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        Split3d->addWidget(spl);
        Containers.append(spl);

        for (int j = i * XNum; j < (i + 1) * XNum && j < ThreeDPlots.size(); j++)
            spl->addWidget(ThreeDPlots[j] = new TPure3dPlot(j, this, Parent, spl));
    }

    QFont font = qApp->font(); font.setPointSize(ThreeDPlots.at(0)->FontSize);
    Slider->Title->setFont(font);
    Slider->Sliders->QwtBar->setFont(font);


#ifndef _ADROID
    TwoDPlot = new T2dPlot(this, true, Split2d);
#else
    TwoDPlot = new T2dPlot(this, false, Split2d);
#endif
    TwoDPlot->PlotControl->hide();
    connect(TwoDPlot->Pure2dPlot, SIGNAL(ShowPlotControl()), SLOT(Show2dPlotControl()));

    Split2d->addWidget(TwoDPlot);
    Split2d->setSizes(QList<int>() << 1 << 0);


    PlotControl = new T3dPlotControl;

    ControlLayout->addWidget(PlotControl);
    ControlLayout->addWidget(TwoDPlot->PlotControl);


    MarkersInfo->GUICreate();

#ifndef _ADROID
    TwoDPlot->PlotControl->GUICreate();
    TwoDPlot->ConnectPlotControlSlots();
#endif
    PlotControl->GUICreate();

    ConnectSlots();
}
void T3dPlot::AdjustAxisLegends()
{
    TwoDPlot->Pure2dPlot->SetAutoLegendEnabled(ThreeDPlots.at(ActivePlot)->AutoLegendEnabled);
    if (ThreeDPlots.at(ActivePlot)->AutoLegendEnabled)
    {
        TwoDPlot->Pure2dPlot->XAxisUnit = ThreeDPlots.at(ActivePlot)->XYAxisUnit;
        TwoDPlot->Pure2dPlot->YAxisUnit = ThreeDPlots.at(ActivePlot)->ZAxisUnit;
    }
    else
    {
        if (ThreeDPlots.at(ActivePlot)->XLegend == ThreeDPlots.at(ActivePlot)->YLegend) TwoDPlot->Pure2dPlot->SetXLegend(ThreeDPlots.at(ActivePlot)->XLegend);
        else TwoDPlot->Pure2dPlot->SetXLegend("Not available");
        TwoDPlot->Pure2dPlot->SetYLegend(ThreeDPlots.at(ActivePlot)->ZLegend);
    }
}
void T3dPlot::ResizeSliders()
{
    if (ThreeDPlots.size() == 1)
        Slider->setMaximumHeight(qBound(ThreeDPlots.at(ActivePlot)->Height(), 200, height()));
}
void T3dPlot::AdjustTwoDPlot()
{
    AdjustAxisLegends();
    if (PlotControl->BCrossection->isChecked() && PlotControl->BMarkers->isChecked()) MakeCrossection();
}
void T3dPlot::SetActivePlot(int N)
{
    PlotControl->show();
    TwoDPlot->PlotControl->hide();

    if (ActivePlot == N) return;

    RestoreEnvironmentState(ActivePlot = N);

    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetPlotActive(i == ActivePlot);
    ThreeDPlots[ActivePlot]->setFocus(Qt::MouseFocusReason);
    AdjustTwoDPlot();
    emit ActivePlotChanged(N);
}
void T3dPlot::RestoreEnvironmentState(int N)
{
    bool block = ThreeDPlots[N]->blockRefresh;
    ThreeDPlots[N]->blockRefresh = true;

    PlotControl->BMarkers-> setChecked(ThreeDPlots.at(N)->BMarkersIsChecked);
    PlotControl->BAngle->   setChecked(ThreeDPlots.at(N)->BAngleIsChecked);
    PlotControl->BGrid->    setChecked(ThreeDPlots.at(N)->BGridIsChecked);
    PlotControl->BFixX->    setChecked(ThreeDPlots.at(N)->BFixXIsChecked);
    PlotControl->BFixY->    setChecked(ThreeDPlots.at(N)->BFixYIsChecked);

    PlotControl->BFixOXDir->setChecked(ThreeDPlots.at(N)->BFixOXDirIsChecked);
    PlotControl->BFixOYDir->setChecked(ThreeDPlots.at(N)->BFixOYDirIsChecked);
    PlotControl->BFixOXDir->setEnabled(ThreeDPlots.at(N)->BFixOXDirIsEnabled);
    PlotControl->BFixOYDir->setEnabled(ThreeDPlots.at(N)->BFixOYDirIsEnabled);
    if (PlotControl->BFixOXDir->isChecked())
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y_On.png"));
    else
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));
    if (PlotControl->BFixOYDir->isChecked())
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X_On.png"));
    else
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));

    PlotControl->BFixOXDir->setVisible(PlotControl->BMarkers->isChecked());
    PlotControl->BFixOYDir->setVisible(PlotControl->BMarkers->isChecked());

    PlotControl->AGSubtr->actions()[ThreeDPlots.at(N)->CurrentSubtrOrder + 1]->setChecked(true);
    PlotControl->AGXFitOrders->actions()[ThreeDPlots.at(N)->CurrentXFitOrder + 1]->setChecked(true);
    PlotControl->AGYFitOrders->actions()[ThreeDPlots.at(N)->CurrentYFitOrder + 1]->setChecked(true);
    PlotControl->MMathProcessing->actions()[0]->setChecked(ThreeDPlots.at(N)->MakeCorrectLineByMedianaXDirIsChecked);
    PlotControl->MMathProcessing->actions()[1]->setChecked(ThreeDPlots.at(N)->MakeCorrectLineByMedianaYDirIsChecked);
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())]->setChecked(ThreeDPlots.at(N)->MeanValueFilterSize >= 0);
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())]->setChecked(ThreeDPlots.at(N)->MedianValueFilterSize >= 0);
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())]->setChecked(ThreeDPlots.at(N)->GaussianFilterSize >= 0);
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())]->setChecked(ThreeDPlots.at(N)->HistogramEqualization);
    SetAllFilterSizesTextInAction();

    ThreeDPlots[N]->blockRefresh = block;

    //if (PlotControl->BDetailedMarkersInfo->isChecked() != ThreeDPlots.at(N)->BDetailedMarkersInfoIsChecked)
    {
        PlotControl->BDetailedMarkersInfo->setChecked(ThreeDPlots.at(N)->BDetailedMarkersInfoIsChecked);
        on_BDetailedMarkersInfo_clicked();
    }
    PlotControl->BCrossection->setChecked(ThreeDPlots.at(N)->BCrossectionIsChecked);
    if (ThreeDPlots.at(N)->NOfPalette < PlotControl->AGPalettes->actions().size())
        PlotControl->AGPalettes->actions()[ThreeDPlots.at(N)->NOfPalette]->setChecked(true);

    ResizeSliders();

    UpdateMinMaxInSlider();
    Slider->Sliders->SetMinMaxRelSlidersValues(ThreeDPlots.at(N)->MinSliderVal, ThreeDPlots.at(N)->MaxSliderVal);
    Slider->Sliders->setAutoMode(ThreeDPlots.at(N)->AutoZScaleOn);
    UpdateSliderHistogram(ThreeDPlots.at(N)->zhistogram);
}


void T3dPlot::InitPalettes()
{
    QFile plfile(QFile::exists("Param/palette.txt") ? "Param/palette.txt" : ":/config/palette.txt");
    plfile.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream stream(&plfile);
    TPalette cv;
    QString name;
    int mode = 0;

    QVector<QString> names;

    if (stream.atEnd()) {
        cv.append(QColor(0, 0, 0));
        cv.append(QColor(190, 190, 190));
        cv.append(QColor(255, 255, 255));
        Palettes.append(cv);
        names.append("Gray");
        cv.clear();
    }
    while (!stream.atEnd()) {
        QString param;
        stream >> param;

        if (param == "#start") {
            stream >> param;
            if (param == "#RGB") {
                mode = 0;
            } else if (param == "#HEX") {
                mode = 1;
            }

            stream.readLine();//skip eol
            name = stream.readLine();
        } else if (param == "#end") {
            Palettes.append(cv);
            names.append(name);
            cv.clear();
        } else {
            if (mode == 0) {
                int r = param.toInt();

                stream >> param;
                int g = param.toInt();

                stream >> param;
                int b = param.toInt();

                cv.append(QColor(r, g, b));
            } else if (mode == 1) {
                cv.append(QColor(param));
            }
        }
    }


    names.append("Spectral 4d");
    for (int i = 0; i < names.size(); i++)
    {
        PlotControl->AGPalettes->addAction(PlotControl->TBPalettes->menu()->addAction(names.at(i)));
        PlotControl->AGPalettes->actions()[i]->setCheckable(true);
    }
    connect(PlotControl->AGPalettes, SIGNAL(triggered(QAction*)), this, SLOT(on_MPalettes_trigered()));

    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetPalette(Palettes.first(), 0, 1);
}
void T3dPlot::on_MPalettes_trigered()
{
    int NOfPalette = FindCheckedItemIn(PlotControl->AGPalettes);
    if (NOfPalette < 0)
        return;

    ThreeDPlots[ActivePlot]->NOfPalette = NOfPalette;
    on_Sliders_ValuesChanged(ThreeDPlots.at(ActivePlot)->MinSliderVal,
                             ThreeDPlots.at(ActivePlot)->MaxSliderVal);
}
void T3dPlot::on_Sliders_ValuesChanged(double Min, double Max)
{
    ThreeDPlots[ActivePlot]->MinSliderVal = Min;
    ThreeDPlots[ActivePlot]->MaxSliderVal = Max;

    TPalette palette;
    if (ThreeDPlots[ActivePlot]->NOfPalette >= Palettes.size()) //isSpectral
        palette = GetSpectralPalette(TwoDPlot->Pure2dPlot->PlotsXLimits,
                                     TwoDPlot->Pure2dPlot->GetSelectedRect().left(),
                                     TwoDPlot->Pure2dPlot->GetSelectedRect().right());
    else
        palette = Palettes.at(ThreeDPlots.at(ActivePlot)->NOfPalette);

    ThreeDPlots[ActivePlot]->SetPalette(palette, Min, Max);
    if (ThreeDPlots.at(ActivePlot)->HistogramEqualization)
        ColorMapHistogramEqualization(ThreeDPlots.at(ActivePlot)->zhistogram, ThreeDPlots.at(ActivePlot)->colormap, Min, Max);

    ThreeDPlots[ActivePlot]->ReplotChanged();
    Slider->Sliders->SetColourMap(ThreeDPlots.at(ActivePlot)->colormap);
}
TPalette T3dPlot::GetSpectralPalette(const Range &ZLimits, double X1, double X2)
{
    double Centre = (X1 + X2) / 2.;
    if (ZLimits[0] == ZLimits[1] || Centre < ZLimits[0] || Centre > ZLimits[1]) {
        TPalette cv;
        cv.append(QColor(0, 0, 0));
        cv.append(QColor(190, 190, 190));
        cv.append(QColor(255, 255, 255));
        return cv;
    }

    double k = 300 / (ZLimits[1] - ZLimits[0]);
    double HalfDeltaH = k * (X2 - X1) / 2.;
    double H = 300 - k * (Centre - ZLimits[0]);

    TPalette palette;
    int Size = 20;
    for (int i = 0; i < Size; i++)
    {
        double p = (double)i / (Size - 1);
        QColor Colour;
        double h = H - HalfDeltaH + 2*HalfDeltaH*p;
        if (HalfDeltaH < 40) {
            if (i < Size / 2)
                Colour.setHsv(h, 255, 255*p*2);
            else
                Colour.setHsv(h, 255*2*(1 - p), 255);
        } else
            Colour.setHsv(h, 255, 255);
        palette.append(Colour);
    }

    return palette;
}
bool T3dPlot::eventFilter(QObject *Object, QEvent *Event)
{
    if (Event->type() == TSelectEvent::id)
        if (ThreeDPlots.at(ActivePlot)->NOfPalette >= Palettes.size()) //isSpectral
            //TSelectEvent *e = (TSelectEvent *)Event; e->SelectedRect.left(), e->SelectedRect.right();
            on_Sliders_ValuesChanged(ThreeDPlots.at(ActivePlot)->MinSliderVal,
                                     ThreeDPlots.at(ActivePlot)->MaxSliderVal);

    if (Event->type() == TSelectEvent::id || Event->type() == TCrossEvent::id || Event->type() == TScalesChangedEvent::id)
        if (Parent) QCoreApplication::sendEvent(Parent, Event);

    return QWidget::eventFilter(Object, Event);
}


void T3dPlot::on_BGrid_clicked()
{
    ThreeDPlots[ActivePlot]->BGrid_clicked(PlotControl->BGrid->isChecked());
    ThreeDPlots[ActivePlot]->BGridIsChecked = PlotControl->BGrid->isChecked();
}
void T3dPlot::on_BDetailedMarkersInfo_clicked()
{
    MarkersInfo->setVisible(PlotControl->BDetailedMarkersInfo->isChecked());
    ThreeDPlots[ActivePlot]->BDetailedMarkersInfoIsChecked = PlotControl->BDetailedMarkersInfo->isChecked();
    UpdateAllMarkersInfo();
}
void T3dPlot::on_BAngle_clicked()
{
    ThreeDPlots[ActivePlot]->BAngle_clicked(PlotControl->BAngle->isChecked());
    ThreeDPlots[ActivePlot]->BAngleIsChecked = PlotControl->BAngle->isChecked();
}
void T3dPlot::on_BMarkers_clicked(bool on)
{
    if (on)
        PlotControl->BCross->change_state_man_with_sig(false);

    ThreeDPlots[ActivePlot]->BMarkersIsChecked = on;
    ThreeDPlots[ActivePlot]->BMarkers_clicked(on);

    if (!on) {
        PlotControl->BDetailedMarkersInfo->setChecked(false);
        PlotControl->BAngle->setChecked(false);
        PlotControl->BCrossection->setChecked(false);
    }
    PlotControl->BMarkers->setIcon(makeHugeIcon(on ? ":/plot/markersOn.png" : ":/plot/markers.png"));
    PlotControl->BDetailedMarkersInfo->setVisible(on);
    if (show_additional_gui() || show_advanced_gui()) {
        PlotControl->BAngle->setVisible(on);
    }
    PlotControl->BCrossection->setVisible(on);
    PlotControl->BFixOYDir->setVisible(on);
    PlotControl->BFixOXDir->setVisible(on);
}
void T3dPlot::on_BFixX_toggled()
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->BFixXIsChecked = PlotControl->BFixX->isChecked();
}
void T3dPlot::on_BFixY_toggled()
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->BFixYIsChecked = PlotControl->BFixY->isChecked();
}
void T3dPlot::on_BRestoreScale_clicked()
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->BRestoreScale_clicked();

    if (Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(0)->PlotsXLimits,
                                                            ThreeDPlots.at(0)->PlotsYLimits,
                                                            TScalesChangedEvent::R_BRestoreScale_clicked));
}
void T3dPlot::on_BAreaZzoom_clicked()
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->AreaZzoom = PlotControl->BAreaZzoom->isChecked();
}
void T3dPlot::on_BCross_clicked()
{
    for (int i = 0; i < ThreeDPlots.size(); i++) {
        ThreeDPlots[i]->Cross->setVisible(ThreeDPlots[i]->BCrossIsChecked = PlotControl->BCross->on_off);
        ThreeDPlots[i]->Replot();
    }
}
void T3dPlot::on_BSelectByRect_clicked()
{
    PlotControl->SetSelectionModeEnabled(PlotControl->BSelectByRect->isChecked());
    if (PlotControl->BSelectByRect->isChecked())
    {
        for (int i = 0; i < ThreeDPlots.size(); i++)
        {
            ThreeDPlots[i]->QPlot->hide();
            ThreeDPlots[i]->SetSelectionModeEnabled(i == ActivePlot);
            ThreeDPlots[i]->SetPlotEnabled(i == ActivePlot);
        }
        if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(ThreeDPlots.at(ActivePlot)->GetSelectedRect(), TSelectEvent::St_SelectionStarted));
    }
    else
    {
        ThreeDPlots[ActivePlot]->SetSelectionModeEnabled(false);
        for (int i = 0; i < ThreeDPlots.size(); i++)
        {
            ThreeDPlots[i]->SetPlotEnabled(true);
            ThreeDPlots[i]->QPlot->show();
            ThreeDPlots[i]->ReAssignPlotOptions();
        }
        ThreeDPlots[ActivePlot]->unsetCursor();
    }
    ThreeDPlots[ActivePlot]->setFocus(Qt::MouseFocusReason);
}
void T3dPlot::on_BApply_clicked()
{
    if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(ThreeDPlots.at(ActivePlot)->GetSelectedRect(), TSelectEvent::St_SelectionApplied));
    PlotControl->on_BApply_clicked();
    on_BSelectByRect_clicked();
    ThreeDPlots[ActivePlot]->SelectDone();
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == ActivePlot) continue;
        ThreeDPlots[i]->SetPlotsXScales(ThreeDPlots.at(ActivePlot)->PlotsXLimits[0], ThreeDPlots.at(ActivePlot)->PlotsXLimits[1]);
        ThreeDPlots[i]->SetPlotsYScales(ThreeDPlots.at(ActivePlot)->PlotsYLimits[0], ThreeDPlots.at(ActivePlot)->PlotsYLimits[1]);
        ThreeDPlots[i]->ResizeQPlot();
    }
    if (ThreeDPlots[ActivePlot]->ZoomBySelection && Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(ActivePlot)->PlotsXLimits,
                                                            ThreeDPlots.at(ActivePlot)->PlotsYLimits,
                                                            TScalesChangedEvent::R_BApply_clicked));
}
void T3dPlot::on_BCancel_clicked()
{
    PlotControl->BSelectByRect->setVisible(true);
    PlotControl->BSelectByRect->setChecked(false);
    on_BSelectByRect_clicked();
    if (Parent) QCoreApplication::postEvent(Parent, new TSelectEvent(ThreeDPlots.at(ActivePlot)->GetSelectedRect(), TSelectEvent::St_SelectionCancled));
}


void T3dPlot::on_BSavePlotPixmap_clicked()
{
    QString SelectedFilter;
    QString FileName;

    FileName = QFileDialog::getSaveFileName(this,
                                            tr("Save plot as picture..."),
                                            QString(),
                                            tr("*.png;;*.jpg;;*.tiff;;*.bmp"),
                                            &SelectedFilter,
                                            QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    if (!FileName.endsWith(SelectedFilter.remove(0, 1)))
        FileName += SelectedFilter;

    QImage graph = QPixmap::grabWidget(ThreeDPlots[ActivePlot]).toImage();
    QImage slider = QPixmap::grabWidget(Slider).toImage().scaledToHeight(graph.height(), Qt::SmoothTransformation);

    QImage composition(graph.width() + slider.width(), graph.height(), graph.format());
    QPainter painter(&composition);
    painter.drawImage(0, 0, graph);
    painter.drawImage(graph.width(), 0, slider);

    composition.save(FileName, SelectedFilter.remove(0, 1).toAscii().data());
}

void T3dPlot::appendAttrib(plot::TMetaStorage &attributes, int N)
{
    attributes.insert("XFit", ThreeDPlots.at(N)->CurrentXFitOrder + 1);
    attributes.insert("YFit", ThreeDPlots.at(N)->CurrentYFitOrder + 1);
    attributes.insert("Substract", ThreeDPlots.at(N)->CurrentSubtrOrder + 1);
    attributes.insert("gwyXFit", ThreeDPlots.at(N)->MakeCorrectLineByMedianaXDirIsChecked);
    attributes.insert("gwyYFit", ThreeDPlots.at(N)->MakeCorrectLineByMedianaYDirIsChecked);
    attributes.insert("Mean", ThreeDPlots.at(N)->MeanValueFilterSize);
    attributes.insert("Median", ThreeDPlots.at(N)->MedianValueFilterSize);
    attributes.insert("Gaussian", ThreeDPlots.at(N)->GaussianFilterSize);
    attributes.insert("HistEq", ThreeDPlots.at(N)->HistogramEqualization);

    attributes.insert("Palette", ThreeDPlots.at(N)->NOfPalette);
    attributes.insert("ZMin View", ThreeDPlots.at(N)->MinSliderVal);
    attributes.insert("ZMax View", ThreeDPlots.at(N)->MaxSliderVal);
    attributes.insert("ZViewAuto", ThreeDPlots.at(N)->AutoZScaleOn);
}
void T3dPlot::restoreAttributes(plot::TMetaStorage attributes, int N)
{
    bool block = ThreeDPlots[N]->blockRefresh;
    ThreeDPlots[N]->blockRefresh = true;

    SetTitle(attributes.value("Name", tr("Scan")).toString(), N);
    SetXYDescAndSIUnit(attributes.value("XYLabel", "").toString(),
                       attributes.value("XYUnit", tr("m")).toString(), N);
    SetZDescAndSIUnit(attributes.value("ZLabel", "").toString(),
                      attributes.value("ZUnit", tr("Counts")).toString(), N);
    SetAllXScales(attributes.value("XMin", 0).toDouble(),
                  attributes.value("XMax", 1).toDouble());
    SetAllYScales(attributes.value("YMin", 0).toDouble(),
                  attributes.value("YMax", 1).toDouble());

    ThreeDPlots.at(N)->CurrentXFitOrder = attributes.value("XFit", 0).toInt() - 1;
    ThreeDPlots.at(N)->CurrentYFitOrder = attributes.value("YFit", 0).toInt() - 1;
    ThreeDPlots.at(N)->CurrentSubtrOrder = attributes.value("Substract", 0).toInt() - 1;
    ThreeDPlots.at(N)->MakeCorrectLineByMedianaXDirIsChecked = attributes.value("gwyXFit", false).toBool();
    ThreeDPlots.at(N)->MakeCorrectLineByMedianaYDirIsChecked = attributes.value("gwyYFit", false).toBool();
    ThreeDPlots.at(N)->MeanValueFilterSize = attributes.value("Mean", -1).toInt();
    ThreeDPlots.at(N)->MedianValueFilterSize = attributes.value("Median", -1).toInt();
    ThreeDPlots.at(N)->GaussianFilterSize = attributes.value("Gaussian", -1).toInt();
    ThreeDPlots.at(N)->HistogramEqualization = attributes.value("HistEq", false).toBool();

    ThreeDPlots.at(N)->NOfPalette = attributes.value("Palette", 0).toInt();
    ThreeDPlots.at(N)->MinSliderVal = attributes.value("ZMin View", 0).toDouble();
    ThreeDPlots.at(N)->MaxSliderVal = attributes.value("ZMax View", 1).toDouble();
    ThreeDPlots.at(N)->AutoZScaleOn = attributes.value("ZViewAuto", false).toBool();

    //apply
    int prevActive = ActivePlot;
    bool blockActive = ThreeDPlots[ActivePlot]->blockRefresh;
    ThreeDPlots[ActivePlot]->blockRefresh = true;

    RestoreEnvironmentState(ActivePlot = N);
    if (N != prevActive)
        RestoreEnvironmentState(ActivePlot = prevActive);

    ThreeDPlots[ActivePlot]->blockRefresh = blockActive; //order for N == Active
    ThreeDPlots[N]->blockRefresh = block;

    ThreeDPlots[N]->Refresh();
}
void T3dPlot::on_BSave_clicked()
{
    if (!ThreeDPlots.at(ActivePlot)->SourceData.size() || !ThreeDPlots.at(ActivePlot)->SourceData.at(0))
        return;

    TMin3dData *data = ThreeDPlots.at(ActivePlot)->SourceData.at(0).data();
    data = data->CropIndex(data->LimsForMathX[0], data->LimsForMathY[0], data->LimsForMathX[1]-1, data->LimsForMathY[1]-1);
    T3dDataOwner src;
    src.append(QSharedPointer<TMin3dData>(data));

    if (data->NYElems < 2 || data->NXElems < 2 ||
            (data->XLimits[1] - data->XLimits[0]) / (data->YLimits[1] - data->YLimits[0]) > 40 ||
            (data->YLimits[1] - data->YLimits[0]) / (data->XLimits[1] - data->XLimits[0]) > 40)
        return;

    TMetaStorage attributes;
    attributes.insert("Preview", QVariant::fromValue(MakePreview(ActivePlot)));

    appendAttrib(attributes, ActivePlot);

    attributes.insert("XPos", data->XLimits[0]);
    attributes.insert("YPos", data->YLimits[0]);

    attributes.insert("XMin", 0);
    attributes.insert("XMax", data->XLimits[1] - data->XLimits[0]);
    attributes.insert("YMin", 0);
    attributes.insert("YMax", data->YLimits[1] - data->YLimits[0]);

    attributes.insert("XYLabel", ThreeDPlots.at(ActivePlot)->XYAxisUnit.Description);
    attributes.insert("XYUnit", ThreeDPlots.at(ActivePlot)->XYAxisUnit.SIUnit);
    attributes.insert("ZLabel", ThreeDPlots.at(ActivePlot)->ZAxisUnit.Description);
    attributes.insert("ZUnit", ThreeDPlots.at(ActivePlot)->ZAxisUnit.SIUnit);

    emit AddDataToContainer(src, attributes);
}
void T3dPlot::AutoSave(int Ind)
{
    int Curr = ActivePlot;

    SetActivePlot(Ind);
    on_BSave_clicked();

    SetActivePlot(Curr);
}

void T3dPlot::on_FontSize_changed()
{
    int size = PlotControl->AGFontSizes->checkedAction()->text().toInt();
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetFontSize(size);

    QFont font = qApp->font(); font.setPointSize(size);
    Slider->Title->setFont(font);
    Slider->Sliders->QwtBar->setFont(font);
}
void T3dPlot::on_BCrossection_clicked()
{
    ThreeDPlots[ActivePlot]->BCrossectionIsChecked = PlotControl->BCrossection->isChecked();
    if (PlotControl->BCrossection->isChecked())
    {
        TwoDPlot->Pure2dPlot->SetTitle(QObject::tr("section"));
        Set2dPlotVisible(true);
        AdjustTwoDPlot();

        PlotControl->BCrossection->setIcon(makeHugeIcon(":/plot/cross_sectionOn.png"));
    }
    else
    {
        curveData.clear();
        Set2dPlotVisible(false);
        PlotControl->BCrossection->setIcon(makeHugeIcon(":/plot/cross_section.png"));
    }
}


void T3dPlot::on_GwyXFit_triggered(bool Checked)
{
    ThreeDPlots[ActivePlot]->MakeCorrectLineByMedianaXDir(Checked);
}
void T3dPlot::on_GwyYFit_triggered(bool Checked)
{
    ThreeDPlots[ActivePlot]->MakeCorrectLineByMedianaYDir(Checked);
}


void T3dPlot::SetFilterSizeTextInAction(QAction *Act, const int Size, const QString &FilterName)
{
    if (Size > 0) Act->setText(FilterName + " (" + QString::number(Size) + "px.)");
    else Act->setText(FilterName);
}
void T3dPlot::SetAllFilterSizesTextInAction()
{
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())],
                              ThreeDPlots.at(ActivePlot)->MeanValueFilterSize, PlotControl->NameOfMeanValueFilter());
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())],
                              ThreeDPlots.at(ActivePlot)->MedianValueFilterSize, PlotControl->NameOfMedianValueFilter());
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())],
                              ThreeDPlots.at(ActivePlot)->GaussianFilterSize, PlotControl->NameOfGaussianFilter());
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())],
                              0, PlotControl->NameOfHistogramFilter());
}
void T3dPlot::MakeMeanValue(int Size)
{
    ThreeDPlots[ActivePlot]->MakeMeanValue(Size);
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())], Size,
                              PlotControl->NameOfMeanValueFilter());
}
void T3dPlot::MakeMedianValue(int Size)
{
    ThreeDPlots[ActivePlot]->MakeMedianValue(Size);
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())], Size,
                              PlotControl->NameOfMedianValueFilter());
}
void T3dPlot::MakeGaussian(int Size)
{
    ThreeDPlots[ActivePlot]->MakeGaussian(Size);
    SetFilterSizeTextInAction(PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())], Size,
                              PlotControl->NameOfGaussianFilter());
}

void T3dPlot::on_MeanValue_triggered(bool Checked)
{
    if (Checked) {
        TFilterSizeWidget FilterSizeWidget(this);
        FilterSizeWidget.setWindowTitle(tr("Mean filter size"));
        connect(&FilterSizeWidget, SIGNAL(SliderValueChanged(int)), this, SLOT(MakeMeanValue(int)));
        FilterSizeWidget.SetFilterSize(TFilterSizeWidget::DefaultFilterSize);
        Checked = FilterSizeWidget.exec();
    }
    if (!Checked) {
        MakeMeanValue(-1);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())]->setChecked(false);
    }
}
void T3dPlot::on_MedianValue_triggered(bool Checked)
{
    if (Checked) {
        TFilterSizeWidget FilterSizeWidget(this);
        FilterSizeWidget.setWindowTitle(tr("Median filter size"));
        connect(&FilterSizeWidget, SIGNAL(SliderValueChanged(int)), this, SLOT(MakeMedianValue(int)));
        FilterSizeWidget.SetFilterSize(TFilterSizeWidget::DefaultFilterSize);
        Checked = FilterSizeWidget.exec();
    }
    if (!Checked) {
        MakeMedianValue(-1);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())]->setChecked(false);
    }
}
void T3dPlot::on_Gaussian_triggered(bool Checked)
{
    if (Checked) {
        TFilterSizeWidget FilterSizeWidget(this);
        FilterSizeWidget.setWindowTitle(tr("Gaussian filter size"));
        connect(&FilterSizeWidget, SIGNAL(SliderValueChanged(int)), this, SLOT(MakeGaussian(int)));
        FilterSizeWidget.SetFilterSize(TFilterSizeWidget::DefaultFilterSize);
        Checked = FilterSizeWidget.exec();
    }
    if (!Checked) {
        MakeGaussian(-1);
        PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())]->setChecked(false);
    }
}
void T3dPlot::on_Histogram_triggered(bool Checked)
{
    ThreeDPlots[ActivePlot]->MakeHistogram(Checked);
}

void T3dPlot::on_FitXOrder_changed()
{
    ThreeDPlots[ActivePlot]->MakeXFit(FindCheckedItemIn(PlotControl->AGXFitOrders) - 1);
}
void T3dPlot::on_FitYOrder_changed()
{
    ThreeDPlots[ActivePlot]->MakeYFit(FindCheckedItemIn(PlotControl->AGYFitOrders) - 1);
}
void T3dPlot::on_SubtrOrder_changed()
{
    ThreeDPlots[ActivePlot]->MakeSubtr(FindCheckedItemIn(PlotControl->AGSubtr) - 1);
}
void T3dPlot::on_AutoZScale_changed(bool on)
{
    ThreeDPlots[ActivePlot]->AutoZScaleOn = on;
    if (on) {
        ThreeDPlots[ActivePlot]->UpdateHistogram(ThreeDPlots.at(ActivePlot)->zhistogram);
        Slider->Sliders->SetMinMaxRelSlidersValues(ThreeDPlots.at(ActivePlot)->MinSliderVal, ThreeDPlots.at(ActivePlot)->MaxSliderVal);
    }
}

void T3dPlot::on_BFixOXDir_clicked()
{
    if (PlotControl->BFixOYDir->isChecked()) {
        ThreeDPlots[ActivePlot]->BFixOYDirIsChecked = false;
        PlotControl->BFixOYDir->setChecked(false);
        ThreeDPlots[ActivePlot]->on_BFixOYDir_clicked();

        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));
    }

    if (PlotControl->BFixOXDir->isChecked())
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y_On.png"));
    else
        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));

    ThreeDPlots[ActivePlot]->BFixOXDirIsChecked = PlotControl->BFixOXDir->isChecked();
    ThreeDPlots[ActivePlot]->on_BFixOXDir_clicked();
}
void T3dPlot::on_BFixOYDir_clicked()
{
    if (PlotControl->BFixOXDir->isChecked()) {
        ThreeDPlots[ActivePlot]->BFixOXDirIsChecked = false;
        PlotControl->BFixOXDir->setChecked(false);
        ThreeDPlots[ActivePlot]->on_BFixOXDir_clicked();

        PlotControl->BFixOXDir->setIcon(makeHugeIcon(":/plot/markers_Y.png"));
    }

    if (PlotControl->BFixOYDir->isChecked())
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X_On.png"));
    else
        PlotControl->BFixOYDir->setIcon(makeHugeIcon(":/plot/markers_X.png"));

    ThreeDPlots[ActivePlot]->BFixOYDirIsChecked = PlotControl->BFixOYDir->isChecked();
    ThreeDPlots[ActivePlot]->on_BFixOYDir_clicked();
}
void T3dPlot::UpdateAllMarkersInfo()
{
    TPure3dPlot *Plt;
    for (int i = 0; i < ThreeDPlots.size(); i++) {
         Plt = ThreeDPlots[i];
         if (Plt->BMarkersIsChecked)
            Plt->UpdateLabels(Plt->BDetailedMarkersInfoIsChecked && i == ActivePlot);
    }

    Plt = ThreeDPlots[ActivePlot];
    if (!Plt->BMarkersIsChecked || !Plt->BDetailedMarkersInfoIsChecked)
        return;

    MarkersInfo->LEdXA->setText  (Plt->AMarkerInfo.XInfo.Value);
    MarkersInfo->LXAUnit->setText(Plt->AMarkerInfo.XInfo.Unit);
    MarkersInfo->LEdYA->setText  (Plt->AMarkerInfo.YInfo.Value);
    MarkersInfo->LYAUnit->setText(Plt->AMarkerInfo.YInfo.Unit);
    MarkersInfo->LEdZA->setText  (Plt->AMarkerInfo.ZInfo.Value);
    MarkersInfo->LZAUnit->setText(Plt->AMarkerInfo.ZInfo.Unit);

    MarkersInfo->LEdXB->setText  (Plt->BMarkerInfo.XInfo.Value);
    MarkersInfo->LXBUnit->setText(Plt->BMarkerInfo.XInfo.Unit);
    MarkersInfo->LEdYB->setText  (Plt->BMarkerInfo.YInfo.Value);
    MarkersInfo->LYBUnit->setText(Plt->BMarkerInfo.YInfo.Unit);
    MarkersInfo->LEdZB->setText  (Plt->BMarkerInfo.ZInfo.Value);
    MarkersInfo->LZBUnit->setText(Plt->BMarkerInfo.ZInfo.Unit);

    double xdist = Plt->BMarkerInfo.XInfo.SrcValue - Plt->AMarkerInfo.XInfo.SrcValue;
    double ydist = Plt->BMarkerInfo.YInfo.SrcValue - Plt->AMarkerInfo.YInfo.SrcValue;
    double zdist = Plt->BMarkerInfo.ZInfo.SrcValue - Plt->AMarkerInfo.ZInfo.SrcValue;

    TValueInfoCreator ValueInfo;
    ValueInfo(Plt->AutoLegendEnabled, Plt->XYAxisUnit, xdist);
    MarkersInfo->LEdDeltaX->setText(ValueInfo.Value); MarkersInfo->LDeltaXUnit->setText(ValueInfo.Unit);

    ValueInfo(Plt->AutoLegendEnabled, Plt->XYAxisUnit, ydist);
    MarkersInfo->LEdDeltaY->setText(ValueInfo.Value); MarkersInfo->LDeltaYUnit->setText(ValueInfo.Unit);

    ValueInfo(Plt->AutoLegendEnabled, Plt->ZAxisUnit, zdist);
    MarkersInfo->LEdDeltaZ->setText(ValueInfo.Value); MarkersInfo->LDeltaZUnit->setText(ValueInfo.Unit);

    double Distance = sqrt(xdist * xdist + ydist * ydist);
    if (Plt->XYAxisUnit.SIUnit == Plt->ZAxisUnit.SIUnit)
        Distance = sqrt(xdist * xdist + ydist * ydist + zdist * zdist);

    ValueInfo(Plt->AutoLegendEnabled, Plt->XYAxisUnit, Distance);
    MarkersInfo->LEdDistance->setText(ValueInfo.Value); MarkersInfo->LDistanceUnit->setText(ValueInfo.Unit);

    MarkersInfo->LEdAngle->setText(QString::number(Plt->CalcAngle(), 'f', 1));
}
void T3dPlot::MakeCrossection()
{
    double xA = ThreeDPlots.at(ActivePlot)->MarkerA->xValue();
    double xB = ThreeDPlots.at(ActivePlot)->MarkerB->xValue();
    double yA = ThreeDPlots.at(ActivePlot)->MarkerA->yValue();
    double yB = ThreeDPlots.at(ActivePlot)->MarkerB->yValue();
    double DeltaX = xB - xA, DeltaY = yB - yA;
    double NumberOfSteps = 1000;//TODO ThreeDPlots.at(ActivePlot)->GetMaxNumberOfPointsXY(xA, yA, xB, yB);
    double x = xA, y = yA, stepx, stepy;

    if (xA > xB)
        qSwap(xA, xB);
    if (yA > yB)
        qSwap(yA, yB);

    ResizeOwner(curveData, 1);
    curveData[0]->YSeq.clear();

    stepx = DeltaX / (NumberOfSteps - 1);
    stepy = DeltaY / (NumberOfSteps - 1);
    curveData[0]->setXCalib(0, QLineF(0, 0, stepx, stepy).length());
    for (int i = 0; i < NumberOfSteps; i++, x += stepx, y += stepy)
        curveData[0]->AddY(ThreeDPlots.at(ActivePlot)->value(qBound(xA, x, xB), qBound(yA, y, yB)));

    TwoDPlot->Pure2dPlot->SourceData = getUnLock(curveData);
    AdjustAxisLegends();
    TwoDPlot->Refresh();
}
void T3dPlot::UpdateMinMaxInSlider()
{
    double ZMult = 1.;
    QString Unit;

    if (ThreeDPlots.at(ActivePlot)->AutoLegendEnabled) {
        ZMult = ThreeDPlots.at(ActivePlot)->ZAxisUnit.Multiplier;
        Unit = ThreeDPlots.at(ActivePlot)->ZAxisUnit.StrMultiplier + ThreeDPlots.at(ActivePlot)->ZAxisUnit.SIUnit;
    }

    Slider->Sliders->SetMinMaxValues(ThreeDPlots.at(ActivePlot)->PlotsZLimits[0] / ZMult, ThreeDPlots.at(ActivePlot)->PlotsZLimits[1] / ZMult);
    Slider->Title->setText(Unit);
}

template <int N>
void ColorMapHistogramEqualization(const THistogram<N> &zhistogram, QwtLinearColorMap *map, double from, double to)
{
    from *= N; //from [0..1]
    to *= N;

    //Piecewise Affine Histogram Equalization, Jose Luis Lisani, Ana Belen Petro, Catalina Sbert 2012
    //http://dx.doi.org/10.5201/ipol.2012.lps-pae
#define HIST_EQUAL_PIECES 20
    int hist_part = (int)(to) - (int)from + 1;
    int pieces = qMin(hist_part, HIST_EQUAL_PIECES);
    if (pieces < 2)
        return;

    QVector<double> cumm_hist_normal(hist_part);
    int acc = 0;
    for (int i = (int)from; i <= (int)(to); i++) {
        double active_part = qMin(1., qMin(to - i, i+1 - from));
        acc += zhistogram.bins[i] * active_part;
        cumm_hist_normal[i - (int)from] = acc;
    }
    if (cumm_hist_normal.last() < 3)
        return;
    //normalize
    for (int i = 0; i < hist_part; i++)
        cumm_hist_normal[i] /= cumm_hist_normal.last();

    QVector<double> newposx(pieces); //partition histogram in equal-volume pieces
    int j = 1; //j: histogram index
    for (int i = 0; i < pieces; i++) {
        double piece_threshold = double(i) / (pieces - 1);
        if (i == 0)
            piece_threshold = 0;
        while (j < hist_part && cumm_hist_normal.at(j) < piece_threshold)
            j++;

        double part = (piece_threshold - cumm_hist_normal.at(j-1)) /
                (cumm_hist_normal.at(j) - cumm_hist_normal.at(j-1));
        double pos = (((j-1) + part) - (from-(int)from)) / (to - from);
        newposx[i] = qBound(0., pos, 1.);
    }
    //normalize range
    for (int i = 0; i < pieces; i++)
        ;//newposx[i] /= newposx.last(); //adds some flicker

    QVector<double> newposy(pieces); //palette partition
    newposy[0] = 0;
    //limit stretch
    for (int i = 1; i < pieces; i++) {
        newposy[i] = i / double(pieces - 1); //regular initially
        double k = newposy[i] - newposy[i-1];
        double scale = newposx[i] - newposx[i-1];
        k /= scale;
        //qDebug() << i << k;
        if (k < 0.3 || k > 3.) {
            k = qBound(0.3, k, 3.); //limit
            newposy[i] = k * scale + newposy[i-1];
        }
    }
    //normalize range
    for (int i = 0; i < pieces; i++) {
        newposy[i] /= newposy.last();
        //qDebug() << newposx[i] << newposy[i] << i / double(pieces - 1);
    }

    QVector<double> stops;
    //split at palette partition borders
    stops = map->colorStops();
    stops = stops.mid(1, stops.size() - 2); //skip 0,1
    for (int i = 1; i < pieces-1; i++) { //skip first,last
        double pos = stops.first() + newposy.at(i) * double(stops.last() - stops.first());
        bool found = false;
        for (int i = 1; i < stops.size()-1; i++) //skip first,last
            if (qFuzzyCompare(pos + 1, stops.at(i) + 1)) {
                found = true;
                break;
            }
        if (!found)
            map->addColorStop(pos, map->rgb(QwtInterval(0, 1), pos));
    }

    //stretch palette from palette partition to histogram partition
    stops = map->colorStops();
    QVector<QRgb> colors(stops.size());
    for (int i = 0; i < stops.size(); i++)
        colors[i] = map->rgb(QwtInterval(0, 1), stops.at(i));

    const double len = stops.at(stops.size()-2) - stops.at(1);
    j = 1; //j: target palette piece index
    for (int i = 2; i < stops.size()-2; i++) { //skip 0,first,last,1
        stops[i] -= stops.at(1); //normalize
        stops[i] /= len;

        while (j < pieces-1 && newposy.at(j) < stops[i])
            j++;
        double cpos = (stops[i] - newposy.at(j-1)) /
                (newposy.at(j) - newposy.at(j-1));
        stops[i] = stops.at(1) + len *
                (newposx.at(j-1) + cpos *
                 (newposx.at(j) - newposx.at(j-1)));
    }

    //apply
    map->setColorInterval(colors.first(), colors.last());
    for (int i = 1; i < stops.size()-1; i++) //skip 0,1
        map->addColorStop(stops.at(i), colors.at(i));
}

template <int Size>
void TPure3dPlot::UpdateHistogram(const THistogram<Size> &zhistogram)
{
    if (!AutoZScaleOn)
        return;

    int sum = 0;
    for (int i = 0; i < Size; i++)
        sum += zhistogram.bins[i];
    if (!sum)
        return;

    //autoscale by throwing out outermost values
#define IGNORE_PART 0.01
    int acc;
    int low = 0;
    acc = 0;
    while (low < Size-1-1) {
        acc += zhistogram.bins[low];
        if (acc > int(sum * IGNORE_PART))
            break;
        low++;
    }

    int high = Size-1;
    acc = 0;
    while (high > low+1) {
        acc += zhistogram.bins[high];
        if (acc > int(sum * IGNORE_PART))
            break;
        high--;
    }

    MinSliderVal = low / (double)(Size-1);
    MaxSliderVal = high / (double)(Size-1);
    SetPalette(active_palette, MinSliderVal, MaxSliderVal);
}
template <int Size>
void T3dPlot::UpdateSliderHistogram(const THistogram<Size> &zhistogram)
{
    int max = 0;
    int sum = 0;
    for (int i = 0; i < Size; i++) {
        sum += zhistogram.bins[i];
        max = qMax(max, zhistogram.bins[i]);
    }

#define HIST_PREC_SIZE 200
    QPixmap bg(HIST_PREC_SIZE, Size);
    bg.fill(palette().color(QPalette::Background));
    if (sum) {
        QPainter p(&bg);
        p.setPen(Qt::lightGray);
        for (int i = 0; i < Size; i++)
            p.drawLine(HIST_PREC_SIZE-2, Size-1 - i,
                       HIST_PREC_SIZE-2 - HIST_PREC_SIZE*.5 * zhistogram.bins[i] / max, Size-1 - i);
        p.end();
    }

    Slider->Sliders->bgItem->setPixmap(bg);


    if (sum && ThreeDPlots.at(ActivePlot)->AutoZScaleOn) {
        ThreeDPlots[ActivePlot]->blockRefresh = true;
        Slider->Sliders->SetMinMaxRelSlidersValues(ThreeDPlots.at(ActivePlot)->MinSliderVal, ThreeDPlots.at(ActivePlot)->MaxSliderVal);
        ThreeDPlots[ActivePlot]->blockRefresh = false;
    }
}

void T3dPlot::Refresh()
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->Refresh();
}
void T3dPlot::SetPlotsXScales(double Min, double Max)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetPlotsXScales(Min, Max);

    if (Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(0)->PlotsXLimits,
                                                            ThreeDPlots.at(0)->PlotsYLimits,
                                                            TScalesChangedEvent::R_SetPlotsXScales_Called));
}
void T3dPlot::SetPlotsYScales(double Min, double Max)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetPlotsYScales(Min, Max);

    if (Parent)
        QCoreApplication::postEvent(Parent,
                                    new TScalesChangedEvent(ThreeDPlots.at(0)->PlotsXLimits,
                                                            ThreeDPlots.at(0)->PlotsYLimits,
                                                            TScalesChangedEvent::R_SetPlotsYScales_Called));
}
void T3dPlot::SetTitle(const QString &Title, int N)
{
    if (N < 0)
        for (int i = 0; i < ThreeDPlots.size(); i++)
            ThreeDPlots[i]->SetTitle(Title);
    else
        ThreeDPlots[N]->SetTitle(Title);
}
void T3dPlot::SetZoomBySelection(bool State)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetZoomBySelection(State);
}
void T3dPlot::SyncZoom(int N, const QPointF &Pos, int Increment, bool /*BFixX_isChecked*/, bool /*BFixY_isChecked*/)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == N) continue;
        ThreeDPlots[i]->ZoomPlot(Pos, Increment);
    }
}
void T3dPlot::SyncPan(int N, const QPointF &Delta)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == N) continue;
        ThreeDPlots[i]->PanPlot(Delta);
    }
}
void T3dPlot::SyncDefineClickInfo(int N, const QPoint &Pos)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
    {
        if (i == N) continue;
        ThreeDPlots[i]->DefineClickInfo(Pos);
    }
}
void T3dPlot::SetCrossCoords(double x, double y)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetCrossCoords(x, y);
}
void T3dPlot::HideCross()
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->HideCross();
}
void T3dPlot::SetDefaultXLimits(double Min, double Max)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetDefaultXLimits(Min, Max);
}
void T3dPlot::SetDefaultYLimits(double Min, double Max)
{
    for (int i = 0; i < ThreeDPlots.size(); i++)
        ThreeDPlots[i]->SetDefaultYLimits(Min, Max);
}
void T3dPlot::SetXYDescAndSIUnit(const QString &Desc, const QString &Unit, int N)
{
    if (N >= 0)
        ThreeDPlots[N]->XYAxisUnit.SetDescAndSIUnit(Desc, Unit);
    else
        for (int i = 0; i < ThreeDPlots.size(); i++)
            ThreeDPlots[i]->XYAxisUnit.SetDescAndSIUnit(Desc, Unit);

    AdjustTwoDPlot();
}
void T3dPlot::SetZDescAndSIUnit(const QString &Desc, const QString &Unit, int N)
{
    if (N >= 0)
        ThreeDPlots[N]->ZAxisUnit.SetDescAndSIUnit(Desc, Unit);
    else
        for (int i = 0; i < ThreeDPlots.size(); i++)
            ThreeDPlots[i]->ZAxisUnit.SetDescAndSIUnit(Desc, Unit);

    AdjustTwoDPlot();
}

void T3dPlot::Set2dPlotVisible(bool Visible)
{
    if (Visible)
    {
        QList<int> WidgetSizes;
        WidgetSizes.append(2 * Split2d->height() * 0.7);
        WidgetSizes.append(2 * Split2d->height() * 0.3);

        if (Split2d->sizes().at(1) == 0)
            Split2d->setSizes(WidgetSizes);
    }
    else
        Split2d->setSizes(QList<int>() << 1 << 0);
}

}
