#pragma once

#include "2d/plot_2dplot.h"
#include "plot_3dtypes.h"
#include "widgets/plot_DoubleSlider.h"
#include "plot_3dWidgets.h"
class QSplitter;

namespace plot
{

class T3dMarkerInfoHandler: public TGeneralMarkerInfoHandler
{
public:
    TValueInfoCreator ZInfo;
    QString FormUniteString() { return TGeneralMarkerInfoHandler::FormUniteString() + "\nZ = " + ZInfo.Value + ZInfo.Unit; }
    void operator()(bool AutoLegendEnabled, const TSIUnit &XYUnitInfo, const TSIUnit &ZUnitInfo, double XVal, double YVal, double ZVal);
};

class T3dMime : public QMimeData
{
    Q_OBJECT
public:
    T3dData data;
};

class T3dPlot;
namespace math { class TMathProcessing; }
class TPure3dPlot : public TCommonPlot, public TCompleteInfoFor3dPlot, public ControlPanelHistory
{
    Q_OBJECT
public slots:
    void s_Refresh();

signals:
    void MathUpdated();

public:
    using TCommonPlot::UpdateLabels;
    void UpdateLabels(bool IsDetailedInfoVisible);
    void HandleAxisMultiplier();
    void UpdateAxisLegends();
    inline void MakeCrossection();

    void dragEnterEvent(QDragEnterEvent *event);
    void dropEvent(QDropEvent *event);

    template <int Size>
    void UpdateHistogram(const THistogram<Size> &zhistogram);

    int N;
    T3dPlot *Child;
    QwtPlotMarker *Cross;
    bool AreaZzoom;
    int zoom_active;
    QPointF areazA;
    QPointF areazAo;
    QPointF areazB;
    QRubberBand *areaRB;
    bool ZoomBySelection;

    int last_resize_x;

    void AreaZzoomST(double x, double y, double xo, double yo, int st);

    int Height() const { return QPlot->size().height(); }

    bool eventFilter(QObject *Object, QEvent *Event);
    void HandleMoveMarkers(const QPointF &APos, const QPointF &BPos);
    void ResizeQPlot(bool move = false);
    QSize FindAvailableSpaceForQPlot() const;

    void on_BFixOXDir_clicked();
    void on_BFixOYDir_clicked();
    void SelectRectChanged(const QRectF &Rect);
    void BRestoreScale_clicked()
    {
        //if (ZoomBySelection) {
        PlotsXLimits = DefaultXLimits;
        PlotsYLimits = DefaultYLimits;
        //}

        TCommonPlot::BRestoreScale_clicked();
        ResizeQPlot();
    }
    void SetZoomBySelection(bool State) { ZoomBySelection = State; }
    void SelectDone();

    void SetCrossCoords(double x, double y);
    void HideCross();

    math::TMathProcessing *MathProcessing;
    bool isMathReady();
    bool IsMathEnabled() const;
    void MakeXFit(const int Order);
    void MakeYFit(const int Order);
    void MakeSubtr(const int Order);
    void MakeCorrectLineByMedianaXDir(const bool Make);
    void MakeCorrectLineByMedianaYDir(const bool Make);
    void MakeMeanValue(const int Size = 10);
    void MakeMedianValue(const int Size = 10);
    void MakeGaussian(const int Size = 10);
    void MakeHistogram(const bool Make);

    T3dMarkerInfoHandler AMarkerInfo;
    T3dMarkerInfoHandler BMarkerInfo;

    Q_INVOKABLE void Refresh();
    bool wait_for_math;
    bool math_delayed_refresh;

    TPure3dPlot(int N, T3dPlot *Chld, QObject *Parent = 0, QWidget *parent = 0);
    ~TPure3dPlot();
    QObject *Parent; //for postEvent(Parent, blablabla);
};

class T3dPlot: public QWidget
{
    Q_OBJECT
public slots:
    void appendAttrib(plot::TMetaStorage &, int N = 0); //customization/filters
    void restoreAttributes(plot::TMetaStorage, int N = 0);

    void UpdateAllMarkersInfo();
    void on_BGrid_clicked();
    void on_BFixX_toggled();
    void on_BFixY_toggled();
    void on_BSave_clicked();
    void on_BCross_clicked();
    void on_BAngle_clicked();
    void on_BMarkers_clicked(bool);
    void on_BFixOXDir_clicked();
    void on_BFixOYDir_clicked();
    void on_GwyXFit_triggered(bool Checked);
    void on_GwyYFit_triggered(bool Checked);
    void MakeGaussian(int Size);
    void MakeMeanValue(int Size);
    void MakeMedianValue(int Size);
    void on_MeanValue_triggered(bool Checked);
    void on_MedianValue_triggered(bool Checked);
    void on_Gaussian_triggered(bool Checked);
    void on_Histogram_triggered(bool Checked);
    void on_MPalettes_trigered();
    void on_BCrossection_clicked();
    void on_BAreaZzoom_clicked();
    void on_BSelectByRect_clicked();
    void on_BRestoreScale_clicked();
    void on_BApply_clicked();
    void on_BSavePlotPixmap_clicked();
    void on_BCancel_clicked();
    void on_BDetailedMarkersInfo_clicked();
    void on_Sliders_ValuesChanged(double Min, double Max);
    void on_BExpandSelectRect_clicked() { ThreeDPlots[ActivePlot]->ExpandSelectionRect(); ThreeDPlots[ActivePlot]->ReDrawSelectedRectangle(TSelectEvent::SRO_Mouse); }
    void on_FontSize_changed();

    void on_FitXOrder_changed();
    void on_FitYOrder_changed();
    void on_SubtrOrder_changed();
    void on_AutoZScale_changed(bool on);

public slots:
    void Show2dPlotControl()
    {
        PlotControl->BCross->change_state_man_with_sig(false);
        PlotControl->hide();
        TwoDPlot->PlotControl->show();
    }
    void Set2dPlotVisible(bool Visible);
    void HideSelectionIfAny() { if (ThreeDPlots.at(ActivePlot)->SelectionModeEnabled) on_BCancel_clicked(); }

signals:
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);
    void ActivePlotChanged(int N);

public:
    T3dPlot(int N = 1, QObject *Parent = NULL, QWidget *wparent = 0);
    void Refresh();
    void AutoSave(int Ind);

    void SetActivePlot(int N);
    void RestoreEnvironmentState(int N);


    void GUICreate(int N);
    void ConnectSlots();
    void ConnectPlotControlSlots();
    void AdjustTwoDPlot();
    void MakeCrossection();
    void AdjustAxisLegends();

    void InitPalettes();
    static TPalette GetSpectralPalette(const Range &ZLimits, double X1, double X2);
    void ResizeSliders();
    void UpdateMinMaxInSlider();
    template <int Size>
    void UpdateSliderHistogram(const THistogram<Size> &zhistogram);
    bool eventFilter(QObject *Object, QEvent *Event);

    QVector<TPalette> Palettes;

    void SyncPan(int N, const QPointF &Delta);
    void SyncDefineClickInfo(int N, const QPoint &Pos);
    void SyncZoom(int N, const QPointF &Pos, int Increment, bool BFixX_isChecked, bool BFixY_isChecked);


    void SetTitle(const QString &Title, int N = 0);
    void SetZoomBySelection(bool State);
    QImage MakePreview(int N = 0) { return ThreeDPlots[N]->MakePreview(PlotControl); }

    void SetPlotsXScales(double Min, double Max);
    void SetPlotsYScales(double Min, double Max);
    void SetAllXScales(double Min, double Max) { SetDefaultXLimits(Min, Max); SetPlotsXScales(Min, Max); }
    void SetAllYScales(double Min, double Max) { SetDefaultYLimits(Min, Max); SetPlotsYScales(Min, Max); }
    void SetDefaultXLimits(double Min, double Max);
    void SetDefaultYLimits(double Min, double Max);

    void SetXYDescAndSIUnit(const QString &Desc, const QString &Unit, int N = 0);
    void SetZDescAndSIUnit(const QString &Desc, const QString &Unit, int N = 0);

    void HideCross();
    void SetCrossCoords(double x, double y);

    void SetAllFilterSizesTextInAction();
    void SetFilterSizeTextInAction(QAction *Act, const int Size, const QString &FilterName);

    void SetSelectWidthRelCenter(qreal W) { ThreeDPlots[ActivePlot]->SetSelectWidthRelCenter(W); }
    void SetSelectHeightRelCenter(qreal H) { ThreeDPlots[ActivePlot]->SetSelectHeightRelCenter(H); }
    void SetSelectWidth(qreal W) { ThreeDPlots[ActivePlot]->SetSelectWidth(W); }
    void SetSelectHeight(qreal H) { ThreeDPlots[ActivePlot]->SetSelectHeight(H); }

    QVector<TPure3dPlot *> ThreeDPlots;
    T2dPlot *TwoDPlot;
    QVector<QSplitter *> Containers;
    TCurvesDataOwner curveData;

    int ActivePlot;

    QObject *Parent;
    TDoubleSlider *Slider;

    QSplitter *Split3d;
    QSplitter *Split2d;

    T3dMarkersInfo *MarkersInfo;
    T3dPlotControl *PlotControl;
};

}
