#ifndef PLOT_MATHFILTERS_H
#define PLOT_MATHFILTERS_H

#include "math_core.h"

namespace plot {
namespace math {

#define EXEC_FILTER_WITH_SIZE(__filter_name) \
    InitVars(Arr, Sz, StartX, StartY, EndX, EndY); \
    if (Size <= 0) return; \
    __filter_name(Arr); \
    on_FilterFinish(Arr);

class TCorrectLinesByMedian: private TMedian
{
    template <class T>
    void AddValueTo(const T F, const T L, double Val) const
    { for (T It = F; It != L; *(It++) += Val); }
    void AddValueTo(T2dArray &Arr, const int X, const int StartY, const int EndY, const double Val) const
    { for (int Y = StartY; Y < EndY; Arr[Y++][X] += Val); }
    void FetchLine(QVector<double> &Line, const int X, const int StartY, const int EndY, const T2dArray &Arr) const
    { Line.clear(); for (int Y = StartY; Y < EndY; Line.push_back(Arr[Y++][X])); }
public:
    TCorrectLinesByMedian(T2dArray *Arr = NULL, TPlotEnums::TAxis Dir = TPlotEnums::A_XAxis, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1)
    { if (Arr) Dir == TPlotEnums::A_XAxis ? CorrectLinesByMedianXDir(*Arr, StartX, StartY, EndX, EndY) : CorrectLinesByMedianYDir(*Arr, StartX, StartY, EndX, EndY); }
    void CorrectLinesByMedianXDir(T2dArray &Arr, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1) const;
    void CorrectLinesByMedianYDir(T2dArray &Arr, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1) const;
};

class TMeanValue: public TMathCore
{
    void MeanValue(const T2dArray &Arr, bool Average = true);
public:
    TMeanValue(T2dArray *Arr = NULL, int Size = 10, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1)
    { if (Arr) MeanValue(*Arr, Size, StartX, StartY, EndX, EndY); }
    void MeanValue(T2dArray &Arr, int Sz, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1)
    { EXEC_FILTER_WITH_SIZE(MeanValue); }
};

class TMedianValue: private TMedian, private TMathCore
{
    void MedianValue(const T2dArray &Arr) const;
public:
    TMedianValue(T2dArray *Arr = NULL, int Size = 10, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1)
    { if (Arr) MedianValue(*Arr, Size, StartX, StartY, EndX, EndY); }
    void MedianValue(T2dArray &Arr, int Sz, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1)
    { EXEC_FILTER_WITH_SIZE(MedianValue); }
};

class TGaussian: private TMathCore
{
    template <class TIter>
    double CalcSum(const TIter B, const TIter E) const        { double R = 0; for (TIter It = B; It != E; R += *(It++)); return R; }
    template <class TIter>
    void MultiplyBy(const TIter B, const TIter E, const double Val) const { for (TIter It = B; It != E; *(It++) *= Val); }
    void MultiplyBy(double *Area, const int StartX, const int StartY, const int EndX, const int EndY, const double Val, const int W) const;

    void Convolve1d(const QVector<double> &kernel, const plot::TPlotEnums::TAxis orientation, const T2dArray &Arr) const;
    void HConvolved(const QVector<double> &kernel, const T2dArray &Arr) const;
    void VConvolved(const QVector<double> &kernel, const T2dArray &Arr) const;

    void Gaussian(const T2dArray &Arr) const;
public:
    TGaussian(T2dArray *Arr = NULL, int Size = 10, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1)
    { if (Arr) Gaussian(*Arr, Size, StartX, StartY, EndX, EndY); }
    void Gaussian(T2dArray &Arr, int Sz, int StartX = -1, int StartY = -1, int EndX = -1, int EndY = -1);
};

}
}

#endif // PLOT_MATHFILTERS_H
