#include "videoplayer.h"
#include "gui/status.hpp"
#include "motors.hpp"
#include "remote_nst_dataid.hpp"
#include "oscilloscope.h"
#include "modules.hpp"
#include "mainwindow.h"

#include "ratiolayout.hpp"
#include "flowlayout.hpp"
#include "gui/mslider.hpp"
#include "qwt_text_label.h"
#include <qwt_plot_marker.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_grid.h>
#include <QCheckBox>
#include <QApplication>
#include <QStyleFactory>
#include <QRubberBand>
#include <QComboBox>
#include <QGroupBox>

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#include <QtConcurrent>
#else
#include <QtCore>
#endif

#ifdef USE_IVIDEO_MOTIC
#include "hardware/motic.hpp"
#endif
#ifdef USE_IVIDEO_MOTIC_X
#include "hardware/moticx.hpp"
#endif


void VideoPlayer::closeEvent(QCloseEvent *event)
{
    PBToNewWindow->click();
    event->ignore();
}

bool VideoPlayer::eventFilter(QObject *obj, QEvent *Event)
{
    if (obj == QPlot->canvas()) {
        if (Event->type() == QEvent::MouseButtonPress &&
                ((QMouseEvent *)Event)->button() == Qt::RightButton) {
            plot_mode = 0;
            active = false;
            BCenterAndSet->change_state_man(false);
            BCenterAndSet2->change_state_man(false);
            BLine->change_state_man(false);
            BLineMeasure->change_state_man(false);
            markerA->hide();
            markerB->hide();
            lineAB->hide();
            markerC->setLabel(QwtText());
            QPlot->replot();

            return true;
        } else if (Event->type() == QEvent::MouseButtonPress) {
            posx = ((QMouseEvent *)Event)->posF();
            active = plot_mode == 0;
            prev_xy_joy_output = QPointF();

            if (plot_mode == 0) {
                qreal x = QPlot->invTransform(QwtPlot::xBottom, posx.x());
                qreal y = QPlot->invTransform(QwtPlot::yLeft, posx.y());
                lineAB->SetData(QPointF(x, y), QPointF(x, y));
                lineAB->show();
            }
            if (plot_mode == 3 || plot_mode == 4) {
                qreal x = QPlot->invTransform(QwtPlot::xBottom, posx.x());
                qreal y = QPlot->invTransform(QwtPlot::yLeft, posx.y());

                if (sqrt(pow(x - markerA->xValue(), 2) +
                         pow(y - markerA->yValue(), 2)) <
                        sqrt(pow(x - markerB->xValue(), 2) +
                             pow(y - markerB->yValue(), 2)))
                    marker = markerA;
                else
                    marker = markerB;

                qreal kx = fabs(QPlot->invTransform(QwtPlot::xBottom, 20) -
                                QPlot->invTransform(QwtPlot::xBottom, 0)) / 20.;
                qreal ky = fabs(QPlot->invTransform(QwtPlot::yLeft, 20) -
                                QPlot->invTransform(QwtPlot::yLeft, 0)) / 20.;
                const int XRange = 15, YRange = 15;
                if (fabs((x - marker->xValue()) / kx) < XRange &&
                        fabs((y - marker->yValue()) / ky) < YRange)
                    active = true;
            }
            return true;
        } else if (Event->type() == QEvent::MouseButtonRelease) {
            if (plot_mode == 0) {
                lineAB->hide();
                QPlot->replot();
                goXYjoy(QPointF());
            } else
            if (plot_mode == 1) {
                QRectF rect;
                rect = head_cant->geometry();
                rect.moveCenter(((QMouseEvent *)Event)->posF());
                head_cant->setGeometry(rect.toRect());
                rect = stage_cant->geometry();
                rect.moveCenter(((QMouseEvent *)Event)->posF());
                stage_cant->setGeometry(rect.toRect());

                (*settings.at(prev_idx))[7] = QVariant(
                            QPlot->invTransform(QwtPlot::xBottom,
                                                head_cant->geometry().center().x())).toString();
                (*settings.at(prev_idx))[8] = QVariant(
                            QPlot->invTransform(QwtPlot::yLeft,
                                                head_cant->geometry().center().y())).toString();

                head_cant->show();
                stage_cant->show();
                BCantileverRange->change_state_man(true);
                sl_save();
            } else if (plot_mode == 2) {
                QRectF rect;
                rect = stage_laser->geometry();
                rect.moveCenter(((QMouseEvent *)Event)->posF());
                stage_laser->setGeometry(rect.toRect());

                (*settings.at(prev_idx))[9] = QVariant(
                            QPlot->invTransform(QwtPlot::xBottom,
                                                stage_laser->geometry().center().x())).toString();
                (*settings.at(prev_idx))[10] = QVariant(
                            QPlot->invTransform(QwtPlot::yLeft,
                                                stage_laser->geometry().center().y())).toString();

                stage_laser->show();
                BLaserRange->change_state_man(true);
                sl_save();
            } else if (plot_mode == 3 && active) {
                QPointF posy = ((QMouseEvent *)Event)->posF();
                marker->setXValue(QPlot->invTransform(QwtPlot::xBottom, posy.x()));
                marker->setYValue(QPlot->invTransform(QwtPlot::yLeft, posy.y()));

                posx.rx() = QPlot->transform(QwtPlot::xBottom, markerA->xValue()) / QPlot->canvas()->width();
                posx.ry() = QPlot->transform(QwtPlot::yLeft, markerA->yValue()) / QPlot->canvas()->height();

                posy.rx() = QPlot->transform(QwtPlot::xBottom, markerB->xValue()) / QPlot->canvas()->width();
                posy.ry() = QPlot->transform(QwtPlot::yLeft, markerB->yValue()) / QPlot->canvas()->height();

                qreal kx = fabs(QPlot->invTransform(QwtPlot::xBottom, 20) -
                                QPlot->invTransform(QwtPlot::xBottom, 0)) / 20.;
                qreal ky = fabs(QPlot->invTransform(QwtPlot::yLeft, 20) -
                                QPlot->invTransform(QwtPlot::yLeft, 0)) / 20.;
                qreal dx = (markerB->xValue() - markerA->xValue()) / kx;
                qreal dy = (markerB->yValue() - markerA->yValue()) / ky;

                qreal dist = sqrt(dx*dx + dy*dy);
                qreal scale = ewsz->value() / dist;

                range_x->setValue(scale * QPlot->canvas()->width());
                range_y->setValue(scale * QPlot->canvas()->height());

                mkm_px_x = (qreal)QPlot->canvas()->width() / range_x->value();
                mkm_px_y = (qreal)QPlot->canvas()->height() / range_y->value();

                QPlot->setAxisScale(QwtPlot::xBottom, 0, range_x->value());
                QPlot->setAxisScale(QwtPlot::yLeft, 0, range_y->value());

                QRectF rect; QPointF center;
                rect = head_cant->geometry(); center = rect.center();
                rect.setSize(QSizeF(mkm_px_x * Xmax, mkm_px_y * Ymax)); rect.moveCenter(center);
                head_cant->setGeometry(rect.toRect());
                rect = stage_cant->geometry(); center = rect.center();
                rect.setSize(QSizeF(mkm_px_x * Xmax2, mkm_px_y * Ymax2)); rect.moveCenter(center);
                stage_cant->setGeometry(rect.toRect());
                rect = stage_laser->geometry(); center = rect.center();
                rect.setSize(QSizeF(mkm_px_x * Xmax2, mkm_px_y * Ymax2)); rect.moveCenter(center);
                stage_laser->setGeometry(rect.toRect());

                BAxes->change_state_man_with_sig(true);
                QPlot->replot();

                markerA->setXValue(QPlot->invTransform(QwtPlot::xBottom, posx.x() * QPlot->canvas()->width()));
                markerA->setYValue(QPlot->invTransform(QwtPlot::yLeft, posx.y() * QPlot->canvas()->height()));
                markerB->setXValue(QPlot->invTransform(QwtPlot::xBottom, posy.x() * QPlot->canvas()->width()));
                markerB->setYValue(QPlot->invTransform(QwtPlot::yLeft, posy.y() * QPlot->canvas()->height()));
                lineAB->SetData(QPointF(markerA->xValue(), markerA->yValue()),
                                QPointF(markerB->xValue(), markerB->yValue()));

                QPlot->replot();
                sl_save();
            }
            active = false;
            return true;
        } else if (Event->type() == QEvent::MouseMove && active) {
            if (plot_mode == 0) {
                qreal x = QPlot->invTransform(QwtPlot::xBottom, posx.x());
                qreal y = QPlot->invTransform(QwtPlot::yLeft, posx.y());
                qreal x2 = QPlot->invTransform(QwtPlot::xBottom, ((QMouseEvent *)Event)->posF().x());
                qreal y2 = QPlot->invTransform(QwtPlot::yLeft, ((QMouseEvent *)Event)->posF().y());
                lineAB->SetData(QPointF(x, y), QPointF(x2, y2));
                QPlot->replot();

                QPointF dxy = ((QMouseEvent *)Event)->posF() - posx;
                dxy.rx() *= 7. * 2 / qMin(QPlot->canvas()->width(), QPlot->canvas()->height());
                dxy.ry() *= -7. * 2 / qMin(QPlot->canvas()->width(), QPlot->canvas()->height());
                goXYjoy(dxy);
            }
            if (plot_mode == 3 || plot_mode == 4) {
                posx = ((QMouseEvent *)Event)->posF();
                marker->setXValue(QPlot->invTransform(QwtPlot::xBottom, posx.x()));
                marker->setYValue(QPlot->invTransform(QwtPlot::yLeft, posx.y()));

                lineAB->SetData(QPointF(markerA->xValue(), markerA->yValue()),
                                QPointF(markerB->xValue(), markerB->yValue()));

                if (plot_mode == 4) {
                    qreal dx = markerB->xValue() - markerA->xValue();
                    qreal dy = markerB->yValue() - markerA->yValue();

                    qreal dist = sqrt(dx*dx + dy*dy);
                    markerC->setXValue((markerA->xValue() +  markerB->xValue()) / 2);
                    markerC->setYValue((markerA->yValue() +  markerB->yValue()) / 2);
                    QwtText text;
                    text.setText(QString::number(dist, 'f', 2) + tr("micro") + tr("m"));
                    text.setColor(Qt::white);
                    markerC->setLabel(text);
                }

                QPlot->replot();
            }
            return true;
        } else if (Event->type() == QEvent::Resize) {
            mkm_px_x = (qreal)QPlot->canvas()->width() / range_x->value();
            mkm_px_y = (qreal)QPlot->canvas()->height() / range_y->value();

            QRectF rect;
            rect = head_cant->geometry();
            rect.setSize(QSizeF(mkm_px_x * Xmax, mkm_px_y * Ymax));
            rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                     settings.at(prev_idx)->at(7).toDouble()),
                                    QPlot->transform(QwtPlot::yLeft,
                                                     settings.at(prev_idx)->at(8).toDouble())));
            head_cant->setGeometry(rect.toRect());

            rect = stage_cant->geometry();
            rect.setSize(QSizeF(mkm_px_x * Xmax2, mkm_px_y * Ymax2));
            rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                     settings.at(prev_idx)->at(7).toDouble()),
                                    QPlot->transform(QwtPlot::yLeft,
                                                     settings.at(prev_idx)->at(8).toDouble())));
            stage_cant->setGeometry(rect.toRect());

            rect = stage_laser->geometry();
            rect.setSize(QSizeF(mkm_px_x * Xmax2, mkm_px_y * Ymax2));
            rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                     settings.at(prev_idx)->at(9).toDouble()),
                                    QPlot->transform(QwtPlot::yLeft,
                                                     settings.at(prev_idx)->at(10).toDouble())));
            stage_laser->setGeometry(rect.toRect());

            QPlot->replot();
        }
        return false;
    }
    else
        return QWidget::eventFilter(obj, Event);
}

void VideoPlayer::sl_center_set(bool value)
{
    if (value) {
        BCenterAndSet2->change_state_man(false);
        BLine->change_state_man(false);
        BLineMeasure->change_state_man(false);
        markerA->hide();
        markerB->hide();
        lineAB->hide();
        markerC->setLabel(QwtText());
        QPlot->replot();
        plot_mode = 1; //cantilever center

        I_MODULE *oscm = ModuleManager::instance->find1("oscilloscope");
        if (!oscm)
            return;
        Oscilloscope *osc = (Oscilloscope *)oscm->basecast();

        osc->DacsWidget->go_to_center(0);
        osc->DacsWidget->go_to_center(1);
    } else {
        plot_mode = 0;
    }
}
void VideoPlayer::sl_center_set2(bool value)
{
    if (value) {
        BCenterAndSet->change_state_man(false);
        BLine->change_state_man(false);
        BLineMeasure->change_state_man(false);
        markerA->hide();
        markerB->hide();
        lineAB->hide();
        markerC->setLabel(QwtText());
        QPlot->replot();
        plot_mode = 2; //laser center

        I_MODULE *oscm = ModuleManager::instance->find1("oscilloscope");
        if (!oscm)
            return;
        Oscilloscope *osc = (Oscilloscope *)oscm->basecast();

        osc->DacsWidget->go_to_center(0);
        osc->DacsWidget->go_to_center(1);
    } else {
        plot_mode = 0;
    }
}
void VideoPlayer::sl_axes(bool value)
{
    QPlot->enableAxis(QwtPlot::yLeft, value);
    QPlot->enableAxis(QwtPlot::xBottom, value);
    QPlot->replot();
}
void VideoPlayer::sl_grid(bool value)
{
    QGrid->setVisible(value);
    QPlot->replot();
}
void VideoPlayer::sl_line(bool value)
{
    if (value) {
        BCenterAndSet->change_state_man(false);
        BCenterAndSet2->change_state_man(false);
        BLineMeasure->change_state_man(false);
        plot_mode = 3; //line mode

        markerA->setXValue(range_x->value()*0.4);
        markerA->setYValue(range_y->value()*0.4);
        markerB->setXValue(range_x->value()*0.6);
        markerB->setYValue(range_y->value()*0.6);
        markerC->setLabel(QwtText());

        markerA->show();
        markerB->show();
        lineAB->SetData(QPointF(markerA->xValue(), markerA->yValue()),
                        QPointF(markerB->xValue(), markerB->yValue()));
        lineAB->show();

        QPlot->replot();
    } else {
        plot_mode = 0;
        markerA->hide();
        markerB->hide();
        lineAB->hide();
        QPlot->replot();
    }
}
void VideoPlayer::sl_line_measure(bool value)
{
    QPlot->canvas()->setCursor(QCursor(value ? Qt::ArrowCursor : Qt::OpenHandCursor));
    if (value) {
        BCenterAndSet->change_state_man(false);
        BCenterAndSet2->change_state_man(false);
        BLine->change_state_man(false);
        plot_mode = 4; //line measure mode

        markerA->setXValue(range_x->value()*0.4);
        markerA->setYValue(range_y->value()*0.4);
        markerB->setXValue(range_x->value()*0.6);
        markerB->setYValue(range_y->value()*0.6);

        qreal dx = markerB->xValue() - markerA->xValue();
        qreal dy = markerB->yValue() - markerA->yValue();

        qreal dist = sqrt(dx*dx + dy*dy);
        markerC->setXValue((markerA->xValue() +  markerB->xValue()) / 2);
        markerC->setYValue((markerA->yValue() +  markerB->yValue()) / 2);
        QwtText text;
        text.setText(QString::number(dist, 'f', 2) + tr("micro") + tr("m"));
        text.setColor(Qt::white);
        markerC->setLabel(text);
        markerC->show();

        markerA->show();
        markerB->show();
        lineAB->SetData(QPointF(markerA->xValue(), markerA->yValue()),
                        QPointF(markerB->xValue(), markerB->yValue()));
        lineAB->show();

        QPlot->replot();
    } else {
        plot_mode = 0;
        markerA->hide();
        markerB->hide();
        lineAB->hide();
        markerC->setLabel(QwtText());
        QPlot->replot();
    }
}
void VideoPlayer::sl_add_config()
{
    settings.append(new QList<QString>());
    settings.at(settings.size() - 1)->append(tr("Default"));
    settings.at(settings.size() - 1)->append("1024");
    settings.at(settings.size() - 1)->append("768");
    settings.at(settings.size() - 1)->append("100");
    settings.at(settings.size() - 1)->append("100");
    settings.at(settings.size() - 1)->append("false");
    settings.at(settings.size() - 1)->append("false");
    settings.at(settings.size() - 1)->append("0");
    settings.at(settings.size() - 1)->append("0");
    settings.at(settings.size() - 1)->append("0");
    settings.at(settings.size() - 1)->append("0");

    CBConfig->addItem(tr("Default"));
    CBConfig->setCurrentIndex(settings.size() - 1);
    sl_select_config(settings.size() - 1);
    sl_save();
}
void VideoPlayer::sl_del_config()
{
    int idx = CBConfig->currentIndex();
    CBConfig->setCurrentIndex(0);
    prev_idx = -1;

    delete settings.at(idx);
    settings.removeAt(idx);
    CBConfig->removeItem(idx);

    if (settings.size() == 0) {
        settings.append(new QList<QString>());
        settings.at(0)->append(tr("Default"));
        settings.at(0)->append("1024");
        settings.at(0)->append("768");
        settings.at(0)->append("100");
        settings.at(0)->append("100");
        settings.at(0)->append("false");
        settings.at(0)->append("false");
        settings.at(0)->append("0");
        settings.at(0)->append("0");
        settings.at(0)->append("0");
        settings.at(0)->append("0");

        CBConfig->addItem(tr("Default"));
    }
    CBConfig->setCurrentIndex(0);
    sl_select_config(0);
    sl_save();
}
void VideoPlayer::sl_select_config(int val)
{
    if (prev_idx != -1) {
        (*settings.at(prev_idx))[0] = ewname->text();
        (*settings.at(prev_idx))[1] = range_x->text();
        (*settings.at(prev_idx))[2] = range_y->text();
        (*settings.at(prev_idx))[3] = ewch->text();
        (*settings.at(prev_idx))[4] = ewcw->text();
    }

    if (val != -1) {
        prev_idx = val;
        QPlot->setTitle(tr("Configuration - ") + settings.at(CBConfig->currentIndex())->at(0));
        ewname->setText(settings.at(CBConfig->currentIndex())->at(0));
        range_x->setValue(settings.at(CBConfig->currentIndex())->at(1).toDouble());
        range_y->setValue(settings.at(CBConfig->currentIndex())->at(2).toDouble());
        ewch->setValue(settings.at(CBConfig->currentIndex())->at(3).toDouble());
        ewcw->setValue(settings.at(CBConfig->currentIndex())->at(4).toDouble());

        QRectF rect;
        rect = head_cant->geometry();
        rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                 settings.at(CBConfig->currentIndex())->at(7).toDouble()),
                                QPlot->transform(QwtPlot::yLeft,
                                                 settings.at(CBConfig->currentIndex())->at(8).toDouble())));
        head_cant->setGeometry(rect.toRect());

        rect = stage_cant->geometry();
        rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                 settings.at(CBConfig->currentIndex())->at(7).toDouble()),
                                QPlot->transform(QwtPlot::yLeft,
                                                 settings.at(CBConfig->currentIndex())->at(8).toDouble())));
        stage_cant->setGeometry(rect.toRect());

        rect = stage_laser->geometry();
        rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                 settings.at(CBConfig->currentIndex())->at(9).toDouble()),
                                QPlot->transform(QwtPlot::yLeft,
                                                 settings.at(CBConfig->currentIndex())->at(10).toDouble())));
        stage_laser->setGeometry(rect.toRect());
    }

    sl_range();
    on_cwh_ch();
    sl_save();
}
void VideoPlayer::sl_save()
{
    QFile data(configPath);
    data.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text);
    QTextStream out(&data);

    int idx = CBConfig->currentIndex();
    (*settings.at(idx))[0] = ewname->text();
    (*settings.at(idx))[1] = range_x->text();
    (*settings.at(idx))[2] = range_y->text();
    (*settings.at(idx))[3] = ewch->text();
    (*settings.at(idx))[4] = ewcw->text();
    (*settings.at(idx))[5] = "false";
    (*settings.at(idx))[6] = "false";
    QPlot->setTitle(tr("Configuration - ") + settings.at(CBConfig->currentIndex())->at(0));

    if ((*settings.at(idx))[0].isEmpty())
        (*settings.at(idx))[0] = "Empty";
    if ((*settings.at(idx))[1].isEmpty())
        (*settings.at(idx))[1] = "1024";
    if ((*settings.at(idx))[2].isEmpty())
        (*settings.at(idx))[2] = "768";
    if ((*settings.at(idx))[3].isEmpty())
        (*settings.at(idx))[3] = "100";
    if ((*settings.at(idx))[4].isEmpty())
        (*settings.at(idx))[4] = "100";
    if ((*settings.at(idx))[7].isEmpty())
        (*settings.at(idx))[7] = "0";
    if ((*settings.at(idx))[8].isEmpty())
        (*settings.at(idx))[8] = "0";
    if ((*settings.at(idx))[9].isEmpty())
        (*settings.at(idx))[9] = "0";
    if ((*settings.at(idx))[10].isEmpty())
        (*settings.at(idx))[10] = "0";

    out << CBConfig->currentIndex()
        << ' ' << (flipx->isChecked() ? 1 : 0)
        << ' ' << (flipy->isChecked() ? 1 : 0)
        << ' ' << (rotxy->isChecked() ? 1 : 0)
        << ' ' << xyMsel->currentIndex() << endl;
    CBConfig->clear();
    for (int i = 0; i < settings.size(); i++) {
        CBConfig->addItem(settings.at(i)->at(0));
        out << (*settings.at(i))[0].replace(" ", "_") << ' ' <<
               settings.at(i)->at(1) << ' ' <<
               settings.at(i)->at(2) << ' ' <<
               settings.at(i)->at(3) << ' ' <<
               settings.at(i)->at(4) << ' ' <<
               settings.at(i)->at(5) << ' ' <<
               settings.at(i)->at(6) << ' ' <<
               settings.at(i)->at(7) << ' ' <<
               settings.at(i)->at(8) << ' ' <<
               settings.at(i)->at(9) << ' ' <<
               settings.at(i)->at(10) << endl;
    }
    CBConfig->setCurrentIndex(idx);

    out.flush();
    data.close();
}

void VideoPlayer::sl_set_XYZ_range(double x, double y, double )
{
    Xmax = x;
    Ymax = y;

    sl_range();
}
void VideoPlayer::sl_set_XYZ_range2(double x, double y, double )
{
    Xmax2 = x;
    Ymax2 = y;

    sl_range();
}

void VideoPlayer::sl_range()
{
    mkm_px_x = (qreal)QPlot->canvas()->width() / range_x->value();
    mkm_px_y = (qreal)QPlot->canvas()->height() / range_y->value();

    QPlot->setAxisScale(QwtPlot::xBottom, 0, range_x->value());
    QPlot->setAxisScale(QwtPlot::yLeft, 0, range_y->value());

    QRectF rect; QPointF center;
    rect = head_cant->geometry(); center = rect.center();
    rect.setSize(QSizeF(mkm_px_x * Xmax, mkm_px_y * Ymax)); rect.moveCenter(center);
    head_cant->setGeometry(rect.toRect());
    rect = stage_cant->geometry(); center = rect.center();
    rect.setSize(QSizeF(mkm_px_x * Xmax2, mkm_px_y * Ymax2)); rect.moveCenter(center);
    stage_cant->setGeometry(rect.toRect());
    rect = stage_laser->geometry(); center = rect.center();
    rect.setSize(QSizeF(mkm_px_x * Xmax2, mkm_px_y * Ymax2)); rect.moveCenter(center);
    stage_laser->setGeometry(rect.toRect());
    QPlot->replot();

    static bool first = true;
    if (first) {
        first = false;

        rect = head_cant->geometry();
        rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                 settings.at(CBConfig->currentIndex())->at(7).toDouble()),
                                QPlot->transform(QwtPlot::yLeft,
                                                 settings.at(CBConfig->currentIndex())->at(8).toDouble())));
        head_cant->setGeometry(rect.toRect());

        rect = stage_cant->geometry();
        rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                 settings.at(CBConfig->currentIndex())->at(7).toDouble()),
                                QPlot->transform(QwtPlot::yLeft,
                                                 settings.at(CBConfig->currentIndex())->at(8).toDouble())));
        stage_cant->setGeometry(rect.toRect());

        rect = stage_laser->geometry();
        rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                                 settings.at(CBConfig->currentIndex())->at(9).toDouble()),
                                QPlot->transform(QwtPlot::yLeft,
                                                 settings.at(CBConfig->currentIndex())->at(10).toDouble())));
        stage_laser->setGeometry(rect.toRect());

        QPlot->replot();
    }

    (*settings.at(prev_idx))[7] = QVariant(
                QPlot->invTransform(QwtPlot::xBottom,
                                    head_cant->geometry().center().x())).toString();
    (*settings.at(prev_idx))[8] = QVariant(
                QPlot->invTransform(QwtPlot::yLeft,
                                    head_cant->geometry().center().y())).toString();
    (*settings.at(prev_idx))[9] = QVariant(
                QPlot->invTransform(QwtPlot::xBottom,
                                    stage_laser->geometry().center().x())).toString();
    (*settings.at(prev_idx))[10] = QVariant(
                QPlot->invTransform(QwtPlot::yLeft,
                                    stage_laser->geometry().center().y())).toString();
    sl_save();
}

int VideoPlayer::times_constructed = 0;
VideoPlayer::VideoPlayer(QWidget *parent)
    : QWidget(parent)
{
    inst_idx = times_constructed++;
    QString suffix = inst_idx == 0 ? QString() : QString::number(inst_idx+1);

    active = false;
    refresh_limit_timer.invalidate();
    refresh_limit_last_elapsed = 0;

    configPath = QString("Param/video%1.conf").arg(suffix);
    isInterrupted = false;

    plot_mode = 0;
    mkm_px_x = 1;
    mkm_px_y = 1;
    prev_idx = 0;
    cw = 1;
    ch = 1;


    QPlot = new QwtPlot(this);
    QPlot->canvas()->setCursor(Qt::OpenHandCursor);
    QPlot->enableAxis(QwtPlot::yLeft, false);
    QPlot->enableAxis(QwtPlot::xBottom, false);
    QPlot->setAxisScale(QwtPlot::xBottom, 0, 1024);
    QPlot->setAxisScale(QwtPlot::yLeft, 0, 768);
    QFont font = qApp->font(); font.setPointSize(font.pointSize() + 2);
    QPlot->titleLabel()->setFont(font);
    QwtText tx(tr("micro")+tr("m")); tx.setFont(font);
    QPlot->setAxisTitle(QwtPlot::xBottom, tx);
    QPlot->setAxisTitle(QwtPlot::yLeft, tx);
    QPlot->setAxisFont(QwtPlot::xBottom, font);
    QPlot->setAxisFont(QwtPlot::yLeft, font);

    QGrid = new QwtPlotGrid();
    QGrid->setMajPen(QPen(Qt::black));
    QGrid->attach(QPlot);
    QGrid->hide();

    markerA = new QwtPlotMarker();
    markerB = new QwtPlotMarker();
    markerC = new QwtPlotMarker();
    markerA->attach(QPlot); markerA->hide();
    markerB->attach(QPlot); markerB->hide();
    markerC->attach(QPlot); markerC->hide();
    markerA->setXValue(10); markerA->setYValue(10);
    markerB->setXValue(20); markerB->setYValue(20);
    markerB->setXValue(15); markerB->setYValue(15);
    markerA->setLineStyle(QwtPlotMarker::Cross); markerA->setLinePen(QPen(Qt::white, 1, Qt::DashLine));
    markerA->setSymbol(new QwtSymbol(QwtSymbol::Diamond, QColor(255,255,255), QColor(255,255,255), QSize(9, 9)));
    markerB->setLineStyle(QwtPlotMarker::Cross); markerB->setLinePen(QPen(Qt::white, 1, Qt::DashLine));
    markerB->setSymbol(new QwtSymbol(QwtSymbol::Diamond, QColor(255,255,255), QColor(255,255,255), QSize(9, 9)));

    lineAB = new plot::TLineSegment;
    lineAB->attach(QPlot);
    lineAB->hide();
    lineAB->setPen(QPen(QColor(255, 33, 33),   1, Qt::DashLine));

    head_cant = new QRubberBand(QRubberBand::Rectangle, QPlot->canvas());
    stage_cant = new QRubberBand(QRubberBand::Rectangle, QPlot->canvas());
    stage_laser = new QRubberBand(QRubberBand::Rectangle, QPlot->canvas());
    head_cant->hide();
    stage_cant->hide();
    stage_laser->hide();

    QPalette palette;
    palette.setBrush( QPalette::Base, QBrush( Qt::cyan ) );
    head_cant->setPalette(palette);
    palette.setBrush( QPalette::Base, QBrush( Qt::yellow ) );
    stage_cant->setPalette(palette);
    palette.setBrush( QPalette::Base, QBrush( Qt::green ) );
    stage_laser->setPalette(palette);

    head_cant->setStyle(QStyleFactory::create("Motif"));
    stage_cant->setStyle(QStyleFactory::create("Motif"));
    stage_laser->setStyle(QStyleFactory::create("Motif"));

    Xmax = Ymax = Xmax2 = Ymax2 = 100;

    QRectF rect;
    rect = head_cant->geometry();
    rect.setSize(QSizeF(mkm_px_x*50, mkm_px_y*50));
    head_cant->setGeometry(rect.toRect());
    rect = stage_cant->geometry();
    rect.setSize(QSizeF(mkm_px_x*100, mkm_px_y*100));
    stage_cant->setGeometry(rect.toRect());
    rect = stage_laser->geometry();
    rect.setSize(QSizeF(mkm_px_x*100, mkm_px_y*100));
    stage_laser->setGeometry(rect.toRect());



    QGroupBox *GBsettings = new QGroupBox(tr("Settings"), this);
    GBsettings->setVisible(false);

    QGroupBox *GrBControl = new QGroupBox(this);

    RatioLayout *lusbcamlay = new RatioLayout;
    lusbcamlay->setAlignment(Qt::AlignRight);
    lusbcamlay->addWidget(QPlot);

    QBoxLayout *layout = new QHBoxLayout(this);
    layout->addWidget(GrBControl);
    layout->addWidget(GBsettings);
    layout->addLayout(lusbcamlay, 1);


    activeCamera = 0;
#ifdef USE_IVIDEO_MOTIC
    CameraList.append(new Motic());
#endif
#ifdef USE_IVIDEO_MOTIC_X
    CameraList.append(new MoticX());
#endif

    foreach (IVideo *v, CameraList)
        connect(v->getObject(), SIGNAL(ImgReceived(QImage)), this, SLOT(on_Camera_ImgReceived(QImage)), Qt::QueuedConnection);

    deviceStatus = new StatusWidget1(tr("Video") + (inst_idx == 0 ? QString() : (" " + suffix)));
    setDeviceStatus(EDno);
    connect(&resetWatch, SIGNAL(finished()), this, SLOT(resetCamerasCommit()));


    PBToNewWindow = new TwoStateButton(":/Detach.png",":/Attach.png");
    PBToNewWindow->setToolTip(QObject::tr("Individual window on/off"));

    m_BStart = new TwoStateButton(":start.png",":stop.png");
    m_BStart->setProperty("id", DataID::VideoOn);
    connect(m_BStart, SIGNAL(state_changed(bool)), this, SLOT(StartStop(bool)));

    BReset = new QPushButton(makeHugeIcon(":/SPMTuning_window/Video.png"), QString());
    BReset->setToolTip(tr("Reinitialize cameras"));
    connect(BReset, SIGNAL(clicked()), SLOT(resetCameras()));

    BAutoFocusOsc = new QPushButton(makeHugeIcon(":/SPMTuning_window/AutoFocusPMT.png"), QString());
    BAutoFocusOsc->setToolTip(tr("Laser Auto Focus"));
    connect(BAutoFocusOsc, SIGNAL(clicked()), SLOT(on_BAutoFocusOsc_clicked()));

    BAutoFocus = new QPushButton(makeHugeIcon(":/SPMTuning_window/AutoFocus.png"), QString());
    BAutoFocus->setToolTip(tr("Auto Focus"));
    connect(BAutoFocus, SIGNAL(clicked()), SLOT(on_BAutoFocus_clicked()));

    BToData = new QPushButton(makeHugeIcon(":/plot/Attach.png"), QString());
    BToData->setToolTip(tr("Convert to grayscale & Send to Data as 3d") + " (Ctrl+S)");
    BToData->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_S));
    connect(BToData, SIGNAL(clicked()), SLOT(on_BToData_clicked()));

    saveB = new QPushButton(makeHugeIcon(":/snap_shot.png"), QString());
    saveB->setToolTip(tr("Save image"));
    saveB->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_P));
    connect(saveB, SIGNAL(clicked()), this, SLOT(sl_saveToFile()));

    BCenterAndSet = new TwoStateButton(":/USBVideo/TipAlignCenterOff.png",":/USBVideo/TipAlignCenterOn.png");
    BCenterAndSet->setToolTip(tr("Tip scan area center alignment"));
    connect(BCenterAndSet, SIGNAL(state_changed(bool)), this, SLOT(sl_center_set(bool)));

    BCenterAndSet2 = new TwoStateButton(":/USBVideo/LaserSpotAlignCenterOff.png",":/USBVideo/LaserSpotAlignCenterOn.png");
    BCenterAndSet2->setToolTip(tr("Laser scan area center alignment"));
    connect(BCenterAndSet2, SIGNAL(state_changed(bool)), this, SLOT(sl_center_set2(bool)));

    BLine = new TwoStateButton(":/USBVideo/sizeByLineOff.png",":/USBVideo/sizeByLineOn.png");
    BLine->setToolTip(tr("Set video size"));
    connect(BLine, SIGNAL(state_changed(bool)), this, SLOT(sl_line(bool)));

    BLineMeasure = new TwoStateButton(":/USBVideo/sizeByLineOff.png",":/USBVideo/sizeByLineOn.png");
    BLineMeasure->setToolTip(tr("Measure distance"));
    connect(BLineMeasure, SIGNAL(state_changed(bool)), this, SLOT(sl_line_measure(bool)));

    BCantileverRange = new TwoStateButton(":/USBVideo/TipAreaShowOff.png",":/USBVideo/TipAreaShowOn.png");
    BCantileverRange->setToolTip(tr("Show tip scan area"));
    connect(BCantileverRange, SIGNAL(state_changed(bool)),head_cant, SLOT(setVisible(bool)));
    connect(BCantileverRange, SIGNAL(state_changed(bool)),stage_cant, SLOT(setVisible(bool)));

    BLaserRange = new TwoStateButton(":/USBVideo/LaserAreaShowOff.png",":/USBVideo/LaserAreaShowOn.png");
    BLaserRange->setToolTip(tr("Show laser spot scan area"));
    connect(BLaserRange, SIGNAL(state_changed(bool)),stage_laser, SLOT(setVisible(bool)));

    BGrid = new TwoStateButton(":/USBVideo/GridOff.png",":/USBVideo/GridOn.png");
    BGrid->setToolTip(tr("Show grid"));
    connect(BGrid, SIGNAL(state_changed(bool)), this, SLOT(sl_grid(bool)));

    BAxes = new TwoStateButton(":/USBVideo/ScaleOff.png",":/USBVideo/ScaleOn.png");
    BAxes->setToolTip(tr("Show Axes"));
    connect(BAxes, SIGNAL(state_changed(bool)), this, SLOT(sl_axes(bool)));

    AviOptions = new QPushButton;
    AviOptions->setIcon(makeHugeIcon(":/SPMTuning_window/tools.png"));
    AviOptions->setToolTip(tr("Show Settings"));
    AviOptions->setCheckable(true);
    connect(AviOptions, SIGNAL(clicked(bool)), GBsettings, SLOT(setVisible(bool)));

    TRectWidget *clrHead = new TRectWidget(Qt::cyan);
    TRectWidget *clrStage = new TRectWidget(Qt::yellow);
    TRectWidget *clrLaser = new TRectWidget(Qt::green);
    clrHead->setFixedSize(m_BStart->sizeHint().width(), 7);
    clrStage->setFixedSize(m_BStart->sizeHint().width(), 7);
    clrLaser->setFixedSize(m_BStart->sizeHint().width(), 7);

    QPushButton *icHead = new QPushButton(makeHugeIcon(":Head_page.png"), QString());
    QPushButton *icStage = new QPushButton(makeHugeIcon(":stage_page.png"), QString());
    QPushButton *icLaser = new QPushButton(makeHugeIcon(":/SPMTuning_window/laser_on.png"), QString());
    icHead->setToolTip(tr("Head"));
    icStage->setToolTip(tr("Stage"));
    icLaser->setToolTip(tr("Laser"));
    icHead->setFlat(true);
    icStage->setFlat(true);
    icLaser->setFlat(true);
    icHead->setEnabled(false);
    icStage->setEnabled(false);
    icLaser->setEnabled(false);

    HFlowLayout *LControls = new HFlowLayout(GrBControl);
    LControls->addWidget(PBToNewWindow);
    LControls->addWidget(m_BStart);
    LControls->addWidget(saveB);
    LControls->addWidget(BToData);
    addDelimeterH(LControls, m_BStart->sizeHint().width());
    LControls->addWidget(BAutoFocus);
    LControls->addWidget(BAutoFocusOsc);
    addDelimeterH(LControls, m_BStart->sizeHint().width());
    LControls->addWidget(BCantileverRange);
    LControls->addWidget(BLaserRange);
    LControls->addWidget(AviOptions);
    LControls->addWidget(BReset);
    addDelimeterH(LControls, m_BStart->sizeHint().width());
    LControls->addWidget(BLineMeasure);
    LControls->addWidget(BAxes);
    LControls->addWidget(BGrid);
    addDelimeterH(LControls, m_BStart->sizeHint().width());
    LControls->addWidget(icHead);
    LControls->addWidget(clrHead);
    LControls->addWidget(icStage);
    LControls->addWidget(clrStage);
    LControls->addWidget(icLaser);
    LControls->addWidget(clrLaser);
    LControls->spaceX = LControls->spaceY = 5;
    LControls->setContentsMargins(3, 3, 3, 3);


    BAddConfig = new QPushButton(makeHugeIcon(":/USBVideo/ConfPlus.png"), QString());
    BAddConfig->setToolTip(tr("Add new configuration"));
    connect(BAddConfig, SIGNAL(clicked(bool)), this, SLOT(sl_add_config()));

    BDelConfig = new QPushButton(makeHugeIcon(":/USBVideo/ConfMinus.png"), QString());
    BDelConfig->setToolTip(tr("Remove current configuration"));
    connect(BDelConfig, SIGNAL(clicked(bool)), this, SLOT(sl_del_config()));


    zPos = new DoubleChanger(0, 10, 4, true);
    zPos->edit->hide();

    QGroupBox *box_Zpos = new QGroupBox(tr("Obj.Z, ")+tr("micro")+tr("m"));
    QLayout *layout_Zpos = new QVBoxLayout(box_Zpos);
    layout_Zpos->addWidget(zPos);

    MotorSlider *mzslider = new MotorSlider();
    connect(mzslider, SIGNAL(newPos(int)), this, SLOT(goZjoy(int)));

    QGroupBox *box_mZpos = new QGroupBox(tr("Z motor"));
    QLayout *layout_mZpos = new QVBoxLayout(box_mZpos);
    layout_mZpos->addWidget(mzslider);

    layout->addWidget(box_mZpos);
    layout->addWidget(box_Zpos);


    CBConfig = new QComboBox;

    int idx = 0, _flipx = 0, _flipy = 0, _rotxy = 0, xymidx = 2;
    { // read video config
        QFile data(configPath);
        if (!data.open(QIODevice::ReadOnly | QIODevice::Text))
            goto nodata;
        QTextStream in(&data);

        int i = 0;
        in >> idx >> _flipx >> _flipy >> _rotxy >> xymidx;
        while (!in.atEnd()) {
            settings.append(new QList<QString>());
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");
            settings.at(i)->append("");

            in >> (*settings.at(i))[0];
            if (in.atEnd()) {
                delete settings.at(i);
                settings.removeAt(i);

                break;
            }
            in >> (*settings.at(i))[1] >>
                  (*settings.at(i))[2] >>
                  (*settings.at(i))[3] >>
                  (*settings.at(i))[4] >>
                  (*settings.at(i))[5] >>
                  (*settings.at(i))[6] >>
                  (*settings.at(i))[7] >>
                  (*settings.at(i))[8] >>
                  (*settings.at(i))[9] >>
                  (*settings.at(i))[10] ;
            (*settings.at(i))[0].replace("_", " ");
            if (QVariant(settings.at(i)->at(5)).toBool()) _flipx = true;
            if (QVariant(settings.at(i)->at(6)).toBool()) _flipy = true;
            CBConfig->addItem(settings.at(i)->at(0));
            i++;
        }

        data.close();
    }
    nodata:

    if (settings.size() == 0) {
        idx = 0;
        xymidx = 2;
        settings.append(new QList<QString>());
        settings.at(0)->append(tr("Default"));
        settings.at(0)->append("1024");
        settings.at(0)->append("768");
        settings.at(0)->append("100");
        settings.at(0)->append("100");
        settings.at(0)->append("false");
        settings.at(0)->append("false");
        settings.at(0)->append("0");
        settings.at(0)->append("0");
        settings.at(0)->append("0");
        settings.at(0)->append("0");

        CBConfig->addItem(tr("Default")) ;
    }
    idx = qBound(0, idx, settings.size()-1);

    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        BAxes->change_state_man_with_sig(settings.value("video/axes", true).toBool());
        BGrid->change_state_man_with_sig(settings.value("video/grid", true).toBool());

        if (!settings.value("display/spectroscopy", false).toBool()) {
            BLaserRange->hide();
            BCenterAndSet2->hide();
            icLaser->hide();
            clrLaser->hide();
        }
        bool head_on = settings.value("display/oscilloscope/head", true).toBool();
        bool stage_on = settings.value("display/oscilloscope/stage", true).toBool();
        if (!head_on) {
            icHead->hide();
            clrHead->hide();
        }
        if (!stage_on) {
            icStage->hide();
            clrStage->hide();
        }

        if (!show_advanced_gui()) {
            foreach (QWidget *wid,
                     QWidgetList() << BLaserRange << BCantileverRange
                     << icHead << clrHead << icStage << clrStage << icLaser << clrLaser
                     << BCenterAndSet << BCenterAndSet2
                     << BAutoFocus << BAutoFocusOsc)
                wid->hide();
        }
    }

    range_x = new Edit(1, 5000, 1);
    range_y = new Edit(1, 5000, 1);
    ewcw = new Edit(10, 100);
    ewch = new Edit(10, 100);
    ewsz = new Edit(1, 5000, 1);
    ewname = new QLineEdit(tr("Default"));
    flipx = new QCheckBox(tr("Flip x"));
    flipy = new QCheckBox(tr("Flip y"));
    rotxy = new QCheckBox(tr("Swap X/Y"));
    xyMsel = new QComboBox;
    {
        xyMsel->addItem(tr("Off"));
        int idxx, idxy;
        getXYMidx(0, &idxx, &idxy);
        xyMsel->addItem(tr("Head: ") + QString::number(idxx) + ", " + QString::number(idxy));
        getXYMidx(1, &idxx, &idxy);
        xyMsel->addItem(tr("Stage: ") + QString::number(idxx) + ", " + QString::number(idxy));
    }
    xymidx = qBound(0, xymidx, xyMsel->count()-1);

    ewsz->setValue(35);
    CBConfig->setCurrentIndex(idx);
    prev_idx = idx;
    QPlot->setTitle(tr("Configuration - ") + settings.at(idx)->at(0));
    ewname->setText(settings.at(idx)->at(0));
    range_x->setValue(settings.at(idx)->at(1).toDouble());
    range_y->setValue(settings.at(idx)->at(2).toDouble());
    ewch->setValue(settings.at(idx)->at(3).toDouble());
    ewcw->setValue(settings.at(idx)->at(4).toDouble());
    cw = ewcw->value() / 100;
    ch = ewch->value() / 100;
    mkm_px_x = (qreal)QPlot->canvas()->width() / range_x->value();
    mkm_px_y = (qreal)QPlot->canvas()->height() / range_y->value();
    xyMsel->setCurrentIndex(xymidx);

    QPlot->setAxisScale(QwtPlot::xBottom, 0, range_x->value());
    QPlot->setAxisScale(QwtPlot::yLeft, 0, range_y->value());

    flipx->setChecked(_flipx);
    flipy->setChecked(_flipy);
    rotxy->setChecked(_rotxy);

    rect = head_cant->geometry();
    rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                             settings.at(idx)->at(7).toDouble()),
                            QPlot->transform(QwtPlot::yLeft,
                                             settings.at(idx)->at(8).toDouble())));
    head_cant->setGeometry(rect.toRect());

    rect = stage_cant->geometry();
    rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                             settings.at(idx)->at(7).toDouble()),
                            QPlot->transform(QwtPlot::yLeft,
                                             settings.at(idx)->at(8).toDouble())));
    stage_cant->setGeometry(rect.toRect());

    rect = stage_laser->geometry();
    rect.moveCenter(QPointF(QPlot->transform(QwtPlot::xBottom,
                                             settings.at(idx)->at(9).toDouble()),
                            QPlot->transform(QwtPlot::yLeft,
                                             settings.at(idx)->at(10).toDouble())));
    stage_laser->setGeometry(rect.toRect());

    connect(CBConfig, SIGNAL(activated(int)), this, SLOT(sl_select_config(int)));
    connect(range_x, SIGNAL(newValue()), this, SLOT(sl_range()));
    connect(range_y, SIGNAL(newValue()), this, SLOT(sl_range()));
    connect(ewch, SIGNAL(newValue()), this, SLOT(on_cwh_ch()));
    connect(ewcw, SIGNAL(newValue()), this, SLOT(on_cwh_ch()));
    connect(ewname, SIGNAL(textEdited(QString)), this, SLOT(sl_save()));
    connect(flipx, SIGNAL(clicked()), this, SLOT(sl_save()));
    connect(flipy, SIGNAL(clicked()), this, SLOT(sl_save()));
    connect(rotxy, SIGNAL(clicked()), this, SLOT(sl_save()));
    connect(xyMsel, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_save()));

    QGridLayout *BGslay = new QGridLayout(GBsettings);
    BGslay->addWidget(BAddConfig, 0, 0);
    BGslay->addWidget(BDelConfig, 0, 1);
    BGslay->addWidget(BLine, 0, 2);
    BGslay->addWidget(BCenterAndSet, 0, 3);
    BGslay->addWidget(BCenterAndSet2, 0, 4);

    BGslay->addWidget(CBConfig, 1, 0, 1, -1);

    BGslay->addWidget(new QLabel(tr("Name: ")), 2, 0, 1, 2);
    BGslay->addWidget(ewname, 2, 2, 1, 2);

    BGslay->addWidget(new QLabel(tr("Range X: ")), 3, 0, 1, 2);
    BGslay->addWidget(range_x, 3, 2, 1, 2);
    BGslay->addWidget(new QLabel(tr("micro")+tr("m")), 3, 4);

    BGslay->addWidget(new QLabel(tr("Range Y: ")), 4, 0, 1, 2);
    BGslay->addWidget(range_y, 4, 2, 1, 2);
    BGslay->addWidget(new QLabel(tr("micro")+tr("m")), 4, 4);

    BGslay->addWidget(new QLabel(tr("Width: ")), 5, 0, 1, 2);
    BGslay->addWidget(ewcw, 5, 2, 1, 2);
    BGslay->addWidget(new QLabel(tr("% max")), 5, 4, 1, 2);

    BGslay->addWidget(new QLabel(tr("Height: ")), 6, 0, 1, 2);
    BGslay->addWidget(ewch, 6, 2, 1, 2);
    BGslay->addWidget(new QLabel(tr("% max")), 6, 4, 1, 2);

    BGslay->addWidget(new QLabel(tr("Line length: ")), 7, 0, 1, 2);
    BGslay->addWidget(ewsz, 7, 2, 1, 2);
    BGslay->addWidget(new QLabel(tr("micro")+tr("m")), 7, 4);

    BGslay->addWidget(flipx, 8, 0, 1, 2);
    BGslay->addWidget(flipy, 8, 2, 1, 2);
    BGslay->addWidget(rotxy, 9, 0, 1, 4);

    BGslay->setAlignment(Qt::AlignTop);

    BGslay->addWidget(new QLabel(tr("XY motors: ")), 11, 0, 1, 2);
    BGslay->addWidget(xyMsel, 11, 2, 1, 3);

    ritem = new plot::ImageItem;
    ritem->image = QImage(":/USBVideo/USB_cam_win.png");
    ritem->attach(QPlot);
    QPlot->setMinimumSize(10, 8);
    QPlot->canvas()->installEventFilter(this);

    setWindowTitle(tr("Video - NSpec"));
}

VideoPlayer::~VideoPlayer()
{
    if (activeCamera)
        activeCamera->Stop();
    activeCamera = 0;

    if (inst_idx == 0) {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        settings.setValue("video/grid", BGrid->on_off);
        settings.setValue("video/axes", BAxes->on_off);
    }

    foreach (IVideo *v, CameraList) {
        while (qobject_cast<const QThread *>(v->getObject()) && qobject_cast<const QThread *>(v->getObject())->isRunning()) {
            qApp->processEvents();
            SleepExport::msleep2(3);
        }
        delete v;
    }
}

void VideoPlayer::on_cwh_ch()
{
    cw = ewcw->value() / 100;
    ch = ewch->value() / 100;

    sl_save();
}


void VideoPlayer::getXYMidx(int set, int *x, int *y)
{
    if (set == 0) {
        *x = 1; //legacy compat
        *y = 2; //legacy compat
        QList<int> l1 = Motors::getRoleMotors(Motors::RoleX1);
        QList<int> l2 = Motors::getRoleMotors(Motors::RoleY1);
        if (l1.size())
            *x = l1.first();
        if (l2.size())
            *y = l2.first();
    } else {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        *x = settings_double(settings, "x_motor", 7); //legacy compat
        *y = settings_double(settings, "y_motor", 8); //legacy compat
        QList<int> l1 = Motors::getRoleMotors(Motors::RoleX2);
        QList<int> l2 = Motors::getRoleMotors(Motors::RoleY2);
        if (l1.size())
            *x = l1.first();
        if (l2.size())
            *y = l2.first();
    }
}

void VideoPlayer::goXYjoy(QPointF speedxy)
{
    if (xyMsel->currentIndex() <= 0)
        return;

    I_MODULE *motorsm = ModuleManager::instance->find1("motors");
    if (!motorsm)
        return;

    Motors *motors = (Motors *)motorsm->basecast();

    QLineF line(0, 0, speedxy.x(), speedxy.y());
    double len = line.length();
    double angle = line.angle();
    double da = angle - prev_xy_joy_output.y();
    while (da >= 180)
        da -= 180;
    while (da <= -180)
        da += 180;

    if (speedxy.isNull() || fabs(da) >= 10 ||
            fabs(len - prev_xy_joy_output.x()) >= 0.2) {
        prev_xy_joy_output.rx() = qRound(len * 5) / 5.;
        prev_xy_joy_output.ry() = qRound(angle / 10.) * 10;
        int idxx, idxy;
        getXYMidx(xyMsel->currentIndex() - 1, &idxx, &idxy);
        double speedx = len > 0.1 ? qFabs(speedxy.x()) / len * motors->getShuttleSpeed(len) : 0;
        double speedy = len > 0.1 ? qFabs(speedxy.y()) / len * motors->getShuttleSpeed(len) : 0;
        motors->go(idxx, speedxy.x() == 0 ? 0 : (speedxy.x() > 0 ? 1 : -1), speedx, 0);
        motors->go(idxy, speedxy.y() == 0 ? 0 : (speedxy.y() > 0 ? 1 : -1), speedy, 0);
    }
    QPlot->canvas()->setCursor(QCursor(!speedxy.isNull() ? Qt::SizeAllCursor : Qt::OpenHandCursor));
}
void VideoPlayer::goZjoy(int speed)
{
    I_MODULE *motorsm = ModuleManager::instance->find1("motors");
    if (!motorsm)
        return;

    Motors *motors = (Motors *)motorsm->basecast();

    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    int idx = settings_double(settings, "z_motor", 4); //legacy compat

    QList<int> l = Motors::getRoleMotors(Motors::RoleZ2Focus);
    if (l.size())
        idx = l.first();

    motors->goShuttle(speed, idx);
}


void VideoPlayer::setDeviceStatus(int idx)
{
    switch (idx) {
    case EDno:
        deviceStatus->status->set(StatusWidget::Cross, Qt::red);
        deviceStatus->status->setToolTip(tr("No device"));
        break;
    case EDconn:
        deviceStatus->status->set(StatusWidget::Circle, QColor(255, 128, 0));
        deviceStatus->status->setToolTip(tr("Connecting"));
        break;
    case EDok:
        deviceStatus->status->set(StatusWidget::Circle, Qt::green);
        deviceStatus->status->setToolTip(tr("Connected"));
        break;
    case EDplay:
        deviceStatus->status->set(StatusWidget::Play, Qt::green);
        deviceStatus->status->setToolTip(tr("Updating"));
        break;
    }
}

void VideoPlayer::resetCameras()
{
    setDeviceStatus(EDconn);

    if (activeCamera)
        activeCamera->closeCamera();
    activeCamera = 0;

    m_BStart->setEnabled(false);
    BAutoFocus->setEnabled(false);
    BAutoFocusOsc->setEnabled(false);

    QFuture<void> f = QtConcurrent::run(this, &VideoPlayer::resetCamerasImpl);
    ManagedThreadedCleaner::registerManagedFuture(f);
    resetWatch.setFuture(f);
}
static QMutex videoreset_threadlock;
void VideoPlayer::resetCamerasImpl()
{
    QMutexLocker lck(&videoreset_threadlock); //don't try simultaneous video driver init

    foreach (IVideo *v, CameraList)
        if (v->openCamera(1)) {
            activeCamera = v;
            break;
        }
}
void VideoPlayer::resetCamerasCommit()
{
    ManagedThreadedCleaner::unregisterManagedFuture(resetWatch.future());

    bool opened = activeCamera != 0;
    if (opened) {
        deviceStatus->label->setToolTip(activeCamera->getName());

        m_BStart->setEnabled(true);
        BAutoFocus->setEnabled(true);
        BAutoFocusOsc->setEnabled(true);
    }

    setDeviceStatus(opened ? EDok : EDno);
    refresh_limit_timer.invalidate();
}

void VideoPlayer::StartStop(bool value)
{
    if (!activeCamera)
        return;

    BReset->setEnabled(!value);
    if (value) {
        activeCamera->Start();

        frames = 0;
    } else {
        activeCamera->Stop();
    }
    setDeviceStatus(value ? EDplay : EDok);
}

void VideoPlayer::on_BToData_clicked()
{
    I_MODULE *datam = ModuleManager::instance->find1("data");
    if (!datam)
        return;

    DataPlot *data = (DataPlot *)datam->basecast();

    plot::TMetaStorage attributes;

    attributes.insert("Name", "Video");
    attributes.insert("Source", "Video");

    attributes.insert("XMin", 0);
    attributes.insert("XMax", 1e-6*range_x->value());
    attributes.insert("YMin", 0);
    attributes.insert("YMax", 1e-6*range_y->value());

    attributes.insert("XYLabel", QString());
    attributes.insert("XYUnit", tr("m"));

    data->Container->AddDataToContainer(ritem->image, attributes);
}
void VideoPlayer::sl_saveToFile()
{
    QString SelectedFilter;
    QString FileName = QFileDialog::getSaveFileName(this, tr("Save to file"), QString(), "*.png;;*.jpg;;*.tiff;;*.bmp", &SelectedFilter, QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty()) return;
    if (!FileName.endsWith(SelectedFilter.remove(0, 1))) FileName += SelectedFilter;

    QPixmap::grabWidget(QPlot).save(FileName, SelectedFilter.remove(0, 1).toAscii().data());
}
