#ifndef VIDEOPLAYER_H
#define VIDEOPLAYER_H

#include <QDialog>
#include <QWidget>
#include <QElapsedTimer>
#include <QLabel>
#include <QGridLayout>
#include <QtEvents>
#include <QFileDialog>
#include <QFutureWatcher>
class QCheckBox;
class QComboBox;
class QRubberBand;
#include "controls.h"
#include "edit.hpp"
#include "qwt_plot.h"
#include <qwt_plot_grid.h>

#include "i_video.hpp"

class StatusWidget1;
namespace plot {
    class ImageItem;
    class TLineSegment;
}
class QwtPlotMarker;
class VideoPlayer : public QWidget
{
    Q_OBJECT
public:
    VideoPlayer(QWidget *parent = 0);
    ~VideoPlayer();
    static int times_constructed;
    int inst_idx;

    TwoStateButton *PBToNewWindow;
    StatusWidget1 *deviceStatus;
    TwoStateButton *m_BStart;

    QPointF posx;
    bool active;
    QwtPlotMarker *marker;

    DoubleChanger *zPos;
    bool isInterrupted;

    QString configPath;

    QFutureWatcher<void> resetWatch;
    void resetCamerasImpl(); //private

public slots:
    void resetCameras();
    void resetCamerasCommit(); //private

    void cancelOperation();
    void on_Camera_ImgReceived(QImage Img);
    void calcOscContrast(QByteArray data);
    void on_BAutoFocus_clicked();
    void on_BAutoFocusOsc_clicked();
    void on_BToData_clicked();
    void StartStop(bool value);
    void sl_saveToFile();
    void on_cwh_ch();
    void sl_center_set(bool value);
    void sl_center_set2(bool value);
    void sl_axes(bool value);
    void sl_grid(bool value);
    void sl_line(bool value);
    void sl_add_config();
    void sl_del_config();
    void sl_select_config(int val);
    void sl_save();
    void sl_line_measure(bool value);

    void sl_set_XYZ_range(double x, double y, double z);
    void sl_set_XYZ_range2(double x, double y, double z);
    void sl_range();

    void goZjoy(int speed);
    void goXYjoy(QPointF speedxy);
    void getXYMidx(int set, int *x, int *y);

private:
    void setDeviceStatus(int);
    bool eventFilter(QObject *, QEvent *);
    void closeEvent(QCloseEvent *);

    int plot_mode;
    QPointF prev_xy_joy_output;
    QRubberBand *head_cant;
    QRubberBand *stage_cant;
    QRubberBand *stage_laser;
    QwtPlotMarker *markerA;
    QwtPlotMarker *markerB;
    QwtPlotMarker *markerC;
    plot::TLineSegment *lineAB;

    QList<QList<QString> *> settings;
    int prev_idx;

    double Xmax, Ymax;
    double Xmax2, Ymax2;
    qreal mkm_px_x;
    qreal mkm_px_y;
    Edit *range_x;
    Edit *range_y;
    Edit *ewcw;
    Edit *ewch;
    Edit *ewsz;
    QLineEdit *ewname;
    TwoStateButton *BCenterAndSet;
    TwoStateButton *BCenterAndSet2;
    TwoStateButton *BLine;
    TwoStateButton *BLineMeasure;
    QComboBox *CBConfig;
    QPushButton *BAddConfig;
    QPushButton *BDelConfig;
    QCheckBox *flipx;
    QCheckBox *flipy;
    QCheckBox *rotxy;
    QComboBox *xyMsel;

    double cw;
    double ch;

    QVector<IVideo *> CameraList;
    IVideo *activeCamera;

    int frames;
    double osc_contrast;
    bool new_osc_contrast;

    QPushButton *AviOptions;
    QPushButton *saveB;
    QPushButton *BAutoFocus;
    QPushButton *BAutoFocusOsc;
    QPushButton *BToData;
    QPushButton *BReset;
    TwoStateButton *BCantileverRange;
    TwoStateButton *BLaserRange;
    TwoStateButton *BGrid;
    TwoStateButton *BAxes;

    QwtPlot *QPlot;
    QwtPlotGrid *QGrid;
    plot::ImageItem *ritem;
    QElapsedTimer refresh_limit_timer;
    qint64 refresh_limit_last_elapsed;
};

#endif
