#pragma once

#include "ccd.h"
#include "controls.h"
#include "edit.hpp"
#include <QElapsedTimer>
#include <QPointer>

class IExternalScan;
class CCDPlot;
IExternalScan *getNewCCDIExtScan(CCDPlot *ccd);

class QwtThermo;
class CoboltGui;
class StatusWidget1;
class CCDMatrix;
class StatusBar;
class QScrollArea;
class QComboBox;
class QCheckBox;
class QGroupBox;
class QLabel;
class QTabWidget;
class QTextEdit;
class QRadioButton;
namespace plot { class TPlot; }
class CCDPlot : public QWidget
{
    Q_OBJECT

private slots:
    void onPanoram(bool);
    void onPanoramMin(double);
    void onPanoramMax(double);
    void PMTScan();

public slots:
    void setDeviceStatus();
    void setDeviceStatus_handler(int);
    void setDeviceStatus2(int);
    void updateSpecBar();

    void monoch_laser_wl();
    void monoch_slit_in();
    void monoch_slit_out();
    void monoch_pre_wl();
    void monoch_wl();
    void monoch_mirror();
    void monoch_grating();
    void monoch_status_update(bool);
    void nd_filter(double);

    void sl_set_X_scale(int);
    void sl_calib_change(bool);
    void s_apply_calib_Settings();
    void load_data();
    void save_data();

    void s_triggerMode(int mode);
    void slot_updateTemperature();
    void enable_options_rm(int mode);
    void enable_options_acm(int mode);
    void sl_show_Monochromator();

    void startAcquisition(bool);
    void sl_aq_but_off();
    void sl_Plot_Refresh();
    void new2dData(QVector<double> ydata, QVector<double> xdata);

    void sl_set_current_acc_num(int);
    void sl_set_current_kin_num(int);

    void UpdateNxNy();
    void sl_set_new_3D_Limits(const Range &XLimits, const Range &YLimits);
    void SetManualBaseLine();

    void intensity_calib_enabled();
    void intensity_calib_open_wl(QString name = QString());
    void intensity_calib_open_px(QString name = QString());

    void initOptics();
    void opticsReset();
    void cancelOperation();

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slotPMT(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);

signals:
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

public:
    QList<QWidget *> getDeviceStatusList();

    QElapsedTimer refresh_limit_timer;

    QVector<QWidget *> ccdControlWidgets; //disabled on acq

    bool isInterrupted;
    QPointer<StatusBar> spec_bar;

    QTimer *acqUpd;
    QTimer *temperatureUpd;
    QTimer *devStatusSync;

    int syncDevStatus, prevDevStatus;

    QPointer<StatusWidget1> deviceStatus; //ccd
    QPointer<StatusWidget1> deviceStatus2; //monoch
    TwoStateButton *PBToNewWindow;

    CCDPlot(QWidget *parent = 0);
    void fillLaserConf();
    ~CCDPlot();

    CCDMatrix *ccdMatrix;
    plot::TPlot *MyPl;
    plot::T2dPlot *PMTPlot;
    bool ScanCCDMode;

    plot::T3dDataOwner imgData;
    plot::TCurvesDataOwner curveData;
    plot::TCurvesDataOwner PMTcurveData;
    plot::TMetaStorage curveCalib;

    QVector<CoboltGui *> cobolt_gui_list;
    QList<double> laser_list_wavelength;
    QList<int> laser_list_edgefilter_idx;
    QList<int> laser_list_beamexpander_wl_idx;
    QList<int> laser_list_beamexpander_diam_idx;
    QList<double> laser_list_monoch_x;
    QList<double> laser_list_monoch_y;
    QList<double> laser_list_monoch_z;
    QList<double> laser_list_confocal_pmt_x;
    QList<double> laser_list_confocal_pmt_y;
    QList<double> laser_list_confocal_pmt_z;

    QScrollArea *LeftScroll;

    QComboBox *CBexitport;
    QComboBox *CBgrating_select;
    QComboBox *CBPreMonochWl;
    QCheckBox *central_wl_reset;

    QComboBox *scaleBox;
    QComboBox *setAcqMode;
    QComboBox *setReadMode;
    QComboBox *setTriggerMode;

    QGroupBox *box_singleTrack;
    QGroupBox *box_accumulation;
    QWidget *box_monochromator;
    QTabWidget *tabs_options;
    Edit *NDEdit;

    Edit *PanoramMin;
    Edit *PanoramMax;
    TwoStateButton *BPanoram;
    QGroupBox *panoramControl;

    QLabel *lb_temp;
    QLabel *lb_auto;

    Edit *lineLaserWl;
    Edit *lineCentrWL;
    Edit *lineDispersion;
    Edit *lineCentralPx;

    Edit *lineExpTime;
    Edit *lineNumAcc;
    Edit *lineAccCycleTime;

    Edit *lineSetTemp;

    Edit *lineCenterRowTrack;
    Edit *lineHeighTrack;

    Edit *lineCentralWlMonoch;
    Edit *linePreCentralWlMonoch;
    Edit *lineSlitInMonoch;
    Edit *lineSlitOutMonoch;

    QCheckBox *calibEn, *pixelCalibEn;
    QLabel *calibLbl, *pixelCalibLbl;
    QPushButton *calibOpn, *pixelCalibOpn;

    QWidget *NLCEdit;
    QRadioButton *BNLCNone, *BNLCAuto, *BNLCFormula, *BNLCFile;
    QPushButton *BNLCCfg;

    Edit *Num_Kin_ser;
    Edit *Time_Kin_ser;
    QGroupBox *Kin_series;
    QComboBox *kinSerDACscan;
    Edit *kinSerDACstep;
    double kinSerDACinitialValue;

    DoubleChanger *zPos;

    QWidget *ccdWidget;
    QWidget *temperature_widget;
    QwtThermo *Termometer;

    TwoStateButton *m_BStart;
    TwoStateButton *One_Cont;
    TwoStateButton *AutoCalibration;
    TwoStateButton *Cut_Pikes;
    TwoStateButton *Base_line;
    QPushButton *Monochr;
    QPushButton *base_line_save;
    QPushButton *base_line_manual;
    QPushButton *BPMTScan;

    bool Nspec_Initalized;
    QMutex *nspectr_mutex;
    bool AutoSend;

    int Nx;
    int Ny;

    QLabel *lb_centralWL;
    QLabel *lb_dispersion;
    QLabel *lb_laser;
    QLabel *sendTemp;
    QLabel *temp_pic;

protected:
    void createPlot();
    void createButtons();
    void createCCDAtt();
    void createLayout();
    bool eventFilter(QObject *Obj, QEvent *Event);
    void closeEvent(QCloseEvent *event);

};

class ARS3000
{
public:
    static
    bool init();
    static
    bool setSlitWidth(int idx, double width); //mm
    static
    bool setLaser(int idx);
    static
    bool isReady();
};


class QScriptEngine;
class CCDNLCEditor : public QWidget
{
    Q_OBJECT
public:
    CCDNLCEditor(CCDPlot *_plot);

signals:
    void newStatus(QString);

public slots:
    void open();
    void save();
    void setMode();
    void testFormula();
    void setStatus(int st = 1);
    void abort();

public:
    CCDPlot *ccdplot;

    QScriptEngine *eng;
    QTimer *timer;
    QTextEdit *edit;
    QLabel *LFormulaStatus;
    plot::T2dPlot *plot2d;
    plot::TCurvesDataOwner curveData;
    int mode;
    QString dataFormula, dataFile;
    QCheckBox *BSpline;

};
