#include "scanplot.h"
#include "ccdplot.h"
#include "ccdmatrix.h"
#include "gui/status.hpp"
#include <qwt_thermo.h>

#include <QApplication>
#include <QSettings>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QMessageBox>

class ICCDScan : public IExternalScan
{
    Q_OBJECT

signals:
    void AddDataToContainer(ccd::TSetOfSpectrum *, plot::TMetaStorage);

public slots:
    void BGetSpectraInfo();
    void save();
    void clear() {
        if (Func_interval->on_off)
            Func_interval->change_state_man_with_sig(false);
        if (Spectrum_part->on_off)
            Spectrum_part->change_state_man_with_sig(false);

        lastNumPoints = 0;
        SetOfSpectrums.Array.clear();
        SetOfSpectrums.Results.at(0)->clear();
        I4dAdd.Section.at(0)->clear();
        IExternalScan::clear();
    }
    void onConvFunc();
    void scaleChanged();
    void sl_Spectrum_range_select(bool st);
    void sl_Func_range_select(bool st);
    void set2dPlotInfo();
    void onRange_valueChanged();
    void onFunc_valueChanged();

    void cancelOperation() {
        isInterrupted = true;
    }
    void addData(QVector<double> data) {
        if (!started || bOom)
            return;

#ifdef USE_CCD_EMULATOR
    QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
    if (settings.value("ccd", false).toBool()) {
            QPoint Point = SetOfSpectrums.Translate(I4dAdd.CurrentXPos, I4dAdd.CurrentYPos);
            double XPos = 1e6 * ((SetOfSpectrums.XLimits[1] - SetOfSpectrums.XLimits[0]) / (double)SetOfSpectrums.NXElems * (double)Point.x() + SetOfSpectrums.XLimits[0]);
            double YPos = 1e6 * ((SetOfSpectrums.YLimits[1] - SetOfSpectrums.YLimits[0]) / (double)SetOfSpectrums.NYElems * (double)Point.y() + SetOfSpectrums.YLimits[0]);

            //int scale = 1. + (rand() % 3);
            ccd::TSpectrum s1 = ccd::GaussSpectrum(data.size(), QPointF(10, 10), QPointF(XPos, YPos), 300);
            ccd::TSpectrum s2 = ccd::GaussSpectrum(data.size(), QPointF(25, 80), QPointF(XPos, YPos), 400);
            ccd::TSpectrum s3 = ccd::GaussSpectrum(data.size(), QPointF(50, 60), QPointF(XPos, YPos), 500);
            ccd::TSpectrum s4 = ccd::GaussSpectrum(data.size(), QPointF(70, 20), QPointF(XPos, YPos), 600);

            for (int i = 0; i < data.size(); i++)
                data[i] = s1[i] + s2[i] + s3[i] + s4[i];
        }
#endif

        bOom = !data.size();
        try {
            data = data.mid(qMax(0., SetOfSpectrums.MinForRecord), SetOfSpectrums.MaxForRecord - qMax(0., SetOfSpectrums.MinForRecord));
            data.squeeze();

            I4dAdd.AddNextSpectrum(data);
        } catch (...) {
            qDebug() << "ccdscan oom";
            bOom = true;
        }

        emit acquired();
    }
    void convUpdated() {
        {
            *Output3d.at(0) = *SetOfSpectrums.Results.at(0);
            I4dAdd.CalcSection();

            int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();
            attributes.insert("ZUnit", T4dDataHelper::getConvZUnits(idx, scale, SetOfSpectrums.YscaleUnit));

            set2dPlotInfo();
            *Section.at(0) = Point_Spectrum->on_off ? *I4dAdd.Section.at(0) : *ccdplot->curveData.at(0);
            Section.at(0)->Pen.setColor(Qt::red);
        }

        int x0, x1, y0, y1;
        GetMathLimits(x0, x1, y0, y1);
        Output3d.at(0)->LimsForMathX[0] = x0;
        Output3d.at(0)->LimsForMathX[1] = x1;
        Output3d.at(0)->LimsForMathY[0] = y0;
        Output3d.at(0)->LimsForMathY[1] = y1;

        emit updated();
    }
    void updateData() {
        lastNumPoints = SetOfSpectrums.CalcResults(lastNumPoints);
    }

public:
    ICCDScan() {
        lastNumPoints = 0;
        scale = 0;
        ActiveHBL = -1;
        I4dAdd.data = &SetOfSpectrums;
        connect(&SetOfSpectrums, SIGNAL(finished()), this, SLOT(convUpdated()), Qt::QueuedConnection);
    }
    bool eventFilter(QObject *, QEvent *);

    bool init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy);
    void finish() {
        //ccdMatrix->s_abortAcq();
        ccdplot->ScanCCDMode = false;
        if (ccdplot->m_BStart->on_off)
            ccdplot->m_BStart->change_state_man_with_sig(false);
        destructStatus(ccdplot->spec_bar);

        disconnect(ccdMatrix, SIGNAL(new2dData(QVector<double>,QVector<double>)), this, SLOT(addData(QVector<double>)));

        IExternalScan::finish();
    }

    QWidget *createInterfaceWidget();

    bool acquireNextPoint() {
        if (bOom) {
            lastError = tr("CCD scan: ") + tr("Out of memory.");
            return false;
        }
        while (ccdplot->m_BStart->on_off) { //sync with sl_aq_but_off()
            qApp->processEvents();
            SleepExport::msleep2(3);
        }
        ccdplot->m_BStart->change_state_man_with_sig(true);
        return true;
    }

    void GetMathLimits(int &x0, int &x1, int &y0, int &y1);

    int ActiveHBL;
    bool isInterrupted;
    int scale;
    int lastNumPoints;

    Edit *ewlamdaRangeMin;
    Edit *ewlamdaRangeMax;
    Edit *ewlamdaFuncMin;
    Edit *ewlamdaFuncMax;

    QLabel *SelectRange;
    QLabel *SelectFunc;

    QComboBox *CBConvFunc;
    TwoStateButton *Spectrum_part;
    TwoStateButton *Func_interval;
    TwoStateButton *Point_Spectrum;

    CCDMatrix *ccdMatrix;
    CCDPlot *ccdplot;
    ccd::TSetOfSpectrum SetOfSpectrums;
    ccd::TSetOfSpectrumConstructor I4dAdd;
    bool bOom;
};
#include <ccdscan.moc>

IExternalScan *getNewCCDIExtScan(CCDPlot *ccd)
{
    ICCDScan *sc = new ICCDScan;
    sc->ccdMatrix = ccd->ccdMatrix;
    sc->ccdplot = ccd;

    sc->disableWidgets << ccd->m_BStart << ccd->zPos << ccd->scaleBox
                       << ccd->ccdControlWidgets;

    QObject::connect(ccd->scaleBox, SIGNAL(currentIndexChanged(int)), sc, SLOT(scaleChanged()));

    return sc;
}


bool ICCDScan::init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy) {
    IExternalScan::init(_nx, _ny, _dir_stpoint, _sx, _sy);

    bOom = false;

    SetOfSpectrums.CalibInfo[0] = ccdMatrix->Laser_WL;
    SetOfSpectrums.CalibInfo[1] = ccdMatrix->Cent_WL_real;
    SetOfSpectrums.CalibInfo[2] = ccdMatrix->Disp_real;
    SetOfSpectrums.CalibInfo[3] = (double)ccdMatrix->pixel_Xsize;
    SetOfSpectrums.CalibInfo[4] = (double)ccdMatrix->Nx_real;
    SetOfSpectrums.CalibInfo[5] = (double)ccdMatrix->Cent_Pixel_real;

    if (ccdplot->setAcqMode->currentIndex() == 2)
        ccdplot->setAcqMode->setCurrentIndex(0);
    if (ccdplot->setReadMode->currentIndex() == 2)
        ccdplot->setReadMode->setCurrentIndex(0);

    if (ccdplot->BPanoram->on_off) {
        lastError = tr("CCD scan: ") + tr("Disable panoram.");
        return false;
    }
    if (ccdplot->BPanoram->on_off)
        ccdplot->BPanoram->change_state_man_with_sig(false);
    if (ccdplot->One_Cont->on_off)
        ccdplot->One_Cont->change_state_man_with_sig(false);

    double xscale = T4dDataHelper::getScaleUnity(scale);
    double MinForFuncApply = ewlamdaFuncMin->value() * xscale;
    double MaxForFuncApply = ewlamdaFuncMax->value() * xscale;
    double MinForRec = ewlamdaRangeMin->value() * xscale;
    double MaxForRec = ewlamdaRangeMax->value() * xscale;

    T4dDataHelper::ReCalcRamanOrWavelengthToPixels(SetOfSpectrums.CalibInfo, MinForFuncApply, MaxForFuncApply, scale);
    T4dDataHelper::ReCalcRamanOrWavelengthToPixels(SetOfSpectrums.CalibInfo, MinForRec, MaxForRec, scale);

    lastNumPoints = 0;
    SetOfSpectrums.XscaleUnit = scale/*ccdMatrix->Scale*/;
    SetOfSpectrums.SetDirStPoint((ccd::TSetOfSpectrum::TDirections)dir, (ccd::TSetOfSpectrum::TStartPoints)start_point);
    I4dAdd.PrepareForNewScan(nx, ny, sx, sy,
                             (int)MinForRec, (int)MaxForRec,
                             (int)MinForFuncApply, (int)MaxForFuncApply);

    ccdplot->ScanCCDMode = true;
    connect(ccdMatrix, SIGNAL(new2dData(QVector<double>,QVector<double>)), this, SLOT(addData(QVector<double>)));

    return true;
}

void ICCDScan::GetMathLimits(int &x0, int &x1, int &y0, int &y1) {
    x0 = y0 = 0;
    x1 = nx;
    y1 = ny;

    int curLine = I4dAdd.GetCurrentLineNumber();
    curLine++;

    switch(dir_stpoint) {
    case spTopLeftAxisX:
    case spTopRightAxisX:
        y0 = qMax(ny - 1 - curLine, 0);
        break;
    case spBottomLeftAxisX:
    case spBottomRightAxisX:
        y1 = qMin(curLine + 1, ny);
        break;
    case spTopLeftAxisY:
    case spBottomLeftAxisY:
        x1 = qMin(curLine + 1, nx);
        break;
    case spTopRightAxisY:
    case spBottomRightAxisY:
        x0 = qMax(nx - 1 - curLine, 0);
        break;
    }
}

QWidget *ICCDScan::createInterfaceWidget() {
    QGroupBox *wid = new QGroupBox(tr("CCD"));
    wid->setObjectName("CCDSCANWIDGET");
    wid->installEventFilter(this);
    QFormLayout *lay = makeFormLayout(wid);

    QPushButton *BClear = new QPushButton(makeHugeIcon(":/CCD/mclear.png"), QString());
    BClear->setToolTip(tr("Clear spectrum"));
    connect(BClear, SIGNAL(clicked()), this, SLOT(clear()));

    QPushButton *BSave = new QPushButton(makeHugeIcon(":/CCD/4DAttach.png"), QString());
    BSave->setToolTip(tr("Send to data"));
    connect(BSave, SIGNAL(clicked()), this, SLOT(save()));

    QPushButton *BGet = new QPushButton(makeHugeIcon(":/AutoCalib/CalImport.png"), QString());
    BGet->setToolTip(tr("Get spectra scale and range"));
    connect(BGet, SIGNAL(clicked()), this, SLOT(BGetSpectraInfo()));

    Point_Spectrum = new TwoStateButton(":/CCD/point_spect_off.png", ":/plot/point.png");
    Point_Spectrum->setToolTip(tr("Switch between section and point spectra"));

    Spectrum_part = new TwoStateButton(":/CCD/Sp_part.png", ":/CCD/Sp_part_on.png");
    Spectrum_part->setToolTip(tr("Select captured spectrum part"));
    connect(Spectrum_part, SIGNAL(state_changed(bool)), this, SLOT(sl_Spectrum_range_select(bool)));

    Func_interval = new TwoStateButton(":/CCD/Func_range.png", ":/CCD/Func_range_on.png");
    Func_interval->setToolTip(tr("Select convolution function range"));
    connect(Func_interval, SIGNAL(state_changed(bool)), this, SLOT(sl_Func_range_select(bool)));

    QHBoxLayout *layB1 = new QHBoxLayout;
    layB1->addWidget(BGet);
    layB1->addWidget(BSave);
    layB1->addWidget(BClear);
    layB1->addWidget(Point_Spectrum);
    layB1->addWidget(Spectrum_part);
    layB1->addWidget(Func_interval);
    layB1->setAlignment(Qt::AlignLeft);

    lay->addRow(layB1);


    CBConvFunc = new QComboBox;
    CBConvFunc->addItem(tr("Integrate"), ccd::TSetOfSpectrum::F_Integrate);
    CBConvFunc->addItem(tr("Maximum"), ccd::TSetOfSpectrum::F_Maximum);
    CBConvFunc->addItem(tr("Maximum Position"), ccd::TSetOfSpectrum::F_MaxPosition);
    CBConvFunc->addItem(tr("MassCenter Position"), ccd::TSetOfSpectrum::F_MassCenterPozition);
    CBConvFunc->addItem(tr("Peak Width"), ccd::TSetOfSpectrum::F_PeakWidth);
    connect(CBConvFunc, SIGNAL(currentIndexChanged(int)), this, SLOT(onConvFunc()));
    lay->addRow(tr("Convolution"), CBConvFunc);


    ewlamdaRangeMin = new Edit(0, 4096, 3);
    ewlamdaRangeMax = new Edit(0, 4096, 3); ewlamdaRangeMax->setValue(ccdMatrix->Nx-1);
    ewlamdaFuncMin = new Edit(0, 4096, 3);
    ewlamdaFuncMax = new Edit(0, 4096, 3); ewlamdaFuncMax->setValue(ccdMatrix->Nx-1);

    connect(ewlamdaRangeMin, SIGNAL(newValue(double)), ewlamdaRangeMax, SLOT(setMinimum(double)));
    connect(ewlamdaRangeMax, SIGNAL(newValue(double)), ewlamdaRangeMin, SLOT(setMaximum(double)));
    connect(ewlamdaFuncMin, SIGNAL(newValue(double)), ewlamdaFuncMax, SLOT(setMinimum(double)));
    connect(ewlamdaFuncMax, SIGNAL(newValue(double)), ewlamdaFuncMin, SLOT(setMaximum(double)));
    connect(ewlamdaRangeMax, SIGNAL(newValue(double)), ewlamdaFuncMax, SLOT(setMaximum(double)));
    connect(ewlamdaRangeMin, SIGNAL(newValue(double)), ewlamdaFuncMin, SLOT(setMinimum(double)));
    connect(ewlamdaRangeMin, SIGNAL(newValue(double)), this, SLOT(onRange_valueChanged()));
    connect(ewlamdaRangeMax, SIGNAL(newValue(double)), this, SLOT(onRange_valueChanged()));
    connect(ewlamdaFuncMin, SIGNAL(newValue(double)), this, SLOT(onFunc_valueChanged()));
    connect(ewlamdaFuncMax, SIGNAL(newValue(double)), this, SLOT(onFunc_valueChanged()));

    SelectRange = new QLabel;
    SelectFunc  = new QLabel;

    QString units = T4dDataHelper::getScaleUnits(scale);
    SelectRange->setText(units);
    SelectFunc->setText(units);

    QHBoxLayout *HBLForFunApplyRange = new QHBoxLayout;
    HBLForFunApplyRange->setSpacing(3);
    HBLForFunApplyRange->addWidget(new QLabel(tr("Function ")));
    HBLForFunApplyRange->addWidget(new QLabel(tr(" from ")));
    HBLForFunApplyRange->addWidget(ewlamdaFuncMin);
    HBLForFunApplyRange->addWidget(new QLabel(tr(" to ")));
    HBLForFunApplyRange->addWidget(ewlamdaFuncMax);
    HBLForFunApplyRange->addWidget(SelectFunc);
    HBLForFunApplyRange->addStretch(1);

    QHBoxLayout *HBLForAllLambdaRange = new QHBoxLayout;
    HBLForAllLambdaRange->setSpacing(3);
    HBLForAllLambdaRange->addWidget(new QLabel(tr("Spectrum ")));
    HBLForAllLambdaRange->addWidget(new QLabel(tr(" from ")));
    HBLForAllLambdaRange->addWidget(ewlamdaRangeMin);
    HBLForAllLambdaRange->addWidget(new QLabel(tr(" to ")));
    HBLForAllLambdaRange->addWidget(ewlamdaRangeMax);
    HBLForAllLambdaRange->addWidget(SelectRange);
    HBLForAllLambdaRange->addStretch(1);

    lay->addRow(HBLForAllLambdaRange);
    lay->addRow(HBLForFunApplyRange);

    disableWidgets << BGet << BSave << BClear << ewlamdaRangeMin << ewlamdaRangeMax << Spectrum_part;

    return wid;
}
void ICCDScan::BGetSpectraInfo()
{
    if (ccdplot->BPanoram->on_off) {
        QMessageBox::warning(displayPlot, tr("CCD"), tr("Disable panoram"));
        return;
    }

    SetOfSpectrums.CalibInfo[0] = ccdMatrix->Laser_WL;
    SetOfSpectrums.CalibInfo[1] = ccdMatrix->Cent_WL_real;
    SetOfSpectrums.CalibInfo[2] = ccdMatrix->Disp_real;
    SetOfSpectrums.CalibInfo[3] = (double)ccdMatrix->pixel_Xsize;
    SetOfSpectrums.CalibInfo[4] = (double)ccdMatrix->Nx_real;
    SetOfSpectrums.CalibInfo[5] = (double)ccdMatrix->Cent_Pixel_real;

    scale = ccdMatrix->Scale_n;

    plot::TCurvesDataOwner Elements_temp = plot::getLock(ccdplot->MyPl->TwoDPlot->Pure2dPlot->SourceData);
    if (Elements_temp.size() > 0 && Elements_temp.at(0) && Elements_temp.at(0)->XSeq.size() > 0) {
        double xscale = T4dDataHelper::getScaleUnity(scale);
        double Min = Elements_temp.at(0)->XSeq.first() / xscale;
        double Max = Elements_temp.at(0)->XSeq.last() / xscale;

        ewlamdaRangeMin->setRange(Min, Max);
        ewlamdaRangeMax->setRange(Min, Max);
        ewlamdaFuncMin->setRange(Min, Max);
        ewlamdaFuncMax->setRange(Min, Max);

        ewlamdaRangeMin->setValue(Min);
        ewlamdaRangeMax->setValue(Max);
        ewlamdaFuncMin->setValue(Min);
        ewlamdaFuncMax->setValue(Max);

        ewlamdaRangeMin->blockSignals(true);
        ewlamdaRangeMax->blockSignals(true);
        ewlamdaFuncMin->blockSignals(true);
        ewlamdaFuncMax->blockSignals(true);
        ewlamdaRangeMin->setNewValue();
        ewlamdaRangeMax->setNewValue();
        ewlamdaFuncMin->setNewValue();
        ewlamdaFuncMax->setNewValue();
        ewlamdaRangeMin->blockSignals(false);
        ewlamdaRangeMax->blockSignals(false);
        ewlamdaFuncMin->blockSignals(false);
        ewlamdaFuncMax->blockSignals(false);
    }

    QString units = T4dDataHelper::getScaleUnits(scale);

    SelectRange->setText(units);
    SelectFunc->setText(units);
}

void ICCDScan::scaleChanged()
{
    double xscale = T4dDataHelper::getScaleUnity(scale);
    double Min0 = ewlamdaRangeMin->minimum() * xscale;
    double Max0 = ewlamdaRangeMin->maximum() * xscale;
    double Min = ewlamdaRangeMin->value() * xscale;
    double Max = ewlamdaRangeMax->value() * xscale;
    double Min1 = ewlamdaFuncMin->value() * xscale;
    double Max1 = ewlamdaFuncMax->value() * xscale;

    T4dDataHelper::ReCalcRamanOrWavelengthToPixels(SetOfSpectrums.CalibInfo, Min0, Max0, scale);
    T4dDataHelper::ReCalcRamanOrWavelengthToPixels(SetOfSpectrums.CalibInfo, Min, Max, scale);
    T4dDataHelper::ReCalcRamanOrWavelengthToPixels(SetOfSpectrums.CalibInfo, Min1, Max1, scale);


    scale = ccdMatrix->Scale_n;
    xscale = T4dDataHelper::getScaleUnity(scale);
    QString units = T4dDataHelper::getScaleUnits(scale);

    SelectRange->setText(units);
    SelectFunc->setText(units);


    T4dDataHelper::ReCalcRamanOrWavelengthFromPixels(SetOfSpectrums.CalibInfo, Min0, Max0, scale);
    T4dDataHelper::ReCalcRamanOrWavelengthFromPixels(SetOfSpectrums.CalibInfo, Min, Max, scale);
    T4dDataHelper::ReCalcRamanOrWavelengthFromPixels(SetOfSpectrums.CalibInfo, Min1, Max1, scale);

    Min0 /= xscale;
    Max0 /= xscale;
    Min /= xscale;
    Max /= xscale;
    Min1 /= xscale;
    Max1 /= xscale;

    ewlamdaRangeMin->setRange(Min0, Max0);
    ewlamdaRangeMax->setRange(Min0, Max0);
    ewlamdaFuncMin->setRange(Min, Max);
    ewlamdaFuncMax->setRange(Min, Max);

    ewlamdaRangeMin->setValue(Min);
    ewlamdaRangeMax->setValue(Max);
    ewlamdaFuncMin->setValue(Min1);
    ewlamdaFuncMax->setValue(Max1);

    ewlamdaRangeMin->blockSignals(true);
    ewlamdaRangeMax->blockSignals(true);
    ewlamdaFuncMin->blockSignals(true);
    ewlamdaFuncMax->blockSignals(true);
    ewlamdaRangeMin->setNewValue();
    ewlamdaRangeMax->setNewValue();
    ewlamdaFuncMin->setNewValue();
    ewlamdaFuncMax->setNewValue();
    ewlamdaRangeMin->blockSignals(false);
    ewlamdaRangeMax->blockSignals(false);
    ewlamdaFuncMin->blockSignals(false);
    ewlamdaFuncMax->blockSignals(false);
}

void ICCDScan::save()
{
    ccd::TSetOfSpectrum *spectr = new ccd::TSetOfSpectrum;
    *(ccd::TInfoForSetOfSpectrums *)spectr = SetOfSpectrums;
    spectr->YscaleUnit = SetOfSpectrums.YscaleUnit;
    spectr->CalibInfo = SetOfSpectrums.CalibInfo;
    *spectr->Results.at(0) = *SetOfSpectrums.Results.at(0);

    plot::TMetaStorage attributes;

    attributes.insert("XPos", spectr->XLimits[0]);
    attributes.insert("YPos", spectr->YLimits[0]);

    spectr->XLimits[1] -= spectr->XLimits[0]; spectr->XLimits[0] = 0;
    spectr->YLimits[1] -= spectr->YLimits[0]; spectr->YLimits[0] = 0;

    attributes.insert("XMin", spectr->XLimits[0]);
    attributes.insert("XMax", spectr->XLimits[1]);
    attributes.insert("YMin", spectr->YLimits[0]);
    attributes.insert("YMax", spectr->YLimits[1]);

    attributes.insert("XYUnit", QObject::tr("m"));
    attributes.insert("YUnit", SetOfSpectrums.YscaleUnit);
    int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();
    attributes.insert("ZUnit", T4dDataHelper::getConvZUnits(idx, scale, SetOfSpectrums.YscaleUnit));

    attributes.insert("Name", "Spectrum");
    attributes.insert("Source", "CCD Scan");


    attributes.insert("X points", nx);
    attributes.insert("Y points", ny);

    attributes.insert("CCD Temperature", ccdMatrix->currentCameraTemperature);
    attributes.insert("Exposure time", ccdMatrix->m_exposureTime);
    attributes.insert("Mode", ccdplot->setAcqMode->currentText());
    attributes.insert("Trigger mode", ccdplot->setTriggerMode->currentText());
    attributes.insert("Read mode", ccdplot->setReadMode->currentText());

    attributes.insert("Accum num", ccdMatrix->numOfAccumulations);
    attributes.insert("Accum cycle time", ccdMatrix->accumulationCycleTime);
    //attributes.insert("Kinetic Series num", ccdMatrix->Kin_ser_number);
    //attributes.insert("Kinetic Cycle num", ccdMatrix->Kin_ser_time);
    attributes.insert("Single track center row", ccdMatrix->m_centralRow);
    attributes.insert("Single track height", ccdMatrix->m_trackHeight);

    attributes.insert("Objective Z", ccdplot->zPos->value());

    attributes.unite(ccdplot->curveCalib);

    displayPlot->appendAttrib(attributes, 1);

    emit AddDataToContainer(spectr, attributes);
}

void ICCDScan::onConvFunc()
{
    int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();

    SetOfSpectrums.CurrentFunc = idx;
    lastNumPoints = 0;
    updateData();
}


void ICCDScan::set2dPlotInfo()
{
    if ((ActiveHBL == -1) && Point_Spectrum->on_off) {
        attributes.insert("Name", tr("CCD 4d scan profile"));
        attributes.insert("XUnit", QObject::tr("m"));
        int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();
        attributes.insert("YUnit", T4dDataHelper::getConvZUnits(idx, scale, SetOfSpectrums.YscaleUnit));
        attributes.insert("XLabel", QString());
        attributes.insert("YLabel", QString());
    } else {
        attributes.insert("Name", tr("CCD 4d point curve"));
        attributes.insert("XUnit", ccdplot->MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SIUnit);
        attributes.insert("YUnit", ccdplot->MyPl->TwoDPlot->Pure2dPlot->YAxisUnit.SIUnit);
        attributes.insert("XLabel", ccdplot->MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.Description);
        attributes.insert("YLabel", ccdplot->MyPl->TwoDPlot->Pure2dPlot->YAxisUnit.Description);
    }
}

void ICCDScan::sl_Spectrum_range_select(bool st)
{
    if (st && Func_interval->on_off) {
        displayPlot->TwoDPlot->SetSelectionModeEnabled(false);
        Func_interval->change_state_man(false);
    }

    ActiveHBL = (st ? 1 : -1);
    set2dPlotInfo();
    emit setSectionEnabled(!st);

    if (st) {
        displayPlot->TwoDPlot->Pure2dPlot->SourceData = ccdplot->MyPl->TwoDPlot->Pure2dPlot->SourceData;
        displayPlot->TwoDPlot->Refresh();
        displayPlot->Set2dPlotVisible(true);
    }

    displayPlot->TwoDPlot->SetSelectionModeEnabled(st);
    displayPlot->TwoDPlot->Pure2dPlot->SetZoomBySelection(!st);
}
void ICCDScan::sl_Func_range_select(bool st)
{
    if (st && Spectrum_part->on_off) {
        displayPlot->TwoDPlot->SetSelectionModeEnabled(false);
        Spectrum_part->change_state_man(false);
    }

    ActiveHBL = (st ? 2 : -1);
    set2dPlotInfo();
    emit setSectionEnabled(!st);

    if (st) {
        displayPlot->TwoDPlot->Pure2dPlot->SourceData = ccdplot->MyPl->TwoDPlot->Pure2dPlot->SourceData;
        displayPlot->TwoDPlot->Refresh();
        displayPlot->Set2dPlotVisible(true);
    }

    if (displayPlot->TwoDPlot->Pure2dPlot->FitOrder != plot::TPlotEnums::FO_None)
        displayPlot->TwoDPlot->Pure2dPlot->s_Refresh();
    displayPlot->TwoDPlot->SetSelectionModeEnabled(st);
    displayPlot->TwoDPlot->Pure2dPlot->SetZoomBySelection(!st);
}
void ICCDScan::onFunc_valueChanged()
{
    double Min = ewlamdaFuncMin->value() * T4dDataHelper::getScaleUnity(scale);
    double Max = ewlamdaFuncMax->value() * T4dDataHelper::getScaleUnity(scale);

    if (ActiveHBL == 2)
        T4dDataHelper::SetSelectGeometry(displayPlot->TwoDPlot, Min, Max);

    T4dDataHelper::ReCalcRamanOrWavelengthToPixels(SetOfSpectrums.CalibInfo, Min, Max, scale);

    SetOfSpectrums.SetMinMaxForFuncApply(Min,Max);
    lastNumPoints = 0;
    updateData();
}
void ICCDScan::onRange_valueChanged()
{
    if (ActiveHBL == 1)
        T4dDataHelper::SetSelectGeometry(displayPlot->TwoDPlot,
                                         ewlamdaRangeMin->value() * T4dDataHelper::getScaleUnity(scale),
                                         ewlamdaRangeMax->value() * T4dDataHelper::getScaleUnity(scale));
}
bool ICCDScan::eventFilter(QObject *obj, QEvent *Event)
{
    if (Event->type() == QEvent::Show && obj->objectName() == "CCDSCANWIDGET") {
        BGetSpectraInfo(); //sync on first show
        obj->removeEventFilter(this); //one time only
    }

    if (Event->type() == plot::TSelectEvent::id && ActiveHBL != -1) {
        double Min = ((plot::TSelectEvent *)Event)->SelectedRect.left();
        double Max = ((plot::TSelectEvent *)Event)->SelectedRect.right();
        if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionStarted) {
            if (ActiveHBL == 1)
                onRange_valueChanged();
            if (ActiveHBL == 2)
                onFunc_valueChanged();
        } else
        if (((plot::TSelectEvent *)Event)->Origin == plot::TSelectEvent::SRO_Mouse || ((plot::TSelectEvent *)Event)->Origin == plot::TSelectEvent::SRO_Unknown) {
            double xscale = T4dDataHelper::getScaleUnity(scale);
            if (ActiveHBL == 1) {
                ewlamdaRangeMin->setValue(Min / xscale);
                ewlamdaRangeMax->setValue(Max / xscale);
                ewlamdaRangeMin->setNewValue();
                ewlamdaRangeMax->setNewValue();
            } else if (ActiveHBL == 2) {
                ewlamdaFuncMin->setValue(Min / xscale);
                ewlamdaFuncMax->setValue(Max / xscale);
                ewlamdaFuncMin->setNewValue();
                ewlamdaFuncMax->setNewValue();
            }
        }
        if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionApplied || ((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionCancled)  {
            if (ActiveHBL == 1)
                Spectrum_part->change_state_man_with_sig(false);
            else if (ActiveHBL == 2)
                Func_interval->change_state_man_with_sig(false);
        }
        return true;
    }
    if (Event->type() == plot::TCrossEvent::id) {
        if (Point_Spectrum->on_off) {
            int XInd, YInd;
            SetOfSpectrums.GetIntegerIndexes(((plot::TCrossEvent *)Event)->Coord.x(), ((plot::TCrossEvent *)Event)->Coord.y(), XInd, YInd);
            T4dDataHelper::ShowSpectr(displayPlot->TwoDPlot, &SetOfSpectrums, XInd, YInd, scale);
            displayPlot->Set2dPlotVisible(true);
            return true;
        }
    }
    return false;
}
