#pragma once

#define ACQMODE_SINGLE 0
#define ACQMODE_ACCUM 1
#define ACQMODE_KINETIC 2

#define RESMODE_FVB 0
#define RESMODE_SINGLETRACK 1
#define RESMODE_IMAGE 2

#define TRGMODE_AUTO 0
#define TRGMODE_EXT 1
#define TRGMODE_EXT2 2

class QWidget;

class I_CCD
{
public:
    I_CCD() : initialized(false), p_data(0), hasTemperature(false),
        temperature(20), minTemperature(-20), maxTemperature(20),
        xPixelsCount(1), yPixelsCount(1), xPixelSize(0.024), yPixelSize(0.024),
        acqMode(0), resMode(2), triggerMode(0),
        centralRow(0), trackHeight(1),
        exposureTime(0.01), accumulateTime(0.1), kineticCycleTime(0.1),
        accumulateCount(1), kineticCyclesCount(1) { }
    virtual ~I_CCD() {}

    virtual bool initialize(int num = 0) = 0;
    virtual bool shutdown() = 0;

    virtual bool setAcquisitionMode(int mode) = 0;
    virtual bool setResultsMode(int mode) = 0;
    virtual bool setTriggerMode(int mode) = 0;

    virtual bool setTemperature(int temp) { temperature = temp; return false; }
    virtual int getTemperature() { return temperature; }

    virtual bool startAcquisition() = 0;
    virtual bool stopAcquisition() = 0;
    virtual bool isReady() = 0;
    virtual bool getEffectiveTimings(double *exposure, double *accumulate, double *kinetic) = 0; //ms
    virtual bool getData() { return false; }

    virtual QWidget *createInterfaceWidget() { return 0; }

    bool initialized;
    long *p_data;

    bool hasTemperature;
    int temperature; //C
    int minTemperature;
    int maxTemperature;

    int xPixelsCount;
    int yPixelsCount;
    double xPixelSize; //mm
    double yPixelSize;

    int acqMode;
    int resMode;
    int triggerMode;

    int centralRow;
    int trackHeight;

    double exposureTime; //s
    double accumulateTime;
    double kineticCycleTime;

    int accumulateCount;
    int kineticCyclesCount;
};
