#include "container.h"
#include "datalist.hpp"
#include "mainwindow.h"
#include "gui/status.hpp"
#include "ccd/ccd.h"
#include "flowlayout.hpp"
#include "gwynetio.hpp"
#include "cont_filesproccessing.h"

#include <qwt_plot_marker.h>
#include <QApplication>
#include <QTimer>
#include <QSettings>
#include <QDragMoveEvent>
#include <QDragEnterEvent>
#include <QProgressBar>
#include <QShortcut>
#include <QSplitter>
#include <QTreeWidget>
#include <QFileDialog>
#include <QHeaderView>
#include <QTimer>
#include <QDateTime>
#include <QUrl>


TContainer::~TContainer()
{
    delete FileProccessing;
}
void TContainer::thread_close()
{
    while (FileProccessing->isRunning()) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    if (gwi) {
        gwi->quit();
        gwi->wait();
        delete gwi;
    }
}
TContainer::TContainer(plot::TPlot *PlotFromDataPlot, QWidget *parent) : QWidget(parent)
{
    Plot = PlotFromDataPlot;
    pl3 = 0;

    new3d = true;
    ActiveHBL = false;
    lastXind = lastYind = 0;

    isAutosave = false;
    preview_update = false;
    copypaste = false;
    func_max = func_min = 0;

    imageitem = new plot::ImageItem;
    imageitem->attach(Plot->ThreeDPlot->ThreeDPlots.at(0)->QPlot);
    imageitem->hide();

    GUICreate();

    FileProccessing = new TFileProccessing(this);
    FileProccessing->setObjectName(QString::fromUtf8("FileProccessing"));
    connect(FileProccessing, SIGNAL(DoAddBlanks(int )), SLOT(AddBlanks(int )));
    connect(FileProccessing, SIGNAL(Error(QString,QString)), SLOT(Error(QString,QString)));

    SetDownButtonsEnabled(false);

    AutosaveTimer = new QTimer(this);
    connect(AutosaveTimer, SIGNAL(timeout()), this, SLOT(AutoSave()));

    QMetaObject::connectSlotsByName(this);

    connect(Plot->TwoDPlot->Pure2dPlot, SIGNAL(MathUpdated()), this, SLOT(Send2dPlotData()));
    connect(Plot->ThreeDPlot->ThreeDPlots.at(0), SIGNAL(MathUpdated()), this, SLOT(Send3dPlotData()));
    connect(Plot->ThreeDPlot->ThreeDPlots.at(0), SIGNAL(MathUpdated()), this, SLOT(on3dUpdate()));
    connect(Plot->ThreeDPlot->ThreeDPlots.at(0), SIGNAL(MathUpdated()), this, SLOT(DelayedPreviewCreation()));
    t3dench = new T3dEnchancement(this);
    t3dench->sourceplot = Plot->ThreeDPlot;
    connect(Plot->ThreeDPlot->Slider->Sliders, SIGNAL(ValuesChanged(double,double)), t3dench, SLOT(queueUpdate()));
    connect(Plot->ThreeDPlot->PlotControl->AGPalettes, SIGNAL(triggered(QAction*)), t3dench, SLOT(queueUpdate()));

    gwi = 0;
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    if (settings.value("net/gwy_import_srv", true).toBool()) {
        gwi = new GwyddionImport();//start gwyddion receive thread
        connect(gwi, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), this, SLOT(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), Qt::QueuedConnection);
        connect(gwi, SIGNAL(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), this, SLOT(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), Qt::QueuedConnection);
        connect(gwi, SIGNAL(Refresh()), this, SLOT(on_FileProccessing_UpdatePreview()));
    }

    setAcceptDrops(true);
}

bool TContainer::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == CBDataLists && event->type() == QEvent::DragEnter)
        event->accept();
    if (obj == CBDataLists && event->type() == QEvent::DragMove) {
        event->ignore();
        QTabBar *tb = 0;
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
        tb = CBDataLists->tabBar();
#else
        tb = CBDataLists->findChild<QTabBar *>(QString());//QLatin1String("qt_tabwidget_tabbar"));
#endif
        int idx = -1;
        if (tb) idx = tb->tabAt(((QDragMoveEvent *)event)->pos() - tb->pos());
        if (idx != -1) CBDataLists->setCurrentIndex(idx);
    }
    return false;
}
void TContainer::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat("text/uri-list"))
        event->acceptProposedAction();
}
void TContainer::dropEvent(QDropEvent *event)
{
    event->acceptProposedAction();

    QList<QString> lst;
    foreach (QUrl url, event->mimeData()->urls()) {
        QString rl = url.toLocalFile();
        if (rl.endsWith(".nstdat"))
            lst.append(rl);
    }

    if (lst.isEmpty()) {
        new Notification(tr("Drag && drop") + ": " + tr("No supported files found"), Notification::Warn, 10);
        doBeep();
        return;
    }

    if (FileProccessing->isRunning()) {
        new Notification(tr("Drag && drop") + ": " + tr("Please wait, another operation is in progress"), Notification::Warn, 10);
        doBeep();
        return;
    }

    if (lst.size() > 1)
        qDebug() << "Only one drag & drop file supported";

    QString FileName = lst.first();
    OpenFile(FileName);
}

void TContainer::GUICreate()
{
    PBBar = new QProgressBar;
    PBBar->hide();

    BToDevice = new QPushButton;
    BToDevice->setObjectName(QString::fromUtf8("BToDevice"));
    BToDevice->setToolTip(tr("Send info to device"));
    BToDevice->setIcon(makeHugeIcon(":/container/todev.png"));
    BToDevice->hide();

    QPushButton *BToGwyddion = new QPushButton;
    BToGwyddion->setObjectName(QString::fromUtf8("BToGwyddion"));
    BToGwyddion->setToolTip(tr("Send selected info to Gwyddion") + " (Ctrl+G)");
    BToGwyddion->setIcon(makeHugeIcon(":/container/GwyExp.png"));
    BToGwyddion->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_G));

    QPushButton *BToGwyddionAll = new QPushButton;
    BToGwyddionAll->setObjectName(QString::fromUtf8("BToGwyddionAll"));
    BToGwyddionAll->setToolTip(tr("Send current tab infos to Gwyddion") + " (Ctrl+Shift+G)");
    BToGwyddionAll->setIcon(makeHugeIcon(":/container/GwyExpTab.png"));
    BToGwyddionAll->setShortcut(QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_G));

    QPushButton *BCopyGroup = new QPushButton;
    BCopyGroup->setObjectName(QString::fromUtf8("BCopyGroup"));
    BCopyGroup->setToolTip(tr("Copy selected group") + " (Ctrl+Shift+C)");
    BCopyGroup->setIcon(makeHugeIcon(":/container/copyGr.png"));
    BCopyGroup->setShortcut(QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_C));

    QPushButton *BSave = new QPushButton;
    BSave->setObjectName(QString::fromUtf8("BSave"));
    BSave->setToolTip(tr("Save") + " (Ctrl+S)");
    BSave->setIcon(makeHugeIcon(":/container/save.png"));
    BSave->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_S));

    QPushButton *BSaveAs = new QPushButton;
    BSaveAs->setObjectName(QString::fromUtf8("BSaveAs"));
    BSaveAs->setToolTip(tr("Save As...") + " (Ctrl+Shift+S)");
    BSaveAs->setIcon(makeHugeIcon(":/container/saveas.png"));
    BSaveAs->setShortcut(QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_S));

    QPushButton *BSaveAll = new QPushButton;
    BSaveAll->setObjectName(QString::fromUtf8("BSaveAll"));
    BSaveAll->setToolTip(tr("Save all"));
    BSaveAll->setIcon(makeHugeIcon(":/container/saveall.png"));

    BToAscii = new QPushButton;
    BToAscii->setObjectName(QString::fromUtf8("BToAscii"));
    BToAscii->setToolTip(tr("Export to ASCII"));
    BToAscii->setIcon(makeHugeIcon(":/container/toTXT.png"));

    QPushButton *BOpen = new QPushButton;
    BOpen->setObjectName(QString::fromUtf8("BOpen"));
    BOpen->setToolTip(tr("Open") + " (Ctrl+O)");
    BOpen->setIcon(makeHugeIcon(":/container/open.png"));
    BOpen->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_O));

    QPushButton *BCopy = new QPushButton;
    BCopy->setObjectName(QString::fromUtf8("BCopy"));
    BCopy->setToolTip(tr("Copy selected data") + " (Ctrl+C)");
    BCopy->setIcon(makeHugeIcon(":/container/copy.png"));
    BCopy->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_C));

    BPaste = new QPushButton;
    BPaste->setObjectName(QString::fromUtf8("BPaste"));
    BPaste->setToolTip(tr("Paste data") + " (Ctrl+V)");
    BPaste->setIcon(makeHugeIcon(":/container/paste.png"));
    BPaste->setEnabled(false);
    BPaste->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_V));

    QPushButton *BMerge = new QPushButton;
    BMerge->setObjectName(QString::fromUtf8("BMerge"));
    BMerge->setToolTip(tr("Merge all tabs"));
    BMerge->setIcon(makeHugeIcon(":/container/merge.png"));

    BRemove = new QPushButton;
    BRemove->setObjectName(QString::fromUtf8("BRemove"));
    BRemove->setToolTip(tr("Delete selected data") + " (Delete)" + "\n" + tr("Hold \"Ctrl\" to delete group"));
    BRemove->setIcon(makeHugeIcon(":/container/delete.png"));
    BRemove->setShortcut(QKeySequence(Qt::Key_Delete));
    new QShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_Delete), BRemove, SLOT(animateClick()), 0, Qt::WindowShortcut);

    BRemoveAll = new QPushButton;
    BRemoveAll->setObjectName(QString::fromUtf8("BRemoveAll"));
    BRemoveAll->setToolTip(tr("Delete all data") + " (Shift+Delete)");
    BRemoveAll->setIcon(makeHugeIcon(":/container/delete_all.png"));
    BRemoveAll->setShortcut(QKeySequence(Qt::SHIFT | Qt::Key_Delete));

    QPushButton *BAddTab = new QPushButton;
    BAddTab->setObjectName(QString::fromUtf8("BAddTab"));
    BAddTab->setToolTip(tr("Add new tab") + " (Ctrl+T)");
    BAddTab->setIcon(makeHugeIcon(":/container/newdata.png"));
    BAddTab->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_T));
    connect(BAddTab, SIGNAL(clicked()), this, SLOT(addTab()));

    BDetach = new QPushButton;
    BDetach->setObjectName(QString::fromUtf8("BDetach"));
    BDetach->setToolTip(tr("Detach current plot"));
    BDetach->setIcon(makeHugeIcon(":/Detach.png"));

    BShowAttributes = new QPushButton;
    BShowAttributes->setObjectName(QString::fromUtf8("BShowAttributes"));
    BShowAttributes->setIcon(makeHugeIcon(":/container/info.png"));
    BShowAttributes->setToolTip(tr("Show attributes panel"));
    BShowAttributes->setCheckable(true);

    BAddAttrib = new QPushButton;
    BAddAttrib->setObjectName(QString::fromUtf8("BAddAttrib"));
    BAddAttrib->setIcon(makeHugeIcon(":/container/infoplus.png"));
    BAddAttrib->setToolTip(tr("Add attribute line"));
    BAddAttrib->hide();

    B3d = new QPushButton;
    B3d->setObjectName(QString::fromUtf8("B3d"));
    B3d->setIcon(makeHugeIcon(":/container/3dimage.png"));
    B3d->setToolTip(tr("Show 3D View"));
    B3d->setCheckable(true);

    Convol_func = new QToolButton;
    Convol_func->setIcon(makeHugeIcon(":/CCD/Func.png"));
    Convol_func->setToolTip(tr("Convolution function"));
    Convol_func->setPopupMode(QToolButton::InstantPopup);
    Convol_func->hide();

    QMenu *M_Convol_func = new QMenu;
    Convol_func->setMenu(M_Convol_func);

    AG_Convol_func = new QActionGroup(this);
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Integrate")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Maximum")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Maximum Position")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("MassCenter Position")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Peak Width")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Stiffness")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Adhesion")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Energy Dissipation")));
    AG_Convol_func->addAction(M_Convol_func->addAction(tr("Baseline")));
    AG_Convol_func->actions()[0]->setCheckable(true);
    AG_Convol_func->actions()[1]->setCheckable(true);
    AG_Convol_func->actions()[2]->setCheckable(true);
    AG_Convol_func->actions()[3]->setCheckable(true);
    AG_Convol_func->actions()[4]->setCheckable(true);
    AG_Convol_func->actions()[5]->setCheckable(true);
    AG_Convol_func->actions()[6]->setCheckable(true);
    AG_Convol_func->actions()[7]->setCheckable(true);
    AG_Convol_func->actions()[8]->setCheckable(true);
    AG_Convol_func->actions()[0]->setChecked(true);

    for (int i = 0; i < 9; i++)
        connect(AG_Convol_func->actions()[i], SIGNAL(triggered()), this, SLOT(onAG_ConvFunc_trigered()));

    Func_interval = new TwoStateButton(":/CCD/Func_range.png",":/CCD/Func_range_on.png");
    Func_interval->setToolTip(tr("Function convolution range selection"));
    Func_interval->setCheckable(true);
    Func_interval->hide();
    connect(Func_interval, SIGNAL(state_changed(bool)), SLOT(sl_Func_range_select(bool)));

    Scale_select = new QToolButton;
    Scale_select->setIcon(makeHugeIcon(":/container/Lunits.png"));
    Scale_select->setToolTip(tr("Display units"));
    Scale_select->setPopupMode(QToolButton::InstantPopup);
    Scale_select->hide();

    QMenu *M_Scale_select = new QMenu;
    Scale_select->setMenu(M_Scale_select);

    AG_Scale_select = new QActionGroup(this);
    AG_Scale_select->addAction(M_Scale_select->addAction(tr("Pixels")))->setCheckable(true);
    AG_Scale_select->addAction(M_Scale_select->addAction(tr("Wavelength")))->setCheckable(true);
    AG_Scale_select->addAction(M_Scale_select->addAction(tr("Raman shift")))->setCheckable(true);
    AG_Scale_select->actions()[0]->setChecked(true);
    connect(AG_Scale_select, SIGNAL(triggered(QAction*)), this, SLOT(onAG_Scale_select_trigered()));


    QGroupBox *ButtonsWid = new QGroupBox(this);
    QSplitter *spl = new QSplitter(this);

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->addWidget(ButtonsWid);
    MainLayout->addWidget(spl);

    QWidget* GrData = new QWidget(spl);
    spl->addWidget(GrData);
    spl->setStretchFactor(0, 1);
    QVBoxLayout *LData = new QVBoxLayout(GrData);
    LData->setContentsMargins(0, 0, 0, 0);


    GTable = new QGroupBox(tr("Info"), spl);
    GTable->hide();
    spl->addWidget(GTable);
    QHBoxLayout *Ltable = new QHBoxLayout(GTable);

    table = new QTreeWidget(GTable);
    table->setMinimumWidth(230);
    Ltable->addWidget(table);
    connect(table, SIGNAL(collapsed(QModelIndex)), this, SLOT(attribEditorResize()));
    connect(table, SIGNAL(expanded(QModelIndex)), this, SLOT(attribEditorResize()));
    connect(table, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(attribConfigEdited(QTreeWidgetItem*,int)));

    QList<QString> header;
    header.push_back(tr("Name"));
    header.push_back(tr("Value"));
    table->setHeaderLabels(header);


    CBDataLists = new QTabWidget(GrData);
    CBDataLists->setDocumentMode(true);
    CBDataLists->setAcceptDrops(true);
    CBDataLists->installEventFilter(this);
    CBDataLists->setTabsClosable(true);
    addTab(tr("Default"));
    CurrentDataList = (TListWidget *)CBDataLists->widget(0);
    connect(CBDataLists, SIGNAL(currentChanged(int)), SLOT(TabChanged(int)));
    connect(CBDataLists, SIGNAL(tabCloseRequested(int)), SLOT(TabClose(int)));

    LData->addWidget(CBDataLists);
    LData->addWidget(PBBar);


    if (!show_additional_gui()) {
        BSaveAll->hide();
        BCopyGroup->hide();
        BMerge->hide();
        BToGwyddionAll->hide();
        BDetach->hide();

        if (!show_advanced_gui()) {
            BSaveAs->hide();
            BShowAttributes->hide();
            BToAscii->hide();
        }
    }

    HFlowLayout *ButtonsLayout = new HFlowLayout(ButtonsWid);
    ButtonsLayout->spaceX = 6; ButtonsLayout->spaceY = 6;
    ButtonsLayout->setContentsMargins(3, 3, 3, 3);

    ButtonsLayout->addWidget(BSave);
    ButtonsLayout->addWidget(BSaveAs);
    ButtonsLayout->addWidget(BSaveAll);
    ButtonsLayout->addWidget(BOpen);

    addDelimeterH(ButtonsLayout, BSave->sizeHint().width());
    ButtonsLayout->addWidget(BAddTab);
    ButtonsLayout->addWidget(BCopy);
    ButtonsLayout->addWidget(BCopyGroup);
    ButtonsLayout->addWidget(BPaste);
    ButtonsLayout->addWidget(BMerge);
    ButtonsLayout->addWidget(BRemove);
    ButtonsLayout->addWidget(BRemoveAll);

    addDelimeterH(ButtonsLayout, BSave->sizeHint().width());
    ButtonsLayout->addWidget(BShowAttributes);
    ButtonsLayout->addWidget(BAddAttrib);
    ButtonsLayout->addWidget(BToDevice);

    addDelimeterH(ButtonsLayout, BSave->sizeHint().width());
    ButtonsLayout->addWidget(B3d);
    ButtonsLayout->addWidget(Convol_func);
    ButtonsLayout->addWidget(Func_interval);
    ButtonsLayout->addWidget(Scale_select);

    addDelimeterH(ButtonsLayout, BSave->sizeHint().width());
    ButtonsLayout->addWidget(BToGwyddion);
    ButtonsLayout->addWidget(BToGwyddionAll);
    ButtonsLayout->addWidget(BToAscii);
    ButtonsLayout->addWidget(BDetach);
}

void TContainer::SetDownButtonsEnabled(bool State)
{
    if (!State) {
        if (BShowAttributes->isChecked()) BShowAttributes->toggle();
        if (B3d->isChecked()) B3d->toggle();
    }

    BRemoveAll->setEnabled(State);
    BRemove->setEnabled(State);
    BShowAttributes->setEnabled(State);
    B3d->setEnabled(State);
    BToAscii->setEnabled(State);
    BDetach->setEnabled(State);
}


void TContainer::SavePreview()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QImage img;
    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {
        if (!Plot->TwoDPlot->Pure2dPlot->isMathReady())
            return;
        img = Plot->TwoDPlot->MakePreview();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d ||
               CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        if (!Plot->ThreeDPlot->ThreeDPlots.at(0)->isMathReady())
            return;
        img = Plot->ThreeDPlot->MakePreview();
    } else
        return;

    CurrentDataList->ClickedItem->attributes.insert("Preview", QVariant::fromValue(img));
    CurrentDataList->blockSignals(true);
    CurrentDataList->ClickedItem->Item->setIcon(QIcon(QPixmap::fromImage(img)));
    CurrentDataList->blockSignals(false);
}

void TContainer::UpdatePreview()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    preview_update = true;
    CurrentDataList->ClickedItem->UpdatePreview = true;
    CurrentDataList->setCurrentItem(CurrentDataList->ClickedItem->Item);
    preview_update = false;
}

void TContainer::SaveFilters()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    TListItem *ClickedItem = CurrentDataList->ClickedItem;
    if (ClickedItem->InfoType == TAbstractInfo::_2d) {
        ClickedItem->attributes.insert("Fit mode", FindCheckedItemIn(Plot->TwoDPlot->PlotControl->AGFitOrders));

        QStringList clrs;
        for (int i = 0; i < ((plot::TCurvesDataOwner *)ClickedItem->Info)->size(); i++)
            clrs.append((*(plot::TCurvesDataOwner *)ClickedItem->Info)[i]->Pen.color().name());

        ClickedItem->attributes.insert("Curve colour", clrs.join(","));
        ClickedItem->attributes.insert("Curve thickness", FindCheckedItemIn(Plot->TwoDPlot->PlotControl->AGCurveThickness));
    } else if (ClickedItem->InfoType == TAbstractInfo::_3d ||
               ClickedItem->InfoType == TAbstractInfo::_4d) {
        Plot->ThreeDPlot->appendAttrib(ClickedItem->attributes);
    }
}


void TContainer::TabChanged(int index)
{
    SaveFilters();
    SavePreview();

    if (index == -1) {
        if (CBDataLists->count() == 0)
            addTab(tr("Default"));

        CBDataLists->setCurrentIndex(0);
        return;
    }

    CurrentDataList = (TListWidget *)CBDataLists->widget(index);

    SetDownButtonsEnabled(CurrentDataList->DataList.count() != 0);

    if (CurrentDataList->ClickedItem) {
        preview_update = true;
        CurrentDataList->on_itemPressed(CurrentDataList->ClickedItem->Item);
        preview_update = false;
    } else {
        Func_interval->hide();
        Convol_func->hide();
        Scale_select->hide();

        updateAttributes();
    }
}

void TContainer::TabClose(int index)
{
    TListWidget *Current = (TListWidget *)CBDataLists->widget(index);
    Current->ClickedItem = NULL;

    if (index == 0) {
        Current->ClearAll();
        Current->filename.clear();

        if (!CurrentDataList->DataList.count())
            SetDownButtonsEnabled(false);

        if (Current == CurrentDataList) {
            Func_interval->hide();
            Convol_func->hide();
            Scale_select->hide();

            updateAttributes();
        }

        return;
    }

    CBDataLists->removeTab(index);

    if (Current == CurrentDataList)
        CurrentDataList = 0;

    delete Current;

    BPaste->setEnabled(false);
}

void TContainer::addTab(QString name)
{
    if (name.isEmpty())
        name = tr("New tab");

    TListWidget *newList = new TListWidget(this);
    newList->setMinimumWidth(200);
    connect(newList, SIGNAL(ItemClicked(const plot::T3dDataOwner*)), SLOT(onItemClicked(const plot::T3dDataOwner*)));
    connect(newList, SIGNAL(ItemClicked(const plot::TCurvesDataOwner*)), SLOT(onItemClicked(const plot::TCurvesDataOwner*)));
    connect(newList, SIGNAL(ItemClicked(const ccd::TSetOfSpectrum *)), SLOT(onItemClicked(const ccd::TSetOfSpectrum *)));
    connect(newList, SIGNAL(ItemClicked(const QImage)), SLOT(onItemClicked(const QImage)));

    int index = CBDataLists->addTab(newList, name);

    CBDataLists->setCurrentIndex(index);
}


void TContainer::on_BCopyGroup_clicked()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    info_copypaste.clear();
    copypaste_type.clear();
    copypaste_attrib.clear();

    QString group = CurrentDataList->ClickedItem->attributes.value("GroupId").toString();
    for (int i = 0; i < CurrentDataList->DataList.size(); i++)
        if (CurrentDataList->DataList[i]->attributes.value("GroupId").toString() == group) {
            info_copypaste.push_back(CurrentDataList->DataList[i]->Info);
            copypaste_type.push_back(CurrentDataList->DataList[i]->InfoType);
            copypaste_attrib.push_back(CurrentDataList->DataList[i]->attributes);
        }

    BPaste->setEnabled(true);
}

void TContainer::on_BCopy_clicked()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    info_copypaste.clear();
    copypaste_type.clear();
    copypaste_attrib.clear();

    info_copypaste.push_back(CurrentDataList->ClickedItem->Info);
    copypaste_type.push_back(CurrentDataList->ClickedItem->InfoType);
    copypaste_attrib.push_back(CurrentDataList->ClickedItem->attributes);

    BPaste->setEnabled(true);
}
void TContainer::on_BPaste_clicked()
{
    if (!info_copypaste.size())
        return;

    BPaste->setEnabled(false);

    copypaste = true;
    for (int i = 0; i < info_copypaste.size(); i++) {
        try {
            if (copypaste_type[i] == TAbstractInfo::_3d) {
                plot::T3dDataOwner *old = ((plot::T3dDataOwner *)info_copypaste[i]);
                plot::T3dDataOwner newOne;
                plot::ResizeOwner(newOne, old->size());
                for (int j = 0; j < newOne.size(); j++)
                    *newOne.at(j) = *old->at(j);

                AddDataToContainer(newOne, copypaste_attrib[i]);
            } else if (copypaste_type[i] == TAbstractInfo::_2d) {
                plot::TCurvesDataOwner *old = ((plot::TCurvesDataOwner *)info_copypaste[i]);
                plot::TCurvesDataOwner newOne;
                plot::ResizeOwner(newOne, old->size());
                for (int j = 0; j < newOne.size(); j++)
                    *newOne.at(j) = *old->at(j);

                AddDataToContainer(newOne, copypaste_attrib[i]);
            } else if (copypaste_type[i] == TAbstractInfo::_4d) {
                ccd::TSetOfSpectrum *spectr = new ccd::TSetOfSpectrum;
                ccd::TSetOfSpectrum *old = ((ccd::TSetOfSpectrum *)info_copypaste[i]);
                *(ccd::TInfoForSetOfSpectrums *)spectr = *old;
                spectr->CalibInfo = old->CalibInfo;
                *spectr->Results[0] = *old->Results[0];

                AddDataToContainer(spectr, copypaste_attrib[i]);
            } else if (copypaste_type[i] == TAbstractInfo::_Image) {
                AddDataToContainer(copypaste_attrib[i].value("Preview").value<QImage>(), copypaste_attrib[i]);
            }
        } catch (...) {
            //Mem fail
        }
    }

    UpdateHierarchy();

    copypaste = false;
}

void TContainer::on_BMerge_clicked()
{
    addTab(tr("Merge tab"));
    int idx_result = CBDataLists->currentIndex();

    setEnabled(false);
    qApp->processEvents();

    info_copypaste.clear();
    copypaste_type.clear();
    copypaste_attrib.clear();

    for (int idx = 0; idx < CBDataLists->count(); idx++) {
        if (idx == idx_result)
            continue;

        TListWidget *w = (TListWidget *)CBDataLists->widget(idx);

        for (int i = 0; i < w->DataList.count(); i++) {
            info_copypaste.push_back(w->DataList[i]->Info);
            copypaste_type.push_back(w->DataList[i]->InfoType);
            copypaste_attrib.push_back(w->DataList[i]->attributes);
        }
    }

    CBDataLists->setCurrentIndex(idx_result);
    on_BPaste_clicked();

    setEnabled(true);
}


void TContainer::OpenFile(QString FileName)
{
    if (FileName.isEmpty())
        return;

    QString name = FileName;
    if (name.lastIndexOf("/") != -1)
        name.remove(0, name.lastIndexOf("/") + 1);
    if (name.lastIndexOf("\\") != -1)
        name.remove(0, name.lastIndexOf("\\") + 1);

    addTab(name);

    CurrentDataList->filename = FileName;

    Load(FileName);

    SetDownButtonsEnabled(true);
}

QString TContainer::getDataDir()
{
    QString datadir = "NST data files";
#ifndef _WIN32
    datadir = QDir::homePath() + "/" + datadir;
#endif
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    datadir = settings.value("data_dir", datadir).toString();
    return datadir;
}

void TContainer::on_BOpen_clicked()
{
    QString FileName;
    FileName = FileDir.isEmpty() ? TContainer::getDataDir() + "/" : FileDir;

    FileName = QFileDialog::getOpenFileName(this, QString(), FileName,
                                            QString("NST files(*.nstdat)"),
                                            0, QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    FileDir = FileName.left(FileName.lastIndexOf("/") + 1);

    OpenFile(FileName);
}

void TContainer::on_BSave_clicked()
{
    SaveFilters();
    SavePreview();

    if (!CurrentDataList->DataList.count())
        return;
    if (!CurrentDataList->isUpdated && isAutosave)
        return;


    QString FileName;
    if (!CurrentDataList->filename.isEmpty())
        FileName = CurrentDataList->filename;
    else if (isAutosave) {
        FileName = TContainer::getDataDir() + QString("/AutoSave%1_").arg(++autosave_idx) + QDateTime::currentDateTime().toString("yyyy-MM-dd_hhap") + ".nstdat";
    } else {
        FileName = (FileDir.isEmpty() ? TContainer::getDataDir() + "/" : FileDir) + QDateTime::currentDateTime().toString("yyyy-MM-dd_hhap") + ".nstdat";

        FileName = QFileDialog::getSaveFileName(this, tr("Save"), FileName,
                                                QString("NST files(*.nstdat)"),
                                                0, QFileDialog::DontUseNativeDialog);

        if (FileName.isEmpty())
            return;

        FileDir = FileName.left(FileName.lastIndexOf("/") + 1);
    }


    Save(FileName);

    while (FileProccessing->isRunning()) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    if (isAutosave)
        return;

    CurrentDataList->filename = FileName;

    if (FileName.lastIndexOf("/") != -1)
        FileName.remove(0, FileName.lastIndexOf("/") + 1);
    if (FileName.lastIndexOf("\\") != -1)
        FileName.remove(0, FileName.lastIndexOf("\\") + 1);

    CBDataLists->setTabText(CBDataLists->indexOf(CurrentDataList), FileName);

    if (CBDataLists->indexOf(CurrentDataList) == 0) //for "Default"
    {
        CBDataLists->blockSignals(true);
        addTab(tr("Default"));
        CBDataLists->findChild<QTabBar *>()->moveTab(CBDataLists->count()-1, 0);
        CBDataLists->setCurrentIndex(1);
        CBDataLists->blockSignals(false);
    }
}
void TContainer::on_BSaveAs_clicked()
{
    SaveFilters();
    SavePreview();

    if (!CurrentDataList->DataList.count())
        return;

    QString FileName;
    if (CurrentDataList->filename.isEmpty())
        FileName = (FileDir.isEmpty() ? TContainer::getDataDir() + "/" : FileDir) + QDateTime::currentDateTime().toString("yyyy-MM-dd_hhap") + ".nstdat";
    else
        FileName = CurrentDataList->filename; //TODO add "_copy"

    FileName = QFileDialog::getSaveFileName(this, tr("Save As"), FileName,
                                            QString("NST files(*.nstdat)"),
                                            0, QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    FileDir = FileName.left(FileName.lastIndexOf("/") + 1);

    Save(FileName);
}
void TContainer::on_BSaveAll_clicked()
{
    SaveFilters();
    SavePreview();

    int old_idx = CBDataLists->currentIndex();
    int count = CBDataLists->count();
    bool def_tab = false;

    for (int idx = 0; idx < count; idx++) {
        CBDataLists->setCurrentIndex(idx);
        on_BSave_clicked();

        //if default tab was saved into another tab
        if (count != CBDataLists->count()) {
            def_tab = true;
            count = CBDataLists->count();
            idx++;
        }
    }

    if (def_tab && old_idx == 0)
        old_idx++;

    CBDataLists->setCurrentIndex(old_idx);
}

void TContainer::on_BRemoveAll_clicked()
{
    CurrentDataList->ClearAll();
    CurrentDataList->filename.clear();

    SetDownButtonsEnabled(false);

    CurrentDataList->ClickedItem = NULL;
    BPaste->setEnabled(false);
    updateAttributes();
}
void TContainer::on_BRemove_clicked()
{
    if (!CurrentDataList->ClickedItem)
        return;

    if (qApp->queryKeyboardModifiers() & Qt::ControlModifier) {
        //remove group instead
        QString group = CurrentDataList->ClickedItem->attributes.value("GroupId").toString();
        QVector<int> lst;
        for (int i = 0; i < CurrentDataList->DataList.size(); i++)
            if (CurrentDataList->DataList[i]->attributes.value("GroupId").toString() == group)
                lst.append(i);
        for (int i = lst.size()-1; i >= 0; i--)
            CurrentDataList->Delete(lst.at(i));
    } else
        CurrentDataList->Delete();

    if (!CurrentDataList->count())
        SetDownButtonsEnabled(false);

    CurrentDataList->ClickedItem = NULL;
    CurrentDataList->setCurrentItem(0);
    BPaste->setEnabled(false);
    updateAttributes();
}

void TContainer::on_BDetach_clicked()
{
    if (Plot->CurrentPlotType == plot::TPlot::_2d) {
        plot::T2dPlot *plot = new plot::T2dPlot;

        plot->setWindowTitle(Plot->TwoDPlot->Pure2dPlot->Title + tr(" - View"));
        plot->setMinimumSize(500, 300);
        plot->resize(700, 500);

        plot->Pure2dPlot->SourceData = Plot->TwoDPlot->Pure2dPlot->SourceData;
        *((plot::TInfoFor2dPlot *)plot->Pure2dPlot) = *((plot::TInfoFor2dPlot *)Plot->TwoDPlot->Pure2dPlot);

        plot->Refresh();
        plot->setAttribute(Qt::WA_DeleteOnClose, true);
        plot->show();
    } else { //_3d
        if (B3d->isChecked()) {
            parentWidget()->setEnabled(false);
            while (!Plot->ThreeDPlot->ThreeDPlots.at(0)->isMathReady()) {
                qApp->processEvents();
                SleepExport::msleep2(3);
            }
            parentWidget()->setEnabled(true);

            T3dViewer *pl = pl3; //backup default plot

            QWidget *Wid3d = create3dPlot();
            T3dEnchancement *ench = new T3dEnchancement(Wid3d);
            ench->pl3 = pl3;
            ((QWidget *)pl3)->installEventFilter(ench);
            ((QWidget *)pl3)->setAcceptDrops(true);

            Wid3d->setWindowTitle(Plot->ThreeDPlot->ThreeDPlots.at(0)->Title + tr(" - 3d View"));
            Wid3d->setMinimumSize(600, 500);
            Wid3d->resize(700, 600);
            Wid3d->setAttribute(Qt::WA_DeleteOnClose, true);
            Wid3d->show();

            new3d = true;
            on_B3d_toggled(true); //update data
            pl3 = pl;
            return;
        }

        plot::T3dPlot *plot = new plot::T3dPlot;

        plot->setWindowTitle(Plot->ThreeDPlot->ThreeDPlots.at(0)->Title + tr(" - View"));
        plot->setMinimumSize(600, 500);
        plot->resize(700, 600);

        plot->ThreeDPlots[0]->SourceData = Plot->ThreeDPlot->ThreeDPlots.at(0)->SourceData;
        *((plot::TInfoFor3dPlot *)plot->ThreeDPlots[0]) = *((const plot::TInfoFor3dPlot *)Plot->ThreeDPlot->ThreeDPlots.at(0));
        if (imageitem->isVisible()) {
            plot::ImageItem *imageitem2 = new plot::ImageItem;
            imageitem2->attach(plot->ThreeDPlots.at(0)->QPlot);
            imageitem2->image = imageitem->image;
            imageitem2->targetRect = imageitem->targetRect;

            plot->Slider->hide();
            plot->PlotControl->setEnabled(false);
        }

        plot->ThreeDPlots[0]->ReSetAxisScales();
        plot->Refresh();
        plot->setAttribute(Qt::WA_DeleteOnClose, true);
        plot->show();
    }
}


void TContainer::on_BShowAttributes_toggled(bool checked)
{
    DataPlot *data = (DataPlot *)ModuleManager::instance->find1("data")->basecast();
    QSplitter *spl = data->Splitter;
    QSplitter *spl2 = ((QSplitter *)GTable->parentWidget());
    int sz1 = spl->sizes().at(0);
    int sz2 = spl->sizes().at(1);
    int sz3 = -GTable->sizeHint().width() - spl->handleWidth();

    BAddAttrib->setVisible(checked);
    BToDevice->setVisible(checked && ModuleManager::instance->find1("spm"));
    GTable->setVisible(checked);

    setUpdatesEnabled(false);
    qApp->processEvents();
    setUpdatesEnabled(true);

    if (checked)
        sz3 = GTable->sizeHint().width() + spl->handleWidth();
    spl2->setSizes(QList<int>() << 100000 << 1);
    spl->setSizes(QList<int>() << sz1+sz3 << sz2-sz3);

    if (!checked) {
        //table->clear();
        return;
    }

    updateAttributes();
}

void TContainer::attribEditorResize()
{
    table->header()->resizeSections(QHeaderView::ResizeToContents);
}

void TContainer::attribConfigEdited(QTreeWidgetItem *, int)
{
    SaveAttrib();
    CurrentDataList->on_itemPressed(CurrentDataList->ClickedItem->Item);
}

void TContainer::addAttribute(QString groupname, QString name, QString val)
{
    QTreeWidgetItem *root = 0;
    if (!groupname.isEmpty()) {
        if (!attribItemsRoots.contains(groupname)) {
            root = new QTreeWidgetItem(table, QStringList() << groupname);

            attribItemsRoots.insert(groupname, root);
        } else
            root = attribItemsRoots.value(groupname);
    }

    QTreeWidgetItem *item;
    if (root) {
        item = new QTreeWidgetItem(root, QStringList() << name << val);
        root->setExpanded(true);
    } else {
        item = new QTreeWidgetItem(table, QStringList() << name << val);
        item->setData(0, Qt::UserRole, true);
    }

    item->setFlags(item->flags() | Qt::ItemIsEditable);
}

QString attribGroup(QString name)
{
    if (name == "Palette" ||
            name == "Curve colour" ||
            name == "Curve thickness")
        return "Plot";

    if (name == "XLabel" ||
            name == "XUnit" ||
            name == "YLabel" ||
            name == "YUnit" ||
            name == "ZLabel" ||
            name == "ZUnit" ||
            name == "XYUnit" ||
            name == "XYLabel")
        return "Labels";

    if (name == "Name" ||
            name == "Date" ||
            name == "Id" ||
            name == "GroupId" ||
            name == "Parent" ||
            name == "Source")
        return "Common";

    if (name == "CCD NX" ||
            name == "CCD NY" ||
            name == "CCD Central Wavelength, nm" ||
            name == "CCD Central Pixel" ||
            name == "CCD Dispersion")
        return "CCD";

    if (name == "CCD X idx" ||
            name == "CCD Y idx" ||
            name == "CCD X screen pos, m" ||
            name == "CCD Y screen pos, m" ||
            name == "CCD X global pos, m" ||
            name == "CCD Y global pos, m")
        return "CCD Scan Pos";

    if (name == "XPos" ||
            name == "YPos" ||
            name .startsWith("Head X") ||
            name .startsWith("Head Y") ||
            name .startsWith("Stage X") ||
            name .startsWith("Stage Y"))
        return "Position";

    if (name == "Nx" ||
            name == "pixelsize" ||
            name == "Laser Wavelength, nm" ||
            name == "Central Wavelength, nm" ||
            name == "Central Pixel" ||
            name == "Dispersion")
        return "Calibration";

    if (name == "XMin" ||
            name == "XMax" ||
            name == "YMin" ||
            name == "YMax" ||
            name == "ZMin View" ||
            name == "ZMax View" ||
            name == "ZViewAuto")
        return "XYZ Limits";

    if (name == "Fit mode" ||
            name == "XFit" ||
            name == "YFit" ||
            name == "Substract" ||
            name == "gwyXFit" ||
            name == "gwyYFit" ||
            name == "Mean" ||
            name == "Median" ||
            name == "Gaussian" ||
            name == "HistEq")
        return "Filters";

    if (name == "Mode" ||
            name == "Accum num" ||
            name == "Accum cycle time" ||
            name == "Trigger mode" ||
            name == "Kinetic Series num" ||
            name == "Kinetic Cycle num" ||
            name == "Read mode" ||
            name == "Single track center row" ||
            name == "Single track height" ||
            name == "Exposure time" ||
            name == "Grating" ||
            name == "ND filter" ||
            name == "CCD Temperature" ||
            name == "Objective Z" ||
            name == "X points" ||
            name == "Y points")
        return "CCD settings";

    if (name == "FB P" ||
            name == "FB I" ||
            name == "FB D" ||
            name == "FB JM Precision" ||
            name == "FB JM Height" ||
            name == "Set point" ||
            name == "From, kHz" ||
            name == "To, kHz" ||
            name == "Current frequency, kHz" ||
            name == "Drive intensity, V" ||
            name == "Low pass filter, kHz" ||
            name == "Phase shift, degree" ||
            name == "ChannelIdx" ||
            name == "DFLIdx" ||
            name == "LFLIdx" ||
            name == "INTIdx" ||
            name == "SPM FB PID" ||
            name == "SPM FB scanner" ||
            name == "SPM FB mode")
        return "SPM settings";

    if (name == "Ch A" ||
            name == "Ch B" ||
            name == "Ch C" ||
            name == "Ch D" ||
            name.startsWith("Lift ") ||
            name == "Velocity, Hz" ||
            name == "XY PID" ||
            name.startsWith("Scan "))
        return "Scan settings";

    return "";
}

void TContainer::updateAttributes()
{
    table->blockSignals(true);
    table->clear();
    if (!CurrentDataList || !CurrentDataList->DataList.count() || !CurrentDataList->ClickedItem) {
        table->setEnabled(false);
        BAddAttrib->setEnabled(false);

        table->blockSignals(false);
        return;
    }
    table->setEnabled(true);
    BAddAttrib->setEnabled(true);

    table->sortItems(-1, Qt::AscendingOrder);

    attribItemsRoots.clear();

    QHash<QString, QVariant>::const_iterator it = CurrentDataList->ClickedItem->attributes.constBegin();
    for (; it != CurrentDataList->ClickedItem->attributes.constEnd(); it++)
        if (it.value().canConvert<QString>())
            addAttribute(attribGroup(it.key()), it.key(), it.value().toString());

    table->sortItems(0, Qt::AscendingOrder);

    table->blockSignals(false);
}

void TContainer::on_BAddAttrib_clicked()
{
    table->blockSignals(true);

    addAttribute(QString(), tr("<unset>"), tr("<unset>"));

    table->blockSignals(false);
}

void TContainer::SaveAttrib()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_Image) {
        QImage data = CurrentDataList->ClickedItem->attributes.value("Preview").value<QImage>();
        CurrentDataList->ClickedItem->attributes.clear();
        CurrentDataList->ClickedItem->attributes.insert("Preview", QVariant::fromValue(data));
    } else
        CurrentDataList->ClickedItem->attributes.clear();

    for (int i = 0; i < table->topLevelItemCount(); i++) {
        QTreeWidgetItem *item = table->topLevelItem(i);
        if (item->data(0, Qt::UserRole).toBool()) {
            if (!item->text(0).isEmpty())
                CurrentDataList->ClickedItem->attributes.insert(item->text(0), item->text(1));
        }

        for (int j = 0; j < item->childCount(); j++) {
            QTreeWidgetItem *it = item->child(j);
            if (!it->text(0).isEmpty())
                CurrentDataList->ClickedItem->attributes.insert(it->text(0), it->text(1));
        }
    }

    CurrentDataList->blockSignals(true);
    CurrentDataList->ClickedItem->Item->setText(CurrentDataList->ClickedItem->attributes.value("Name").toString());
    CurrentDataList->blockSignals(false);

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        ccd::TSetOfSpectrum *info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

        info->XLimits[0] = CurrentDataList->ClickedItem->attributes.value("XMin").toDouble();
        info->XLimits[1] = CurrentDataList->ClickedItem->attributes.value("XMax").toDouble();
        info->YLimits[0] = CurrentDataList->ClickedItem->attributes.value("YMin").toDouble();
        info->YLimits[1] = CurrentDataList->ClickedItem->attributes.value("YMax").toDouble();
        info->UpdateDxDy();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
        plot::T3dDataOwner *info = (plot::T3dDataOwner *)CurrentDataList->ClickedItem->Info;

        info->at(0)->XLimits[0] = CurrentDataList->ClickedItem->attributes.value("XMin").toDouble();
        info->at(0)->XLimits[1] = CurrentDataList->ClickedItem->attributes.value("XMax").toDouble();
        info->at(0)->YLimits[0] = CurrentDataList->ClickedItem->attributes.value("YMin").toDouble();
        info->at(0)->YLimits[1] = CurrentDataList->ClickedItem->attributes.value("YMax").toDouble();
        info->at(0)->UpdateDxDy();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {
        //plot::TCurvesDataOwner *info = (plot::TCurvesDataOwner *)CurrentDataList->ClickedItem->Info;

    }
}


QDateTime itemDate(const TListItem *s, QDateTime default_val = QDateTime::currentDateTime())
{
    QString id = s->attributes.value("Date").toString();
    QDateTime tt = QDateTime::fromString(id, "yyyy-MM-dd hh:mm:ss:zzz");
    if (tt.isValid())
        return tt;

    return default_val;
}
bool dateCompareItem(const TListItem *s1, const TListItem *s2)
{
    QDateTime t1 = itemDate(s1, QDateTime());
    QDateTime t2 = itemDate(s2, QDateTime());
    return t1 > t2;
}
bool dateCompareItemList(const TListItems &s1, const TListItems &s2)
{
    if (s1.size() == 0 || s2.size() == 0) { qDebug() << "fail"; return false; }
    return dateCompareItem(s1.first(), s2.first());
}

void TContainer::UpdateHierarchy()
{
    setEnabled(false);

    //fill groups
    TListItem *citem;
    QVector<TListItems> igroups;
    QVector<QString> groups;

    for (int j = 0; j < CurrentDataList->DataList.count(); j++) {
        citem = CurrentDataList->DataList[j];

        QString id = citem->attributes.value("GroupId", "empty").toString();

        int idx = 0;
        for (; idx < groups.size(); idx++)
            if (groups[idx] == id) {
                igroups[idx].append(citem);
                break;
            }

        if (idx == groups.size()) {
            groups.append(id);
            igroups.append(TListItems() << citem);
        }
    }
    groups.clear();

    //sort items
    for (int i = 0; i < igroups.size(); i++)
        qStableSort(igroups[i].begin(), igroups[i].end(), dateCompareItem);

    //sort groups
    qStableSort(igroups.begin(), igroups.end(), dateCompareItemList);

    CurrentDataList->blockSignals(true);

    //color & name
    static QColor colours[5] = { QColor(234, 99, 30),
                                 QColor(157, 57, 26),
                                 QColor(111, 159, 36),
                                 QColor(106, 197, 220),
                                 QColor(101, 58, 140) };
    int in = 0;
    for (int idx = igroups.size()-1; idx >= 0 ; idx--) {
        bool grouped = igroups[idx].size() > 1;
        for (int i = 0; i < igroups[idx].size(); i++) {
            igroups[idx][i]->Item->setBackground(grouped ? colours[(in % 5)] : Qt::white);

            QString name = igroups[idx][i]->attributes.value("Name", "-").toString();
            igroups[idx][i]->Item->setText(name);
        }
        if (grouped) in++;
    }

    //parents
    //NYI

    //clear list
    while (CurrentDataList->count())
        CurrentDataList->takeItem(0);

    //fill list
    for (int idx = 0; idx < igroups.size(); idx++)
        for (int i = 0; i < igroups[idx].size(); i++)
            CurrentDataList->addItem(igroups[idx][i]->Item);

    if (CurrentDataList->ClickedItem)
        CurrentDataList->setCurrentItem(CurrentDataList->ClickedItem->Item);

    CurrentDataList->blockSignals(false);

    setEnabled(true);
}



void TContainer::on_FileProccessing_started()
{
    setEnabled(false);
    PBBar->setVisible(true);
}
void TContainer::on_FileProccessing_finished()
{
    setEnabled(true);
    PBBar->setVisible(false);
}
void TContainer::on_FileProccessing_UpdatePreview()
{
    parentWidget()->setEnabled(false);
    preview_update = true;

    CurrentDataList->ClickedItem = NULL;
    CurrentDataList->setCurrentItem(0); //reset current to always get update even for single data

    for (int i = 0; i < CurrentDataList->DataList.count(); i++) {
        CurrentDataList->ClickedItem = CurrentDataList->DataList[i];
        CurrentDataList->ClickedItem->UpdatePreview = true;
        CurrentDataList->setCurrentItem(CurrentDataList->ClickedItem->Item);

        qApp->processEvents();
        while (CurrentDataList && CurrentDataList->ClickedItem &&
               CurrentDataList->ClickedItem->UpdatePreview) {
            qApp->processEvents();
            SleepExport::msleep2(3);
        }
    }

    preview_update = false;
    parentWidget()->setEnabled(true);
}
void TContainer::on_FileProccessing_FileProccessed(double TotalNumber, double Current)
{
    PBBar->setUpdatesEnabled(false);
    if (TotalNumber < 0 || Current < 0)
        PBBar->reset();
    else {
        PBBar->setMaximum(int(TotalNumber * 100));
        PBBar->setValue(int(Current * 100));
    }
    PBBar->setUpdatesEnabled(true);
}
void TContainer::on_FileProccessing_AllDataLoaded(const TAbstractInfos *DataList)
{
    for (int i = 0; i < DataList->size(); i++) {
        if ((*DataList)[i].InfoType == TAbstractInfo::_2d)
            CurrentDataList->Add(DataList->at(i).attributes, (plot::TCurvesDataOwner *)DataList->at(i).Info, DataList->at(i).UpdatePreview);
        else if ((*DataList)[i].InfoType == TAbstractInfo::_3d)
            CurrentDataList->Add(DataList->at(i).attributes, (plot::T3dDataOwner *)DataList->at(i).Info, DataList->at(i).UpdatePreview);
        else if ((*DataList)[i].InfoType == TAbstractInfo::_4d) {
            ((ccd::TSetOfSpectrum *)DataList->at(i).Info)->CalcResults();
            CurrentDataList->Add(DataList->at(i).attributes, (ccd::TSetOfSpectrum *)DataList->at(i).Info, DataList->at(i).UpdatePreview);
        }
        else if ((*DataList)[i].InfoType == TAbstractInfo::_Image)
            CurrentDataList->Add(DataList->at(i).attributes, DataList->at(i).attributes.value("Preview").value<QImage>(), false);
    }
    CurrentDataList->isUpdated = false;

    delete DataList;

    UpdateHierarchy();
    setEnabled(true);

    SendContainerData();
}

void TContainer::AddBlanks(int num)
{
    CurrentDataList->AddBlanks(num);
}
void TContainer::Error(QString com, QString msg)
{
    new Notification(com + ": " + msg, Notification::Bad, 20);
    doBeep();
}

void TContainer::Save(QString &FileName)
{
    if (FileName.isEmpty())
        return;

    if (!FileName.endsWith(".nstdat"))
        FileName += ".nstdat";

    CurrentDataList->isUpdated = false;
    FileProccessing->Save(FileName, GenerateDataForSave());
}
void TContainer::AutoSave()
{
    isAutosave = true;
    autosave_idx = 0;
    on_BSaveAll_clicked();
    isAutosave = false;
}
void TContainer::SetAutoSaveTime(int t)
{
    AutosaveTimer->stop();

    AutoSaveTime = t;
    AutosaveTimer->setInterval(t);

    if (AutoSaveTime != 0)
        AutosaveTimer->start();
}

TAbstractInfos *TContainer::GenerateDataForSave()
{
    TAbstractInfos *DataForSave = new TAbstractInfos;
    DataForSave->resize(CurrentDataList->DataList.size());

    for (int i = 0; i < CurrentDataList->DataList.size(); i++)
    {
        (*DataForSave)[i].Info     = CurrentDataList->DataList[i]->Info;
        (*DataForSave)[i].InfoType = CurrentDataList->DataList[i]->InfoType;
        (*DataForSave)[i].attributes = CurrentDataList->DataList[i]->attributes;
    }
    return DataForSave;
}
void TContainer::Load(const QString &FileName)
{
    if (FileName.isEmpty())
        return;

    TAbstractInfos *LoadedData = new TAbstractInfos;
    FileProccessing->Load(FileName, LoadedData);
}


void TContainer::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QString date = getDateHash();
    QString uniqid = date + "Data";
    quint16 id = qChecksum(uniqid.toStdString().data(), uniqid.length());
    attributes.insert("Id", QString::number(id, 16).toUpper());
    attributes.insert("GroupId", CurrentDataList->ClickedItem->attributes.value("GroupId"));
    attributes.insert("Date", CurrentDataList->ClickedItem->attributes.value("Date"));
    attributes.insert("Parent", CurrentDataList->ClickedItem->attributes.value("Id"));
    attributes.insert("Name", CurrentDataList->ClickedItem->attributes.value("Name"));
    attributes.insert("Source", "Data");

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {

    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
        attributes.insert("Marker A x", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerA->xValue()*1e6);
        attributes.insert("Marker A y", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerA->yValue()*1e6);

        attributes.insert("Marker B x", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerB->xValue()*1e6);
        attributes.insert("Marker B y", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerB->yValue()*1e6);

    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        attributes.insert("Name", Plot->TwoDPlot->Pure2dPlot->Title);

        int _x = lastXind;
        int _y = lastYind;
        attributes.insert("CCD X idx", _x);
        attributes.insert("CCD Y idx", _y);

        ccd::TSetOfSpectrum *spec = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;
        spec->UpdateDxDy();
        double XInd, YInd;
        if ((bool)spec->Direction) {
            switch (spec->StartPoint) {
            case ccd::TSetOfSpectrum::TopLeft:     XInd = spec->XLimits[0]+_x*spec->dx; YInd = spec->YLimits[1]-_y*spec->dy; break;
            case ccd::TSetOfSpectrum::TopRight:    XInd = spec->XLimits[1]-_x*spec->dx; YInd = spec->YLimits[1]-_y*spec->dy; break;
            case ccd::TSetOfSpectrum::BottomRight: XInd = spec->XLimits[1]-_x*spec->dx; YInd = spec->YLimits[0]+_y*spec->dy; break;
            case ccd::TSetOfSpectrum::BottomLeft:  XInd = spec->XLimits[0]+_x*spec->dx; YInd = spec->YLimits[0]+_y*spec->dy; break;
            }
        } else {
            switch (spec->StartPoint) {
            case ccd::TSetOfSpectrum::TopLeft:    XInd = spec->YLimits[1]-_y*spec->dy; YInd = spec->XLimits[0]+_x*spec->dx;break;
            case ccd::TSetOfSpectrum::TopRight:   XInd = spec->YLimits[1]-_y*spec->dy; YInd = spec->XLimits[1]-_x*spec->dx;break;
            case ccd::TSetOfSpectrum::BottomRight:XInd = spec->YLimits[0]+_y*spec->dy; YInd = spec->XLimits[1]-_x*spec->dx;break;
            case ccd::TSetOfSpectrum::BottomLeft: XInd = spec->YLimits[0]+_y*spec->dy; YInd = spec->XLimits[0]+_x*spec->dx;break;
            }
        }

        attributes.insert("CCD X screen pos, m", XInd);
        attributes.insert("CCD Y screen pos, m", YInd);

        if (CurrentDataList->ClickedItem->attributes.contains("XMin"))
            attributes.insert("CCD X global pos, m", XInd + CurrentDataList->ClickedItem->attributes.value("XMin").toDouble());

        if (CurrentDataList->ClickedItem->attributes.contains("YMin"))
            attributes.insert("CCD Y global pos, m", YInd + CurrentDataList->ClickedItem->attributes.value("YMin").toDouble());

/*
        attributes.insert("Marker A x", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerA->xValue()*1e6);
        attributes.insert("Marker A y", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerA->yValue()*1e6);

        attributes.insert("Marker B x", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerB->xValue()*1e6);
        attributes.insert("Marker B y", Plot->ThreeDPlot->ThreeDPlots.at(0)->MarkerB->yValue()*1e6);
*/

        attributes.insert("Laser Wavelength, nm", CurrentDataList->ClickedItem->attributes.value("Laser Wavelength, nm"));
        attributes.insert("Central Wavelength, nm", CurrentDataList->ClickedItem->attributes.value("Central Wavelength, nm"));
        attributes.insert("Dispersion", CurrentDataList->ClickedItem->attributes.value("Dispersion"));
        attributes.insert("pixelsize", CurrentDataList->ClickedItem->attributes.value("pixelsize"));
        attributes.insert("Nx", CurrentDataList->ClickedItem->attributes.value("Nx"));
        attributes.insert("Central Pixel", CurrentDataList->ClickedItem->attributes.value("Central Pixel").toDouble() - spec->MinForRecord);
    }

    AddDataToContainer(src, attributes);
}
void TContainer::AddDataToContainer_slot(plot::T3dDataOwner src, plot::TMetaStorage attributes)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QString date = getDateHash();

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
        attributes = CurrentDataList->ClickedItem->attributes;

        attributes.insert("XMin", 0);
        attributes.insert("YMin", 0);
        attributes.insert("XMax", src.at(0)->XLimits[1] - src.at(0)->XLimits[0]);
        attributes.insert("YMax", src.at(0)->YLimits[1] - src.at(0)->YLimits[0]);

        QString name = Plot->ThreeDPlot->ThreeDPlots.at(0)->Title;
        attributes.insert("Name", name + tr(" - Crop"));

    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        attributes.insert("Name", Plot->ThreeDPlot->ThreeDPlots.at(0)->Title);

        attributes.insert("Date", date);
        attributes.insert("Source", "Data");
    }

    QString uniqid = date + "Data";
    quint16 id = qChecksum(uniqid.toStdString().data(), uniqid.length());
    attributes.insert("Id", QString::number(id, 16).toUpper());
    attributes.insert("GroupId", CurrentDataList->ClickedItem->attributes.value("GroupId"));
    attributes.insert("Parent", CurrentDataList->ClickedItem->attributes.value("Id"));

    AddDataToContainer(src, attributes);
}


QString TContainer::getDateHash()
{
    static QDateTime stamp(QDate(1, 1, 1), QTime(1, 1));

    if (stamp.msecsTo(QDateTime::currentDateTime()) >= 1000)
        stamp = QDateTime::currentDateTime();

    return stamp.toString("yyyy-MM-dd hh:mm:ss");
}
void TContainer::addDateIdsToAttributes(plot::TMetaStorage &attributes)
{ //requires unique "Source" for unique "Id"
    if (attributes.contains("Date"))
        return;

    QString date = getDateHash();
    attributes.insert("Date", date);

    quint16 gid = qChecksum(date.toStdString().data(), date.length());
    attributes.insert("GroupId", QString::number(gid, 16).toUpper());

    QString uniqid = date + attributes.value("Source").toString();
    quint16 id = qChecksum(uniqid.toStdString().data(), uniqid.length());
    attributes.insert("Id", QString::number(id, 16).toUpper());
}
void TContainer::AddDataToContainer(ccd::TSetOfSpectrum *src, plot::TMetaStorage attributes)
{
    if (!src->Array.size()) {
        delete src;
        return;
    }

    if (!copypaste) {
        CurrentDataList = (TListWidget *)CBDataLists->widget(0);
        CBDataLists->setCurrentIndex(0);
    }

    addDateIdsToAttributes(attributes);

    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, src);
    if (!copypaste)
        UpdateHierarchy();

    SetDownButtonsEnabled(true);
}
void TContainer::AddDataToContainer(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    if (!src.size())
        return;

    if (!copypaste) {
        CBDataLists->setCurrentIndex(0);
        TabChanged(0);
    }

    addDateIdsToAttributes(attributes);

    plot::TCurvesDataOwner *data = new plot::TCurvesDataOwner;
    *data = src;
    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, data);
    if (!copypaste) {
        UpdateHierarchy();

        CurrentDataList->ClickedItem = CurrentDataList->DataList.last();
        UpdatePreview();
    }

    SetDownButtonsEnabled(true);
}
void TContainer::AddDataToContainer(plot::T3dDataOwner src, plot::TMetaStorage attributes)
{
    if (!copypaste) {
        CurrentDataList = (TListWidget *)CBDataLists->widget(0);
        CBDataLists->setCurrentIndex(0);
    }

    addDateIdsToAttributes(attributes);

    plot::T3dDataOwner *data = new plot::T3dDataOwner;
    *data = src;
    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, data, true);
    if (!copypaste) {
        UpdateHierarchy();

        CurrentDataList->ClickedItem = CurrentDataList->DataList.last();
        UpdatePreview();
    }

    SetDownButtonsEnabled(true);
}
void TContainer::AddDataToContainer(QImage src, plot::TMetaStorage attributes)
{
    if (!copypaste) {
        CurrentDataList = (TListWidget *)CBDataLists->widget(0);
        CBDataLists->setCurrentIndex(0);
    }

    addDateIdsToAttributes(attributes);

    CurrentDataList->AddBlanks(1);
    CurrentDataList->Add(attributes, src, true);
    if (!copypaste) {
        UpdateHierarchy();

        CurrentDataList->ClickedItem = CurrentDataList->DataList.last();
        //UpdatePreview();
    }

    SetDownButtonsEnabled(true);
}


void TContainer::Send2dPlotData()
{
    send2dPlot(PlotID::Data, Plot->TwoDPlot);
}
void TContainer::Send3dPlotData()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    send3dPlot(PlotID::Data, Plot->ThreeDPlot, 0, CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d);
}

void TContainer::SendContainerData()
{
    QStringList lst;
    QVariantList previews;
    foreach (QListWidgetItem *wit, CurrentDataList->findItems("*", Qt::MatchWildcard)) {
        TListItem *it = CurrentDataList->FindItem(wit);
        if (!it) continue;
        lst.append(((it->InfoType == TAbstractInfo::_2d) ? "2D" : (it->InfoType == TAbstractInfo::_3d) ? "3D" : "4D") +
                   QString(": ") + (it->Item->text()));
        previews.append(it->Item->icon().pixmap(100, 100).toImage());
    }
    MobileServer::sendData(DataID::DataDataList, lst);
    MobileServer::sendData(DataID::DataPreviewList, previews);
}

