#pragma once

#include <plot.h>
#include <QWidget>
class QTreeWidget;
class QTreeWidgetItem;
class QProgressBar;

class GwyddionImport;
class TQTableWidget;
class TListWidget;
class TFileProccessing;
class TAbstractInfo;
typedef QVector<TAbstractInfo> TAbstractInfos;
class T3dViewer;
class T3dEnchancement;
namespace ccd { class TSetOfSpectrum; }

class TContainer: public QWidget
{
    Q_OBJECT

signals:
    void restoreAttributes(plot::TMetaStorage);

public slots:
    void OpenFile(QString name);

    void Send2dPlotData();
    void Send3dPlotData();
    void SendContainerData();

    void thread_close();
    void on_BOpen_clicked();
    void on_BSave_clicked();
    void on_BSaveAs_clicked();
    void on_BSaveAll_clicked();
    void on_BRemove_clicked();
    void on_BRemoveAll_clicked();

    void on_BCopy_clicked();
    void on_BCopyGroup_clicked();
    void on_BPaste_clicked();
    void on_BMerge_clicked();

    void on_BShowAttributes_toggled(bool checked);
    void updateAttributes();
    void on_BAddAttrib_clicked();
    void SaveAttrib();
    void SaveFilters();
    void SavePreview();

    void on_BToDevice_clicked();
    void on_BToAscii_clicked();
    void on_BDetach_clicked();
    void on_BToGwyddion_clicked();
    void on_BToGwyddionAll_clicked();
    void gwyddion_error();

    void on3dUpdate();
    void on_B3d_toggled(bool checked);
    QWidget *create3dPlot(QWidget *parent = 0);
    static QImage get3dTexture(plot::T3dDataOwner Info, double zmin, double zmax, QwtLinearColorMap *colorMap);

    void TabChanged(int index);
    void TabClose(int index);
    void addTab(QString name = QString());
    void AddBlanks(int num);

    void on_FileProccessing_started();
    void on_FileProccessing_finished();
    void on_FileProccessing_UpdatePreview();
    void on_FileProccessing_FileProccessed(double TotalNumber, double Current);
    void on_FileProccessing_AllDataLoaded(const TAbstractInfos *DataList);
    void AutoSave();

    static QString getDateHash();
    static void addDateIdsToAttributes(plot::TMetaStorage &attributes);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(ccd::TSetOfSpectrum *, plot::TMetaStorage);
    void AddDataToContainer(QImage, plot::TMetaStorage);

    //for in data section&copy
    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);

    void UpdateHierarchy();
    void UpdatePreview();

    void attribEditorResize();
    void attribConfigEdited(QTreeWidgetItem *item, int column);


    void onItemClicked(const plot::TCurvesDataOwner *);
    void onItemClicked(const plot::T3dDataOwner *);
    void onItemClicked(const ccd::TSetOfSpectrum *);
    void onItemClicked(const QImage);
    void show3d(const plot::T3dData data, bool is4d = false);

    void sl_Func_range_select(bool);
    void onAG_ConvFunc_trigered();
    void onAG_Scale_select_trigered();

    void DelayedPreviewCreation();
    void CrossEvent(QPointF pt);
    bool SelectEvent(QRectF SelectedRect, int State, int Origin);
    void Error(QString,QString);

public:
    TContainer(plot::TPlot *PlotFromDataPlot, QWidget *parent = 0);
    ~TContainer();
    void GUICreate();
    bool eventFilter(QObject *obj, QEvent *event);
    void dropEvent(QDropEvent *event);
    void dragEnterEvent(QDragEnterEvent *);


    TFileProccessing *FileProccessing;
    TAbstractInfos *GenerateDataForSave();
    void Load(const QString &FileName);
    void Save(QString &FileName);
static QString getDataDir();
    void SetDownButtonsEnabled(bool State);
    void addAttribute(QString groupname, QString name, QString val);

    plot::TPlot *Plot;
    plot::ImageItem *imageitem;
    T3dViewer *pl3;
    T3dEnchancement *t3dench;
    GwyddionImport *gwi;

    TListWidget *CurrentDataList;

    QMap<QString, QTreeWidgetItem *> attribItemsRoots;
    bool isAutosave; int autosave_idx;
    QString FileDir;
    QProgressBar *PBBar;
    QTreeWidget *table;
    QTabWidget *CBDataLists;
    QGroupBox* GTable;

    QVector<void *> info_copypaste;
    QVector<int> copypaste_type;
    QVector<plot::TMetaStorage> copypaste_attrib;
    bool copypaste;
    bool preview_update;

    bool ActiveHBL;
    double func_min, func_max;
    int lastXind, lastYind;

    QPushButton *BPaste;
    QPushButton *BRemove;
    QPushButton *BRemoveAll;

    QPushButton *BShowAttributes;
    QPushButton *BAddAttrib;

    QPushButton *BDetach;
    QPushButton *BToAscii;
    QPushButton *BToDevice;
    QPushButton *B3d;
    bool new3d;

    QToolButton *Convol_func;
    QActionGroup *AG_Convol_func;

    TwoStateButton *Func_interval;

    QToolButton *Scale_select;
    QActionGroup *AG_Scale_select;

    QTimer *AutosaveTimer;
    int AutoSaveTime;
    void SetAutoSaveTime(int);
};

class T3dEnchancement : public QObject
{
    Q_OBJECT
public slots:
    void updateTexture();
    void queueUpdate();

public:
    T3dViewer *pl3;
    plot::T3dPlot *sourceplot;

    T3dEnchancement(QObject *parent = 0) : QObject(parent) { pl3 = 0; sourceplot = 0; upd = 0; }
    QTimer *upd;

    //D&D
    bool eventFilter(QObject *, QEvent *);
    //do installEventFilter and setAcceptDrops on 3dviewer
};
