#include "container.h"
#include "datalist.hpp"
#include "gui/t3dcontrol.hpp"
#include "gui/viewer3d.hpp"
#include "gui/status.hpp"
#include "modules.hpp"
#include "dataplot.h"
#include "tools.hpp"
#include "ccd/ccd.h"
#include "ccd/ccdmatrix.h"
#include "gwynetio.hpp"
#include "spcfile.h"
#include "ascifile.h"
#include <QFileDialog>
#include <QVBoxLayout>
#include <QSplitter>
#include <QTimer>
#include <QApplication>


void TContainer::on_BToGwyddion_clicked()
{
    if (!CurrentDataList)
        return;

    GwyddionExport *exp = new GwyddionExport();
    connect(exp, SIGNAL(no_gwyddion()), this, SLOT(gwyddion_error()));

    if (CurrentDataList->ClickedItem)
        exp->AddData(CurrentDataList->ClickedItem->Info, CurrentDataList->ClickedItem->InfoType, CurrentDataList->ClickedItem->attributes);

    exp->start();
}
void TContainer::gwyddion_error()
{
    new Notification(tr("Can't start Gwyddion."), Notification::Bad, 10);
    doBeep();
}

void TContainer::on_BToGwyddionAll_clicked()
{
    if (!CurrentDataList)
        return;

    GwyddionExport *exp = new GwyddionExport();
    connect(exp, SIGNAL(no_gwyddion()), this, SLOT(gwyddion_error()));

    for (int i = 0; i < CurrentDataList->DataList.size(); i++)
        exp->AddData(CurrentDataList->DataList.at(i)->Info, CurrentDataList->DataList.at(i)->InfoType, CurrentDataList->DataList.at(i)->attributes);

    exp->start();
}
void TContainer::on_BToDevice_clicked()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    emit restoreAttributes(CurrentDataList->ClickedItem->attributes);
}

void TContainer::on_BToAscii_clicked()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    QString exr_str, FileName;
    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d)
        FileName = QFileDialog::getSaveFileName(this, tr("Export"), FileName, "ASCII (*.txt);;SPC (*.spc)",
                                                &exr_str, QFileDialog::DontUseNativeDialog);
    else
        FileName = QFileDialog::getSaveFileName(this, tr("Export"), FileName, "ASCII (*.txt)",
                                                &exr_str, QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    if (exr_str == "ASCII (*.txt)" && FileName.endsWith(".txt", Qt::CaseInsensitive)) FileName.chop(4);
    if (exr_str == "SPC (*.spc)" && FileName.endsWith(".spc", Qt::CaseInsensitive)) FileName.chop(4);

    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {
        int ActiveCurve = Plot->TwoDPlot->Pure2dPlot->ActiveCurve;
        QString suffix;
        plot::TCurvesDataOwner Elements_export = plot::getLock(Plot->TwoDPlot->Pure2dPlot->SourceData);

        for (int counter = 0; counter < Elements_export.size(); counter++) {
            if (Elements_export.at(counter) && Elements_export.at(counter)->size()) {
                if (ActiveCurve == -1 || ActiveCurve == counter) {
                    if (ActiveCurve == -1 && Elements_export.size() != 1)
                        suffix = QString::number(counter+1);

                    if (exr_str == "ASCII (*.txt)") {
                        ascifile ascif (FileName+suffix+".txt");
                        ascif.set_instrument("Data ASCII export");
                        ascif.set_data_name(CurrentDataList->ClickedItem->attributes.value("Name", tr("2d curve")).toString());
                        Plot->TwoDPlot->Pure2dPlot->XAxisUnit.StrMultiplier.clear();
                        Plot->TwoDPlot->Pure2dPlot->XAxisUnit.UpdateLegend();
                        Plot->TwoDPlot->Pure2dPlot->YAxisUnit.StrMultiplier.clear();
                        Plot->TwoDPlot->Pure2dPlot->YAxisUnit.UpdateLegend();
                        ascif.set_x_type(Plot->TwoDPlot->Pure2dPlot->XAxisUnit.Legend);
                        ascif.set_y_type(Plot->TwoDPlot->Pure2dPlot->YAxisUnit.Legend);
                        ascif.set_xpts(Elements_export.at(counter)->size());

                        QVector<double> xseq = Elements_export.at(counter)->XSeq;
                        if (Elements_export.at(counter)->isCompressedXCurve()) {
                            xseq.resize(Elements_export.at(counter)->size());
                            for (int i = 0; i < xseq.size(); i++)
                                xseq[i] = Elements_export.at(counter)->getXat(i);
                        }
                        ascif.set_x_data(xseq.constData());
                        ascif.set_y_data(Elements_export.at(counter)->YSeq.constData());
                        ascif.write_file();
                    } else {
                        QVector<float> xseq(Elements_export.at(counter)->size());
                        QVector<float> yseq(Elements_export.at(counter)->size());

                        spcfile spc(FileName+suffix+".spc");
                        spc.set_npts(Elements_export.at(counter)->size());
                        spc.set_y_type(YCOUNT);
                        spc.set_exper(SPCRMN);

                        int Scale = FindCheckedItemIn(AG_Scale_select);
                        if (!Scale_select->isVisibleTo(this)) {
                            if (Plot->TwoDPlot->Pure2dPlot->XAxisUnit.Description == QObject::tr("pixels"))
                                Scale = 0;
                            if (Plot->TwoDPlot->Pure2dPlot->XAxisUnit.Description == QObject::tr("WL"))
                                Scale = 1;
                            if (Plot->TwoDPlot->Pure2dPlot->XAxisUnit.Description == QObject::tr("Raman Shift, 1/cm"))
                                Scale = 2;
                            if (Scale == -1) {
                                qDebug() << "spc export not supported x axis scale";
                                continue;
                            }
                        }

                        switch (Scale) {
                        case 0:
                            spc.set_x_type(XPOINT);
                            break;
                        case 1:
                            spc.set_x_type(XMETERS);
                            break;
                        case 2:
                            spc.set_x_type(XRAMANS);
                            break;
                        }
                        for (int i = 0; i < Elements_export.at(counter)->size(); i++) {
                            xseq[i] = Elements_export.at(counter)->getXat(i);
                            yseq[i] = Elements_export.at(counter)->YSeq.at(i);
                        }

                        spc.set_x_data(xseq.constData());
                        spc.set_y_data(yseq.constData());
                        spc.write_file();
                    }
                }
            }
        }
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_Image) {
        CurrentDataList->ClickedItem->attributes.value("Preview").value<QImage>().save(FileName+".png");
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_3d) {
        plot::T3dDataOwner *info = (plot::T3dDataOwner *)CurrentDataList->ClickedItem->Info;

        ascifile ascif (FileName+".txt");
        ascif.set_instrument("Data ASCII export");
        ascif.set_data_name(CurrentDataList->ClickedItem->attributes.value("Name", tr("Scan")).toString());
        ascif.set_x_type(CurrentDataList->ClickedItem->attributes.value("XYUnit", tr("m")).toString());
        ascif.set_y_type(CurrentDataList->ClickedItem->attributes.value("ZUnit", tr("Counts")).toString());
        ascif.set_xsz(info->at(0)->XLimits);
        ascif.set_ysz(info->at(0)->YLimits);
        ascif.set_xpts(info->at(0)->NXElems);
        ascif.set_ypts(info->at(0)->NYElems);

        ascif.set_array(info->at(0)->Array);

        ascif.write_file_3d();
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        ccd::TSetOfSpectrum *info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

        QDir::current().mkpath(FileName + "/");

        {
            ascifile ascif (FileName+"/3d.txt");
            ascif.set_instrument("Data ASCII export");
            ascif.set_data_name(CurrentDataList->ClickedItem->attributes.value("Name", tr("Scan")).toString());
            ascif.set_x_type(CurrentDataList->ClickedItem->attributes.value("XYUnit", tr("m")).toString());
            ascif.set_y_type(CurrentDataList->ClickedItem->attributes.value("ZUnit", tr("Counts")).toString());
            ascif.set_xsz(info->XLimits);
            ascif.set_ysz(info->YLimits);
            ascif.set_xpts(info->NXElems);
            ascif.set_ypts(info->NYElems);

            ascif.set_array(info->Results.at(0)->Array);

            ascif.write_file_3d();
        }

        if (info->CalibInfo.size() >= 6) {
            QFile dataFile(FileName+"/spectra_calibrations.txt");
            dataFile.open(QIODevice::WriteOnly | QIODevice::Text);
            QTextStream outData(&dataFile);

            outData << "# Nano Scan Technologies Ltd." << endl;
            outData << "# ASCII Data" << endl << endl;
            outData << "# 4d Calibration Data" << endl << endl;
            outData << "# Laser Wavelength, nm = " << info->CalibInfo[0] << endl;
            outData << "# Central Wavelength, nm = " << info->CalibInfo[1] << endl;
            outData << "# Dispersion, nm/mm = " << info->CalibInfo[2] << endl;
            outData << "# pixelsize, mm = " << info->CalibInfo[3] << endl;
            outData << "# Nx = " << info->CalibInfo[4] << endl;
            outData << "# Central Pixel = " << info->CalibInfo[5] << endl;

            dataFile.close();
        }

        QDir::current().mkpath(FileName + "/data/");
        for (int i = 0; i < (int)info->NYElems && i < info->Array.size(); i++) {
            for (int j = 0; j < (int)info->NXElems && j < info->Array[i].size(); j++) {
                QString suffix = "/data/spectra_at_" + QString::number(i) + "_" + QString::number(j);
                ccd::TSpectrum &sp = info->Array[i][j];


                ascifile ascif (FileName+suffix+".txt");
                ascif.set_instrument("Data ASCII export");
                ascif.set_data_name(suffix);
                Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->XAxisUnit.StrMultiplier.clear();
                Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->XAxisUnit.UpdateLegend();
                ascif.set_x_type(Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->XAxisUnit.Legend);
                ascif.set_y_type(tr("Counts"));
                ascif.set_xpts(sp.size());

                QVector<double> xseq(sp.size());
                int Scale = FindCheckedItemIn(AG_Scale_select);
                for (int i = 0; i < sp.size(); i++) {
                    double val = i + info->MinForRecord;
                    if (info->CalibInfo.size() == 6) {
                        if (Scale == 1) {
                            val = 1e-9*wavelength(info->CalibInfo[1],
                                    info->CalibInfo[2], info->CalibInfo[3],
                                    val, (int)info->CalibInfo[5]);
                        } else if (Scale == 2) {
                            val = raman_shift(info->CalibInfo[0], wavelength(info->CalibInfo[1],
                                        info->CalibInfo[2], info->CalibInfo[3],
                                        val, (int)info->CalibInfo[5]));
                        }
                    }
                    xseq[i] = val;
                }

                ascif.set_x_data(&xseq[0]);
                ascif.set_y_data(&sp[0]);
                ascif.write_file();
            }
        }
    }
}



void TContainer::CrossEvent(QPointF pt)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem || CurrentDataList->ClickedItem->InfoType != TAbstractInfo::_4d)
        return;

    ccd::TSetOfSpectrum *Info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

    int XInd, YInd;
    Info->GetIntegerIndexes(pt.x(), pt.y(), XInd, YInd);

    T4dDataHelper::ShowSpectr(Plot->ThreeDPlot->TwoDPlot, Info, lastXind = XInd, lastYind = YInd, FindCheckedItemIn(AG_Scale_select));

    Plot->ThreeDPlot->PlotControl->BMarkers->setChecked(false);
    Plot->ThreeDPlot->Set2dPlotVisible(true);
}

bool TContainer::SelectEvent(QRectF SelectedRect, int State, int Origin)
{
    if (!ActiveHBL)
        return false;

    if (!CurrentDataList || !CurrentDataList->ClickedItem || CurrentDataList->ClickedItem->InfoType != TAbstractInfo::_4d)
        return false;

    ccd::TSetOfSpectrum *Info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

    double Min = SelectedRect.left(), Max = SelectedRect.right();

    if (State == plot::TSelectEvent::St_SelectionStarted) {
        plot::TCurvesDataOwner Elements_temp = plot::getLock(Plot->TwoDPlot->Pure2dPlot->SourceData);
        if (Elements_temp.size() > 0 && Elements_temp.at(0) && Elements_temp.at(0)->XSeq.size() != 0) {
            Min = Elements_temp.at(0)->XSeq.first();
            Max = Elements_temp.at(0)->XSeq.last();

            Min = (Min+0.25*(Max-Min));
            Max = (Min+0.75*(Max-Min));
        }
        Min = (func_min ? T4dDataHelper::ReCalibrate(func_min, Info->CalibInfo, FindCheckedItemIn(AG_Scale_select)) : Min);
        Max = (func_max ? T4dDataHelper::ReCalibrate(func_max, Info->CalibInfo, FindCheckedItemIn(AG_Scale_select)) : Max);
    }

    if (Origin == plot::TSelectEvent::SRO_Mouse || Origin == plot::TSelectEvent::SRO_Unknown)
    {
        T4dDataHelper::SetSelectGeometry(Plot->ThreeDPlot->TwoDPlot, Min, Max);
        T4dDataHelper::ReCalcRamanOrWavelengthToPixels(Info->CalibInfo, Min, Max, FindCheckedItemIn(AG_Scale_select));
        func_min = Min;
        func_max = Max;

        Info->SetMinMaxForFuncApply(Min, Max);
        Info->CalcResults();

        Plot->ThreeDPlot->ThreeDPlots[0]->Refresh();
    }

    if (State == plot::TSelectEvent::St_SelectionApplied ||
            State == plot::TSelectEvent::St_SelectionCancled) //TODO restore prev conv.range on cancel
        Func_interval->change_state_man_with_sig(false);

    return true;
}

void TContainer::sl_Func_range_select(bool st)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    if (st) {
        Plot->ThreeDPlot->PlotControl->BCrossection->setChecked(false);
        Plot->ThreeDPlot->PlotControl->BMarkers->setChecked(false);
        Plot->ThreeDPlot->PlotControl->BMarkers->setEnabled(false);

        if (Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->Title == QObject::tr("section")) {
            ccd::TSetOfSpectrum *Info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

            T4dDataHelper::ShowSpectr(Plot->ThreeDPlot->TwoDPlot, Info, lastXind, lastYind, FindCheckedItemIn(AG_Scale_select));
            Plot->ThreeDPlot->Set2dPlotVisible(true);
            Plot->Refresh();
        }
    } else
        Plot->ThreeDPlot->PlotControl->BMarkers->setEnabled(true);

    ActiveHBL = st;
    Plot->ThreeDPlot->TwoDPlot->SetSelectionModeEnabled(st);
    Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->SetZoomBySelection(!st);
}
void TContainer::onAG_Scale_select_trigered()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    int Scale = FindCheckedItemIn(AG_Scale_select);
    if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        ccd::TSetOfSpectrum *Info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

        switch (Scale) {
        case 0:
            Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
            break;
        case 1:
            Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("WL"), QObject::tr("m"));
            break;
        case 2:
            Plot->ThreeDPlot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("Raman Shift, 1/cm"));
            break;
        }

        Info->XscaleUnit = FindCheckedItemIn(AG_Scale_select);

        Plot->Refresh();

        T4dDataHelper::ShowSpectr(Plot->ThreeDPlot->TwoDPlot, Info, lastXind, lastYind, Scale);
    } else if (CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d) {
        switch (Scale) {
        case 0:
            Plot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
            break;
        case 1:
            Plot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("WL"), QObject::tr("m"));
            break;
        case 2:
            Plot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("Raman Shift, 1/cm"));
            break;
        }

        for (int i = 0; i < Plot->TwoDPlot->Pure2dPlot->SourceData.size(); i++)
            set_spectra_calib(*Plot->TwoDPlot->Pure2dPlot->SourceData[i].data(), CurrentDataList->ClickedItem->attributes, Scale);

        Plot->Refresh();
    }
}

void TContainer::onAG_ConvFunc_trigered()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem || CurrentDataList->ClickedItem->InfoType != TAbstractInfo::_4d)
        return;

    ccd::TSetOfSpectrum *Info = (ccd::TSetOfSpectrum *)CurrentDataList->ClickedItem->Info;

    int idx = FindCheckedItemIn(AG_Convol_func);
    Plot->ThreeDPlot->SetZDescAndSIUnit(QString(), T4dDataHelper::getConvZUnits(idx, FindCheckedItemIn(AG_Scale_select), Info->YscaleUnit), 0);

    Info->CurrentFunc = idx;

    Info->CalcResults();
    onItemClicked(Info);
}



QWidget *TContainer::create3dPlot(QWidget *parent)
{
    QWidget *Wid3d = new QWidget(parent);
    if (parent && qobject_cast<QBoxLayout *>(parent->layout()))
        qobject_cast<QBoxLayout *>(parent->layout())->insertWidget(0, Wid3d, 1);

    QHBoxLayout *pl_l = new QHBoxLayout(Wid3d);
    pl_l->setSpacing(3);
    pl_l->setContentsMargins(0, 3, 3, 3);

    pl3 = new T3dViewer(Wid3d);
    pl3->invertY = true;
    pl3->showAxes = true;
    pl3->setMinimumWidth(400);

    T3DControl *pl3Control = new T3DControl(pl3, Wid3d);

    pl_l->addWidget(pl3, 1);
    pl_l->addWidget(pl3Control);

    return Wid3d;
}

bool T3dEnchancement::eventFilter(QObject *, QEvent *e)
{
    if (e->type() == QEvent::DragEnter)
        e->accept();
    if (e->type() == QEvent::Drop) {
        plot::T3dPlot plot;
        plot.hide();
        plot.ThreeDPlots[0]->dropEvent((QDropEvent *)e);

        plot::T3dPlot *prev = sourceplot;
        sourceplot = &plot;
        updateTexture();
        sourceplot = prev;

        e->accept();
        return true;
    }
    return false;
}

QImage TContainer::get3dTexture(plot::T3dDataOwner Info, double zmin, double zmax, QwtLinearColorMap *colorMap)
{
    if (!Info.at(0))
        return QImage();

    int nx = Info.at(0)->NXElems;
    int ny = Info.at(0)->NYElems;
    int xstride = 1, ystride = 1;
    while (nx / xstride > 1500)
        xstride++;
    while (ny / ystride > 1500)
        ystride++;
    nx /= xstride;
    ny /= ystride;

    QImage texture(nx, ny, QImage::Format_ARGB32);
    for (int i = 0; i < ny; i++) {
        QRgb *p = (QRgb *)texture.scanLine(i);
        for (int j = 0; j < nx; j++)
            *(p++) = colorMap->rgb(QwtInterval(zmin, zmax), Info.at(0)->Array[i*ystride][j*xstride]);
    }

    return texture;
}
void T3dEnchancement::queueUpdate()
{
    if (!pl3 || !sourceplot)
        return;

    if (!upd) {
        upd = new QTimer(this);
        upd->setSingleShot(true);
        upd->setInterval(50);
        connect(upd, SIGNAL(timeout()), this, SLOT(updateTexture()));
    }

    if (!upd->isActive())
        upd->start();
}
void T3dEnchancement::updateTexture()
{
    if (!pl3 || !sourceplot)
        return;

    plot::T3dDataOwner Info = plot::getLock(sourceplot->ThreeDPlots.at(0)->PaintData);
    double min = sourceplot->ThreeDPlots.at(0)->PlotsZLimits[0];
    double max = sourceplot->ThreeDPlots.at(0)->PlotsZLimits[1];
    pl3->updateTexture(TContainer::get3dTexture(Info, min, max,
                                                sourceplot->ThreeDPlots.at(0)->colormap));
    pl3->updateGL();
}

void TContainer::on3dUpdate()
{
    if (B3d->isChecked()) {
        parentWidget()->setEnabled(false);
        while (!Plot->ThreeDPlot->ThreeDPlots.at(0)->isMathReady()) {
            qApp->processEvents();
            SleepExport::msleep2(3);
        }
        parentWidget()->setEnabled(true);

        on_B3d_toggled(true);
    }
}
void TContainer::on_B3d_toggled(bool checked)
{
    foreach (QObject *obj, Plot->ThreeDPlot->PlotControl->children()) {
        QWidget *wid = qobject_cast<QWidget *>(obj);
        if (wid && wid->isVisible() &&
            wid != Plot->ThreeDPlot->PlotControl->TBMathProcessing &&
            wid != Plot->ThreeDPlot->PlotControl->TBMath2 &&
            wid != Plot->ThreeDPlot->PlotControl->TBPalettes)
            wid->setEnabled(!checked);
    }

    if (!checked) {
        B3d->setIcon(makeHugeIcon(":/container/3dimage.png"));

        if (pl3) pl3->parentWidget()->hide();
        Plot->show();
        Plot->ThreeDPlot->ResizeSliders();

        float data[4];
        if (pl3) pl3->updateData(data, 2, 2);
        if (pl3) pl3->updateTexture(QImage());
        t3dench->pl3 = 0;
        return;
    }

    if (!CurrentDataList->ClickedItem ||
            CurrentDataList->ClickedItem->InfoType == TAbstractInfo::Blank ||
            CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_2d ||
            CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_Image)
        return;

    plot::T3dDataOwner Info = plot::getLock(Plot->ThreeDPlot->ThreeDPlots.at(0)->PaintData);
    double min = Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsZLimits[0];
    double max = Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsZLimits[1];
    int nx = Info.at(0)->NXElems;
    int ny = Info.at(0)->NYElems;
    int xstride = 1, ystride = 1;
    while (nx / xstride > 1500)
        xstride++;
    while (ny / ystride > 1500)
        ystride++;
    nx /= xstride;
    ny /= ystride;

    float *data = 0;
try {
    data = new float[nx * ny];

    for (int i = 0; i < ny; i++)
        for (int j = 0; j < Info.at(0)->Array[i].size() / xstride; j++)
            data[j*ny + i] = (Info.at(0)->Array[i*ystride][j*xstride] - min) / (max - min) - 0.5;

    if (!pl3) {
        DataPlot *data = (DataPlot *)ModuleManager::instance->find1("data")->basecast();
        create3dPlot(data->Splitter->widget(1));
    }
    if (!t3dench->pl3) //do not overwrite on detach
        t3dench->pl3 = pl3;

    pl3->updateData(data, nx, ny,
                    (Info.at(0)->XLimits[1] - Info.at(0)->XLimits[0]) * xstride /
                    (Info.at(0)->YLimits[1] - Info.at(0)->YLimits[0]) / ystride);
    pl3->updateTexture(get3dTexture(Info, min, max,
                                    Plot->ThreeDPlot->ThreeDPlots.at(0)->colormap));

} catch(...) {
    qDebug() << "3d oom";
    float data[4];
    if (pl3) pl3->updateData(data, 2, 2);
    if (pl3) pl3->updateTexture(QImage());
}

    if (data)
        delete [] data;

    Plot->hide();
    Plot->ThreeDPlot->Slider->setMaximumHeight(100000);
    pl3->parentWidget()->show();
    B3d->setIcon(makeHugeIcon(":/container/3dimage_on.png"));

    if (new3d) {
        pl3->x_rot = 30;
        pl3->y_rot = 0;
        pl3->z_rot = 10;
        pl3->x_pos = pl3->y_pos = pl3->z_pos = 0;
        pl3->z_scale = 0.2;
        pl3->scale = 1;
    }
    new3d = false;

    /*
    double scale = 1/Plot->ThreeDPlot->ThreeDPlots.at(0)->XYAxisUnit.Multiplier;
    double zscale = 1/Plot->ThreeDPlot->ThreeDPlots.at(0)->ZAxisUnit.Multiplier;
    pl3->coordinates()->axes[2].setLimits(min * zscale, max * zscale);

    pl3->coordinates()->axes[i].setLabelString(Plot->ThreeDPlot->ThreeDPlots.at(0)->XYAxisUnit.Legend);
    pl3->coordinates()->axes[2].setLabelString(Plot->ThreeDPlot->ThreeDPlots.at(0)->ZAxisUnit.Legend);

    QFont font = Plot->ThreeDPlot->ThreeDPlots.at(0)->QPlot->axisFont(QwtPlot::xBottom);
    pl3->coordinates()->setLabelFont(font);
    */

    QFont font = qApp->font();
    font.setPointSize(Plot->ThreeDPlot->ThreeDPlots.at(0)->FontSize);
    pl3->setTitle(Plot->ThreeDPlot->ThreeDPlots.at(0)->Title, font);

    plot::TValueInfoCreator ValueInfo;

    ValueInfo(Plot->ThreeDPlot->ThreeDPlots.at(0)->AutoLegendEnabled,
              Plot->ThreeDPlot->ThreeDPlots.at(0)->XYAxisUnit,
              Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsXLimits[1] -
            Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsXLimits[0]);
    pl3->setAxisTitle(0, ValueInfo.Value + ValueInfo.Unit, font);

    ValueInfo(Plot->ThreeDPlot->ThreeDPlots.at(0)->AutoLegendEnabled,
              Plot->ThreeDPlot->ThreeDPlots.at(0)->XYAxisUnit,
              Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsYLimits[1] -
            Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsYLimits[0]);
    pl3->setAxisTitle(1, ValueInfo.Value + ValueInfo.Unit, font);

    ValueInfo(Plot->ThreeDPlot->ThreeDPlots.at(0)->AutoLegendEnabled,
              Plot->ThreeDPlot->ThreeDPlots.at(0)->ZAxisUnit,
              Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsZLimits[1] -
            Plot->ThreeDPlot->ThreeDPlots.at(0)->PlotsZLimits[0]);
    pl3->setAxisTitle(2, ValueInfo.Value + ValueInfo.Unit, font);

    pl3->resizeGL(pl3->width(), pl3->height());
    pl3->updateGL();
}



void TContainer::DelayedPreviewCreation()
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem || !CurrentDataList->ClickedItem->UpdatePreview)
        return;

    CurrentDataList->ClickedItem->UpdatePreview = false;

    SavePreview();
    CurrentDataList->setFlow(QListView::LeftToRight);
}

void TContainer::onItemClicked(const plot::TCurvesDataOwner *Info)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    Plot->ThreeDPlot->Slider->hide();
    Plot->SwitchPlots(plot::TPlot::_2d);
    Plot->TwoDPlot->Pure2dPlot->blockRefresh = true;

    QStringList clrs = CurrentDataList->ClickedItem->attributes.value("Curve colour").toString().split(",");
    for (int i = 0; i < Info->size(); i++) {
        QString clr = "#000000";
        if (clrs.size() > i)
            clr = clrs.at(i);
        Info->at(i)->Pen.setColor(QColor(clr));
    }

    Plot->TwoDPlot->Pure2dPlot->Title = CurrentDataList->ClickedItem->attributes.value("Name", tr("2d curve")).toString();
    Plot->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(CurrentDataList->ClickedItem->attributes.value("XLabel").toString(),
                                                           CurrentDataList->ClickedItem->attributes.value("XUnit").toString());
    Plot->TwoDPlot->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CurrentDataList->ClickedItem->attributes.value("YLabel").toString(),
                                                           CurrentDataList->ClickedItem->attributes.value("YUnit").toString());

    Plot->TwoDPlot->SetAutoScalingEnabled(true);
    Plot->TwoDPlot->Pure2dPlot->AutoLegendEnabled = true;
    Plot->TwoDPlot->Pure2dPlot->UpdateAxisLegends();

    Plot->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(*Info);
    //Plot->ThreeDPlot->ThreeDPlots[0]->MathData.clear(); //ensure no math running, no event fuck-up

    Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[CurrentDataList->ClickedItem->attributes.value("Fit mode", 0).toInt()]->trigger();
    Plot->TwoDPlot->PlotControl->AGCurveThickness->actions()[CurrentDataList->ClickedItem->attributes.value("Curve thickness", 0).toInt()]->trigger();
    Plot->TwoDPlot->PlotControl->CBChooseActiveCurve->setCurrentIndex(Plot->TwoDPlot->PlotControl->CBChooseActiveCurve->count()-1);

    //parentWidget()->setEnabled(false);
    while (!Plot->TwoDPlot->Pure2dPlot->isMathReady()) { //lots of dependencies\breakage with events\preview_update
        //qApp->processEvents();
        SleepExport::msleep2(3);
    }
    //if (!preview_update)
        //parentWidget()->setEnabled(true);

    Plot->TwoDPlot->Pure2dPlot->blockRefresh = false;
    Plot->Refresh();

    if (CurrentDataList->ClickedItem->UpdatePreview)
    {
        CurrentDataList->ClickedItem->UpdatePreview = false;
        SavePreview();
    }

    if (B3d->isChecked())
        B3d->toggle();

    B3d->hide();
    Func_interval->hide();
    Convol_func->hide();

    //x scale recalc for spectrum, if available
    bool spectra_calib =
            CurrentDataList->ClickedItem->attributes.contains("Laser Wavelength, nm") &&
            CurrentDataList->ClickedItem->attributes.contains("Central Wavelength, nm") &&
            CurrentDataList->ClickedItem->attributes.contains("Dispersion") &&
            CurrentDataList->ClickedItem->attributes.contains("pixelsize") &&
            CurrentDataList->ClickedItem->attributes.contains("Nx") &&
            CurrentDataList->ClickedItem->attributes.contains("Central Pixel");
    if (spectra_calib) {
        int scale = 0;
        if (CurrentDataList->ClickedItem->attributes.value("XLabel").toString() == QObject::tr("pixels"))
            scale = 0;
        if (CurrentDataList->ClickedItem->attributes.value("XLabel").toString() == QObject::tr("WL, nm"))
            scale = 1;
        if (CurrentDataList->ClickedItem->attributes.value("XLabel").toString() == QObject::tr("Raman Shift, 1/cm"))
            scale = 2;

        AG_Scale_select->actions()[scale]->setChecked(true);
        //onAG_Scale_select_trigered();
    }
    Scale_select->setVisible(spectra_calib);

    if (Plot->TwoDPlot->PlotControl->BMarkers->isChecked())
        Plot->TwoDPlot->PlotControl->BMarkers->click();

    updateAttributes();
}
void TContainer::onItemClicked(const QImage img)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    if (B3d->isChecked())
        B3d->toggle();

    //fake info - background
    plot::T3dDataOwner *Info = new plot::T3dDataOwner;
    plot::ResizeOwner(*Info, 1);
    Info->at(0)->setSize(1, 1);
    Info->at(0)->UpdateNxNy();
    Info->at(0)->resetMathLims();

    Info->at(0)->XLimits[0] = CurrentDataList->ClickedItem->attributes.value("XMin", 0).toDouble();
    Info->at(0)->XLimits[1] = CurrentDataList->ClickedItem->attributes.value("XMax", 1).toDouble();
    Info->at(0)->YLimits[0] = CurrentDataList->ClickedItem->attributes.value("YMin", 0).toDouble();
    Info->at(0)->YLimits[1] = CurrentDataList->ClickedItem->attributes.value("YMax", 1).toDouble();
    Info->at(0)->UpdateDxDy();

    show3d(plot::getUnLock(*Info));
    B3d->hide();
    Plot->ThreeDPlot->Slider->hide();

    imageitem->image = img;
    imageitem->targetRect = QRectF(Info->at(0)->XLimits[0], Info->at(0)->YLimits[0],
            Info->at(0)->XLimits[1] - Info->at(0)->XLimits[0],
            Info->at(0)->YLimits[1] - Info->at(0)->YLimits[0]);
    imageitem->show();
}
void TContainer::onItemClicked(const plot::T3dDataOwner *Info)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    Info->at(0)->XLimits[0] = CurrentDataList->ClickedItem->attributes.value("XMin", 0).toDouble();
    Info->at(0)->XLimits[1] = CurrentDataList->ClickedItem->attributes.value("XMax", 1).toDouble();
    Info->at(0)->YLimits[0] = CurrentDataList->ClickedItem->attributes.value("YMin", 0).toDouble();
    Info->at(0)->YLimits[1] = CurrentDataList->ClickedItem->attributes.value("YMax", 1).toDouble();
    Info->at(0)->UpdateDxDy();

    show3d(plot::getUnLock(*Info));
}
void TContainer::onItemClicked(const ccd::TSetOfSpectrum *Info)
{
    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    func_min = Info->MinForFuncApply;
    func_max = Info->MaxForFuncApply;
    ((ccd::TSetOfSpectrum *)Info)->XscaleUnit = FindCheckedItemIn(AG_Scale_select);

    show3d(plot::getUnLock(Info->Results), true);
    Plot->ThreeDPlot->SetZDescAndSIUnit(QString(), T4dDataHelper::getConvZUnits(FindCheckedItemIn(AG_Convol_func), FindCheckedItemIn(AG_Scale_select), Info->YscaleUnit), 0);
    Plot->Refresh();

    T4dDataHelper::ShowSpectr(Plot->ThreeDPlot->TwoDPlot, (ccd::TSetOfSpectrum *)Info, lastXind = 0, lastYind = 0, FindCheckedItemIn(AG_Scale_select));
}
void TContainer::show3d(const plot::T3dData data, bool is4d)
{
    //deactivate image display
    imageitem->hide();
    imageitem->image = QImage();

    if (!CurrentDataList || !CurrentDataList->ClickedItem)
        return;

    new3d = true;
    t3dench->pl3 = 0;
    if (!preview_update)
        CurrentDataList->ClickedItem->UpdatePreview = true;

    Plot->SwitchPlots(plot::TPlot::_3d);
    Plot->ThreeDPlot->Slider->show();
    Plot->ThreeDPlot->ThreeDPlots[0]->blockRefresh = true;

    Plot->ThreeDPlot->Set2dPlotVisible(is4d);
    Plot->ThreeDPlot->PlotControl->BSave->setVisible(is4d);
    Plot->ThreeDPlot->PlotControl->BCross->setVisible(is4d);
    if (Plot->ThreeDPlot->PlotControl->BCross->on_off)
        Plot->ThreeDPlot->PlotControl->BCross->click();
    Plot->ThreeDPlot->PlotControl->BMarkers->setChecked(false);

    if (Func_interval->on_off)
        Func_interval->change_state_man_with_sig(false);


    Plot->ThreeDPlot->ThreeDPlots[0]->SourceData = data;
    //Plot->ThreeDPlot->ThreeDPlots[0]->MathData.clear(); //ensure no math running, no event fuck-up

    if (is4d && !CurrentDataList->ClickedItem->attributes.contains("ZViewAuto"))
        CurrentDataList->ClickedItem->attributes.insert("ZViewAuto", true);

    Plot->ThreeDPlot->restoreAttributes(CurrentDataList->ClickedItem->attributes);


    //parentWidget()->setEnabled(false);
    while (!Plot->ThreeDPlot->ThreeDPlots.at(0)->isMathReady()) { //lots of dependencies\breakage with events\preview_update
        //qApp->processEvents();
        SleepExport::msleep2(3);
    }
    //if (!preview_update)
        //parentWidget()->setEnabled(true);

    Plot->ThreeDPlot->ThreeDPlots[0]->blockRefresh = false;
    Plot->Refresh();
    Plot->ThreeDPlot->ThreeDPlots[0]->ResizeQPlot();

    B3d->show();
    Func_interval->setVisible(is4d);
    Convol_func->setVisible(is4d);
    Scale_select->setVisible(is4d);

    updateAttributes();

    Plot->ThreeDPlot->PlotControl->show();
    Plot->ThreeDPlot->TwoDPlot->PlotControl->hide();

    if (!B3d->isChecked())
        foreach (QObject *obj, Plot->ThreeDPlot->PlotControl->children()) {
            QWidget *wid = qobject_cast<QWidget *>(obj);
            if (wid && wid->isVisible())
                wid->setEnabled(true);
        }
}
