#include "datalist.hpp"
#include "container.h"
#include "ccd/ccd.h"
#include <QsKineticScroller.h>
#include <QScrollBar>
#include <QDragEnterEvent>
#include <QDebug>


TListItem::TListItem(plot::TMetaStorage _attributes, void *data, int type, TListWidget *Parent, bool UpdPr)
{
    UpdatePreview = UpdPr;
    Info = data;
    attributes = _attributes;
    InfoType = type;

    QPixmap img = QPixmap::fromImage(attributes.value("Preview").value<QImage>());
    if (img.isNull())
        img = QPixmap(":/container/question.png");
    if (img.isNull()) {
        img = QPixmap(100, 100);
        img.fill(Qt::darkGray);
    }

    Item = Parent->FindFirstBlank();
    Parent->blockSignals(true);
    if (!Item)
        Item = new QListWidgetItem(Parent);

    Item->setIcon(img);
    Item->setText(attributes.value("Name", TContainer::tr("%1d data").arg(type+1)).toString());
    Item->setData(Qt::UserRole, InfoType);
    Item->setFlags(Item->flags() | Qt::ItemIsEditable);
    Parent->blockSignals(false);
}
TListItem::~TListItem()
{
    if (InfoType == TAbstractInfo::_2d)
        delete (plot::TCurvesDataOwner *)Info;
    else if (InfoType == TAbstractInfo::_3d)
        delete (plot::T3dDataOwner *)Info;
    else if (InfoType == TAbstractInfo::_4d)
        delete (ccd::TSetOfSpectrum *)Info;

    delete Item;
}

//----------------------------------------------------------------------------

TListWidget::TListWidget(TContainer *_cont)
{
    cont = _cont;
    ClickedItem = 0;
    isUpdated = false;

    setResizeMode(QListWidget::Adjust);
    setViewMode(QListWidget::IconMode);
    setMovement(QListWidget::Static);
    setIconSize(QSize(110, 110));
    //setSpacing(10);
    setGridSize(QSize(140, 150));

    setDragEnabled(true);
    viewport()->setAcceptDrops(true);
    setDropIndicatorShown(false);

    addScroller(this);

    connect(this, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), this, SLOT(on_itemPressed(QListWidgetItem*)));
    connect(this, SIGNAL(itemChanged(QListWidgetItem*)), this, SLOT(on_nameChanged(QListWidgetItem*)));
}
TListWidget::~TListWidget()
{
    disconnect();
    ClearAll();
}
void TListWidget::ClearAll()
{
    isUpdated = false;
    blockSignals(true);
    for (; DataList.size(); delete DataList[0], DataList.removeAt(0));
    blockSignals(false);
}
void TListWidget::Delete(int N)
{
    if (N < 0 || N >= DataList.size()) {
        N = DataList.indexOf(FindItem(currentItem()));
        if (N == -1)
            return;
    }

    blockSignals(true);
    delete DataList[N];
    blockSignals(false);

    //isUpdated = false;
    DataList.removeAt(N);
}

void TListWidget::AddBlanks(int N)
{
    isUpdated = true;
    blockSignals(true);
    for (int i = 0; i < N; i++)
        (new QListWidgetItem(QIcon(":/container/question.png"), "Proccessing", this))->setData(Qt::UserRole, TAbstractInfo::Blank);
    blockSignals(false);
}
QListWidgetItem *TListWidget::FindFirstBlank()
{
    for (int i = 0; i < count(); i++)
        if (item(i)->data(Qt::UserRole) == TAbstractInfo::Blank)
            return item(i);

    return 0;
}
TListItem *TListWidget::FindItem(QListWidgetItem *Item) const
{
    for (int i = 0; i < DataList.size(); i++)
        if (DataList[i]->Item == Item)
            return DataList[i];

    return 0;
}


void TListWidget::Add(plot::TMetaStorage attributes, plot::TCurvesDataOwner *data, bool UpdatePreview)
{
    isUpdated = true;
    DataList.append(new TListItem(attributes, data, TAbstractInfo::_2d, this, UpdatePreview));
}
void TListWidget::Add(plot::TMetaStorage attributes, plot::T3dDataOwner *data, bool UpdatePreview)
{
    isUpdated = true;
    DataList.append(new TListItem(attributes, data, TAbstractInfo::_3d, this, UpdatePreview));
}
void TListWidget::Add(plot::TMetaStorage attributes, ccd::TSetOfSpectrum *Info, bool UpdatePreview)
{
    isUpdated = true;
    if (attributes.value("Preview").value<QImage>().size() == QSize(50, 50))
        attributes.remove("Preview");
    DataList.append(new TListItem(attributes, Info, TAbstractInfo::_4d, this, UpdatePreview));
}
void TListWidget::Add(plot::TMetaStorage attributes, QImage Info, bool /*UpdatePreview*/)
{
    isUpdated = true;
    attributes.insert("Preview", QVariant::fromValue(Info));
    DataList.append(new TListItem(attributes, 0, TAbstractInfo::_Image, this, false));
}


void TListWidget::on_nameChanged(QListWidgetItem *Item)
{
    TListItem *it = FindItem(Item);
    if (!it)
        return;

    QString txt = Item->text();
    if (it->attributes.value("Name").toString() == txt) {
        qDebug() << "cont name e1";
        return;
    }

    it->attributes.insert("Name", txt);

    cont->preview_update = true;
    on_itemPressed(Item);
    cont->preview_update = false;
}
void TListWidget::on_itemPressed(QListWidgetItem *Item)
{
    if (!cont->preview_update) {
        cont->SaveFilters();
        cont->SavePreview();
    }

    if (!(ClickedItem = FindItem(Item)))
        return;

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    verticalScrollBar()->setSingleStep(qMax(verticalScrollBar()->maximum() / (count() + 1), 2));
#endif
    ClickedItem->InfoType == TAbstractInfo::_2d ?
                emit ItemClicked((const plot::TCurvesDataOwner *)ClickedItem->Info) :
                ClickedItem->InfoType == TAbstractInfo::_3d ?
                    emit ItemClicked((const plot::T3dDataOwner *)ClickedItem->Info) :
                    ClickedItem->InfoType == TAbstractInfo::_4d ?
                        emit ItemClicked((const ccd::TSetOfSpectrum *)ClickedItem->Info) :
                        emit ItemClicked(ClickedItem->attributes.value("Preview").value<QImage>());//(const QImage *)ClickedItem->Info);
}


void TListWidget::dragEnterEvent(QDragEnterEvent *event)
{
    if (cont && event->mimeData()->hasFormat("text/uri-list"))
        return event->acceptProposedAction();

    if (event->source() == this)
        return;

    if (event->mimeData()->hasImage())
        return event->accept();

    QListWidget::dragEnterEvent(event);
}
void TListWidget::dragMoveEvent(QDragMoveEvent *event)
{
    return dragEnterEvent((QDragEnterEvent *)event);
}
void TListWidget::dropEvent(QDropEvent *event)
{
    if (cont && event->mimeData()->hasFormat("text/uri-list"))
        return cont->dropEvent(event);

    if (event->source() == this)
        return;

    QListWidget::dropEvent(event);
}
QStringList TListWidget::mimeTypes() const { return QStringList() << "nstdatamime" << "image/jpeg" << "image/png"; }
QMimeData *TListWidget::mimeData(const QList<QListWidgetItem *> items) const
{
    for (int i = 0; i < items.size(); i++) {
        TListItem *it = FindItem(items.at(i));
        if (!it) continue;

        if (it->InfoType == TAbstractInfo::_2d) {
            plot::T2dMime *mimeData = new plot::T2dMime;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setData("nstdatamime", QByteArray());
            mimeData->data = plot::getUnLock(*(plot::TCurvesDataOwner *)it->Info);
            return mimeData;
        }
        if (it->InfoType == TAbstractInfo::_3d) {
            plot::T3dMime *mimeData = new plot::T3dMime;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setData("nstdatamime", QByteArray());
            mimeData->data = plot::getUnLock(*(plot::T3dDataOwner *)it->Info);
            return mimeData;
        }
        if (it->InfoType == TAbstractInfo::_Image) {
            QMimeData *mimeData = new QMimeData;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setImageData(it->attributes.value("Preview"));
            return mimeData;
        }
    }

    return 0;
}
bool TListWidget::dropMimeData(int /*index*/, const QMimeData *data, Qt::DropAction action)
{
    if (action == Qt::IgnoreAction)
        return true;

    const plot::T2dMime *m2d = qobject_cast<const plot::T2dMime *>(data);
    const plot::T3dMime *m3d = qobject_cast<const plot::T3dMime *>(data);
    bool image = data->hasImage();
    if (!m2d && !m3d && !image)
        return false;

    cont->copypaste = true;
    if (m2d) {
        plot::TCurvesDataOwner old = plot::getLock(m2d->data);
        plot::TCurvesDataOwner newOne;
        plot::ResizeOwner(newOne, old.size());
        for (int i = 0; i < newOne.size(); i++)
            *newOne.at(i) = *old.at(i);

        cont->AddDataToContainer(newOne, m2d->property("nstmetadata").value<plot::TMetaStorage>());
    } else
    if (m3d) {
        plot::T3dDataOwner old = plot::getLock(m3d->data);
        plot::T3dDataOwner newOne;
        plot::ResizeOwner(newOne, old.size());
        for (int i = 0; i < newOne.size(); i++)
            *newOne.at(i) = *old.at(i);

        cont->AddDataToContainer(newOne, m3d->property("nstmetadata").value<plot::TMetaStorage>());
    } else
    if (image) {
        QImage img = data->imageData().value<QImage>();

        plot::TMetaStorage attributes = data->property("nstmetadata").value<plot::TMetaStorage>();
        if (attributes.isEmpty()) {
            attributes.insert("Name", "Image");
            attributes.insert("Source", "External");

            TContainer::addDateIdsToAttributes(attributes);

            attributes.insert("XMin", 0);
            attributes.insert("XMax", 100);
            attributes.insert("YMin", 0);
            attributes.insert("YMax", 100);

            attributes.insert("XYLabel", "%");
            attributes.insert("XYUnit", QString());
        }

        cont->AddDataToContainer(img, attributes);
    }
    cont->copypaste = false;

    cont->UpdateHierarchy();

    return true;
}
