#include "deflect_widget.hpp"
#include <QtEvents>
#include <QGestureEvent>
#include <QPainter>

DeflectWid::DeflectWid()
{
    setBackgroundRole(QPalette::Base);
    setAutoFillBackground(true);

    pixmap.load(":/SPMTuning_window/laser_spot3.png");
    TuneFork.load(":/SPMTuning_window/approach/TunFork.png");

    setInt(0);
    setXY(0, 0);
    Zpos = 0.5;
    FBscanner = 0;
    bSTM = false;
    bSnotra = false;

    font.setFamily("arial");

    setMinimumSize(200,200);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

    grabGesture(Qt::TapAndHoldGesture);
}

bool DeflectWid::event(QEvent *event)
{
    if (event->type() == QEvent::Gesture) {
        return gestureEvent(static_cast<QGestureEvent*>(event));
    }
    return QWidget::event(event);
}
void DeflectWid::mouseDoubleClickEvent(QMouseEvent * /*Event*/)
{
    emit doubleClicked();
}
bool DeflectWid::gestureEvent(QGestureEvent *event)
{
    if (event->gesture(Qt::TapAndHoldGesture))
        mouseDoubleClickEvent(0);

    return true;
}
void DeflectWid::setTuningFork(bool id)
{
    bSnotra=id;
    update();
}
void DeflectWid::setSTM(bool id)
{
    bSTM=id;
    update();
}

void DeflectWid::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);

    QString s;
    painter.setRenderHint(QPainter::SmoothPixmapTransform, true);
    painter.setPen(QColor(0, 191, 255));
    painter.setBrush(QBrush(QColor(32, 32, 128)));
    painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
    painter.setBrush(QBrush());

    int minsz = qMin(width(), height());
    int size = qMax<int>(10, minsz * intens / SPOT_SCALE);
    int fontsize = minsz / (FONT_SCALE);
    font.setPixelSize(fontsize);
    painter.setFont(font);

    if (!bSnotra && !bSTM) {
        double Cx = 0.5 + qRound(width()/2.);
        double Cy = 0.5 + qRound(height()/2.);
        int x = Cx + width()*X - size/2;
        int y = Cy - height()*Y - size/2;
        int R = qMax(Cx, Cy) / 6;

        painter.setRenderHint(QPainter::Antialiasing, true);
        for (int i = 6; i > 1; i--)
            painter.drawEllipse(QRectF(Cx-R*i,Cy-R*i, 2*R*i, 2*R*i));
        painter.setRenderHint(QPainter::Antialiasing, false);

        painter.setPen(QColor(0, 191, 255));
        painter.drawLine(Cx, 0, Cx, height());
        painter.drawLine(0, Cy, width(), Cy);

        painter.drawPixmap(x, y, size, size, pixmap);
    }
    painter.setPen(Qt::white);
    if (bSTM) {
        painter.drawText(width()/2.2, height()/2 - 0.8*fontsize, tr("STM MODE"));
    } else
    if (bSnotra) {
        int TF_size = minsz / 4;
        painter.drawText(width()/2.2, height()/2 - 0.8*fontsize, tr("TF MODE"));
        painter.drawPixmap(0.1*width(), 0.15*height(), 1.5*TF_size, 3*TF_size, TuneFork);
    }

    painter.setPen(QPen(Qt::white, height() / 150));
    painter.drawLine(0.6*width(), 0.13*height(), 0.95*width(), 0.13*height());
    painter.drawLine(0.6*width(),0.13*height(), 0.6*width(), 0.15*height());
    painter.drawLine(0.69*width(),0.13*height(),0.69*width(),0.15*height());
    painter.drawLine(0.78*width(),0.13*height(),0.78*width(),0.15*height());
    painter.drawLine(0.86*width(),0.13*height(),0.86*width(),0.15*height());
    painter.drawLine(0.95*width(),0.13*height(),0.95*width(),0.15*height());


    QColor color = (Zpos > 0.5) ? Qt::green : ((Zpos > 0.25) ? Qt::yellow : Qt::red);
    painter.setPen(QPen(color, height() / 40));
    painter.drawLine((0.95-0.35*Zpos)*width(), 0.1*height(), 0.95*width(), 0.1*height());

    if (FBscanner >= 0) {
        s = tr("FB Z: ") + (FBscanner==0 ? tr("Tip") : tr("Sample"));
        painter.setPen(FBscanner==0 ? Qt::green : Qt::red);
        painter.drawText(5, 0.84*height(), s);
    }
    painter.setPen(Qt::white);

    if (!bSnotra && !bSTM)
    {
        s = tr("LFL: %1").arg(LFL);
        painter.drawText(5, 5 + 2*fontsize, s);

        s = tr("DFL: %1").arg(DFL);
        painter.drawText(5, 5 + fontsize, s);

        s = tr("INT, %: %1").arg(intens*100, 6, 'f', 1);
        painter.drawText(5, height() + 5 - fontsize, s);
    }
    if (!bSTM) {
        s = QString(tr("MAG: %1")).arg(QString());
        painter.drawText(width()- 10 - QFontMetrics(font).width(s), height() - 5 - 2*fontsize, s);
        s = MAG;
        painter.drawText(width()- 10 - QFontMetrics(font).width(s), height() + 5 - fontsize, s);
    } else {
        s = QString(tr("ITIP: %1")).arg(DFL);
        painter.drawText(width()- 10 - QFontMetrics(font).width(s), height() + 5 - fontsize, s);
    }

    fontsize /= 2;
    font.setPixelSize(fontsize);
    painter.setFont(font);

    s = QString(tr("Z Range(%):%1")).arg(Zpos*100, 6, 'f', 1);
    painter.drawText(0.62*width(), 0.2*height(), s);

}
void DeflectWid::setFbscanner(int value)
{
    if (FBscanner == value)
        return;

    FBscanner = value;
    update();
}
void DeflectWid::setZpos(float Zpos_new)
{
    if (Zpos == Zpos_new)
        return;

    Zpos = Zpos_new;
    update();
}
void DeflectWid::setMag(QString Mag_new)
{
    if (MAG == Mag_new)
        return;

    MAG = Mag_new;
    update();
}
void DeflectWid::setDfl(QString Mag_new)
{
    if (DFL == Mag_new)
        return;

    DFL = Mag_new;
    update();
}
void DeflectWid::setLfl(QString Mag_new)
{
    if (LFL == Mag_new)
        return;

    LFL = Mag_new;
    update();
}
void DeflectWid::setXY(float a, float b)
{
    if (X == a && Y == b)
        return;

    X = a;
    Y = b;
    update();
}
void DeflectWid::setInt(float i)
{
    if (intens == i)
        return;

    intens = i;
    update();
}
