#pragma once

#include <QIcon>
#include <QLineEdit>
#include <QFrame>
#include <QPushButton>
class QFormLayout;

bool show_additional_gui();
bool show_advanced_gui();
QIcon makeHugeIcon(QString path);
QFormLayout *makeFormLayout(QWidget *parent = 0);

class QDoubleValidator;
class DoubleEdit : public QLineEdit
{
    Q_OBJECT

public slots:
    void stepUp();
    void stepDown();
    void setMin(double value_min);
    void setMax(double value_max);
    void setValue(double value);
    void setNewValue();
    void resetValue();

signals:
    void editingStarted();
    void newValue();
    void newValue(int);
    void newValue(double);

public:
    DoubleEdit(double min, double max, int acc);
    double value() const;

protected:
    void focusInEvent(QFocusEvent *event);
    void focusOutEvent(QFocusEvent *event);

private:
    double prev_value;
    QDoubleValidator *range_validator;
};

class NSTSlider;
class Edit;
class DoubleChanger : public QFrame
{
    Q_OBJECT

public slots:
    void setDecimals(int decimals);
    void setMinimum(double value_min);
    void setMaximum(double value_max);
    void setValue(double value);
    void setNewValue();

signals:
    void editingStarted();
    void editingFinished();
    void newValue();
    void newValue(int);
    void newValue(double);
    void newMinimum(double);
    void newMaximum(double);
    void newDecimals(int);

public:
    DoubleChanger(double min, double max, int acc = 3, bool vertical = false);
    double value() const;

    Edit *edit;
    NSTSlider *slider;
};
void linkDoubleChangers(DoubleChanger *master, DoubleChanger *slave, bool linkRange = true);

class TwoStateButton : public QPushButton
{
    Q_OBJECT

public slots:
    void change_state();
    void change_state_man(bool);
    void change_state_man_with_sig(bool st);

signals:
    void state_changed(bool);

public:
    TwoStateButton(QString off, QString on);
    TwoStateButton(const QString & text,QString off, QString on);

    QString offstring;
    QString onstring;
    bool on_off;
};
