#include "t3dcontrol.hpp"
#include "viewer3d.hpp"
#include "flowlayout.hpp"
#include "controls.h"

#include <QPushButton>
#include <QFileDialog>
#include <QTimer>
#include <QGroupBox>

void T3DControl::top_view()
{
    pl3->x_rot = 90;
    pl3->y_rot = 0;
    pl3->z_rot = 0;
    pl3->updateGL();
}

void T3DControl::save_click()
{
    QString SelectedFilter;
    QString FileName;

    FileName = QFileDialog::getSaveFileName(this,
                                            tr("Save as..."),
                                            QString(),
                                            tr("*.png;;*.jpg;;*.tiff;;*.bmp"),
                                            &SelectedFilter,
                                            QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    if (!FileName.endsWith(SelectedFilter.remove(0, 1)))
        FileName += SelectedFilter;

    pl3->grabFrameBuffer().save(FileName, SelectedFilter.remove(0, 1).toAscii().data());
}

void T3DControl::move_toggle(bool checked)
{
    pl3->mousemode = checked ? 1 : 0;
    if (!checked) {
        BMoveFlag->setIcon(makeHugeIcon(":/3D/move.png"));
        return;
    }

    BMoveFlag->setIcon(makeHugeIcon(":/3D/move_on.png"));
}

void T3DControl::scale_click()
{
    pl3->z_scale = 0.2;
    pl3->scale = 1;
    pl3->x_pos = pl3->y_pos = pl3->z_pos = 0;
    pl3->resizeGL(pl3->width(), pl3->height());
    pl3->updateGL();
}

void T3DControl::background_toggle(bool checked)
{
    float bg = palette().color(QPalette::Background).value();
    if (checked) bg = 256 - bg;
    pl3->bgColor = QColor(bg, bg, bg);

    float fg = (bg > 127) ? 0 : 255;
    pl3->fgColor = QColor(fg, fg, fg);

    if (isVisible()) pl3->updateGL();
}
void T3DControl::box_toggle(bool checked)
{
    if (!checked) {
        BBox->setIcon(makeHugeIcon(":/3D/wireOn.png"));
        pl3->showAxes = true;
        pl3->updateGL();
        return;
    }

    BABox->setChecked(false);
    BBox->setIcon(makeHugeIcon(":/3D/wireOF.png"));
    pl3->showAxes = false;
    pl3->updateGL();
}
void T3DControl::allbox_toggle(bool checked)
{
    if (!checked) {
        BABox->setIcon(makeHugeIcon(":/3D/scaleOn.png"));
        BBox->setIcon(makeHugeIcon(":/3D/wireOn.png"));
        pl3->showAxes = true;
        pl3->updateGL();
        return;
    }

    BBox->setChecked(false);
    BABox->setIcon(makeHugeIcon(":/3D/scaleOf.png"));
    BBox->setIcon(makeHugeIcon(":/3D/wireOF.png"));
    pl3->showAxes = false;
    pl3->updateGL();
}

void T3DControl::rotate_click()
{
    if (BRotate->isChecked()) {
        rot_timer->start(50);
        BRotate->setIcon(makeHugeIcon(":/3D/rotateOn.png"));
    } else {
        rot_timer->stop();
        BRotate->setIcon(makeHugeIcon(":/3D/rotate.png"));
    }
}

void T3DControl::rotate()
{
    if (!BRotate->isChecked())
        return;

    pl3->z_rot += 1;
    pl3->updateGL();
}

T3DControl::T3DControl(T3dViewer *pl, QWidget *parent) : QWidget(parent)
{
    pl3 = pl;
    background_toggle(false);

    QPushButton *BTop = new QPushButton;
    BTop->setIcon(makeHugeIcon(":/container/TopView.png"));
    BTop->setToolTip(QObject::tr("Top view"));

    BMoveFlag = new QPushButton;
    BMoveFlag->setIcon(makeHugeIcon(":/3D/move.png"));
    BMoveFlag->setToolTip(QObject::tr("Move mode"));
    BMoveFlag->setCheckable(true);

    QPushButton *BBackground = new QPushButton;
    BBackground->setIcon(makeHugeIcon(":/3D/blackwhite.png"));
    BBackground->setToolTip(QObject::tr("Toggle black\\white background"));
    BBackground->setCheckable(true);

    BBox = new QPushButton;
    BBox->setIcon(makeHugeIcon(":/3D/wireOn.png"));
    BBox->setToolTip(QObject::tr("Show\\hide graph box"));
    BBox->setCheckable(true);

    BABox = new QPushButton;
    BABox->setIcon(makeHugeIcon(":/3D/scaleOn.png"));
    BABox->setToolTip(QObject::tr("Show\\hide graph scale"));
    BABox->setCheckable(true);
    BABox->hide();

    QPushButton *BScale = new QPushButton;
    BScale->setIcon(makeHugeIcon(":/3D/restore_scale.png"));
    BScale->setToolTip(QObject::tr("Restore scale"));

    QPushButton *BSave = new QPushButton;
    BSave->setIcon(makeHugeIcon(":/snap_shot.png"));
    BSave->setToolTip(QObject::tr("Save image"));

    BRotate = new QPushButton;
    BRotate->setIcon(makeHugeIcon(":/3D/rotate.png"));
    BRotate->setToolTip(QObject::tr("Auto rotation"));
    BRotate->setCheckable(true);


    QVBoxLayout *ll = new QVBoxLayout(this);
    ll->setContentsMargins(3, 0, 3, 6);

    QGroupBox *gr = new QGroupBox(this);
    ll->addWidget(gr);
    HFlowLayout *layout = new HFlowLayout(gr);
    layout->spaceX = layout->spaceY = 5;
    layout->setContentsMargins(3, 3, 3, 3);

    layout->addWidget(BMoveFlag);
    layout->addWidget(BBackground);
    layout->addWidget(BBox);
    layout->addWidget(BABox);
    layout->addWidget(BScale);
    layout->addWidget(BSave);
    layout->addWidget(BRotate);
    layout->addWidget(BTop);


    rot_timer = new QTimer(this);
    connect(rot_timer, SIGNAL(timeout()), SLOT(rotate()));

    connect(BMoveFlag, SIGNAL(toggled(bool)), SLOT(move_toggle(bool)));
    connect(BBackground, SIGNAL(toggled(bool)), SLOT(background_toggle(bool)));
    connect(BBox, SIGNAL(toggled(bool)), SLOT(box_toggle(bool)));
    connect(BABox, SIGNAL(toggled(bool)), SLOT(allbox_toggle(bool)));
    connect(BScale, SIGNAL(clicked()), SLOT(scale_click()));
    connect(BSave, SIGNAL(clicked()), SLOT(save_click()));
    connect(BRotate, SIGNAL(clicked()), SLOT(rotate_click()));
    connect(BTop, SIGNAL(clicked()), SLOT(top_view()));
}
