#include "moticx.hpp"
#include <QNetworkReply>
#include <QTcpSocket>

MoticX::MoticX()
{
    pos_ba00 = -1;
    init = false;
    reply = 0;

    manager.moveToThread(this);
    moveToThread(this);
}

MoticX::~MoticX()
{
    disconnect();
    terminate();
    if (reply) {
        reply->abort();
    }

    closeCamera();
}

void MoticX::Start()
{
    if (!init)
        return;

    start();
}

void MoticX::Stop()
{
    if (!init)
        return;

    quit();
}

void MoticX::newData()
{
    const unsigned char ba00[4] = {0xFF, 0xd8, 0xFF, 0}; //jpg start
    const unsigned char ba01[3] = {0xFF, 0xd9, 0}; //jpg end
    //"--boundary" - separator

    if (!reply || reply->bytesAvailable() <= 0)
        return;

    int prev_size = ba.size();
    ba.append(reply->readAll());

    if (pos_ba00 == -1)
        pos_ba00 = ba.indexOf((const char *)ba00, qMax(0, prev_size - 2));
    if (pos_ba00 == -1)
        return;

    int pos_ba01 = ba.indexOf((const char *)ba01, pos_ba00 + 3);
    if (pos_ba01 == -1)
        return;
    pos_ba01 += 2;

    QImage img;
    if (img.loadFromData((const uchar *)ba.constData() + pos_ba00, ba.size() - pos_ba00))
        emit ImgReceived(img);

    ba.remove(0, pos_ba01);
    pos_ba00 = -1;
}

void MoticX::run()
{
    reply = manager.get(QNetworkRequest(QUrl("http://" + ip + ":8080/?action=stream")));
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(newData()));

    exec();

    reply->abort();
    delete reply;
    reply = 0;

    ba.clear();
    pos_ba00 = -1;
}

void MoticX::closeCamera()
{
    init = false;
}

bool MoticX::openCamera()
{
    closeCamera();

    /*
    QNetworkAccessManager manager;
    QNetworkReply *reply = manager.get(QNetworkRequest(QUrl("http://192.168.1.151:8080/"))); //fucking qt is freezing for 1 sec somewhere!
    QElapsedTimer t; t.start();
    while (!reply->isFinished() && t.elapsed() < 60) {
        qApp->processEvents();
        SleepExport::msleep2(2);
    }

    if (reply->error() != QNetworkReply::NoError || t.elapsed() >= 60) {
        reply->abort();
        reply->deleteLater();
        return false;
    }
    reply->abort();
    reply->deleteLater();
    //qDebug() << t.elapsed();
    */
    QTcpSocket sock;
    sock.connectToHost(ip = "192.168.1.151", 8080);
    if (sock.waitForConnected(100))
        return init = true;
#ifndef _WIN32
    sock.connectToHost(ip = "192.168.10.1", 8080);
    if (sock.waitForConnected(100))
        return init = true;
#endif

    return false;
}
