#include "modules.hpp"
#include "spmtunning.h"
#include "scanplot.h"
#include "ccd/ccd.h"
#include <qwt_thermo.h>
#include <QHBoxLayout>
#include <QFormLayout>

class IJMScan : public IExternalScan
{
    Q_OBJECT

signals:
    void AddDataToContainer(ccd::TSetOfSpectrum *, plot::TMetaStorage);

public slots:
    void save();
    void clear() {
        lastNumPoints = 0;
        SetOfSpectrums.Array.clear();
        SetOfSpectrums.Results.at(0)->clear();
        I4dAdd.Section.at(0)->clear();
        IExternalScan::clear();
    }
    void onConvFunc();

    void convUpdated() {
        {
            *Output3d.at(0) = *SetOfSpectrums.Results.at(0);
            I4dAdd.CalcSection();

            int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();
            QString u = T4dDataHelper::getConvZUnits(idx, ccd::nm, SetOfSpectrums.YscaleUnit);
            attributes.insert("ZUnit", u);

            if (Point_Spectrum->on_off) {
                *Section.at(0) = *I4dAdd.Section.at(0);
                attributes.insert("Name", tr("JM 4d scan profile"));
                attributes.insert("XUnit", QObject::tr("m"));
                attributes.insert("YUnit", u);
            } else {
                *Section.at(0) = *spmtunning->curveData_force.at(0);
                if (spmtunning->curveData_force.size() >= 2) {
                    plot::TCurveData *curve = spmtunning->curveData_force.at(1).data();
                    for (int i = 0; i < curve->YSeq.size(); i++)
                        Section.at(0)->AddY(curve->YSeq.at(curve->YSeq.size() - i - 1));
                }
                attributes.insert("Name", tr("JM 4d point curve"));
                attributes.insert("XUnit", QObject::tr("m"));
                attributes.insert("YUnit", SetOfSpectrums.YscaleUnit);
            }
            Section.at(0)->Pen.setColor(Qt::red);
        }

        int x0, x1, y0, y1;
        GetMathLimits(x0, x1, y0, y1);
        Output3d.at(0)->LimsForMathX[0] = x0;
        Output3d.at(0)->LimsForMathX[1] = x1;
        Output3d.at(0)->LimsForMathY[0] = y0;
        Output3d.at(0)->LimsForMathY[1] = y1;

        emit updated();
    }

public:
    bool eventFilter(QObject *, QEvent *);
    bool init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy);

    QWidget *createInterfaceWidget();

    bool acquireNextPoint() {
        ccd::TSpectrum TmpSpectrum;
        if (spmtunning->curveData_force.size() >= 2) {
            int sz1 = spmtunning->curveData_force.at(0)->YSeq.size();
            int sz2 = spmtunning->curveData_force.at(1)->YSeq.size();
            TmpSpectrum.reserve(sz1 + sz2);

            for (int i = 0; i < sz1; i++)
                TmpSpectrum.push_back(spmtunning->curveData_force.at(0)->YSeq.at(i));

            for (int i = 0; i < sz2; i++)
                TmpSpectrum.push_back(spmtunning->curveData_force.at(1)->YSeq.at(sz2 - i - 1));

            TmpSpectrum.squeeze();
        }

        I4dAdd.AddNextSpectrum(TmpSpectrum);

        bool ok = spmtunning->BFeedbackOnOff->on_off && spmtunning->BFeedbackOnOffMode->on_off;
        if (!spmtunning->BFeedbackOnOff->on_off)
            lastError = tr("JM scan: SPM feedback is disabled.");
        else if (!spmtunning->BFeedbackOnOffMode->on_off)
            lastError = tr("JM scan: JM mode is disabled.");

        if (ok)
            emit acquired();

        return ok;
    }
    int getDataChannel() { return spmtunning->FB_channel; }
    void updateData() {
        lastNumPoints = SetOfSpectrums.CalcResults(lastNumPoints);
    }

    void GetMathLimits(int &x0, int &x1, int &y0, int &y1);

    int lastNumPoints;

    QComboBox *CBConvFunc;
    TwoStateButton *Point_Spectrum;

    ccd::TSetOfSpectrum SetOfSpectrums;
    ccd::TSetOfSpectrumConstructor I4dAdd;

    SPMTunning *spmtunning;
};
#include <jm_scan.moc>

IExternalScan *getNewJMIExtScan(SPMTunning *spm)
{
    IJMScan *sc = new IJMScan;
    sc->spmtunning = spm;
    sc->I4dAdd.data = &sc->SetOfSpectrums;
    sc->lastNumPoints = 0;

    sc->disableWidgets << spm->CB_Points_spectr << spm->PBSpectr_start_stop;

    QObject::connect(&sc->SetOfSpectrums, SIGNAL(finished()), sc, SLOT(convUpdated()), Qt::QueuedConnection);

    return sc;
}


bool IJMScan::init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy) {
    IExternalScan::init(_nx, _ny, _dir_stpoint, _sx, _sy);

    spmtunning->s_spectr_start(false); //sync points count
    int jm_points = spmtunning->CB_Points_spectr->currentText().toInt();

    SetOfSpectrums.CalibInfo[0] = 1; //lwl
    SetOfSpectrums.CalibInfo[1] = 0; //cwl
    SetOfSpectrums.CalibInfo[2] = 1; //disp
    SetOfSpectrums.CalibInfo[3] = (double)1 * qMax(0.1, spmtunning->DCfbPrec->value()); //pxsize
    SetOfSpectrums.CalibInfo[4] = (double)2*jm_points; //nx
    SetOfSpectrums.CalibInfo[5] = (double)jm_points-1; //cpx

    lastNumPoints = 0;
    SetOfSpectrums.SetDirStPoint((ccd::TSetOfSpectrum::TDirections)dir, (ccd::TSetOfSpectrum::TStartPoints)start_point);
    I4dAdd.PrepareForNewScan(nx, ny, sx, sy, 0, 2*jm_points, 0, 2*jm_points);

    ScanPlot *scan = (ScanPlot *)ModuleManager::instance->find1("scan")->basecast();
    SetOfSpectrums.YscaleUnit = scan->scan_ch[1] == srcHeight ? QObject::tr("m") : QObject::tr("V");

    if (!spmtunning->BFeedbackOnOff->on_off)
        lastError = tr("JM scan: SPM feedback is disabled.");
    else if (!spmtunning->BFeedbackOnOffMode->isVisible())
        lastError = tr("JM scan: JM mode is not available.");
    else if (!spmtunning->BFeedbackOnOffMode->on_off)
        lastError = tr("JM scan: JM mode is disabled.");
    return spmtunning->BFeedbackOnOff->on_off && spmtunning->BFeedbackOnOffMode->on_off;
}

void IJMScan::GetMathLimits(int &x0, int &x1, int &y0, int &y1) {
    x0 = y0 = 0;
    x1 = nx;
    y1 = ny;

    int curLine = I4dAdd.GetCurrentLineNumber();
    curLine++;

    switch(dir_stpoint) {
    case spTopLeftAxisX:
    case spTopRightAxisX:
        y0 = qMax(ny - 1 - curLine, 0);
        break;
    case spBottomLeftAxisX:
    case spBottomRightAxisX:
        y1 = qMin(curLine + 1, ny);
        break;
    case spTopLeftAxisY:
    case spBottomLeftAxisY:
        x1 = qMin(curLine + 1, nx);
        break;
    case spTopRightAxisY:
    case spBottomRightAxisY:
        x0 = qMax(nx - 1 - curLine, 0);
        break;
    }
}

QWidget *IJMScan::createInterfaceWidget() {
    QGroupBox *wid = new QGroupBox(tr("JM force curve"));
    QFormLayout *lay = makeFormLayout(wid);

    QPushButton *BClear = new QPushButton(makeHugeIcon(":/CCD/mclear.png"), QString());
    BClear->setToolTip(tr("Clear spectrum"));
    connect(BClear, SIGNAL(clicked()), this, SLOT(clear()));

    QPushButton *BSave = new QPushButton(makeHugeIcon(":/CCD/4DAttach.png"), QString());
    BSave->setToolTip(tr("Send to data"));
    connect(BSave, SIGNAL(clicked()), this, SLOT(save()));

    Point_Spectrum = new TwoStateButton(":/CCD/point_spect_off.png", ":/plot/point.png");
    Point_Spectrum->setToolTip(tr("Switch between section and point spectra"));


    QHBoxLayout *layB1 = new QHBoxLayout;
    layB1->addWidget(BSave);
    layB1->addWidget(BClear);
    layB1->addWidget(Point_Spectrum);
    layB1->addStretch(1);

    lay->addRow(layB1);


    CBConvFunc = new QComboBox;
    CBConvFunc->addItem(tr("Stiffness"), ccd::TSetOfSpectrum::F_Stiffness);
    CBConvFunc->addItem(tr("Adhesion"), ccd::TSetOfSpectrum::F_Adhesion_F);
    CBConvFunc->addItem(tr("Energy Dissipation"), ccd::TSetOfSpectrum::F_Energy);
    CBConvFunc->addItem(tr("Baseline"), ccd::TSetOfSpectrum::F_Baseline);
    connect(CBConvFunc, SIGNAL(currentIndexChanged(int)), this, SLOT(onConvFunc()));
    lay->addRow(tr("Convolution"), CBConvFunc);


    QComboBox *CBPoints = new QComboBox;
    CBPoints->addItem("50");  CBPoints->addItem("100");
    CBPoints->addItem("200"); CBPoints->addItem("300");
    CBPoints->addItem("400"); CBPoints->addItem("500");
    CBPoints->addItem("600"); CBPoints->addItem("700");
    CBPoints->addItem("800"); CBPoints->addItem("900");
    CBPoints->addItem("1000");
    lay->addRow(tr("Z points"), CBPoints);
    connect(CBPoints, SIGNAL(currentIndexChanged(int)), spmtunning->CB_Points_spectr, SLOT(setCurrentIndex(int)));
    connect(spmtunning->CB_Points_spectr, SIGNAL(currentIndexChanged(int)), CBPoints, SLOT(setCurrentIndex(int)));

    disableWidgets << BSave << BClear << CBPoints;

    return wid;
}

void IJMScan::save()
{
    ccd::TSetOfSpectrum *spectr = new ccd::TSetOfSpectrum;
    *(ccd::TInfoForSetOfSpectrums *)spectr = SetOfSpectrums;
    spectr->YscaleUnit = SetOfSpectrums.YscaleUnit;
    spectr->CalibInfo = SetOfSpectrums.CalibInfo;
    *spectr->Results.at(0) = *SetOfSpectrums.Results.at(0);

    plot::TMetaStorage attributes;

    attributes.insert("XPos", spectr->XLimits[0]);
    attributes.insert("YPos", spectr->YLimits[0]);

    spectr->XLimits[1] -= spectr->XLimits[0]; spectr->XLimits[0] = 0;
    spectr->YLimits[1] -= spectr->YLimits[0]; spectr->YLimits[0] = 0;

    attributes.insert("XMin", spectr->XLimits[0]);
    attributes.insert("XMax", spectr->XLimits[1]);
    attributes.insert("YMin", spectr->YLimits[0]);
    attributes.insert("YMax", spectr->YLimits[1]);

    attributes.insert("XYUnit", QObject::tr("m"));
    attributes.insert("YUnit", SetOfSpectrums.YscaleUnit);
    int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();
    attributes.insert("ZUnit", T4dDataHelper::getConvZUnits(idx, ccd::nm, SetOfSpectrums.YscaleUnit));

    attributes.insert("Name", "JM force curve");
    attributes.insert("Source", "JM Scan");


    attributes.insert("X points", nx);
    attributes.insert("Y points", ny);

    //TODO nm calib in attrib

    displayPlot->appendAttrib(attributes, 1);

    emit AddDataToContainer(spectr, attributes);
}

void IJMScan::onConvFunc()
{
    int idx = CBConvFunc->itemData(CBConvFunc->currentIndex()).toInt();

    SetOfSpectrums.CurrentFunc = idx;
    lastNumPoints = 0;
    updateData();
}

bool IJMScan::eventFilter(QObject *, QEvent *Event)
{
    if (Event->type() == plot::TCrossEvent::id) {
        if (Point_Spectrum->on_off) {
            int XInd, YInd;
            SetOfSpectrums.GetIntegerIndexes(((plot::TCrossEvent *)Event)->Coord.x(), ((plot::TCrossEvent *)Event)->Coord.y(), XInd, YInd);
            T4dDataHelper::ShowSpectr(displayPlot->TwoDPlot, &SetOfSpectrums, XInd, YInd, ccd::nm);
            displayPlot->Set2dPlotVisible(true);
            return true;
        }
    }
    return false;
}
