#pragma once

#include <defs.h>
#include <QWidget>
#include <QPointer>
#include "controls.h"
#include "edit.hpp"
#include "tools.hpp"

class StatusWidget1;
class MotorSlider;
class QVBoxLayout;
class QGroupBox;
class QCheckBox;
class Motors : public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT

signals:
    void m123stopped();

public slots:
    //num 1-based
    void go(int num, int dir, double speed, double steps); //mkm
    void goUpE(int num = -1, double speed = 0);
    void goUp(int num = -1, double speed = 0, double dist = 0); //mkm
    void goDownE(int num = -1, double speed = 0);
    void goDown(int num = -1, double speed = 0, double dist = 0); //mkm
    void goStop(int num = -1);
    void goShuttleInt(int speedidx, int num = -1); //-5..5, with intra/extrapolation
    void goShuttle(double speedidx, int num = -1); //-5..5, with intra/extrapolation
    void goShuttleStep(int dir, int num = -1); //1/-1
    double getShuttleSpeed(double speedidx); //-5..5, with intra/extrapolation

    void init();
    void setBLspeed();
    void Slot_Set_Motor_Status_handler();
    void Slot_Set_Motor_Status();
    void capBits(int, long);

    void set_raw_motors_inv_end(unsigned char, unsigned short);
    void set_raw_motors_control(unsigned char, unsigned short);
    void set_step_motors_leveling(int,unsigned char, double, double);

    void save_params();
    void save_device();
    void save_current();
    void set_name_idx(QAction *a);
    void SetCurrent(int num = 0); //num 0-based

    void sett_showhide();
    void show_sett();
    void addCustomSett(QWidget *w);

public:
    Motors(QWidget *parent = 0);

    static //1-based
    QList<int> getRoleMotors(int role); //see MotorRoles enum

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size);
    bool sendData(const void *handle);
    void resetWriter();

    long motor_step[16+1];
    long motor_speed[16+1];
    WORD motor_cont_byte[16+1];
    WORD motor_inv_byte[16];
    DWORD motor_bl_steps[16];
    WORD motor_bl_speed;
    DWORD motor_land_safe_calib;
    BYTE rawmotoraddr;
    WORD rawmotordata;

    TMessageFlag sendReset;
    TMessageFlag updMotor[16+1];
    bool sendLMheight;

    BYTE csb_motor_addr;
    TMessageFlag updateCsbMotorAddr;
    BYTE csb_motor_count;
    TMessageFlag updateCsbMotorCount;
    BYTE remap_motor_to[16];
    TMessageFlag updateRemapMotor[16];

    BYTE motorRst;
    TMessageFlag send_control[16+1], update_inv_end[16], send_bl_steps[16], sendBLspeed, sendLandSafeCalib, sendCustom;


    Edit *Enum;
    QPushButton *PBselector[16];
    int active_motor;
    int last_command;

    Edit *Espeed;
    Edit *Edist;

    QWidgetList showhidelist;
    QPushButton *BShowHideSett;
    QWidget *Wsettings;
    QPushButton *BSave;
    QGroupBox *WsettGB;
    QLineEdit *LEname;
    Edit *Eidx;
    QCheckBox *CBinv;
    QCheckBox *CBdir;
    Edit *EBLdist; //backlash
    Edit *EMaxSpeed;

    Edit *Ecsbmotors;
    Edit *Epitch;
    Edit *Estepsrev;
    Edit *EBLspeed; //backlash
    bool orient123;
    bool orient[16];

    QWidget *WsettingsLeft;
    QWidget *WsettingsShuttle;
    Edit *EdistShuttle;
    Edit *EspeedShuttle[5+1];

    QPointer<StatusWidget1> SWmotor1;
    QTimer *motorSync;
    BYTE Motor[16];
    bool Motor_on[16];
    int motorStatusLimit;

    bool blocksave;
    bool reqsent;

public:
    enum MotorRoles {
        RoleNone,
        RoleX1, RoleY1, RoleZ1Land,
        RoleX2, RoleY2, RoleZ2Focus,
        RoleX3, RoleY3, RoleZ3,
        RoleUnused
    };

};
