#include "options_window.h"
#include "mainwindow.h"
#include "ccd/ccdplot.h"
#include "scanplot.h"
#include "spmtunning.h"
#include "autocalib.hpp"
#include "gui/status.hpp"
#include "gui/ttabwidget.hpp"

#include <QStyledItemDelegate>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QCheckBox>
#include <QSettings>
#include <QTreeWidget>
#include <QHeaderView>
#include <QFontDialog>
#include <QFileDialog>
#include <QToolBar>
#include <QApplication>

class EditorDelegate: public QStyledItemDelegate
{
public:
    EditorDelegate(QObject *parent) : QStyledItemDelegate(parent) {}
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {/*
        if (index.data().toString() == "false" || index.data().toString() == "true") {
            QCheckBox *box = new QCheckBox(parent);
            box->setAutoFillBackground(true);
            return box;
        }*/
        return index.column() != 1 ? 0 : QStyledItemDelegate::createEditor(parent, option, index);
    }/*
    void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::setModelData(editor, model, index);

        model->setData(index, edit->isChecked(), Qt::EditRole);
    }
    void setEditorData(QWidget *editor, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::setEditorData(editor, index);

        edit->setChecked(index.model()->data(index, Qt::EditRole).toBool());
    }
    void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::updateEditorGeometry(editor, option, index);

        edit->setGeometry(option.rect);
    }*/
};

void TOptionsWindow::selectCustomTR()
{
    if (Ctranslation->currentIndex() != 2)
        return;

    QString f = QFileDialog::getOpenFileName(this, tr("Select custom translation"), QString(), "*.qm", 0, QFileDialog::DontUseNativeDialog);
    if (f.isEmpty())
        f = qApp->property("translation").toString();

    if (f.endsWith(":/translation_base"))
        Ctranslation->setCurrentIndex(0);
    else if (f.endsWith(":/translation_russian"))
        Ctranslation->setCurrentIndex(1);
    else {
        Ctranslation->setCurrentIndex(2);
        Ctranslation->setItemData(2, f);
    }
}

TOptionsWindow::TOptionsWindow(QWidget *parent) : QWidget(parent, Qt::Dialog)
{
    setWindowFlags((Qt::WindowFlags)(windowFlags() & (~Qt::WindowContextHelpButtonHint)));
    setWindowModality(Qt::WindowModal);
    setWindowTitle(tr("Interface options - NSpec"));

    QVBoxLayout *MainLayout = new QVBoxLayout(this);
    MainLayout->setSizeConstraint(QLayout::SetFixedSize);

    TTabWidget *tabWidget = new TTabWidget(this);
    QHBoxLayout *Blayout = new QHBoxLayout();
    Blayout->setAlignment(Qt::AlignRight);

    MainLayout->addWidget(tabWidget);
    MainLayout->addWidget(new QLabel("* - " + tr("Requires program restart")), 0, Qt::AlignRight | Qt::AlignHCenter);
    MainLayout->addLayout(Blayout);


    QWidget *optWid1 = new QWidget(tabWidget);
    QWidget *optWid2 = new QWidget(tabWidget);
    QWidget *optWid3 = new QWidget(tabWidget);
    QFormLayout *Boplayout = makeFormLayout(optWid1);
    Boplayout->setLabelAlignment(Qt::AlignLeft);
    QVBoxLayout *Boplayout2 = new QVBoxLayout(optWid2);
    QFormLayout *Boplayout3 = makeFormLayout(optWid3);

    tabWidget->addTab(optWid1, tr("General"));
    tabWidget->addTab(optWid2, tr("Modules"));
    tabWidget->addTab(optWid3, tr("Network"));
    if (!show_additional_gui() &&
            !show_advanced_gui())
        tabWidget->setTabVisible(2, false);


    QPushButton *Bok = new QPushButton(tr("Apply"));
    QPushButton *Bcancel = new QPushButton(tr("Cancel"));
    Bcancel->setShortcut(QKeySequence(Qt::Key_Escape));
    QPushButton *Bfont = new QPushButton(tr("Select font"));
    QPushButton *Bdata = new QPushButton(tr("Select"));
    QPushButton *Bedit = new QPushButton(tr("Configuration"));
    connect(Bok, SIGNAL(clicked()), this, SLOT(Apply()));
    connect(Bcancel, SIGNAL(clicked()), this, SLOT(hide()));
    connect(Bfont, SIGNAL(clicked()), this, SLOT(BFont()));
    connect(Bdata, SIGNAL(clicked()), this, SLOT(BData()));

    if (!show_additional_gui() &&
            !show_advanced_gui()) {
        Bedit->setFlat(true);
        Bedit->setText(QString());
    }

    Blayout->addWidget(Bedit);
    Blayout->addWidget(Bok);
    Blayout->addWidget(Bcancel);


    szbox = new Edit(10, 500);
    as_time = new Edit(0, 500);
    as_time->setSpecialValueText(tr("Disabled"));

    Ctranslation = new QComboBox();
    Ctranslation->addItem(tr("English"), QString(":/translation_base"));
    Ctranslation->addItem(tr("Russian"), QString(":/translation_russian"));
    Ctranslation->addItem(tr("Custom..."), QString("translation_russian.qm"));
    connect(Ctranslation, SIGNAL(activated(int)), this, SLOT(selectCustomTR()));

    Boplayout->addRow(tr("Language *"), Ctranslation);
    Boplayout->addRow(tr("Buttons size *"), szbox);
    Boplayout->addRow(tr("Font *"), Bfont);
    Boplayout->addRow(tr("Data directory"), Bdata);
    Boplayout->addRow(tr("Data autosave time: "), as_time);


    ch1 = new QCheckBox(tr("Data") + " *");
    ch2 = new QCheckBox(tr("Spectroscopy") + " *");
    ch3 = new QCheckBox(tr("Scan") + " *");
    ch4 = new QCheckBox(tr("SPM") + " *");
    ch5 = new QCheckBox(tr("Oscilloscope") + " *");
    ch6 = new QCheckBox(tr("Lithography") + " *");
    ch8 = new QCheckBox(tr("Video") + " *");
    ch9 = new QCheckBox(tr("Video") + " 2" " *");

    ch090 = new QCheckBox(tr("About button") + " *");
    ch091 = new QCheckBox(tr("Shuttle Status") + " *");
    ch092 = new QCheckBox(tr("Motors") + " *");

    ch03 = new QCheckBox(tr("Head scanner") + " *");
    ch04 = new QCheckBox(tr("Stage scanner") + " *");
    ch041 = new QCheckBox(tr("Z3 scanner") + " *");
    if (!show_additional_gui() &&
            !show_advanced_gui()) {
        ch9->hide();
        ch041->hide();
    }

    Boplayout2->addWidget(ch1);
    Boplayout2->addWidget(ch2);
    Boplayout2->addWidget(ch3);
    Boplayout2->addWidget(ch4);
    Boplayout2->addWidget(ch5);
    Boplayout2->addWidget(ch092);
    Boplayout2->addWidget(ch8);
    Boplayout2->addWidget(ch9);
    Boplayout2->addWidget(ch6);
    Boplayout2->addWidget(ch090);
    Boplayout2->addSpacing(ch091->sizeHint().height() / 2);
    Boplayout2->addWidget(ch091);
    Boplayout2->addWidget(ch03);
    Boplayout2->addWidget(ch04);
    Boplayout2->addWidget(ch041);


    ch11 = new QCheckBox(tr("Enable Auto-send to Data from Scan"));
    ch12 = new QCheckBox(tr("Enable Sensors autocalibration on start"));
    ch13 = new QCheckBox(tr("Enable Optics initialization on start"));
    ch14 = new QCheckBox(tr("Enable PID autotuning on start"));
    ch15 = new QCheckBox(tr("Show advanced functions") + " *");
    ch16 = new QCheckBox(tr("Show additional functions") + " *");
    ch17 = new QCheckBox(tr("Use sound notifications"));
    ch18 = new QCheckBox(tr("Unlock toolbars"));
    if (!show_additional_gui() &&
            !show_advanced_gui()) {
        ch17->hide();
        ch18->hide();
    }
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        if (!settings.value("display/spectroscopy", false).toBool())
            ch13->hide();
    }

    Boplayout->addRow(ch16);
    Boplayout->addRow(ch15);
    Boplayout->addRow(ch18);
    Boplayout->addRow(ch17);

    Boplayout->addItem(new QSpacerItem(0, 0));

    Boplayout->addRow(ch11);
    Boplayout->addRow(ch12);
    Boplayout->addRow(ch14);
    Boplayout->addRow(ch13);


    Boplayout3->addRow(CBgwy_imp = new QCheckBox(tr("Enable Gwyddion import server") + " *"));
    Boplayout3->addRow(CBremote_net = new QCheckBox(tr("Enable Nspec Light server") + " *"));
    Boplayout3->addRow(CBremote_log = new QCheckBox(tr("Enable Nspec Light access logging")));
    Boplayout3->addRow(CBremote_guest = new QCheckBox(tr("Nspec Light is read-only")));
    Boplayout3->addRow(CBremote_pwd = new QCheckBox(tr("Nspec Light needs password")));
    Boplayout3->addRow(tr("Nspec Light password"), remote_pwd = new QLineEdit);
    remote_pwd->setEchoMode(QLineEdit::PasswordEchoOnEdit); remote_pwd->setPlaceholderText(tr("empty"));
    Boplayout3->addRow(tr("Nspec Light port *"), remote_port = new Edit(1, 65535));


    //config editor
    editor = new QTreeWidget();
    editor->setItemDelegate(new EditorDelegate(this));

    QWidget *configWidget = new QWidget;
    QVBoxLayout *cl1 = new QVBoxLayout(configWidget);
    configWidget->setMinimumSize(800, 600);
    configWidget->setWindowTitle(tr("Configuration"));

    cl1->addWidget(editor);

    QPushButton *cwC = new QPushButton(tr("Close"));
    connect(cwC, SIGNAL(clicked()), configWidget, SLOT(hide()));
    QHBoxLayout *cwL = new QHBoxLayout;
    cwL->addWidget(cwC);
    cwL->addWidget(new QLabel(tr("* - requires restart of program/device to apply")));
    cl1->addLayout(cwL);


    QStringList header;
    header.append(tr("File")+"/"+tr("Variable"));
    header.append(tr("Value"));
    header.append(tr("Comment"));
    editor->setColumnCount(3);
    editor->setHeaderLabels(header);


    //TODO per-module-enabled configs
#ifdef USE_EG3K_EMULATOR
    addConfig("Emulator.conf", "usb", false, "* " + tr("Use EG3000 emulator"));
#endif
#ifdef USE_CCD_EMULATOR
    addConfig("Emulator.conf", "ccd", false, "* " + tr("Use CCD emulator"));
#endif

    { QSettings settings("Param/ccd.conf", QSettings::IniFormat);
        addConfig(settings, "ccd.conf", "/", tr("Device selection, parameters"));
    addConfig(settings, "ccd.conf", "scale_reverse", true, tr("CCD x axis mirror"));
    addConfig(settings, "ccd.conf", "use_pmt", false, "* " + tr("Use internal PMTs"));
    addConfig(settings, "ccd.conf", "use_ormins", false, "* " + tr("Use Ormins CCD"));
    addConfig(settings, "ccd.conf", "use_sol_ccd", false, "* " + tr("Use Sol CCD"));
    addConfig(settings, "ccd.conf", "use_ars", true, "* " + tr("Use ARS3000 confocal module"));
    addConfig(settings, "ccd.conf", "use_m266", false, "* " + tr("Use M266 instead of M833 spectrograph"));
    addConfig(settings, "ccd.conf", "use_shamrock", false, "* " + tr("Use Shamrock spectrograph"));
    addConfig(settings, "ccd.conf", "use_solart2", false, "* " + tr("Use SolarT2 spectrograph"));
    addConfig(settings, "ccd.conf", "baseline_before_intensity", false, "* ");
    addConfig(settings, "ccd.conf", "andor_fast", false, tr("Enable fast Andor mode"));
    addConfig(settings, "ccd.conf", "shutter_open", 30, tr("opening time, ms"));
    addConfig(settings, "ccd.conf", "shutter_close", 30, tr("closing time, ms"));

    if (settings.value("use_ars", true).toBool()) {
        QSettings settings("Param/ars.conf", QSettings::IniFormat);
        addConfig(settings, "ars.conf", "motor_k/", tr("Distance correction factor from phys to virtual(global), mm/mm"));
        for (int i = 0; i < 6; i++)
            addConfig(settings, "ars.conf", "motor_k/"+QString::number(i), 1.);
        addConfig(settings, "ars.conf", "laser_select_pos/", tr("Motor pos for each laser, mm"));
        for (int i = 0; i < 3; i++)
            addConfig(settings, "ars.conf", "laser_select_pos/"+QString::number(i), i*10);
        addConfig(settings, "ars.conf", "slit_k/", tr("Slit correction factor from width to motor dist, mm/mm"));
        for (int i = 0; i < 2; i++)
            addConfig(settings, "ars.conf", "slit_k/"+QString::number(i), 1);
        addConfig(settings, "ars.conf", "slit_zeropos/", tr("Slit dist from limit switch to zero pos, mm"));
        for (int i = 0; i < 2; i++)
            addConfig(settings, "ars.conf", "slit_zeropos/"+QString::number(i), 0.); //fuck msvc
    }
    }

    { QSettings settings("Param/controller.conf", QSettings::IniFormat);
        addConfig(settings, "controller.conf", "/", tr("Device selection, limits, autocalib parameters, etc."));
    addConfig(settings, "controller.conf", "use_eg4000", true, tr("Enable libusb - EG4000 and later"));
    addConfig(settings, "controller.conf", "csb_speed_settings_idx", 0, "* " + tr("sensor rate index in rate table"));
    addConfig(settings, "controller.conf", "double_c_output", 0, "* " + tr("C output with two sensors on channel. 0=C1-C2, 1=C1, 2=C2"));
    addConfig(settings, "controller.conf", "limit_scan_rate", 10, tr("Maximum scan rate, Hz"));
    addConfig(settings, "controller.conf", "sensors_autocalib_gain", 100, tr("Gain to use in calibration process"));
    addConfig(settings, "controller.conf", "sensors_autocalib_reserve", 0.025, tr("% of overcalibration, 1.0 = 100%"));
    QString ctrps = tr("Center position with account for nonlinearity");
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_x_h", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_y_h", 0.48, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_z_h", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_x_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_y_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_z_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_optical_range_correction", 1.1, tr("divider for optical sensor ranges"));
    addConfig(settings, "controller.conf", "auto_i_divisor", 500, tr("I = P/divisor - when I is auto-adjusted"));
    addConfig(settings, "controller.conf", "pi_autocalib_p_scale", 0.45, tr("Multiplier for P"));
    addConfig(settings, "controller.conf", "pi_autocalib_i_scale", 10, tr("Divisor for I"));
    addConfig(settings, "controller.conf", "pi_autocalib_spm_p_scale", 0.44, tr("Multiplier for P"));
    addConfig(settings, "controller.conf", "pi_autocalib_spm_i_scale", 2, tr("Divisor for I"));
    addConfig(settings, "controller.conf", "hvsplit/z2_k", 0, tr("scale from slow to fast. 0 means disabled"));
    addConfig(settings, "controller.conf", "hvsplit/z2_freq", 1, tr("lpf freq khz"));
    }

    { QSettings settings("Param/motor.conf", QSettings::IniFormat);
    addConfig(settings, "motor.conf", "csb_motors_addr", -1, tr("-1, 0 - leave default, 1-head, 2-stage, 3-z3, 5-motors"));
    addConfig(settings, "motor.conf", "divider/123", 1, tr("Landing motor physical/logical step divider"));
    }

    QString mult = tr("Mult for shuttle position ");
    addConfig("shuttle.conf", "shuttle_xpress", false, "* " + tr("Enable small model buttons"));
    addConfig("shuttle.conf", "piezo/delay", 50, "* " + tr("Delay between steps, ms"));
    addConfig("shuttle.conf", "piezo/step", 1e-1, "* " + tr("Base step size, mkm"));
    addConfig("shuttle.conf", "piezo/scale0", 1e-1, "* " + mult + "1");
    addConfig("shuttle.conf", "piezo/scale1", 1e-1, "* " + mult + "2");
    addConfig("shuttle.conf", "piezo/scale2", 1e-1, "* " + mult + "3");
    addConfig("shuttle.conf", "piezo/scale3", 1, "* " + mult + "4");
    addConfig("shuttle.conf", "piezo/scale4", 2, "* " + mult + "5");
    addConfig("shuttle.conf", "piezo/scale5", 5, "* " + mult + "6");
    addConfig("shuttle.conf", "piezo/scale6", 10, "* " + mult + "7");
    addConfig("shuttle.conf", "remap/", tr("-1 - no remap\n 0..14 button from original mapping\n 101.. motor (index-100)\n 250 land motors\n 251.. piezo channels"));
    for (int i = 0; i < 15; i++)
        addConfig("shuttle.conf", "remap/"+QString::number(i), -1);

    {
        QSettings settings("Param/laser.conf", QSettings::IniFormat);
        if (settings.allKeys().size())
            addConfig(settings, "laser.conf", "/", tr("See readme.txt"));

        foreach (QString key, settings.allKeys())
            addConfig(settings, "laser.conf", key);
    }

    { QSettings settings("Param/analog.conf", QSettings::IniFormat);
        addConfig(settings, "analog.conf", "/", tr("Analog inputs/outputs calibrations"));
    QString smin = tr("Min/max voltage for ");
    QString sadc = tr("ADC");
    QString sdac = tr("DAC");
    QStringList l, c;
    l << "1" << "1mag" << "2" << "2mag" << "3" << "4" << "5" << "1hr" << "2hr";
    c << "1/DFL" << "1-MAG" << "2/LFL/I_TIP" << "2-MAG" << "3/INT" << "4" << "5" << "AUX 1" << "AUX 2";
    addConfig(settings, "analog.conf", "adc/", smin + sadc);
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "adc/v" + l.at(i) + "_min", -10, c.at(i));
        addConfig(settings, "analog.conf", "adc/v" + l.at(i) + "_max", 10, c.at(i));
    }

    addConfig(settings, "analog.conf", "dac/", smin + sdac);
    addConfig(settings, "analog.conf", "dac/vdrive_min", 0, "* " + tr("Drive"));
    addConfig(settings, "analog.conf", "dac/vdrive_max", 10, "* " + tr("Drive"));

    l.clear(); c.clear();
    l << "tip" << "1" << "2" << "3hr" << "4hr";
    c << tr("Tip") << "1" << "2" << "AUX 1" << "AUX 2";
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "dac/v" + l.at(i) + "_min", -10, "* " + c.at(i));
        addConfig(settings, "analog.conf", "dac/v" + l.at(i) + "_max", 10, "* " + c.at(i));
    }
    }

    editor->header()->resizeSections(QHeaderView::ResizeToContents);

    connect(editor, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(configEdited(QTreeWidgetItem*,int)));
    connect(editor, SIGNAL(collapsed(QModelIndex)), this, SLOT(editorResize()));
    connect(editor, SIGNAL(expanded(QModelIndex)), this, SLOT(editorResize()));

    connect(Bedit, SIGNAL(clicked()), configWidget, SLOT(show()));
    connect(Bedit, SIGNAL(clicked()), Bok, SLOT(click()));
}

void TOptionsWindow::addConfig(QString file, QString path, QString comment)
{
    addConfig(file, path, QVariant(), comment);
}
void TOptionsWindow::addConfig(QString file, QString path, QVariant default_val, QString comment)
{
    QSettings settings("Param/" + file, QSettings::IniFormat);
    addConfig(settings, file, path, default_val, comment);
}
void TOptionsWindow::addConfig(QSettings &settings, QString file, QString path, QString comment)
{
    addConfig(settings, file, path, QVariant(), comment);
}
void TOptionsWindow::addConfig(QSettings &settings, QString file, QString path, QVariant default_val, QString comment)
{
    QVariant value = settings.value(path, default_val);

    QStringList pathparts = path.split("/");
    QString lastpart = pathparts.last();
    pathparts.removeLast();
    pathparts.prepend(file);

    QTreeWidgetItem *root;
    QString acc;
    while (!pathparts.isEmpty()) {
        if (!items.contains(acc + pathparts.first())) {
            if (acc.isEmpty())
                root = new QTreeWidgetItem(editor, QStringList() << file);
            else
                root = new QTreeWidgetItem(root, QStringList() << pathparts.first());

            items.insert(acc + pathparts.first(), root);
        } else
            root = items.value(acc + pathparts.first());

        acc += pathparts.first();
        pathparts.removeFirst();
    }
    if (lastpart.isEmpty()) {
        root->setText(2, comment);
        return;
    }

    QTreeWidgetItem *item = new QTreeWidgetItem(root, QStringList() << lastpart << value.toString() << comment);
    item->setData(0, Qt::UserRole, file);
    item->setData(0, Qt::UserRole+1, path);
    item->setData(0, Qt::UserRole+3, default_val);

    if (value.toString() == "false" || value.toString() == "true") {
        item->setCheckState(1, value.toBool() ? Qt::Checked : Qt::Unchecked);
        item->setData(1, Qt::DisplayRole, QVariant());
        item->setData(0, Qt::UserRole+2, true);
    } else
        item->setFlags(item->flags() | Qt::ItemIsEditable);
}

void TOptionsWindow::editorResize()
{
    editor->header()->resizeSections(QHeaderView::ResizeToContents);
}


void TOptionsWindow::configEdited(QTreeWidgetItem *item, int)
{
    QSettings settings("Param/" + item->data(0, Qt::UserRole).toString(), QSettings::IniFormat);
    QString path = item->data(0, Qt::UserRole+1).toString();
    QVariant default_val = item->data(0, Qt::UserRole+3);
    if (!item->data(0, Qt::UserRole+2).toBool()) { // !bool
        if (item->text(1).isEmpty() || item->text(1) == default_val.toString()) {
            item->setText(1, default_val.toString());
            settings.remove(path);
        } else
            settings.setValue(path, item->text(1));
    } else {
        if ((item->checkState(1) == Qt::Checked) == default_val.toBool())
            settings.remove(path);
        else
            settings.setValue(path, item->checkState(1) == Qt::Checked);
    }
}

void TOptionsWindow::BFont()
{
    bool ok;
    QFont font_tmp = QFontDialog::getFont(&ok, font, this);

    if (ok) {
        font = font_tmp;
    }
}
void TOptionsWindow::BData()
{
    QString newdir = QFileDialog::getExistingDirectory(this, tr("Default data directory"), datadir,
                                                       QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog);

    if (!newdir.isEmpty())
        datadir = newdir;
}

bool checkSetConfig(QSettings &settings, QString name, QVariant def_v, QVariant new_v)
{
    if (settings.value(name, def_v) != new_v) {
        if (new_v == def_v)
            settings.remove(name);
        else
            settings.setValue(name, new_v);
        return true;
    }
    return false;
}

void TOptionsWindow::Apply()
{
    bool restart = false;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    restart = checkSetConfig(settings, "net/gwy_import_srv", true, CBgwy_imp->isChecked()) || restart;
    restart = checkSetConfig(settings, "net/srv_on", false, CBremote_net->isChecked()) || restart;
    settings.setValue("net/srv_guest", CBremote_guest->isChecked());
    settings.setValue("net/srv_log", CBremote_log->isChecked());
    settings.setValue("net/srv_pwd_on", CBremote_pwd->isChecked());

    settings.setValue("net/srv_pwd", remote_pwd->text());
    restart = checkSetConfig(settings, "net/srv_port", 33331, remote_port->value()) || restart;

    //TODO table/loop
    restart = checkSetConfig(settings, "display/data", true, ch1->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/spectroscopy", false, ch2->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/scan", true, ch3->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/spm", true, ch4->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/motors", false, ch092->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope", true, ch5->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/video", true, ch8->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/video2", false, ch9->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/lithography", false, ch6->isChecked()) || restart;

    restart = checkSetConfig(settings, "display/oscilloscope/head", true, ch03->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope/stage", true, ch04->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope/z3", false, ch041->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/about", true, ch090->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/shuttle_status", true, ch091->isChecked()) || restart;

    settings.setValue("scan/autosend_data", ch11->isChecked());
    settings.setValue("autosave_time", as_time->value());

    settings.setValue("autocalib_sensors", ch12->isChecked());
    settings.setValue("autoinit_ccd", ch13->isChecked());
    settings.setValue("autocalib_pid", ch14->isChecked());

    //TODO reloadConfig() signal
    if (I_MODULE *scanm = ModuleManager::instance->find1("scan")) {
        ScanPlot *scan = (ScanPlot *)scanm->basecast();
        scan->Autosend = ch11->isChecked();
    }
    if (I_MODULE *spmm = ModuleManager::instance->find1("spm")) {
        SPMTunning *spm = (SPMTunning *)spmm->basecast();
        spm->AutoSend = ch11->isChecked();
    }
    if (I_MODULE *ccdm = ModuleManager::instance->find1("ccdplot")) {
        CCDPlot *ccd = (CCDPlot *)ccdm->basecast();
        ccd->AutoSend = ch11->isChecked();
    }
    if (I_MODULE *datam = ModuleManager::instance->find1("data")) {
        DataPlot *data = (DataPlot *)datam->basecast();
        data->Container->SetAutoSaveTime(as_time->value()*60*1000);
    }


    restart = checkSetConfig(settings, "show_additional", false, ch16->isChecked()) || restart;
    restart = checkSetConfig(settings, "show_advanced", false, ch15->isChecked()) || restart;
    if (checkSetConfig(settings, "gui_unlock", false, ch18->isChecked()))
    {
        MainWindow *mainWindow = (MainWindow *)ModuleManager::instance->find1("mainwindow");
        foreach (QToolBar *t, mainWindow->findChildren<QToolBar *>())
            t->setMovable(settings.value("gui_unlock", false).toBool());
    }
    settings.setValue("sound_notification", ch17->isChecked());

    if (font != qApp->font()) {
        settings.setValue("font", font.toString());
        restart = true;
    }
    if (datadir != settings.value("data_dir", TContainer::getDataDir()).toString())
        settings.setValue("data_dir", datadir);


    if (Ctranslation->itemData(Ctranslation->currentIndex()) !=
            settings.value("translation", QString(":/translation_base")).toString())
    {
        settings.setValue("translation", Ctranslation->itemData(Ctranslation->currentIndex()));
        restart = true;
    }

    int Bsize = szbox->value();
    if (Bsize != qApp->property("buttonsize").toInt()) {
        settings.setValue("buttonsize", Bsize);
        restart = true;
    }

    if (restart) {
        new Notification(tr("Settings") + ": " + tr("Please restart application to apply changes."), Notification::Good, 10);
        doBeep();
    }

    hide();
}

void TOptionsWindow::Show()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    CBgwy_imp->setChecked(settings.value("net/gwy_import_srv", true).toBool());
    CBremote_net->setChecked(settings.value("net/srv_on", false).toBool());
    CBremote_guest->setChecked(settings.value("net/srv_guest", true).toBool());
    CBremote_log->setChecked(settings.value("net/srv_log", true).toBool());
    CBremote_pwd->setChecked(settings.value("net/srv_pwd_on", false).toBool());

    remote_pwd->setText(settings.value("net/srv_pwd", "").toString());
    remote_port->setValue(settings.value("net/srv_port", 33331).toInt());

    ch1->setChecked(settings.value("display/data", true).toBool());
    ch2->setChecked(settings.value("display/spectroscopy", false).toBool());
    ch3->setChecked(settings.value("display/scan", true).toBool());
    ch4->setChecked(settings.value("display/spm", true).toBool());
    ch092->setChecked(settings.value("display/motors", false).toBool());
    ch5->setChecked(settings.value("display/oscilloscope", true).toBool());
    ch8->setChecked(settings.value("display/video", true).toBool());
    ch9->setChecked(settings.value("display/video2", false).toBool());
    ch6->setChecked(settings.value("display/lithography", false).toBool());

    ch03->setChecked(settings.value("display/oscilloscope/head", true).toBool());
    ch04->setChecked(settings.value("display/oscilloscope/stage", true).toBool());
    ch041->setChecked(settings.value("display/oscilloscope/z3", false).toBool());
    ch090->setChecked(settings.value("display/about", true).toBool());
    ch091->setChecked(settings.value("display/shuttle_status", true).toBool());

    ch11->setChecked(settings.value("scan/autosend_data", true).toBool());
    as_time->setValue(settings.value("autosave_time", 30).toInt());

    ch12->setChecked(settings.value("autocalib_sensors", false).toBool());
    ch13->setChecked(settings.value("autoinit_ccd", false).toBool());
    ch14->setChecked(settings.value("autocalib_pid", false).toBool());

    ch15->setChecked(settings.value("show_advanced", false).toBool());
    ch16->setChecked(settings.value("show_additional", false).toBool());
    ch17->setChecked(settings.value("sound_notification", true).toBool());
    ch18->setChecked(settings.value("gui_unlock", false).toBool());

    font = qApp->font();
    datadir = settings.value("data_dir", TContainer::getDataDir()).toString();

    szbox->setValue(qApp->property("buttonsize").toInt());

    QString transl = qApp->property("translation").toString();

    if (transl.endsWith(":/translation_base"))
        Ctranslation->setCurrentIndex(0);
    else if (transl.endsWith(":/translation_russian"))
        Ctranslation->setCurrentIndex(1);
    else {
        Ctranslation->setCurrentIndex(2);
        Ctranslation->setItemData(2, transl);
    }

    show();
}
