#pragma once

#include <QWidget>
#include <QElapsedTimer>
#include <QPointer>
#include "controls.h"
#include "edit.hpp"
#include "plot.h"
#include "defs.h"
#include "xyzcalibrations.hpp"
#include "tools.hpp"

class TRectWidget: public QWidget
{
    Q_OBJECT
signals:
    void clicked();

public:
    QColor m_Colour;
    void SetColour(const QColor &c) { m_Colour = c; update(); }

    TRectWidget(const QColor &c = Qt::black): m_Colour(c) { setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed); setMinimumSize(5, 7); }

    void paintEvent(QPaintEvent *) { QPainter p(this); p.fillRect(rect(), m_Colour); }
    void mouseDoubleClickEvent(QMouseEvent *) { emit clicked(); }
};

void fillCBchannels_(QComboBox *box, bool want_xyz = true);
void changeCBname(int srcidx, QString name);


class XYZCalibrations;
class DacsControl;
class AutoCalibPlot;
class AutoCalib;
class Notification;
class Oscilloscope: public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT

public:
    Oscilloscope(QWidget *parent = 0);

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size);
    bool sendData(const void *handle);
    void resetWriter();

    TMessageFlag sendOscChannels, sendOscOnOff;
    bool osc_on;

    void get_curr_data(int ch);
    TMessageFlag doGetData;
    int getDataCh;

public slots:
    void s_setT(int t);
    void sl_osc_start();
    void sl_osc_stop();
    void slot_Osc_Start_Stop(bool);
    void change_ch();
    void setChA(int);
    void newOscData(QByteArray data);
    void GUIRefresh();
    void on_MyPl_CurveColourChanged(QColor, int);
    void colorClick();
    void sett_showhide();

    void reset();
    void calib_plot_change(bool);
    void inject_test_packet();

    void device_status(int code);

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);

signals:
    void newOscData_(QByteArray);
    void oscRefreshed();

    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

public:
    void GUICreate();
    void closeEvent(QCloseEvent *event);
    QPointer<Notification> device_status_notice;


    TwoStateButton *start;
    Edit *SBT;
    QPushButton *BToCalib;
    TwoStateButton *PBCalibMode;
    TwoStateButton *PBToNewWindow;
    QPushButton *BShowHideSett;
    QCheckBox *CBShowRelative;
    bool bshowrelative;

    plot::T2dPlot *MyPl;
    XYZCalibrations *CalibWidget;
    DacsControl *DacsWidget;
    AutoCalibPlot *acp;
    AutoCalib *acf;
    QStackedWidget *OscStackedWidget;

    QComboBox *CBChannel[4];
    TRectWidget *LEdColour[4];
    QLabel *LAverage[4];
    QLabel *LStDev[4];

    QLineEdit *LEdAverage[4];
    QLineEdit *LEdStDev[4];

    double k_m_nm[4];
    int osc_data_idx[4];
    int last_osc_ch;

    plot::TCurvesDataOwner curveData;
    long current_t_count;
    int last_updated_idx;
    int oscMaxLength;
    int OscLineLength;
    int oscRate;
    double time_dilation;
    QElapsedTimer refreshRateLimit;
    QByteArray lsbData;

    bool eventFilter(QObject *Object, QEvent *Event);
};


class QStackedLayout;
class DacsControl: public QWidget, public IWriter
{
    Q_OBJECT

public slots:
    DoubleChanger *get_dacs_channel(QComboBox *CB);
    void fill_dacs_channel_CB(QComboBox *CB);
    void s_dacs_channel(int ch);

    void sl_xyz(double, double, double);
    void sl_xyz2(double, double, double);
    void sl_xyz3(double, double, double);

    void go_to_center(int scanner);

    void Go_to_XY(double, double);
    void Go_to_XY2(double, double);
    void Go_to_Z(double);
    void Go_to_Z2(double);
    void Go_to_Z3(double);
    void Go_to_X(double);
    void Go_to_X2(double);
    void Go_to_Y(double);
    void Go_to_Y2(double);

    void set_Ex_dac1(double);
    void set_Ex_dac2(double);
    void set_Ex_dac3(double);
    void set_Ex_dac4(double);

signals:
    void xUpdate(double);
    void x2Update(double);
    void yUpdate(double);
    void y2Update(double);
    void zUpdate(double);
    void z2Update(double);
    void z3Update(double);

public:
    DacsControl(QWidget *parent = 0);

    bool sendData(const void *handle);
    void resetWriter();

    TMessageFlag sendSP[3];
    double Xsp, Ysp, Zsp;
    double Xsp2, Ysp2, Zsp2;
    double Zsp3;

    TMessageFlag sendExDAC[4];
    double Ex_dac[4];


    QComboBox *DacsChannel;
    QGroupBox *box_Ex_Dac1;
    QStackedLayout *layout_Ex_Dac;

    //takes ownership of dc
    void insertChannel(QString ch_name, QString unit, DoubleChanger *dc, int idx = -1);
    QVector<DoubleChanger *> dacs;
    QVector<QString> units;

    DoubleChanger *Extra_Dac_1;
    DoubleChanger *Extra_Dac_2;
    DoubleChanger *Extra_Dac_3;
    DoubleChanger *Extra_Dac_4;
    DoubleChanger *X;
    DoubleChanger *Y;
    DoubleChanger *Z;
    DoubleChanger *X2;
    DoubleChanger *Y2;
    DoubleChanger *Z2;
    DoubleChanger *Z3;
    DoubleChanger *Zobj;
};
