#pragma once

#include <plot.h>
#include "controls.h"
#include "edit.hpp"
#include <defs.h>
#include "tools.hpp"
#include <QPointer>
#include <QElapsedTimer>
class QCheckBox;

class IExternalScan : public QObject
{
    Q_OBJECT

public:
    IExternalScan() : displayPlot(0) {
        started = false;
        plot::ResizeOwner(Output3d, 1);
        plot::ResizeOwner(Section, 1);
        Section[0]->Pen.setColor(Qt::red);
    }
    virtual ~IExternalScan() {}
    void SetDirStPoint(int _dir, int _stpoint) { dir = _dir; start_point = _stpoint; }
    void SetStPointDirection(int value) {
        switch(value) {
        case spTopLeftAxisX: SetDirStPoint(Horiz, TopLeft); break;
        case spTopRightAxisX: SetDirStPoint(Horiz, TopRight); break;
        case spBottomRightAxisX: SetDirStPoint(Horiz, BottomRight); break;
        case spBottomLeftAxisX: SetDirStPoint(Horiz, BottomLeft); break;
        case spTopLeftAxisY: SetDirStPoint(Vert, TopLeft); break;
        case spTopRightAxisY: SetDirStPoint(Vert, TopRight); break;
        case spBottomRightAxisY: SetDirStPoint(Vert, BottomRight); break;
        case spBottomLeftAxisY: SetDirStPoint(Vert, BottomLeft); break;
        }
    }
    enum TDirections { Vert, Horiz };
    enum TStartPoints { BottomLeft, BottomRight, TopLeft, TopRight };

    virtual bool init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy) {
        nx = _nx;
        ny = _ny;
        dir_stpoint = _dir_stpoint;
        SetStPointDirection(dir_stpoint);
        sx = _sx;
        sy = _sy;
        started = true;

        clear();

        Output3d.at(0)->XLimits = sx;
        Output3d.at(0)->YLimits = sy;
        Output3d.at(0)->setSize(nx, ny);

        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(false);

        return true;
    }
    virtual void finish() {
        started = false;
        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(true);
    }
    virtual void save() { }
    virtual void clear() {
        lastError.clear();
        Output3d.at(0)->clear();
        Section[0]->clear();
    }
    virtual int getDataChannel() { return srcCCD; }
    virtual void updateData() { emit updated(); }

    virtual QWidget *createInterfaceWidget() { return 0; }

    virtual bool acquireNextPoint() = 0;
signals:
    void acquired();
    void updated();
    void setSectionEnabled(bool);

public:
    int nx, ny, dir, start_point, dir_stpoint;
    Range sx, sy;
    bool started;
    plot::T3dPlot *displayPlot;

    //data interface
    QString lastError;
    plot::TCurvesDataOwner Section;
    plot::T3dDataOwner Output3d;
    plot::TMetaStorage attributes;
    QVector<QWidget *> disableWidgets;
};

class SPMTunning;
IExternalScan *getNewJMIExtScan(SPMTunning *);

class QwtThermo;
class StatusBar;
class TTabWidget;
class ScanPlot: public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT

signals:
    void sig_XYPosition(double x, double y);
    void sig_XYPosition2(double x, double y);

    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

    //litho
    void SendBackground(const QImage &Img);
    void SendRect(const QRectF &Rect);

    void recalibrateSensors(int);

public:
    BYTE scan_on_off;
    float scan_lm_h;
    double Step_X;
    double Step_Y;
    BYTE scan_dir_st_point;
    BYTE scan_ch[4];

    bool LmEn, LmEn_ch[4];
    double LmHeight;

    bool scan_precise_mode;
    int scanner_scan;
    int ccdNX,ccdNY;
    int ccdMode;

    WORD m_spm_pid_precision;
    TMessageFlag updateSPMPrecision;
    WORD m_scan_first_delay;
    TMessageFlag updateFirstDelay;
    WORD m_scan_line_delay;
    TMessageFlag updateLineDelay;
    WORD m_scan_point_delay;
    TMessageFlag updatePointDelay;
    bool m_scan_fast_return;
    TMessageFlag updateFastReturn;
    int m_scan_sync_mode;
    TMessageFlag updateSyncMode;

    TMessageFlag sendSpeed, sendLMheight, sendConfig, sendExtConfig, sendStartStop, sendExtStartStop, sendExtReady;

public slots:
    void reset_scan_test();
    void injectTestData();

    void scan_first_delay(double time);
    void scan_line_delay(double time);
    void scan_point_delay(double time);
    void spm_pid_precision(double delta);
    void scan_fast_return(bool);
    void scan_sync_mode(int);
    void scan_lm_height(double);

public:
    Q_SIGNAL void sync_scan_x(double);
    Q_SIGNAL void sync_scan_y(double);
    Q_SIGNAL void sync_scan_x2(double);
    Q_SIGNAL void sync_scan_y2(double);
    void sendXYsync(double x, double y, int sc);

public slots:
    void reset();
    void capBits(int, long);
    void s_scanError();
    void recalibDone(bool);

    bool sl_fast_scan_correction(int ch);
    void sl_ScanCorrection();
    void sl_motor_level();

    void Send2dPlotData();
    void Send3dPlotData1();
    void Send3dPlotData2();
    void Send3dPlotData3();
    void Send3dPlotData4();
    void BSendToGraphEditor_clicked();

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);
    void FillDataAttributes();
    void FillDataAttributes2();
    void restoreAttributes(plot::TMetaStorage);
    void slAutosaveData();

    void sl_setCurrentX(double v);
    void sl_setCurrentY(double v);
    void sl_setCurrentX2(double v);
    void sl_setCurrentY2(double v);
    void sl_syncXY();
    void sl_Update_XYZ_pos();
    void sendXYPos();
    void onLEdDeltaX_editingFinished();
    void onLEdDeltaY_editingFinished();
    void sl_set_XYZ_range(double, double, double);
    void sl_set_XYZ_range2(double, double, double);
    void sl_set_XYZ_range_sync();
    void slUpdZscanner(double zpos);

    void updateDirection();
    void check_nx_ny();
    void check_speed();
    void onBApply_clicked();

    void scan_start_stop(bool);
    void s_stopScan();
    void sl_stop_scan();
    bool removeOldData(int ch);
    void sl_update_progress(QByteArray data);
    void sl_scan_refresh_ch(int n);

    void acquireNextPoint();
    void sl_ext_scan_acquired();
    void sl_ext_scan_update();
    void sl_ext_scan_refresh();
    void sl_ext_mode();
    void sl_dacscan();
    void sl_dacscan_advance();
    bool is_dacscan();

    void setScanner(int sc);
    void LmOnOff(bool value);
    void sl_CB_update();
    void sl_CB_filters_update();

    void sl_section_legend(int);
    void sl_set_section_enabled(bool);
    void SetSectionUnits();

public:
    ScanPlot(QWidget *parent = 0);
    ~ScanPlot();

    void GUICreate();
    void closeEvent(QCloseEvent *event);
    bool eventFilter(QObject *Object, QEvent *Event);

    bool sendData(const void *handle);
    void resetWriter();

    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size);
    void Fill_Data_Array(plot::T2dArray& Array, const EG_ScanData *Scan_Data, const EG_ScanData *lsb_Scan_Data = 0, plot::TCurveData *Section = 0, bool raw = false);
    QByteArray lsbData;
    EG_ScanData *ScanDataPart[8];
    EG_ScanData *ScanDataPart_lsb[8];


    QVector<QWidget *> scanControlWidgets; //disabled on scan
    QPointer<StatusBar> scan_bar;
    QPointer<StatusBar> ccd_scan_bar;
    bool scanFailed;

    bool Autosend;
    QTimer *AutosaveTimer;
    QTimer *RefreshTimer4d;
    QElapsedTimer refresh_3d_limit_timer[4];
    qint64 refresh_limit_last_elapsed[4];
    bool refresh_limit_need_math_update[4];
    int channelsActive;
    unsigned int Ch_activated;

    QGroupBox *GrBDACscan;
    QComboBox *CBDACscanCh;
    Edit *EDACcount;
    Edit *EDACstep;
    double DACinitialValue;
    int DACcurrentNum;


    QComboBox *CBChannel[4];
    QCheckBox *CBSec[4];
    bool sectionEnabled;

    bool l2k_controller, l2k_controller_v22;
    bool is_optical_scanner[3];

    double requested_scan_speed; //to hw
    double user_scan_speed;
    QLabel *LScanRateWarn;
    DoubleChanger *DCScanRate;

    Edit *LEdX;
    Edit *LEdY;
    Edit *LEdDeltaX;
    Edit *LEdDeltaY;
    double x, y, x2, y2;
    double Xmax, Ymax;
    double Xmax2, Ymax2;
    QwtThermo *Zposition;
    QLabel *ZposValue;

    double xLeft, xRight, yTop, yBottom;
    unsigned short int Nx_points, Ny_points;
    int line_number[4];
    QwtPlotMarker *scanLineInd[4];
    bool ready[4];
    int nlcupd[4];
    bool scan_direction;
    double stPointX, stPointY;

    TTabWidget *tabgroup1;
    QWidget *BoxExternal;
    QComboBox *CBExternalMode;
    TwoStateButton *BCCDXYLock;
    QComboBox *CBPointsOYext;
    QComboBox *CBPointsOXext;
    QVector<IExternalScan *> externalScans;
    QVector<QWidget *> externalScanWidgets;
    IExternalScan *currExtScan;
    void addExternalScan(QString name, IExternalScan *iface);

    BYTE fast_scan_correction;
    bool fast_scan_CLZ_disabled;


    DoubleChanger *DCfbP;
    DoubleChanger *DCfbI;
    DoubleChanger *DCfbSetPoint;

    DoubleChanger *spmPrecision;

    DoubleChanger *firstDelay;
    DoubleChanger *lineDelay;
    DoubleChanger *pointDelay;

    QCheckBox *fastReturn;
    QComboBox *pointSync;

    QLabel *LScanner;
    QComboBox *CBScanner;
    QComboBox *CBPointsOY;
    QComboBox *CBPointsOX;
    void Set_CBPoints(QComboBox *box);

    QComboBox *CBStartPos;


    plot::T3dPlot *Plot;

    plot::TMetaStorage scan_attributes;
    plot::T3dDataOwner *dataScan;
    plot::T3dDataOwner *dataScan_src;
    plot::T2dArray *dataScan_X;
    plot::T2dArray *dataScan_Xb;
    plot::T2dArray *dataScan_Y;
    plot::T2dArray *dataScan_Yb;

    plot::T3dDataOwner scanData[2][4];
    plot::TCurvesDataOwner Section_data;


    QScrollArea *LeftScroll;

    QLabel *lb_ccd_scan;
    QLabel *lb_ccd_scan_sm;

    TwoStateButton *LiftMode;
    QCheckBox *LmSwitch[4];
    QLabel *LblLiftMode;
    Edit *LEd_LFmodeHeight;

    TwoStateButton *PBToNewWindow;
    TwoStateButton *m_BStart;
    QToolButton *FastScanCorrection;
    TwoStateButton *BPreciseScan;
    TwoStateButton *BContinuousScan;
    TwoStateButton *BXYLock;
    QPushButton *BMotorLevel;

};
