#include "spmtunning.h"
#include "spm_motors.hpp"
#include "edit.hpp"

#include <QTimer>
#include <QApplication>
#include <QCheckBox>
#include <QGroupBox>
#include <QComboBox>
#include <QVBoxLayout>
#include <QSettings>
#include <QProgressBar>
#include <QFormLayout>
#include "flowlayout.hpp"

#include "hardware/EG3000.h"
#include "gui/status.hpp"
#include "remote_nst_dataid.hpp"
#include "mobile/remote_mobile.hpp"
#include "hardware/shuttle.h"
#include "motors.hpp"


typedef QPair<int, int> FBinfo;
Q_DECLARE_METATYPE(FBinfo)


TOptionsForMotor::TOptionsForMotor()
{
    LEdUpSpeed = new Edit(0, 10000);
    LEdUpDist = new Edit(0.1, 100, 1);
    connect(LEdUpSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdUpDist, SIGNAL(newValue()), this, SLOT(sl_update()));

    LEdLandingSpeed = new Edit(0, 1000);
    LEdLandingSpeedClose = new Edit(0, 1000);
    LEdLandingSpeedCloseC = new Edit(0, 1, 2);
    connect(LEdLandingSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdLandingSpeedClose, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdLandingSpeedCloseC, SIGNAL(newValue()), this, SLOT(sl_update()));


    CB_landing_up = new QCheckBox(tr("Land upward"));
    connect(CB_landing_up, SIGNAL(clicked(bool)), this, SLOT(sl_update()));

    QVBoxLayout *HBLParams = new QVBoxLayout(this);
    HBLParams->setSizeConstraint(QLayout::SetFixedSize);
    HBLParams->setSpacing(3);
    HBLParams->setContentsMargins(6, 3, 6, 6);

    QGroupBox *GrBUp = new QGroupBox(tr("Various movements"), this);
    QFormLayout *VBLUpSpeed = makeFormLayout(GrBUp);

    QGroupBox *GrBLanding = new QGroupBox(tr("Landing"), this);
    QFormLayout *VBLLanding = makeFormLayout(GrBLanding);

    HBLParams->addWidget(GrBLanding);
    HBLParams->addWidget(GrBUp);

    VBLUpSpeed->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEdUpSpeed);
    VBLUpSpeed->addRow(tr("Distance (mm)"), LEdUpDist);
    VBLUpSpeed->setContentsMargins(6, 3, 6, 6);

    VBLLanding->addRow(CB_landing_up);
    VBLLanding->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEdLandingSpeed);
    VBLLanding->addRow(tr("Slow Speed (")+tr("micro")+tr("m/s)"), LEdLandingSpeedClose);
    VBLLanding->addRow(tr("Slow SP %[0..1]"), LEdLandingSpeedCloseC);
    VBLLanding->setContentsMargins(6, 3, 6, 6);

    setWindowTitle(tr("Motor Options - NSpec"));
}
void TOptionsForMotor::sl_save_to_file()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    settings.setValue("up_speed", up_speed);
    settings.setValue("up_dist", up_dist / 1e3); //from mkm

    settings.setValue("land_speed", land_speed);
    settings.setValue("land_speed_close", land_speed_close);
    settings.setValue("land_close_perc", land_close_perc);
    settings.setValue("landing_upward", landUpward);
}
void TOptionsForMotor::sl_load_from_file()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);

    up_speed = settings.value("up_speed", 500).toDouble();
    up_dist = settings.value("up_dist", 1).toDouble() * 1e3; //to mkm

    land_speed = settings.value("land_speed", 50).toDouble();
    land_speed_close = settings.value("land_speed_close", 5).toDouble();
    land_close_perc = settings.value("land_close_perc", 0.7).toDouble();
    landUpward = settings.value("landing_upward", false).toBool();

    LEdUpSpeed->setValue(up_speed);
    LEdUpDist->setValue(up_dist / 1e3); //from mkm

    LEdLandingSpeed->setValue(land_speed);
    LEdLandingSpeedClose->setValue(land_speed_close);
    LEdLandingSpeedCloseC->setValue(land_close_perc);
    CB_landing_up->setChecked(landUpward);
}
void TOptionsForMotor::sl_update()
{
    up_speed = LEdUpSpeed->value();
    up_dist = LEdUpDist->value() * 1e3; //to mkm

    land_speed = LEdLandingSpeed->value();
    land_speed_close = LEdLandingSpeedClose->value();
    land_close_perc = LEdLandingSpeedCloseC->value();
    landUpward = CB_landing_up->isChecked();

    sl_save_to_file();
}

BYTE SPMTunning::getLandingMode()
{
    QList<int> l = Motors::getRoleMotors(Motors::RoleZ1Land);
    if (l.size())
        return l.contains(1) || l.contains(2) ? lm04 : lm01;

    //legacy compat
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    return settings.value("land_mode", 4).toInt();// != lm01 ? lm04 : lm01;
}



void SPMTunning::slot_m123stopped()
{
    last_command = 0;
}

void SPMTunning::motors_stop()
{
    motors->goStop(250); //123 motors
    motors->goStop(4); //4 motor
}

void SPMTunning::slot_DoMoveUp()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = getLandingMode() != lm01 ? 250 : 3;
    if (last_command == 1) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = 1;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goUp(CurrentMotor, OptionsForMotor->up_speed * k, OptionsForMotor->up_dist * k);
}

void SPMTunning::slot_DoFastMoveUp()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = getLandingMode() != lm01 ? 250 : 3;
    if (last_command == 2) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = 2;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goUpE(CurrentMotor, OptionsForMotor->up_speed * k);
}

void SPMTunning::slot_DoMoveDown()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = getLandingMode() != lm01 ? 250 : 3;
    if (last_command == -1) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = -1;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goDown(CurrentMotor, OptionsForMotor->up_speed * k, OptionsForMotor->up_dist * k);
}

void SPMTunning::slot_DoFastMoveDown()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = getLandingMode() != lm01 ? 250 : 3;
    if (last_command == -2) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = -2;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goDownE(CurrentMotor, OptionsForMotor->up_speed * k);
}


void SPMTunning::slot_DoLanding(bool st)
{
    QString text = tr("Landing Status: Waiting for response....");
    MobileServer::sendData(DataID::Land_status, text);

    last_command = 0;
    if (st) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);

        BYTE direction = (motors->orient123 != OptionsForMotor->landUpward) ? 0 : 1;
        double k = motors->Estepsrev->value() / 16 / motors->Epitch->value() / 1e3;
        k *= settings_double(settings, "divider/123", 1);
        Landing_mode = getLandingMode();
        Landing_direction = direction;
        Landing_speed = OptionsForMotor->land_speed * k;
        Landing_speed_close = OptionsForMotor->land_speed_close * k;
        Landing_close_perc = OptionsForMotor->land_close_perc;
        updLanding = true;


        switch (FB_channel) {
        case srcDFL: {
            double cur = 1.1*nDFL;
            double sp = 2.2*DCfbSetPoint->value() * spm_sp_units_gain;
            double pad = fabs(input_gain(srcDFL)) * (B_PID_sign->on_off || cur < sp ? -1 : 1);
            OscYrange[0] = qMin(qMin(sp, pad), cur);
            OscYrange[1] = qMax(qMax(sp, cur), pad);
            OscYrange[0] /= input_gain(srcDFL);
            OscYrange[1] /= input_gain(srcDFL);
        }
            break;
        case srcLFL: {
            double cur = 1.1*nLFL;
            double sp = 2.2*DCfbSetPoint->value() * spm_sp_units_gain;
            double pad = fabs(input_gain(srcLFL)) * (B_PID_sign->on_off || cur < sp ? -1 : 1);
            OscYrange[0] = qMin(qMin(sp, pad), cur);
            OscYrange[1] = qMax(qMax(sp, cur), pad);
            OscYrange[0] /= input_gain(srcLFL);
            OscYrange[1] /= input_gain(srcLFL);
        }
            break;
        case srcMag1:
            OscYrange[0] = 0;
            OscYrange[1] = qMax(1.1*nMAG, 2.2*DCfbSetPoint->value() * spm_sp_units_gain);
            OscYrange[0] /= input_gain(srcMag1);
            OscYrange[1] /= input_gain(srcMag1);
            break;
        }

        PlotMode = mode_Osc;
        sl_setPlotMode();


        if (!landing_bar) {
            landing_bar = new StatusBar(text, 0);
            connect(landing_bar, SIGNAL(canceled()), BLanding, SLOT(click()));
            landing_bar->add();
        } else
            landing_bar->statusLabel->setText(text);
    } else {
        Landing_mode = lmOff;
        updLanding = true;

        destructStatus(landing_bar);
    }
    destructStatus(landing_notification);
}

void SPMTunning::blanding_sync()
{
    if (BLanding->on_off != blanding_status)
        doBeep();

    BLanding->change_state_man(blanding_status);
    MobileServer::sendData(DataID::LandingOn, blanding_status);
}

void SPMTunning::slot_set_Landing_status(int status)
{
    destructStatus(landing_notification);

    QString text;
    switch (status)
    {
    case lndOff:
        text = tr("Landing Status: Canceled...");
        destructStatus(landing_bar);

        if (auto_land_state)
            PBAutoLand->change_state_man_with_sig(false);
        break;
    case lndLanding:
        text = tr("Landing Status: In progress...");
        if (landing_bar) {
            landing_bar->bar->setMaximum(0);
            Progress_timer->stop();
        }
        blanding_status = true;
        blanding_syncT->start(100);
        break;
    case lndDone:
        text = tr("Landing Status: Done...");
        destructStatus(landing_bar);
        landing_notification = new Notification(text, Notification::Good, 10);
        blanding_status = false;
        blanding_syncT->start(100);

        if (auto_land_state == 1)
            doAutoLand(); //to next step
        break;
    case lndError:
        text = tr("Landing Status: Error...");
        if (!BFeedbackOnOff->on_off) text += " " + tr("Feedback is disabled!");
        destructStatus(landing_bar);
        landing_notification = new Notification(text, Notification::Bad, 60);
        blanding_status = false;
        blanding_syncT->start(100);

        if (auto_land_state) {
            blanding_sync();
            PBAutoLand->change_state_man_with_sig(false);
        }
        break;
    case lndWaitDrift:
        text = tr("Landing Status: Checking stability");
        if (landing_bar) {
            landing_bar->bar->setMaximum(5*1000);
            landing_bar->bar->setValue(0);
            landing_bar->bar->setFormat("%p%");
            Progress_timer->start(100);
        }
        blanding_status = true;
        blanding_syncT->start(100);
        break;
    }

    if (landing_bar) landing_bar->statusLabel->setText(text);
    else Progress_timer->stop();
    MobileServer::sendData(DataID::Land_status, text);
}

void SPMTunning::s_updLandProgBar()
{
    if (!landing_bar)
        return;

    if (landing_bar->bar->maximum() && landing_bar->bar->value() < 5*1000)
        landing_bar->setValue(landing_bar->bar->value() + 100);
    else {
        Progress_timer->stop();
        landing_bar->bar->setMaximum(0);
    }
}

void SPMTunning::InitMotorControlSpace()
{
    last_command = 0;

    QPushButton *BMoveDown = new QPushButton(makeHugeIcon(":/SPMTuning_window/Down.png"), QString());
    BMoveDown->setToolTip(tr("Move Tip Down"));

    QPushButton *BMoveUp = new QPushButton(makeHugeIcon(":/SPMTuning_window/Up.png"), QString());
    BMoveUp->setToolTip(tr("Move Tip Up"));

    QPushButton *BFastMoveDown = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastDown.png"), QString());
    BFastMoveDown->setToolTip(tr("Fast Tip Down till end"));

    QPushButton *BFastMoveUp = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastUp.png"), QString());
    BFastMoveUp->setToolTip(tr("Fast Tip Up till end"));

    auto_land_state = 0;
    PBAutoLand = new TwoStateButton(":AutoStart.png", ":stop.png");
    PBAutoLand->setToolTip(tr("Smart landing"));
    connect(PBAutoLand, SIGNAL(state_changed(bool)), this, SLOT(doAutoLand()));

    BLanding = new TwoStateButton(":start.png",":stop.png");
    BLanding->setProperty("id", DataID::LandingOn);
    BLanding->setToolTip(tr("Tip Landing"));

    BMotors_leveling = new QPushButton();
    BMotors_leveling->setIcon(makeHugeIcon(":/SPMTuning_window/Motors_leveling.png"));
    BMotors_leveling->setToolTip(tr("Motors control"));

    BMotors_Options = new QPushButton();
    BMotors_Options->setIcon(makeHugeIcon(":/SPMTuning_window/tools.png"));
    BMotors_Options->setToolTip(tr("SPM/Motor Options"));

    QPushButton *BMotorsStop = new QPushButton();
    BMotorsStop->setIcon(makeHugeIcon(":/SPMTuning_window/approach/landing_done.png"));
    BMotorsStop->setToolTip(tr("Stop all motors"));

    connect(BMoveDown,     SIGNAL(clicked()), this, SLOT(slot_DoMoveDown()));
    connect(BMoveUp,       SIGNAL(clicked()), this, SLOT(slot_DoMoveUp()));
    connect(BFastMoveDown, SIGNAL(clicked()), this, SLOT(slot_DoFastMoveDown()));
    connect(BFastMoveUp,   SIGNAL(clicked()), this, SLOT(slot_DoFastMoveUp()));
    connect(BLanding,      SIGNAL(state_changed(bool)), this, SLOT(slot_DoLanding(bool)));
    connect(BMotorsStop, SIGNAL(clicked()), SLOT(motors_stop()));

    if (!show_additional_gui() && !show_advanced_gui()) {
        //very simple mode
        BMoveDown->hide();
        BFastMoveDown->hide();
        BMotorsStop->hide();
        BMotors_leveling->hide();
        BMotors_Options->setFlat(true);
        BMotors_Options->setIcon(QIcon());
    }

    GrBMotorControl = new QGroupBox(this);
    QVBoxLayout *Motor_Layout = new QVBoxLayout(GrBMotorControl);
    Motor_Layout->setContentsMargins(3, 3, 3, 3);
    Motor_Layout->setSpacing(6);
    Motor_Layout->addWidget(PBAutoLand);
    Motor_Layout->addWidget(BLanding);
    Motor_Layout->addWidget(BMoveUp);
    Motor_Layout->addWidget(BMoveDown);
    Motor_Layout->addWidget(BFastMoveUp);
    Motor_Layout->addWidget(BFastMoveDown);
    Motor_Layout->addWidget(BMotorsStop);

    addDelimeterH(Motor_Layout, BMoveUp->sizeHint().width());
    Motor_Layout->addWidget(BMotors_leveling);
    Motor_Layout->addWidget(BMotors_Options);
}



void SPMTunning::shuttle_button(int num)
{
    {
        QSettings settings("Param/shuttle.conf", QSettings::IniFormat);
        int remap = settings_double(settings, "remap/"+QString::number(num), -1);
        if (remap >= 0)
            num = remap;
    }

    //first row
    if (num == 0) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        active_shuttle_motor = settings_double(settings, "z_motor", 4); //legacy compat
        QList<int> l = Motors::getRoleMotors(Motors::RoleZ2Focus);
        if (l.size())
            active_shuttle_motor = l.first();
        piezo_shuttle = 0;
    }
    if (num == 1)
        active_shuttle_motor = 251;//piezo Z

    if (num == 2) {
        BFeedbackOnOff->change_state_man_with_sig(true);
    }
    if (num == 3) {
        BFeedbackOnOff->change_state_man_with_sig(false);
    }

    //second row
    if (num == 4) {
        active_shuttle_motor = getLandingMode() != lm01 ? 250 : 3; // all motors
        piezo_shuttle = 0;
    }
    if (num == 5) {
        active_shuttle_motor = 1;
        piezo_shuttle = 0;
    }
    if (num == 6) {
        active_shuttle_motor = 2;
        piezo_shuttle = 0;
    }
    if (num == 7) {
        active_shuttle_motor = 3;
        piezo_shuttle = 0;
    }

    if (num == 8) {
        BLanding->click();
        blanding_status = BLanding->on_off;
    }

    if (num == 9) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        active_shuttle_motor = settings_double(settings, "x_motor", 7); //legacy compat
        QList<int> l = Motors::getRoleMotors(Motors::RoleX2);
        if (l.size())
            active_shuttle_motor = l.first();
        piezo_shuttle = 0;
    }

    if (num == 10)
        active_shuttle_motor = 252;//piezo X
    if (num == 11)
        active_shuttle_motor = 254;//piezo X2
    if (num == 13)
        active_shuttle_motor = 253;//piezo Y
    if (num == 12)
        active_shuttle_motor = 255;//piezo Y2

    if (num == 14) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        active_shuttle_motor = settings_double(settings, "y_motor", 8); //legacy compat
        QList<int> l = Motors::getRoleMotors(Motors::RoleY2);
        if (l.size())
            active_shuttle_motor = l.first();
        piezo_shuttle = 0;
    }


    if (num > 100 && num < 120) {
        active_shuttle_motor = qMin(16, num-100);
        piezo_shuttle = 0;
    }

    if (num == 250) {
        active_shuttle_motor = getLandingMode() != lm01 ? 250 : 3; // all motors
        piezo_shuttle = 0;
    }

    if (num > 250)
        active_shuttle_motor = num;


    upd_shuttle_status();
}
void SPMTunning::upd_shuttle_status()
{
    if (!shuttleStatus)
        return;

    int idx = -1;
    QString name;

    if (active_shuttle_motor == 251) //piezo Z
        name = "Z-P";
    if (active_shuttle_motor == 252) //piezo X
        name = "X-P";
    if (active_shuttle_motor == 254) //piezo X2
        name = "X2-P";
    if (active_shuttle_motor == 253) //piezo Y
        name = "Y-P";
    if (active_shuttle_motor == 255) //piezo Y2
        name = "Y2-P";

    if (active_shuttle_motor == 250)
        name = "123";

    if (active_shuttle_motor < 250)
        idx = active_shuttle_motor;

    QString n1 = tr("Shuttle: ");
    if (idx > 0) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        QString stridx = QString::number(idx);
        QString name1 = settings.value("name/" + stridx).toString();
        if (!name1.isEmpty()) name = name1;

        n1 += stridx;
        if (!name.isEmpty()) n1 += " - ";
    }
    n1 += name;
    if (idx > 0 || !name.isEmpty())
        shuttleStatus->setText(n1);
}


#include "oscilloscope.h"
void SPMTunning::shuttle_wheel(int num)
{
    num *= -1;
    if (active_shuttle_motor < 251) {
        motors->goShuttleStep(num, active_shuttle_motor);
    } else {
        double sign = num > 0 ? 1 : -1;
        double speed = piezo_speed;
        if (abs(num) != 1)
            speed = speeds[abs(num)-2];

        I_MODULE *oscm = ModuleManager::instance->find1("oscilloscope");
        if (!oscm)
            return;
        Oscilloscope *osc = (Oscilloscope *)oscm->basecast();
        DoubleChanger *dc = 0;

        switch (active_shuttle_motor) {
        case 251:
            dc = osc->DacsWidget->Zobj;
            break;
        case 252:
            dc = osc->DacsWidget->X;
            break;
        case 253:
            dc = osc->DacsWidget->Y;
            break;
        case 254:
            dc = osc->DacsWidget->X2;
            break;
        case 255:
            dc = osc->DacsWidget->Y2;
            break;
        default:
            return;
        }

        dc->setValue(dc->value() + sign*speed);
        dc->setNewValue();
    }
}

void SPMTunning::shuttle_position_piezo_timeout()
{
    if (active_shuttle_motor >= 251 && piezo_shuttle != 0)
        shuttle_wheel(piezo_shuttle);
}

void SPMTunning::shuttle_position(int num)
{
    num *= -1;
    if (active_shuttle_motor < 251) {
        motors->goShuttle(num, active_shuttle_motor);
    } else {
        piezo_shuttle = num;
        double sign = piezo_shuttle > 0 ? 1 : -1;
        if (num != 0)
            piezo_shuttle = sign*(abs(piezo_shuttle) + 1);
    }
}

void SPMTunning::InitShuttleGuiLink()
{
    shuttleStatus = new QLabel(tr("Shuttle: ") + "123");
    active_shuttle_motor = getLandingMode() != lm01 ? 250 : 3; // all motors
    piezo_shuttle = 0;

    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);

    piezo_speed = settings_double(settings, "piezo/step", 1e-1);
    speeds[0] = speeds[1] = speeds[2] = 1e-1;
    speeds[3] = 1; speeds[4] = 2; speeds[5] = 5; speeds[6] = 10;
    for (int i = 0; i < 7; i++)
        speeds[i] = settings_double(settings, "piezo/scale"
                                   + QString::number(i), speeds[i]) /
                (1000.0/settings_double(settings, "piezo/delay", 50));

    QTimer *piezo_shuttle_timer = new QTimer(this);
    connect(piezo_shuttle_timer, SIGNAL(timeout()), this, SLOT(shuttle_position_piezo_timeout()));
    piezo_shuttle_timer->start(settings_double(settings, "piezo/delay", 50));

    ShuttleControllerNative *hidinput = new ShuttleControllerNative(this);
    connect(hidinput, SIGNAL(button_press(int)), this, SLOT(shuttle_button(int)));
    connect(hidinput, SIGNAL(wheel_event(int)), this, SLOT(shuttle_wheel(int)));
    connect(hidinput, SIGNAL(shuttle_position(int)), this, SLOT(shuttle_position(int)));
}

void SPMTunning::doAutoLand()
{
    destructStatus(autoland_notification);
    if (!PBAutoLand->on_off) {
        if (BLanding->on_off)
            BLanding->change_state_man_with_sig(false);
        if (BFeedbackOnOffMode->on_off && BFeedbackOnOff->on_off) {
            BFeedbackOnOff->change_state_man_with_sig(false);
            BFeedbackOnOffMode->change_state_man_with_sig(false);
        }
        //TODO state-related reset
        if (auto_land_state > 0) {
            slot_DoMoveUp(); //recede from landed state
        }
        auto_land_state = 0;
        return;
    }

    //sanity checks, then landing
    if (auto_land_state == 0) {
        bool fault = false;
        QString error;

        if (BFeedbackOnOff->on_off && nZ < 0.9) {
            slot_DoMoveUp(); //up

            QElapsedTimer t; t.start();
            while (last_command == 1 && t.elapsed() < 250) {
                qApp->processEvents();
                SleepExport::msleep2(10);
            }
            if (last_command == 1)
                slot_DoMoveUp(); //stop
        }

        bool is_tuning_fork = currModeIdx == 10;
        bool is_mag = (currModeIdx >= 2 && currModeIdx <= 5) || (currModeIdx == 10);
        bool is_stm = currModeIdx >= 6 && currModeIdx <= 9;
        if (!is_stm && !is_tuning_fork && (nDFLr > 0.1 || nDFLr < -0.1 || nLFLr > 0.1 || nLFLr < -0.1)) {
            fault = true;
            error = tr("Laser not centered!");
        }
        if (!is_stm && !is_tuning_fork && nInt < 0.2) {
            fault = true;
            error = tr("Weak laser signal!");
        }
        if (is_mag && !fault) {
            double oldmag = nMAG;

            CB_fr_Channel->setCurrentIndex(CB_fr_Channel->findData(srcMag1));
            PlotMode = mode_ResCurve;
            sl_setPlotMode();

            if (!s_FreqScanStartAuto(true) && !s_FreqScanStartAuto()) {
                fault = true;
                error = tr("No resonance!");
            } else {
                setEnabled(false);
                QElapsedTimer t; t.start();
                while (t.elapsed() < 250) {
                    qApp->processEvents();
                    SleepExport::msleep2(10);
                }
                setEnabled(true);

                if (nMAG / input_gain(srcMag1) < 0.3) {
                    fault = true;
                    error = tr("Weak resonance!");
                }

                if (qFabs(nMAG - oldmag) > 0.1 * nMAG && !fault) {
                    DCfbSetPoint->setValue(nMAG * 0.6 / spm_sp_units_gain);
                    DCfbSetPoint->setNewValue();
                }
            }
        }

        if (BFeedbackOnOff->on_off && !fault) {
            bool spcheck = true;
            double dp = DCfbSetPoint->value() * spm_sp_units_gain;
            switch (FB_channel) {
            case srcDFL:
                dp -= nDFL;
                break;
            case srcLFL:
                dp -= nLFL;
                break;
            case srcMag1:
                dp -= nMAG;
                break;
            default:
                spcheck = false;
                break;
            }
            bool sign = Mode_data[currModeIdx][4];
            if (sign)
                dp *= -1;
            if (nZ < 0.2 || (spcheck && dp / input_gain(FB_channel) >= 0.01)) {
                fault = true;
                error = tr("Wrong feedback sign or setpoint!");
            }
        }

        if (fault) {
            autoland_notification = new Notification(tr("Smart landing") + ": " + error, Notification::Warn, 30);
            doBeep();
            PBAutoLand->change_state_man(false);
            return;
        }

        if (!BFeedbackOnOffMode->on_off)
            BFeedbackOnOffMode->change_state_man_with_sig(true);
        if (!BFeedbackOnOff->on_off) {
            double sp = DCfbSetPoint->value() * spm_sp_units_gain;
            switch (FB_channel) {
            case srcDFL:
                sp = nDFL - 1 * input_gain(FB_channel);
                break;
            case srcMag1:
                sp = nMAG * 0.6;
                break;
            }
            DCfbSetPoint->setValue(sp / spm_sp_units_gain);
            DCfbSetPoint->setNewValue();

            BFeedbackOnOff->change_state_man_with_sig(true);

            setEnabled(false);
            QElapsedTimer t; t.start();
            while (t.elapsed() < 1000) {
                qApp->processEvents();
                SleepExport::msleep2(10);
            }
            setEnabled(true);
        }
        if (DCfbHeight->value() < 0.5) {
            DCfbHeight->setValue(0.5);
            DCfbHeight->setNewValue();
        }

        BLanding->change_state_man_with_sig(true);

        auto_land_state = 1;
        return;
    }

    //landing has completed, start spectro
    if (auto_land_state == 1) {
        PlotMode = mode_Spectroscopy;
        sl_setPlotMode();
        BSpectrJM->change_state_man_with_sig(true);
        JMdiv->setValue(qFloor(DCfbHeight->value() * 1e3 / DCfbPrec->value() / CB_Points_spectr->currentText().toInt()));
        JMdiv->setNewValue();
        PBSpectr_start_stop->change_state_man_with_sig(true);

        auto_land_state = 2;
        return;
    }

    //spectroscopy has completed, calc PI
    if (auto_land_state == 2) {
        if (!doPICalib()) {
            PBAutoLand->change_state_man_with_sig(false);

            autoland_notification = new Notification(tr("Smart landing") + ": " + tr("PI calc error"), Notification::Bad, 30);
            doBeep();
            return;
        }

        BFeedbackOnOffMode->change_state_man_with_sig(false);

        if (!doCheckFixFBosc()) {
            BFeedbackOnOff->change_state_man_with_sig(false);
            PBAutoLand->change_state_man_with_sig(false);

            autoland_notification = new Notification(tr("Smart landing") + ": " + tr("FB is oscillating or noisy"), Notification::Bad, 30);
            doBeep();
            return;
        }

        autoland_notification = new Notification(tr("Smart landing") + ": " + tr("Success"), Notification::Good, 30);
        doBeep();

        auto_land_state = 0;
        PBAutoLand->change_state_man(false);
        return;
    }
}
